;;; loaddefs.el --- define standard autoloads of other files

;; Copyright (C) 1985, 1986, 1987, 1992-1995 Free Software Foundation, Inc.

;; Maintainer: XEmacs
;; Keywords: internal

;; This file is part of XEmacs.

;; XEmacs is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; XEmacs is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with XEmacs; see the file COPYING.  If not, write to the Free
;; Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Synched up with: Not synched with FSF.

;;; Commentary:

;;; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;;; Special formatting conventions are used in this file!
;;;
;;; a backslash-newline is used at the beginning of a documentation string
;;; when that string should be stored in the file lib-src/DOCnnn, not in core.
;;;
;;; Such strings read into Lisp as numbers (during the pure-loading phase).
;;;
;;; But you must obey certain rules to make sure the string is understood
;;; and goes into lib-src/DOCnnn properly.  Otherwise, the string will not go
;;; anywhere!
;;;
;;; The doc string must appear in the standard place in a call to
;;; defun, autoload, defvar or defconst.  No Lisp macros are recognized.
;;; The open-paren starting the definition must appear in column 0.
;;;
;;; In defvar and defconst, there is an additional rule:
;;; The double-quote that starts the string must be on the same
;;; line as the defvar or defconst.
;;; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

;;; **********************************************************************
;;; You should never need to write autoloads by hand and put them here.
;;;
;;; It is no longer necessary.  Instead use autoload.el to maintain them
;;; for you.  Just insert ";;;###autoload" before defuns or defmacros you
;;; want to be autoloaded, or other forms you want copied into loaddefs.el
;;; (defvars, key definitions, etc.).  For example, 
;;;	;;;###autoload
;;;	(defun foobar () ....)
;;;	;;;###autoload (define-key global-map "f" 'foobar)
;;;	;;;###autoload
;;;	(defvar foobar-var nil "\
;;;	This is foobar-var's doc-string.")
;;;
;;; Then do M-x update-file-autoloads on the file to update loaddefs.el.
;;;
;;; You can also use M-x update-directory-autoloads to update the autoloads
;;; in loaddefs.el for all .el files in the lisp/ directory, or M-x
;;; update-autoloads-here to update the autoloads for each file that
;;; already has an autoload section in this file.
;;; **********************************************************************

;;; Code:

;; These variables are used by autoloadable packages.
;; They are defined here so that they do not get overridden
;; by the loading of those packages.

;; Names in directory that end in one of these
;; are ignored in completion,
;; making it more likely you will get a unique match.
(setq completion-ignored-extensions
      (mapcar 'purecopy
	      (if (eq system-type 'vax-vms)
		  '(".obj" ".elc" ".exe" ".bin" ".lbin" ".sbin"
		    ".dvi" ".toc" ".log" ".aux"
		    ".lof" ".brn" ".rnt" ".mem" ".lni" ".lis"
		    ".olb" ".tlb" ".mlb" ".hlb" ".glo" ".idx" ".lot" ".fmt")
		'(".o" ".elc" "~" ".bin" ".lbin" ".fasl"
		  ".dvi" ".toc" ".log" ".aux" ".a" ".ln"
		  ".lof" ".blg" ".bbl" ".glo" ".idx" ".lot" ".fmt"
		  ".diff" ".oi"))))

(make-variable-buffer-local 'indent-tabs-mode)

;;; This code also was not generated by autoload.el, because VM goes out
;;; of its way to be perverse.

(autoload 'vm "vm"
   "\
View Mail: an alternate mail reader for emacs.
Optional first arg FOLDER specifies the folder to visit.  It defaults
to the value of vm-primary-inbox.  The folder buffer is put into VM
mode, a major mode for reading mail.

Prefix arg or optional second arg READ-ONLY non-nil indicates
that the folder should be considered read only.  No attribute
changes, messages additions or deletions will be allowed in the
visited folder.

Visiting the primary inbox causes any contents of the system mailbox to
be moved and appended to the resulting buffer.

All the messages can be read by repeatedly pressing SPC.  Use `n'ext and
`p'revious to move about in the folder.  Messages are marked for
deletion with `d', and saved to another folder with `s'.  Quitting VM
with `q' expunges deleted messages and saves the buffered folder to
disk.

See the documentation for vm-mode for more information."
 t)

(autoload 'vm-mode "vm" 
  "\
View Mail: an alternate mail reader for emacs.

Commands:
   h - summarize folder contents
   j - discard cached information about the current message

   n - go to next message
   p - go to previous message
   N - like `n' but ignores skip-variable settings
   P - like `p' but ignores skip-variable settings
 M-n - go to next unread message
 M-p - go to previous unread message
 RET - go to numbered message (uses prefix arg or prompts in minibuffer)
 TAB - go to last message seen
 M-s - incremental search through the folder

   t - display hidden headers
 SPC - scroll forward a page (if at end of message, then display next message)
   b - scroll backward a page
   < - go to beginning of current message
   > - go to end of current message

   d - delete message, prefix arg deletes messages forward (flag as deleted)
 C-d - delete message, prefix arg deletes messages backward (flag as deleted)
   u - undelete
   k - flag for deletion all messages with same subject as the current message

   r - reply (only to the sender of the message)
   R - reply with included text for current message
 M-r - extract and resend bounced message
   f - followup (reply to all recipients of message)
   F - followup with included text from the current message
   z - forward the current message
   m - send a message
   B - resend the current message to another user.
   c - continue composing the most recent message you were composing

   @ - digestify and mail entire folder contents (the folder is not modified)
   * - burst a digest into individual messages, and append and assimilate these
       message into the current folder.

   G - sort messages by various keys

   g - get any new mail that has arrived in the system mailbox
       (new mail is appended to the disk and buffer copies of the
       primary inbox.)
   v - visit another mail folder
   V - visit a virtual folder

   e - edit the current message

   s - save current message in a folder (appends if folder already exists)
   w - write current message to a file without its headers (appends if exists)
   S - save entire folder to disk, expunging deleted messages
   A - save unfiled messages to their vm-auto-folder-alist specified folders
   # - expunge deleted messages (without saving folder)
   q - quit VM, deleted messages are expunged, folder saved to disk
   x - exit VM with no change to the folder

 M N - use marks; the next vm command will affect only marked messages
       if it makes sense for the command to do so

       M M - mark the current message
       M U - unmark the current message
       M m - mark all messages
       M u - unmark all messages
       M ? - help for the mark commands

 W S - save the current window configuration to a name
 W D - delete a window configuration
 W W - apply a configuration
 W ? - help for the window configuration commands

 C-_ - undo, special undo that retracts the most recent
             changes in message attributes.  Expunges and saves
             cannot be undone.  C-x u is also bound to this
             command.

   L - reload your VM init file, ~/.vm

   ? - help

   ! - run a shell command
   | - run a shell command with the current message as input

 M-C - view conditions under which you may redistribute VM
 M-W - view the details of VM's lack of a warranty

Variables:
   vm-auto-center-summary
   vm-auto-folder-alist
   vm-auto-folder-case-fold-search
   vm-auto-get-new-mail
   vm-auto-next-message
   vm-berkeley-mail-compatibility
   vm-check-folder-types
   vm-convert-folder-types
   vm-circular-folders
   vm-confirm-new-folders
   vm-confirm-quit
   vm-crash-box
   vm-delete-after-archiving
   vm-delete-after-bursting
   vm-delete-after-saving
   vm-delete-empty-folders
   vm-digest-burst-type
   vm-digest-center-preamble
   vm-digest-preamble-format
   vm-digest-send-type
   vm-folder-directory
   vm-folder-read-only
   vm-follow-summary-cursor
   vm-forwarded-headers
   vm-forwarding-digest-type
   vm-forwarding-subject-format
   vm-gargle-uucp
   vm-highlighted-header-regexp
   vm-honor-page-delimiters
   vm-in-reply-to-format
   vm-included-text-attribution-format
   vm-included-text-prefix
   vm-inhibit-startup-message
   vm-invisible-header-regexp
   vm-jump-to-new-messages
   vm-jump-to-unread-messages
   vm-keep-sent-messages
   vm-mail-header-from
   vm-mail-mode-hook
   vm-mail-window-percentage
   vm-mode-hook
   vm-move-after-deleting
   vm-move-after-undeleting
   vm-mutable-windows
   vm-preview-lines
   vm-preview-read-messages
   vm-primary-inbox
   vm-recognize-pop-maildrops
   vm-reply-ignored-addresses
   vm-reply-subject-prefix
   vm-resend-bounced-headers
   vm-resend-bounced-discard-header-regexp
   vm-resend-headers
   vm-resend-discard-header-regexp
   vm-retain-message-order
   vm-rfc1153-digest-discard-header-regexp
   vm-rfc1153-digest-headers
   vm-rfc934-digest-discard-header-regexp
   vm-rfc934-digest-headers
   vm-search-using-regexps
   vm-skip-deleted-messages
   vm-skip-read-messages
   vm-spool-files
   vm-startup-with-summary
   vm-strip-reply-headers
   vm-summary-format
   vm-unforwarded-header-regexp
   vm-virtual-folder-alist
   vm-virtual-mirror
   vm-visible-headers
   vm-visit-when-saving
   vm-window-configuration-file
"
 t)

(autoload 'vm-visit-folder "vm" 
  "\
Visit a mail file with View Mail, an alternate mail reader for emacs.
See the description of the `vm' and `vm-mode' functions.

VM will parse and present its messages to you in the usual way.

First arg FOLDER specifies the mail file to visit.  When this
command is called interactively the file name is read from the
minibuffer.

Prefix arg or optional second arg READ-ONLY non-nil indicates
that the folder should be considered read only.  No attribute
changes, messages additions or deletions will be allowed in the
visited folder."
  t)

(autoload 'vm-mail "vm"
  "\
Send a mail message from within View Mail, or from without."
  t)



;;; Generated autoloads follow (made by autoload.el).

;;; To sort them, execute the following after narrowing
;;; to a region starting just after the following formfeed (control-l)
;;; and ending just after the last formfeed in the file.

;;;(sort-regexp-fields nil "\n*.*\n.*from \\(.*\\)[^]*" "\\1"
;;;		    (point-min) (point-max))

;;;***

;;;### (autoloads nil "byte-optimize" "bytecomp/byte-optimize.el" (12558 60872))
;;; Generated autoloads from bytecomp/byte-optimize.el

;;;***

;;;### (autoloads nil "bytecomp-runtime" "bytecomp/bytecomp-runtime.el" (12558 60873))
;;; Generated autoloads from bytecomp/bytecomp-runtime.el

;;;### (autoloads (batch-byte-recompile-directory batch-byte-recompile-directory-norecurse batch-byte-compile display-call-tree byte-compile-sexp byte-compile compile-defun byte-compile-file byte-recompile-file byte-recompile-directory byte-force-recompile) "bytecomp" "bytecomp/bytecomp.el" (12639 8562))
;;; Generated autoloads from bytecomp/bytecomp.el

(autoload 'byte-force-recompile "bytecomp" "\
Recompile every `.el' file in DIRECTORY that already has a `.elc' file.
Files in subdirectories of DIRECTORY are processed also." t nil)

(autoload 'byte-recompile-directory "bytecomp" "\
Recompile every `.el' file in DIRECTORY that needs recompilation.
This is if a `.elc' file exists but is older than the `.el' file.
Files in subdirectories of DIRECTORY are processed also unless argument
NORECURSION is non-nil.

If the `.elc' file does not exist, normally the `.el' file is *not* compiled.
But a prefix argument (optional second arg) means ask user,
for each such `.el' file, whether to compile it.  Prefix argument 0 means
don't ask and compile the file anyway.

A nonzero prefix argument also means ask about each subdirectory.

If the fourth argument FORCE is non-nil,
recompile every `.el' file that already has a `.elc' file." t nil)

(autoload 'byte-recompile-file "bytecomp" "\
Recompile a file of Lisp code named FILENAME if it needs recompilation.
This is if the `.elc' file exists but is older than the `.el' file.

If the `.elc' file does not exist, normally the `.el' file is *not*
compiled.  But a prefix argument (optional second arg) means ask user
whether to compile it.  Prefix argument 0 don't ask and recompile anyway." t nil)

(autoload 'byte-compile-file "bytecomp" "\
Compile a file of Lisp code named FILENAME into a file of byte code.
The output file's name is made by appending `c' to the end of FILENAME.
With prefix arg (noninteractively: 2nd arg), load the file after compiling." t nil)

(autoload 'compile-defun "bytecomp" "\
Compile and evaluate the current top-level form.
Print the result in the minibuffer.
With argument, insert value in current buffer after the form." t nil)

(autoload 'byte-compile "bytecomp" "\
If FORM is a symbol, byte-compile its function definition.
If FORM is a lambda or a macro, byte-compile it as a function." nil nil)

(autoload 'byte-compile-sexp "bytecomp" "\
Compile and return SEXP." nil nil)

(autoload 'display-call-tree "bytecomp" "\
Display a call graph of a specified file.
This lists which functions have been called, what functions called
them, and what functions they call.  The list includes all functions
whose definitions have been compiled in this Emacs session, as well as
all functions called by those functions.

The call graph does not include macros, inline functions, or
primitives that the byte-code interpreter knows about directly (eq,
cons, etc.).

The call tree also lists those functions which are not known to be called
\(that is, to which no calls have been compiled), and which cannot be
invoked interactively." t nil)

(autoload 'batch-byte-compile "bytecomp" "\
Run `byte-compile-file' on the files remaining on the command line.
Use this from the command line, with `-batch';
it won't work in an interactive Emacs.
Each file is processed even if an error occurred previously.
For example, invoke \"emacs -batch -f batch-byte-compile $emacs/ ~/*.el\"" nil nil)

(autoload 'batch-byte-recompile-directory-norecurse "bytecomp" "\
Same as `batch-byte-recompile-directory' but without recursion." nil nil)

(autoload 'batch-byte-recompile-directory "bytecomp" "\
Runs `byte-recompile-directory' on the dirs remaining on the command line.
Must be used only with `-batch', and kills Emacs on completion.
For example, invoke `xemacs -batch -f batch-byte-recompile-directory .'." nil nil)

;;;***

;;;### (autoloads (disassemble) "disass" "bytecomp/disass.el" (12376 19373))
;;; Generated autoloads from bytecomp/disass.el

(autoload 'disassemble "disass" "\
Print disassembled code for OBJECT in (optional) BUFFER.
OBJECT can be a symbol defined as a function, or a function itself
\(a lambda expression or a compiled-function object).
If OBJECT is not already compiled, we compile it, but do not
redefine OBJECT if it is a symbol." t nil)

;;;***

;;;### (autoloads nil "appt" "calendar/appt.el" (12376 19375))
;;; Generated autoloads from calendar/appt.el

;;;***

;;;### (autoloads nil "cal-french" "calendar/cal-french.el" (12376 19376))
;;; Generated autoloads from calendar/cal-french.el

;;;***

;;;### (autoloads nil "cal-mayan" "calendar/cal-mayan.el" (12376 19375))
;;; Generated autoloads from calendar/cal-mayan.el

;;;### (autoloads nil "cal-x" "calendar/cal-x.el" (12376 19380))
;;; Generated autoloads from calendar/cal-x.el

(defvar calendar-setup 'one-frame "\
The frame set up of the calendar.
The choices are `one-frame' (calendar and diary together in one separate,
dediciated frame) or `two-frames' (calendar and diary in separate, dedicated
frames); with any other value the current frame is used.")

;;;***

;;;### (autoloads nil "cal-xemacs" "calendar/cal-xemacs.el" (12558 60876))
;;; Generated autoloads from calendar/cal-xemacs.el

;;;### (autoloads nil "cal-dst" "calendar/cal-dst.el" (12376 19378))
;;; Generated autoloads from calendar/cal-dst.el

(put 'calendar-daylight-savings-starts 'risky-local-variable t)

(put 'calendar-daylight-savings-ends 'risky-local-variable t)

;;;***

;;;### (autoloads (list-yahrzeit-dates calendar) "calendar" "calendar/calendar.el" (12478 30570))
;;; Generated autoloads from calendar/calendar.el

(defvar calendar-week-start-day 0 "\
*The day of the week on which a week in the calendar begins.
0 means Sunday (default), 1 means Monday, and so on.")

(defvar calendar-offset 0 "\
*The offset of the principal month from the center of the calendar window.
0 means the principal month is in the center (default), -1 means on the left,
+1 means on the right.  Larger (or smaller) values push the principal month off
the screen.")

(defvar view-diary-entries-initially nil "\
*Non-nil means display current date's diary entries on entry.
The diary is displayed in another window when the calendar is first displayed,
if the current date is visible.  The number of days of diary entries displayed
is governed by the variable `number-of-diary-entries'.")

(defvar number-of-diary-entries 1 "\
*Specifies how many days of diary entries are to be displayed initially.
This variable affects the diary display when the command M-x diary is used,
or if the value of the variable `view-diary-entries-initially' is t.  For
example, if the default value 1 is used, then only the current day's diary
entries will be displayed.  If the value 2 is used, then both the current
day's and the next day's entries will be displayed.

The value can also be a vector such as [0 2 2 2 2 4 1]; this value
says to display no diary entries on Sunday, the display the entries
for the current date and the day after on Monday through Thursday,
display Friday through Monday's entries on Friday, and display only
Saturday's entries on Saturday.

This variable does not affect the diary display with the `d' command
from the calendar; in that case, the prefix argument controls the
number of days of diary entries displayed.")

(defvar mark-diary-entries-in-calendar nil "\
*Non-nil means mark dates with diary entries, in the calendar window.
The marking symbol is specified by the variable `diary-entry-marker'.")

(defvar view-calendar-holidays-initially nil "\
*Non-nil means display holidays for current three month period on entry.
The holidays are displayed in another window when the calendar is first
displayed.")

(defvar mark-holidays-in-calendar nil "\
*Non-nil means mark dates of holidays in the calendar window.
The marking symbol is specified by the variable `calendar-holiday-marker'.")

(defvar all-hebrew-calendar-holidays nil "\
*If nil, show only major holidays from the Hebrew calendar.
This means only those Jewish holidays that appear on secular calendars.

If t, show all the holidays that would appear in a complete Hebrew calendar.")

(defvar all-christian-calendar-holidays nil "\
*If nil, show only major holidays from the Christian calendar.
This means only those Christian holidays that appear on secular calendars.

If t, show all the holidays that would appear in a complete Christian
calendar.")

(defvar all-islamic-calendar-holidays nil "\
*If nil, show only major holidays from the Islamic calendar.
This means only those Islamic holidays that appear on secular calendars.

If t, show all the holidays that would appear in a complete Islamic
calendar.")

(defvar calendar-load-hook nil "\
*List of functions to be called after the calendar is first loaded.
This is the place to add key bindings to `calendar-mode-map'.")

(defvar initial-calendar-window-hook nil "\
*List of functions to be called when the calendar window is first opened.
The functions invoked are called after the calendar window is opened, but
once opened is never called again.  Leaving the calendar with the `q' command
and reentering it will cause these functions to be called again.")

(defvar today-visible-calendar-hook nil "\
*List of functions called whenever the current date is visible.
This can be used, for example, to replace today's date with asterisks; a
function `calendar-star-date' is included for this purpose:
    (setq today-visible-calendar-hook 'calendar-star-date)
It can also be used to mark the current date with `calendar-today-marker';
a function is also provided for this:
    (setq today-visible-calendar-hook 'calendar-mark-today)

The corresponding variable `today-invisible-calendar-hook' is the list of
functions called when the calendar function was called when the current
date is not visible in the window.

Other than the use of the provided functions, the changing of any
characters in the calendar buffer by the hooks may cause the failure of the
functions that move by days and weeks.")

(defvar today-invisible-calendar-hook nil "\
*List of functions called whenever the current date is not visible.

The corresponding variable `today-visible-calendar-hook' is the list of
functions called when the calendar function was called when the current
date is visible in the window.

Other than the use of the provided functions, the changing of any
characters in the calendar buffer by the hooks may cause the failure of the
functions that move by days and weeks.")

(defvar diary-file "~/diary" "\
*Name of the file in which one's personal diary of dates is kept.

The file's entries are lines in any of the forms

            MONTH/DAY
            MONTH/DAY/YEAR
            MONTHNAME DAY
            MONTHNAME DAY, YEAR
            DAYNAME

at the beginning of the line; the remainder of the line is the diary entry
string for that date.  MONTH and DAY are one or two digit numbers, YEAR is
a number and may be written in full or abbreviated to the final two digits.
If the date does not contain a year, it is generic and applies to any year.
DAYNAME entries apply to any date on which is on that day of the week.
MONTHNAME and DAYNAME can be spelled in full, abbreviated to three
characters (with or without a period), capitalized or not.  Any of DAY,
MONTH, or MONTHNAME, YEAR can be `*' which matches any day, month, or year,
respectively.

The European style (in which the day precedes the month) can be used
instead, if you execute `european-calendar' when in the calendar, or set
`european-calendar-style' to t in your .emacs file.  The European forms are

            DAY/MONTH
            DAY/MONTH/YEAR
            DAY MONTHNAME
            DAY MONTHNAME YEAR
            DAYNAME

To revert to the default American style from the European style, execute
`american-calendar' in the calendar.

A diary entry can be preceded by the character
`diary-nonmarking-symbol' (ordinarily `&') to make that entry
nonmarking--that is, it will not be marked on dates in the calendar
window but will appear in a diary window.

Multiline diary entries are made by indenting lines after the first with
either a TAB or one or more spaces.

Lines not in one the above formats are ignored.  Here are some sample diary
entries (in the default American style):

     12/22/1988 Twentieth wedding anniversary!!
     &1/1. Happy New Year!
     10/22 Ruth's birthday.
     21: Payday
     Tuesday--weekly meeting with grad students at 10am
              Supowit, Shen, Bitner, and Kapoor to attend.
     1/13/89 Friday the thirteenth!!
     &thu 4pm squash game with Lloyd.
     mar 16 Dad's birthday
     April 15, 1989 Income tax due.
     &* 15 time cards due.

If the first line of a diary entry consists only of the date or day name with
no trailing blanks or punctuation, then that line is not displayed in the
diary window; only the continuation lines is shown.  For example, the
single diary entry

     02/11/1989
      Bill Blattner visits Princeton today
      2pm Cognitive Studies Committee meeting
      2:30-5:30 Lizzie at Lawrenceville for `Group Initiative'
      4:00pm Jamie Tappenden
      7:30pm Dinner at George and Ed's for Alan Ryan
      7:30-10:00pm dance at Stewart Country Day School

will appear in the diary window without the date line at the beginning.  This
facility allows the diary window to look neater, but can cause confusion if
used with more than one day's entries displayed.

Diary entries can be based on Lisp sexps.  For example, the diary entry

      %%(diary-block 11 1 1990 11 10 1990) Vacation

causes the diary entry \"Vacation\" to appear from November 1 through November
10, 1990.  Other functions available are `diary-float', `diary-anniversary',
`diary-cyclic', `diary-day-of-year', `diary-iso-date', `diary-french-date',
`diary-hebrew-date', `diary-islamic-date', `diary-mayan-date',
`diary-yahrzeit', `diary-sunrise-sunset', `diary-phases-of-moon',
`diary-parasha', `diary-omer', `diary-rosh-hodesh', and
`diary-sabbath-candles'.  See the documentation for the function
`list-sexp-diary-entries' for more details.

Diary entries based on the Hebrew and/or the Islamic calendar are also
possible, but because these are somewhat slow, they are ignored
unless you set the `nongregorian-diary-listing-hook' and the
`nongregorian-diary-marking-hook' appropriately.  See the documentation
for these functions for details.

Diary files can contain directives to include the contents of other files; for
details, see the documentation for the variable `list-diary-entries-hook'.")

(defvar diary-nonmarking-symbol "&" "\
*Symbol indicating that a diary entry is not to be marked in the calendar.")

(defvar hebrew-diary-entry-symbol "H" "\
*Symbol indicating a diary entry according to the Hebrew calendar.")

(defvar islamic-diary-entry-symbol "I" "\
*Symbol indicating a diary entry according to the Islamic calendar.")

(defvar diary-include-string "#include" "\
*The string indicating inclusion of another file of diary entries.
See the documentation for the function `include-other-diary-files'.")

(defvar sexp-diary-entry-symbol "%%" "\
*The string used to indicate a sexp diary entry in diary-file.
See the documentation for the function `list-sexp-diary-entries'.")

(defvar abbreviated-calendar-year t "\
*Interpret a two-digit year DD in a diary entry as either 19DD or 20DD.
For the Gregorian calendar; similarly for the Hebrew and Islamic calendars.
If this variable is nil, years must be written in full.")

(defvar european-calendar-style nil "\
*Use the European style of dates in the diary and in any displays.
If this variable is t, a date 1/2/1990 would be interpreted as February 1,
1990.  The accepted European date styles are

            DAY/MONTH
            DAY/MONTH/YEAR
            DAY MONTHNAME
            DAY MONTHNAME YEAR
            DAYNAME

Names can be capitalized or not, written in full, or abbreviated to three
characters with or without a period.")

(defvar american-date-diary-pattern '((month "/" day "[^/0-9]") (month "/" day "/" year "[^0-9]") (monthname " *" day "[^,0-9]") (monthname " *" day ", *" year "[^0-9]") (dayname "\\W")) "\
*List of pseudo-patterns describing the American patterns of date used.
See the documentation of `diary-date-forms' for an explanation.")

(defvar european-date-diary-pattern '((day "/" month "[^/0-9]") (day "/" month "/" year "[^0-9]") (backup day " *" monthname "\\W+\\<[^*0-9]") (day " *" monthname " *" year "[^0-9]") (dayname "\\W")) "\
*List of pseudo-patterns describing the European patterns of date used.
See the documentation of `diary-date-forms' for an explanation.")

(defvar european-calendar-display-form '((if dayname (concat dayname ", ")) day " " monthname " " year) "\
*Pseudo-pattern governing the way a date appears in the European style.
See the documentation of calendar-date-display-form for an explanation.")

(defvar american-calendar-display-form '((if dayname (concat dayname ", ")) monthname " " day ", " year) "\
*Pseudo-pattern governing the way a date appears in the American style.
See the documentation of `calendar-date-display-form' for an explanation.")

(defvar print-diary-entries-hook 'lpr-buffer "\
*List of functions called after a temporary diary buffer is prepared.
The buffer shows only the diary entries currently visible in the diary
buffer.  The default just does the printing.  Other uses might include, for
example, rearranging the lines into order by day and time, saving the buffer
instead of deleting it, or changing the function used to do the printing.")

(defvar list-diary-entries-hook nil "\
*List of functions called after diary file is culled for relevant entries.
It is to be used for diary entries that are not found in the diary file.

A function `include-other-diary-files' is provided for use as the value of
this hook.  This function enables you to use shared diary files together
with your own.  The files included are specified in the diary file by lines
of the form

        #include \"filename\"

This is recursive; that is, #include directives in files thus included are
obeyed.  You can change the \"#include\" to some other string by changing
the variable `diary-include-string'.  When you use `include-other-diary-files'
as part of the list-diary-entries-hook, you will probably also want to use the
function `mark-included-diary-files' as part of `mark-diary-entries-hook'.

For example, you could use

     (setq list-diary-entries-hook
       '(include-other-diary-files sort-diary-entries))
     (setq diary-display-hook 'fancy-diary-display)

in your `.emacs' file to cause the fancy diary buffer to be displayed with
diary entries from various included files, each day's entries sorted into
lexicographic order.")

(defvar diary-hook nil "\
*List of functions called after the display of the diary.
Can be used for appointment notification.")

(defvar diary-display-hook nil "\
*List of functions that handle the display of the diary.
If nil (the default), `simple-diary-display' is used.  Use `ignore' for no
diary display.

Ordinarily, this just displays the diary buffer (with holidays indicated in
the mode line), if there are any relevant entries.  At the time these
functions are called, the variable `diary-entries-list' is a list, in order
by date, of all relevant diary entries in the form of ((MONTH DAY YEAR)
STRING), where string is the diary entry for the given date.  This can be
used, for example, a different buffer for display (perhaps combined with
holidays), or produce hard copy output.

A function `fancy-diary-display' is provided as an alternative
choice for this hook; this function prepares a special noneditable diary
buffer with the relevant diary entries that has neat day-by-day arrangement
with headings.  The fancy diary buffer will show the holidays unless the
variable `holidays-in-diary-buffer' is set to nil.  Ordinarily, the fancy
diary buffer will not show days for which there are no diary entries, even
if that day is a holiday; if you want such days to be shown in the fancy
diary buffer, set the variable `diary-list-include-blanks' to t.")

(defvar nongregorian-diary-listing-hook nil "\
*List of functions called for listing diary file and included files.
As the files are processed for diary entries, these functions are used to cull
relevant entries.  You can use either or both of `list-hebrew-diary-entries'
and `list-islamic-diary-entries'.  The documentation for these functions
describes the style of such diary entries.")

(defvar mark-diary-entries-hook nil "\
*List of functions called after marking diary entries in the calendar.

A function `mark-included-diary-files' is also provided for use as the
mark-diary-entries-hook; it enables you to use shared diary files together
with your own.  The files included are specified in the diary file by lines
of the form
        #include \"filename\"
This is recursive; that is, #include directives in files thus included are
obeyed.  You can change the \"#include\" to some other string by changing the
variable `diary-include-string'.  When you use `mark-included-diary-files' as
part of the mark-diary-entries-hook, you will probably also want to use the
function `include-other-diary-files' as part of `list-diary-entries-hook'.")

(defvar nongregorian-diary-marking-hook nil "\
*List of functions called for marking diary file and included files.
As the files are processed for diary entries, these functions are used to cull
relevant entries.  You can use either or both of `mark-hebrew-diary-entries'
and `mark-islamic-diary-entries'.  The documentation for these functions
describes the style of such diary entries.")

(defvar diary-list-include-blanks nil "\
*If nil, do not include days with no diary entry in the list of diary entries.
Such days will then not be shown in the the fancy diary buffer, even if they
are holidays.")

(defvar holidays-in-diary-buffer t "\
*Non-nil means include holidays in the diary display.
The holidays appear in the mode line of the diary buffer, or in the
fancy diary buffer next to the date.  This slows down the diary functions
somewhat; setting it to nil makes the diary display faster.")

(put 'general-holidays 'risky-local-variable t)

(defvar general-holidays '((holiday-fixed 1 1 "New Year's Day") (holiday-float 1 1 3 "Martin Luther King Day") (holiday-fixed 2 2 "Ground Hog Day") (holiday-fixed 2 14 "Valentine's Day") (holiday-float 2 1 3 "President's Day") (holiday-fixed 3 17 "St. Patrick's Day") (holiday-fixed 4 1 "April Fool's Day") (holiday-float 5 0 2 "Mother's Day") (holiday-float 5 1 -1 "Memorial Day") (holiday-fixed 6 14 "Flag Day") (holiday-float 6 0 3 "Father's Day") (holiday-fixed 7 4 "Independence Day") (holiday-float 9 1 1 "Labor Day") (holiday-float 10 1 2 "Columbus Day") (holiday-fixed 10 31 "Halloween") (holiday-fixed 11 11 "Veteran's Day") (holiday-float 11 4 4 "Thanksgiving")) "\
*General holidays.  Default value is for the United States.
See the documentation for `calendar-holidays' for details.")

(put 'local-holidays 'risky-local-variable t)

(defvar local-holidays nil "\
*Local holidays.
See the documentation for `calendar-holidays' for details.")

(put 'other-holidays 'risky-local-variable t)

(defvar other-holidays nil "\
*User defined holidays.
See the documentation for `calendar-holidays' for details.")

(put 'hebrew-holidays-1 'risky-local-variable t)

(defvar hebrew-holidays-1 '((holiday-rosh-hashanah-etc) (if all-hebrew-calendar-holidays (holiday-julian 11 (let* ((m displayed-month) (y displayed-year) (year)) (increment-calendar-month m y -1) (let ((year (extract-calendar-year (calendar-julian-from-absolute (calendar-absolute-from-gregorian (list m 1 y)))))) (if (zerop (% (1+ year) 4)) 22 21))) "\"Tal Umatar\" (evening)"))))

(put 'hebrew-holidays-2 'risky-local-variable t)

(defvar hebrew-holidays-2 '((if all-hebrew-calendar-holidays (holiday-hanukkah) (holiday-hebrew 9 25 "Hanukkah")) (if all-hebrew-calendar-holidays (holiday-hebrew 10 (let ((h-year (extract-calendar-year (calendar-hebrew-from-absolute (calendar-absolute-from-gregorian (list displayed-month 28 displayed-year)))))) (if (= (% (calendar-absolute-from-hebrew (list 10 10 h-year)) 7) 6) 11 10)) "Tzom Teveth")) (if all-hebrew-calendar-holidays (holiday-hebrew 11 15 "Tu B'Shevat"))))

(put 'hebrew-holidays-3 'risky-local-variable t)

(defvar hebrew-holidays-3 '((if all-hebrew-calendar-holidays (holiday-hebrew 11 (let ((m displayed-month) (y displayed-year)) (increment-calendar-month m y 1) (let* ((h-year (extract-calendar-year (calendar-hebrew-from-absolute (calendar-absolute-from-gregorian (list m (calendar-last-day-of-month m y) y))))) (s-s (calendar-hebrew-from-absolute (if (= (% (calendar-absolute-from-hebrew (list 7 1 h-year)) 7) 6) (calendar-dayname-on-or-before 6 (calendar-absolute-from-hebrew (list 11 17 h-year))) (calendar-dayname-on-or-before 6 (calendar-absolute-from-hebrew (list 11 16 h-year)))))) (day (extract-calendar-day s-s))) day)) "Shabbat Shirah"))))

(put 'hebrew-holidays-4 'risky-local-variable t)

(defvar hebrew-holidays-4 '((holiday-passover-etc) (if (and all-hebrew-calendar-holidays (let* ((m displayed-month) (y displayed-year) (year)) (increment-calendar-month m y -1) (let ((year (extract-calendar-year (calendar-julian-from-absolute (calendar-absolute-from-gregorian (list m 1 y)))))) (= 21 (% year 28))))) (holiday-julian 3 26 "Kiddush HaHamah")) (if all-hebrew-calendar-holidays (holiday-tisha-b-av-etc))))

(put 'hebrew-holidays 'risky-local-variable t)

(defvar hebrew-holidays (append hebrew-holidays-1 hebrew-holidays-2 hebrew-holidays-3 hebrew-holidays-4) "\
*Jewish holidays.
See the documentation for `calendar-holidays' for details.")

(put 'christian-holidays 'risky-local-variable t)

(defvar christian-holidays '((if all-christian-calendar-holidays (holiday-fixed 1 6 "Epiphany")) (holiday-easter-etc) (if all-christian-calendar-holidays (holiday-greek-orthodox-easter)) (if all-christian-calendar-holidays (holiday-fixed 8 15 "Assumption")) (if all-christian-calendar-holidays (holiday-advent)) (holiday-fixed 12 25 "Christmas") (if all-christian-calendar-holidays (holiday-julian 12 25 "Eastern Orthodox Christmas"))) "\
*Christian holidays.
See the documentation for `calendar-holidays' for details.")

(put 'islamic-holidays 'risky-local-variable t)

(defvar islamic-holidays '((holiday-islamic 1 1 (format "Islamic New Year %d" (let ((m displayed-month) (y displayed-year)) (increment-calendar-month m y 1) (extract-calendar-year (calendar-islamic-from-absolute (calendar-absolute-from-gregorian (list m (calendar-last-day-of-month m y) y))))))) (if all-islamic-calendar-holidays (holiday-islamic 1 10 "Ashura")) (if all-islamic-calendar-holidays (holiday-islamic 3 12 "Mulad-al-Nabi")) (if all-islamic-calendar-holidays (holiday-islamic 7 26 "Shab-e-Mi'raj")) (if all-islamic-calendar-holidays (holiday-islamic 8 15 "Shab-e-Bara't")) (holiday-islamic 9 1 "Ramadan Begins") (if all-islamic-calendar-holidays (holiday-islamic 9 27 "Shab-e Qadr")) (if all-islamic-calendar-holidays (holiday-islamic 10 1 "Id-al-Fitr")) (if all-islamic-calendar-holidays (holiday-islamic 12 10 "Id-al-Adha"))) "\
*Islamic holidays.
See the documentation for `calendar-holidays' for details.")

(put 'solar-holidays 'risky-local-variable t)

(defvar solar-holidays '((if (fboundp 'atan) (solar-equinoxes-solstices)) (if (progn (require 'cal-dst) t) (funcall 'holiday-sexp calendar-daylight-savings-starts '(format "Daylight Savings Time Begins %s" (if (fboundp 'atan) (solar-time-string (/ calendar-daylight-savings-starts-time (float 60)) calendar-standard-time-zone-name) "")))) (funcall 'holiday-sexp calendar-daylight-savings-ends '(format "Daylight Savings Time Ends %s" (if (fboundp 'atan) (solar-time-string (/ calendar-daylight-savings-ends-time (float 60)) calendar-daylight-time-zone-name) "")))) "\
*Sun-related holidays.
See the documentation for `calendar-holidays' for details.")

(put 'calendar-holidays 'risky-local-variable t)

(autoload 'calendar "calendar" "\
Display a three-month calendar in another window.
The three months appear side by side, with the current month in the middle
surrounded by the previous and next months.  The cursor is put on today's date.

If called with an optional prefix argument, prompts for month and year.

This function is suitable for execution in a .emacs file; appropriate setting
of the variable `view-diary-entries-initially' will cause the diary entries for
the current date to be displayed in another window.  The value of the variable
`number-of-diary-entries' controls the number of days of diary entries
displayed upon initial display of the calendar.

An optional prefix argument ARG causes the calendar displayed to be ARG
months in the future if ARG is positive or in the past if ARG is negative;
in this case the cursor goes on the first day of the month.

Once in the calendar window, future or past months can be moved into view.
Arbitrary months can be displayed, or the calendar can be scrolled forward
or backward.

The cursor can be moved forward or backward by one day, one week, one month,
or one year.  All of these commands take prefix arguments which, when negative,
cause movement in the opposite direction.  For convenience, the digit keys
and the minus sign are automatically prefixes.  The window is replotted as
necessary to display the desired date.

Diary entries can be marked on the calendar or displayed in another window.

Use M-x describe-mode for details of the key bindings in the calendar window.

The Gregorian calendar is assumed.

After loading the calendar, the hooks given by the variable
`calendar-load-hook' are run.  This is the place to add key bindings to the
calendar-mode-map.

After preparing the calendar window initially, the hooks given by the variable
`initial-calendar-window-hook' are run.

The hooks given by the variable `today-visible-calendar-hook' are run
everytime the calendar window gets scrolled, if the current date is visible
in the window.  If it is not visible, the hooks given by the variable
`today-invisible-calendar-hook' are run.  Thus, for example, setting
`today-visible-calendar-hook' to 'calendar-star-date will cause today's date
to be replaced by asterisks to highlight it whenever it is in the window." t nil)

(autoload 'list-yahrzeit-dates "calendar" "\
List Yahrzeit dates for *Gregorian* DEATH-DATE from START-YEAR to END-YEAR.
When called interactively from the calendar window, the date of death is taken
from the cursor position." t nil)

;;;***

;;;### (autoloads nil "diary-ins" "calendar/diary-ins.el" (12376 19379))
;;; Generated autoloads from calendar/diary-ins.el

;;;### (autoloads (diary) "diary-lib" "calendar/diary-lib.el" (12376 19380))
;;; Generated autoloads from calendar/diary-lib.el

(autoload 'diary "diary-lib" "\
Generate the diary window for ARG days starting with the current date.
If no argument is provided, the number of days of diary entries is governed
by the variable `number-of-diary-entries'.  This function is suitable for
execution in a `.emacs' file." t nil)

;;;***

;;;### (autoloads (holidays) "holidays" "calendar/holidays.el" (12376 19378))
;;; Generated autoloads from calendar/holidays.el

(autoload 'holidays "holidays" "\
Display the holidays for last month, this month, and next month.
If called with an optional prefix argument, prompts for month and year.

This function is suitable for execution in a .emacs file." t nil)

;;;***

;;;### (autoloads (phases-of-moon) "lunar" "calendar/lunar.el" (12376 19377))
;;; Generated autoloads from calendar/lunar.el

(autoload 'phases-of-moon "lunar" "\
Display the quarters of the moon for last month, this month, and next month.
If called with an optional prefix argument, prompts for month and year.

This function is suitable for execution in a .emacs file." t nil)

;;;***

;;;### (autoloads (solar-equinoxes-solstices sunrise-sunset) "solar" "calendar/solar.el" (12376 19379))
;;; Generated autoloads from calendar/solar.el

(defvar calendar-time-display-form '(12-hours ":" minutes am-pm (if time-zone " (") time-zone (if time-zone ")")) "\
*The pseudo-pattern that governs the way a time of day is formatted.

A pseudo-pattern is a list of expressions that can involve the keywords
`12-hours', `24-hours', and `minutes',  all numbers in string form,
and `am-pm' and `time-zone',  both alphabetic strings.

For example, the form

  '(24-hours \":\" minutes
    (if time-zone \" (\") time-zone (if time-zone \")\"))

would give military-style times like `21:07 (UTC)'.")

(defvar calendar-latitude nil "\
*Latitude of `calendar-location-name' in degrees.

The value can be either a decimal fraction (one place of accuracy is
sufficient), + north, - south, such as 40.7 for New York City, or the value
can be a vector [degrees minutes north/south] such as [40 50 north] for New
York City.

This variable should be set in site-local.el.")

(defvar calendar-longitude nil "\
*Longitude of `calendar-location-name' in degrees.

The value can be either a decimal fraction (one place of accuracy is
sufficient), + east, - west, such as -73.9 for New York City, or the value
can be a vector [degrees minutes east/west] such as [73 55 west] for New
York City.

This variable should be set in site-local.el.")

(defvar calendar-location-name '(let ((float-output-format "%.1f")) (format "%s%s, %s%s" (if (numberp calendar-latitude) (abs calendar-latitude) (+ (aref calendar-latitude 0) (/ (aref calendar-latitude 1) 60.0))) (if (numberp calendar-latitude) (if (> calendar-latitude 0) "N" "S") (if (equal (aref calendar-latitude 2) 'north) "N" "S")) (if (numberp calendar-longitude) (abs calendar-longitude) (+ (aref calendar-longitude 0) (/ (aref calendar-longitude 1) 60.0))) (if (numberp calendar-longitude) (if (> calendar-longitude 0) "E" "W") (if (equal (aref calendar-latitude 2) 'east) "E" "W")))) "\
*Expression evaluating to name of `calendar-longitude', calendar-latitude'.
For example, \"New York City\".  Default value is just the latitude, longitude
pair.

This variable should be set in site-local.el.")

(autoload 'sunrise-sunset "solar" "\
Local time of sunrise and sunset for today.  Accurate to +/- 2 minutes.
If called with an optional prefix argument, prompt for date.

If called with an optional double prefix argument, prompt for longitude,
latitude, time zone, and date, and always use standard time.

This function is suitable for execution in a .emacs file." t nil)

(autoload 'solar-equinoxes-solstices "solar" "\
Date and time of equinoxes and solstices, if visible in the calendar window.
Requires floating point." nil nil)

;;;***

;;;### (autoloads nil "cl-autoload" "cl/cl-autoload.el" (12639 8884))
;;; Generated autoloads from cl/cl-autoload.el

;;;***

;;;### (autoloads nil "cl-compat" "cl/cl-compat.el" (12559 34923))
;;; Generated autoloads from cl/cl-compat.el

;;;***

;;;### (autoloads nil "cl-defs" "cl/cl-defs.el" (12639 8885))
;;; Generated autoloads from cl/cl-defs.el

;;;***

;;;### (autoloads nil "cl-extra" "cl/cl-extra.el" (12648 18630))
;;; Generated autoloads from cl/cl-extra.el

;;;***

;;;### (autoloads nil "cl-macs" "cl/cl-macs.el" (12639 56810))
;;; Generated autoloads from cl/cl-macs.el

;;;***

;;;### (autoloads nil "cl-seq" "cl/cl-seq.el" (12639 8882))
;;; Generated autoloads from cl/cl-seq.el

;;;***

;;;### (autoloads nil "cl" "cl/cl.el" (12639 8883))
;;; Generated autoloads from cl/cl.el

;;;***

;;;### (autoloads nil "background" "comint/background.el" (12675 57061))
;;; Generated autoloads from comint/background.el

;;;### (autoloads nil "dbx" "comint/dbx.el" (12376 19381))
;;; Generated autoloads from comint/dbx.el

;;;### (autoloads (gdb) "gdb" "comint/gdb.el" (12727 30092))
;;; Generated autoloads from comint/gdb.el

(defvar gdb-command-name "gdb" "\
Pathname for executing gdb.")

(autoload 'gdb "gdb" "\
Run gdb on program FILE in buffer *gdb-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for GDB.  If you wish to change this, use
the GDB commands `cd DIR' and `directory'." t nil)

;;;***

;;;### (autoloads (gdbsrc) "gdbsrc" "comint/gdbsrc.el" (12743 11566))
;;; Generated autoloads from comint/gdbsrc.el

(autoload 'gdbsrc "gdbsrc" "\
Activates a gdb session with gdbsrc-mode turned on.  A numeric prefix
argument can be used to specify a running process to attach, and a non-numeric
prefix argument will cause you to be prompted for a core file to debug." t nil)

;;;***

;;;### (autoloads (perldb xdb dbx sdb) "gud" "comint/gud.el" (12376 19387))
;;; Generated autoloads from comint/gud.el

(autoload 'sdb "gud" "\
Run sdb on program FILE in buffer *gud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger." t nil)

(autoload 'dbx "gud" "\
Run dbx on program FILE in buffer *gud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger." t nil)

(autoload 'xdb "gud" "\
Run xdb on program FILE in buffer *gud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger.

You can set the variable 'gud-xdb-directories' to a list of program source
directories if your program contains sources from more than one directory." t nil)

(autoload 'perldb "gud" "\
Run perldb on program FILE in buffer *gud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger." t nil)

;;;***

;;;### (autoloads nil "history" "comint/history.el" (12376 19384))
;;; Generated autoloads from comint/history.el

;;;### (autoloads nil "inf-lisp" "comint/inf-lisp.el" (12546 50428))
;;; Generated autoloads from comint/inf-lisp.el

(add-hook 'same-window-buffer-names "*inferior-lisp*")

;;;***

;;;### (autoloads nil "kermit" "comint/kermit.el" (12546 50426))
;;; Generated autoloads from comint/kermit.el

;;;### (autoloads (rlogin) "rlogin" "comint/rlogin.el" (12282 38441))
;;; Generated autoloads from comint/rlogin.el

(add-hook 'same-window-regexps "^\\*rlogin-.*\\*\\(\\|<[0-9]+>\\)")

(autoload 'rlogin "rlogin" "\
Open a network login connection to HOST via the `rlogin' program.
Input is sent line-at-a-time to the remote connection.

Communication with the remote host is recorded in a buffer `*rlogin-HOST*'
\(or `*rlogin-USER@HOST*' if the remote username differs).
If a prefix argument is given and the buffer `*rlogin-HOST*' already exists,
a new buffer with a different connection will be made.

When called from a program, if the optional second argument is a string or 
buffer, it names the buffer to use.

The variable `rlogin-program' contains the name of the actual program to
run.  It can be a relative or absolute path.

The variable `rlogin-explicit-args' is a list of arguments to give to
the rlogin when starting.  They are added after any arguments given in
INPUT-ARGS.

If the default value of `rlogin-directory-tracking-mode' is t, then the
default directory in that buffer is set to a remote (FTP) file name to
access your home directory on the remote machine.  Occasionally this causes
an error, if you cannot access the home directory on that machine.  This
error is harmless as long as you don't try to use that default directory.

If `rlogin-directory-tracking-mode' is neither t nor nil, then the default
directory is initially set up to your (local) home directory.
This is useful if the remote machine and your local machine
share the same files via NFS.  This is the default.

If you wish to change directory tracking styles during a session, use the
function `rlogin-directory-tracking-mode' rather than simply setting the
variable." t nil)

;;;***

;;;### (autoloads (shell) "shell" "comint/shell.el" (12558 60879))
;;; Generated autoloads from comint/shell.el

(defvar shell-prompt-pattern (purecopy "^[^#$%>\n]*[#$%>] *") "\
Regexp to match prompts in the inferior shell.
Defaults to \"^[^#$%>\\n]*[#$%>] *\", which works pretty well.
This variable is used to initialise `comint-prompt-regexp' in the
shell buffer.

The pattern should probably not match more than one line.  If it does,
shell-mode may become confused trying to distinguish prompt from input
on lines which don't start with a prompt.

This is a fine thing to set in your `.emacs' file.")

(autoload 'shell "shell" "\
Run an inferior shell, with I/O through buffer *shell*.
If buffer exists but shell process is not running, make new shell.
If buffer exists and shell process is running, 
 just switch to buffer `*shell*'.
Program used comes from variable `explicit-shell-file-name',
 or (if that is nil) from the ESHELL environment variable,
 or else from SHELL if there is no ESHELL.
If a file `~/.emacs_SHELLNAME' exists, it is given as initial input
 (Note that this may lose due to a timing error if the shell
  discards input when it starts up.)
The buffer is put in Shell mode, giving commands for sending input
and controlling the subjobs of the shell.  See `shell-mode'.
See also the variable `shell-prompt-pattern'.

The shell file name (sans directories) is used to make a symbol name
such as `explicit-csh-args'.  If that symbol is a variable,
its value is used as a list of arguments when invoking the shell.
Otherwise, one argument `-i' is passed to the shell.

\(Type \\[describe-mode] in the shell buffer for a list of commands.)" t nil)

(add-hook 'same-window-buffer-names "*shell*")

;;;***

;;;### (autoloads (rsh telnet) "telnet" "comint/telnet.el" (12546 50428))
;;; Generated autoloads from comint/telnet.el

(add-hook 'same-window-regexps "\\*telnet-.*\\*\\(\\|<[0-9]+>\\)")

(autoload 'telnet "telnet" "\
Open a network login connection to host named HOST (a string).
With a prefix argument, prompts for the port name or number as well.
Communication with HOST is recorded in a buffer `*HOST-telnet*'.
Normally input is edited in Emacs and sent a line at a time.
See also `\\[rsh]'." t nil)

(add-hook 'same-window-regexps "\\*rsh-[^-]*\\*\\(\\|<[0-9]*>\\)")

(autoload 'rsh "telnet" "\
Open a network login connection to host named HOST (a string).
Communication with HOST is recorded in a buffer `*rsh-HOST*'.
Normally input is edited in Emacs and sent a line at a time.
See also `\\[telnet]'." t nil)

;;;***

;;;### (autoloads (ange-ftp-hook-function) "ange-ftp" "dired/ange-ftp.el" (12639 8565))
;;; Generated autoloads from dired/ange-ftp.el

(defvar ange-ftp-path-format '("^/\\(\\([^@/:]*\\)@\\)?\\([^@/:]*\\):\\(.*\\)" 3 2 4) "\
*Format of a fully expanded remote pathname.  This is a cons
\(REGEXP . (HOST USER PATH)), where REGEXP is a regular expression matching
the full remote pathname, and HOST, USER, and PATH are the numbers of
parenthesized expressions in REGEXP for the components (in that order).")

(autoload 'ange-ftp-hook-function "ange-ftp" nil nil nil)

(or (assoc (car ange-ftp-path-format) file-name-handler-alist) (setq file-name-handler-alist (cons (cons (car ange-ftp-path-format) 'ange-ftp-hook-function) file-name-handler-alist)))

;;;***

;;;### (autoloads nil "dired-cd" "dired/dired-cd.el" (12376 19387))
;;; Generated autoloads from dired/dired-cd.el

;;;***

;;;### (autoloads nil "dired-chmod" "dired/dired-chmod.el" (12376 19393))
;;; Generated autoloads from dired/dired-chmod.el

;;;***

;;;### (autoloads nil "dired-guess" "dired/dired-guess.el" (12376 19389))
;;; Generated autoloads from dired/dired-guess.el

;;;***

;;;### (autoloads nil "dired-link" "dired/dired-link.el" (12376 19387))
;;; Generated autoloads from dired/dired-link.el

;;;***

;;;### (autoloads nil "dired-lisp" "dired/dired-lisp.el" (12376 19389))
;;; Generated autoloads from dired/dired-lisp.el

;;;***

;;;### (autoloads nil "dired-nstd" "dired/dired-nstd.el" (12376 19389))
;;; Generated autoloads from dired/dired-nstd.el

;;;***

;;;### (autoloads nil "dired-trns" "dired/dired-trns.el" (12376 19390))
;;; Generated autoloads from dired/dired-trns.el

;;;***

;;;### (autoloads nil "dired-vms" "dired/dired-vms.el" (12376 19391))
;;; Generated autoloads from dired/dired-vms.el

;;;***

;;;### (autoloads nil "dired-xemacs-highlight" "dired/dired-xemacs-highlight.el" (12745 48553))
;;; Generated autoloads from dired/dired-xemacs-highlight.el

;;;### (autoloads (dired-cwd-make-magic) "dired-cwd" "dired/dired-cwd.el" (12376 19388))
;;; Generated autoloads from dired/dired-cwd.el

(autoload 'dired-cwd-make-magic "dired-cwd" "\
Modify COMMAND so that it's working directory is the current dired directory.
This works by binding `default-directory' to `(default-directory)'s value.
See also function `default-directory'." t nil)

;;;***

;;;### (autoloads (dired-do-rename-list dired-do-rename-numeric) "dired-num" "dired/dired-num.el" (12376 19390))
;;; Generated autoloads from dired/dired-num.el

(autoload 'dired-do-rename-numeric "dired-num" "\
Rename all marked (or next ARG) files using numbers.
You are prompted for a format string, e.g \"part_%d_of_8\", and a starting
number, e.g. 1.  If there are 8 marked files, this example will rename them to

    part_1_of_8
    part_2_of_8
    ...
    part_8_of_8" t nil)

(autoload 'dired-do-rename-list "dired-num" "\
Rename all marked (or next ARG) files using elements from LIST.
You are prompted for a format string, e.g \"x_%s\", and the list,
e.g. '(foo bar zod).  This example will rename the marked files to

    x_foo
    x_bar
    x_zod

It is an error if LIST has not as many elements as there are files." t nil)

;;;***

;;;### (autoloads (dired-rcs-mark-rcs-files dired-rcs-mark-rcs-locked-files) "dired-rcs" "dired/dired-rcs.el" (12376 19395))
;;; Generated autoloads from dired/dired-rcs.el

(autoload 'dired-rcs-mark-rcs-locked-files "dired-rcs" "\
Mark all files that are under RCS control and RCS-locked.
With prefix argument, unflag all those files.
Mentions RCS files for which a working file was not found in this buffer.
Type \\[dired-why] to see them again." t nil)

(autoload 'dired-rcs-mark-rcs-files "dired-rcs" "\
Mark all files that are under RCS control.
With prefix argument, unflag all those files.
Mentions RCS files for which a working file was not found in this buffer.
Type \\[dired-why] to see them again." t nil)

;;;***

;;;### (autoloads (dired-extra-startup) "dired-x" "dired/dired-x.el" (12376 19392))
;;; Generated autoloads from dired/dired-x.el

(autoload 'dired-extra-startup "dired-x" "\
Automatically put on dired-mode-hook to get extra dired features:
\\<dired-mode-map>
  \\[dired-vm]	-- VM on folder
  \\[dired-rmail]	-- Rmail on folder
  \\[dired-do-insert-subdir]	-- insert all marked subdirs
  \\[dired-do-find-file]	-- visit all marked files simultaneously
  \\[dired-set-marker-char], \\[dired-restore-marker-char]	-- change and display dired-marker-char dynamically.
  \\[dired-omit-toggle]	-- toggle omitting of files
  \\[dired-mark-sexp]	-- mark by lisp expression
  \\[dired-do-unmark]	-- replace existing marker with another.
  \\[dired-mark-rcs-files]	-- mark all RCS controlled files
  \\[dired-mark-files-compilation-buffer]	-- mark compilation files
  \\[dired-copy-filename-as-kill]	-- copy the file or subdir names into the kill ring.
  	   You can feed it to other commands using \\[yank].

For more features, see variables

  dired-omit-files
  dired-omit-extenstions
  dired-dangerous-shell-command
  dired-mark-keys
  dired-local-variables-file
  dired-find-subdir
  dired-guess-have-gnutar
  dired-auto-shell-command-alist

See also functions

  dired-sort-on-size
  dired-do-relsymlink
  dired-flag-extension
  dired-virtual
  dired-jump-back
  dired-jump-back-other-window
" t nil)

;;;***

;;;### (autoloads nil "dired-xemacs-menu" "dired/dired-xemacs-menu.el" (12322 8756))
;;; Generated autoloads from dired/dired-xemacs-menu.el

;;;### (autoloads (dired-noselect dired-other-window dired) "dired" "dired/dired.el" (12727 30096))
;;; Generated autoloads from dired/dired.el

(defvar dired-listing-switches (purecopy "-al") "\
*Switches passed to ls for dired. MUST contain the `l' option.
Can contain even `F', `b', `i' and `s'.")

(defvar dired-chown-program (purecopy (if (memq system-type '(dgux-unix hpux usg-unix-v silicon-graphics-unix irix)) "chown" "/etc/chown")) "\
*Name of chown command (usully `chown' or `/etc/chown').")

(defvar dired-ls-program (purecopy "ls") "\
*Absolute or relative name of the ls program used by dired.")

(defvar dired-ls-F-marks-symlinks t "\
*Informs dired about how ls -lF marks symbolic links.
Set this to t if `dired-ls-program' with -lF marks the symbolic link
itself with a trailing @ (usually the case under Ultrix).

Example: if `ln -s foo bar; ls -F bar' gives `bar -> foo', set it to
nil, if it gives `bar@ -> foo', set it to t.

Dired checks if there is really a @ appended.  Thus, if you have a
marking ls program on one host and a non-marking on another host, and
don't care about symbolic links which really end in a @, you can
always set this variable to t.")

(defvar dired-trivial-filenames (purecopy "^\\.\\.?$\\|^#") "\
*Regexp of files to skip when moving point to the first file of a new directory listing.
Nil means move to the subdir line, t means move to first file.")

(defvar dired-keep-marker-move t "\
If t, moved marked files are marked if their originals were.
If a character, those files (marked or not) are marked with that character.")

(defvar dired-keep-marker-copy 67 "\
If t, copied files are marked if their source files were.
If a character, those files are always marked with that character.")

(defvar dired-keep-marker-hardlink 72 "\
If t, hard-linked files are marked if the linked-to files were.
If a character, those files are always marked with that character.")

(defvar dired-keep-marker-symlink 89 "\
If t, symlinked marked files are marked if the linked-to files were.
If a character, those files are always marked with that character.")

(defvar dired-dwim-target nil "\
*If non-nil, dired tries to guess a default target directory:
If there is a dired buffer displayed in the next window, use
its current subdir, instead of the current subdir of this dired
buffer.

The target is used in the prompt for file copy, move etc.")

(defvar dired-copy-preserve-time nil "\
*If non-nil, Dired preserves the last-modified time in a file copy.
\(This works on only some systems.)\\<dired-mode-map>
Use `\\[dired-do-copy]' with a zero prefix argument to toggle its value.")

(define-key ctl-x-map "d" 'dired)

(autoload 'dired "dired" "\
\"Edit\" directory DIRNAME--delete, rename, print, etc. some files in it.
With an optional prefix argument you can specify the ls SWITCHES that are used.
Dired displays a list of files in DIRNAME (which may also have
  shell wildcards appended to select certain files).
You can move around in it with the usual commands.
You can flag files for deletion with \\<dired-mode-map>\\[dired-flag-file-deleted] and then delete them by
  typing \\[dired-do-deletions].
Type \\[describe-mode] after entering dired for more info.

If DIRNAME is already in a dired buffer, that buffer is used without refresh." t nil)

(define-key ctl-x-4-map "d" 'dired-other-window)

(autoload 'dired-other-window "dired" "\
\"Edit\" directory DIRNAME.  Like `dired' but selects in another window." t nil)

(autoload 'dired-noselect "dired" "\
Like `dired' but returns the dired buffer as value, does not select it." nil nil)

;;;***

;;;### (autoloads (find-grep-dired find-name-dired find-dired) "find-dired" "dired/find-dired.el" (12675 57063))
;;; Generated autoloads from dired/find-dired.el

(defvar find-ls-option (purecopy (if (eq system-type 'berkeley-unix) '("-ls" . "-gilsb") '("-exec ls -ld {} \\;" . "-ld"))) "\
*Description of the option to `find' to produce an `ls -l'-type listing.
This is a cons of two strings (FIND-OPTION . LS-SWITCHES).  FIND-OPTION
gives the option (or options) to `find' that produce the desired output.
LS-SWITCHES is a list of `ls' switches to tell dired how to parse the output.")

(defvar find-grep-options (purecopy (if (eq system-type 'berkeley-unix) "-s" "-q")) "\
*Option to grep to be as silent as possible.
On Berkeley systems, this is `-s'; on Posix, and with GNU grep, `-q' does it.
On other systems, the closest you can come is to use `-l'.")

(defvar find-dired-multiple-buffers nil "\
*If non-nil, generates a new buffer for each find")

(autoload 'find-dired "find-dired" "\
Run `find' and go into dired-mode on a buffer of the output.
The command run (after changing into DIR) is

    find . \\( ARGS \\) -ls" t nil)

(autoload 'find-name-dired "find-dired" "\
Search DIR recursively for files matching the globbing pattern PATTERN,
and run dired on those files.
PATTERN is a shell wildcard (not an Emacs regexp) and need not be quoted.
The command run (after changing into DIR) is

    find . -name 'PATTERN' -ls" t nil)

(autoload 'find-grep-dired "find-dired" "\
Find files in DIR containing a regexp ARG and start Dired on output.
The command run (after changing into DIR) is

    find . -type f -exec test -r {} \\; -exec egrep -s ARG {} \\; -ls

Thus ARG can also contain additional grep options." t nil)

;;;***

;;;### (autoloads nil "gmhist-app" "dired/gmhist-app.el" (12376 19392))
;;; Generated autoloads from dired/gmhist-app.el

;;;***

;;;### (autoloads nil "gmhist-cmp" "dired/gmhist-cmp.el" (12376 19393))
;;; Generated autoloads from dired/gmhist-cmp.el

;;;***

;;;### (autoloads nil "gmhist-mh" "dired/gmhist-mh.el" (12376 19393))
;;; Generated autoloads from dired/gmhist-mh.el

;;;***

;;;### (autoloads nil "gmhist" "dired/gmhist.el" (12639 8568))
;;; Generated autoloads from dired/gmhist.el

;;;***

;;;### (autoloads nil "cl-read" "edebug/cl-read.el" (12639 8664))
;;; Generated autoloads from edebug/cl-read.el

;;;***

;;;### (autoloads nil "cl-specs" "edebug/cl-specs.el" (12546 50556))
;;; Generated autoloads from edebug/cl-specs.el

;;;***

;;;### (autoloads nil "cust-print" "edebug/cust-print.el" (12546 50558))
;;; Generated autoloads from edebug/cust-print.el

;;;***

;;;### (autoloads nil "edebug-cl-read" "edebug/edebug-cl-read.el" (12546 50559))
;;; Generated autoloads from edebug/edebug-cl-read.el

;;;***

;;;### (autoloads nil "edebug-test" "edebug/edebug-test.el" (12546 50592))
;;; Generated autoloads from edebug/edebug-test.el


;;;***

;;;### (autoloads (edebug-eval-top-level-form def-edebug-spec) "edebug" "edebug/edebug.el" (12639 8668))
;;; Generated autoloads from edebug/edebug.el

(autoload 'def-edebug-spec "edebug" "\
Set the edebug-form-spec property of SYMBOL according to SPEC.
Both SYMBOL and SPEC are unevaluated. The SPEC can be 0, t, a symbol
\(naming a function), or a list." nil 'macro)

(defalias 'edebug-defun 'edebug-eval-top-level-form)

(autoload 'edebug-eval-top-level-form "edebug" "\
Evaluate a top level form, such as a defun or defmacro.
This is like `eval-defun', but the code is always instrumented for Edebug.
Print its name in the minibuffer and leave point where it is,
or if an error occurs, leave point after it with mark at the original point." t nil)

;;;***

;;;### (autoloads nil "eval-reg" "edebug/eval-reg.el" (12546 50591))
;;; Generated autoloads from edebug/eval-reg.el


;;;***

;;;### (autoloads nil "ediff-diff" "ediff/ediff-diff.el" (12747 30846))
;;; Generated autoloads from ediff/ediff-diff.el

;;;### (autoloads nil "ediff-hook" "ediff/ediff-hook.el" (12675 57179))
;;; Generated autoloads from ediff/ediff-hook.el

;;;### (autoloads nil "ediff-init" "ediff/ediff-init.el" (12675 57185))
;;; Generated autoloads from ediff/ediff-init.el

;;;***

;;;### (autoloads nil "ediff-merg" "ediff/ediff-merg.el" (12675 57177))
;;; Generated autoloads from ediff/ediff-merg.el

;;;***

;;;### (autoloads (ediff-show-registry) "ediff-mult" "ediff/ediff-mult.el" (12675 57187))
;;; Generated autoloads from ediff/ediff-mult.el

(autoload 'ediff-show-registry "ediff-mult" "\
Display Ediff's registry." t nil)

(defalias 'eregistry 'ediff-show-registry)

;;;***

;;;### (autoloads nil "ediff-ptch" "ediff/ediff-ptch.el" (12639 8876))
;;; Generated autoloads from ediff/ediff-ptch.el

;;;***

;;;### (autoloads nil "ediff-util" "ediff/ediff-util.el" (12740 28878))
;;; Generated autoloads from ediff/ediff-util.el

;;;***

;;;### (autoloads nil "ediff-vers" "ediff/ediff-vers.el" (12675 57186))
;;; Generated autoloads from ediff/ediff-vers.el

;;;### (autoloads nil "ediff-wind" "ediff/ediff-wind.el" (12730 62153))
;;; Generated autoloads from ediff/ediff-wind.el

;;;### (autoloads (electric-buffer-list) "ebuff-menu" "electric/ebuff-menu.el" (12698 33526))
;;; Generated autoloads from electric/ebuff-menu.el

(autoload 'electric-buffer-list "ebuff-menu" "\
Pops up a buffer describing the set of Emacs buffers.
Vaguely like ITS lunar select buffer; combining typeoutoid buffer
listing with menuoid buffer selection.

If the very next character typed is a space then the buffer list
window disappears.  Otherwise, one may move around in the
buffer list window, marking buffers to be selected, saved or deleted.

To exit and select a new buffer, type a space when the cursor is on the
appropriate line of the buffer-list window.

Other commands are much like those of buffer-menu-mode.

Calls value of `electric-buffer-menu-mode-hook' on entry if non-nil.

Non-null optional arg FILES-ONLY means mention only file buffers.
When called from Lisp code, FILES-ONLY may be a regular expression,
in which case only buffers whose names match that expression are listed,
or an arbitrary predicate function.

\\{electric-buffer-menu-mode-map}" t nil)

;;;***

;;;### (autoloads (electric-command-history Electric-command-history-redo-expression) "echistory" "electric/echistory.el" (12657 40690))
;;; Generated autoloads from electric/echistory.el

(autoload 'Electric-command-history-redo-expression "echistory" "\
Edit current history line in minibuffer and execute result.
With prefix argument NOCONFIRM, execute current line as-is without editing." t nil)

(autoload 'electric-command-history "echistory" "\
\\<electric-history-map>Major mode for examining and redoing commands from `command-history'.
This pops up a window with the Command History listing.
The number of command listed is controlled by `list-command-history-max'.
The command history is filtered by `list-command-history-filter' if non-nil.
Combines typeout Command History list window with menu like selection
of an expression from the history for re-evaluation in the *original* buffer.

The history displayed is filtered by `list-command-history-filter' if non-nil.

This pops up a window with the Command History listing.  If the very
next character typed is Space, the listing is killed and the previous
window configuration is restored.  Otherwise, you can browse in the
Command History with  Return  moving down and  Delete  moving up, possibly
selecting an expression to be redone with Space or quitting with `Q'.

Like Emacs-Lisp mode except that characters do not insert themselves and
Tab and Linefeed do not indent.  Instead these commands are provided:
\\{electric-history-map}

Calls the value of `electric-command-history-hook' if that is non-nil.
The Command History listing is recomputed each time this mode is invoked." t nil)

;;;***

;;;### (autoloads (electric-helpify with-electric-help) "ehelp" "electric/ehelp.el" (12657 40690))
;;; Generated autoloads from electric/ehelp.el

(autoload 'with-electric-help "ehelp" "\
Pop up an \"electric\" help buffer.
The arguments are THUNK &optional BUFFER NOERASE MINHEIGHT.
THUNK is a function of no arguments which is called to initialize the
contents of BUFFER.  BUFFER defaults to `*Help*'.  BUFFER will be
erased before THUNK is called unless NOERASE is non-nil.  THUNK will
be called while BUFFER is current and with `standard-output' bound to
the buffer specified by BUFFER.

If THUNK returns nil, we display BUFFER starting at the top, and
shrink the window to fit.  If THUNK returns non-nil, we don't do those things.

After THUNK has been called, this function \"electrically\" pops up a window
in which BUFFER is displayed and allows the user to scroll through that buffer
in electric-help-mode. The window's height will be at least MINHEIGHT if
this value is non-nil.

If THUNK returns nil, we display BUFFER starting at the top, and
shrink the window to fit.  If THUNK returns non-nil, we don't do those
things.

When the user exits (with `electric-help-exit', or otherwise) the help
buffer's window disappears (i.e., we use `save-window-excursion')
BUFFER is put into `default-major-mode' (or `fundamental-mode') when we exit." nil nil)

(autoload 'electric-helpify "ehelp" nil nil nil)

;;;***

;;;### (autoloads nil "electric" "electric/electric.el" (12657 40689))
;;; Generated autoloads from electric/electric.el

;;;### (autoloads (Helper-help Helper-describe-bindings) "helper" "electric/helper.el" (12657 40691))
;;; Generated autoloads from electric/helper.el

(autoload 'Helper-describe-bindings "helper" "\
Describe local key bindings of current mode." t nil)

(autoload 'Helper-help "helper" "\
Provide help for current mode." t nil)

;;;***

;;;### (autoloads nil "edt" "emulators/edt.el" (12376 19400))
;;; Generated autoloads from emulators/edt.el

;;;### (autoloads (evi) "evi" "emulators/evi.el" (12376 19401))
;;; Generated autoloads from emulators/evi.el

(autoload 'evi "evi" "\
Start vi emulation in this buffer." t nil)

;;;***

;;;### (autoloads (convert-mocklisp-buffer) "mlconvert" "emulators/mlconvert.el" (12376 19400))
;;; Generated autoloads from emulators/mlconvert.el

(autoload 'convert-mocklisp-buffer "mlconvert" "\
Convert buffer of Mocklisp code to real Lisp that GNU Emacs can run." t nil)

;;;***

;;;### (autoloads nil "mlsupport" "emulators/mlsupport.el" (12546 50433))
;;; Generated autoloads from emulators/mlsupport.el

;;;***

;;;### (autoloads (teco-command) "teco" "emulators/teco.el" (11903 15655))
;;; Generated autoloads from emulators/teco.el

(autoload 'teco-command "teco" "\
Read and execute a Teco command string." t nil)

;;;***

;;;### (autoloads nil "tpu-doc" "emulators/tpu-doc.el" (12376 19403))
;;; Generated autoloads from emulators/tpu-doc.el

;;;### (autoloads (tpu-edt-on) "tpu-edt" "emulators/tpu-edt.el" (12657 40693))
;;; Generated autoloads from emulators/tpu-edt.el

(fset 'tpu-edt-mode 'tpu-edt-on)

(fset 'tpu-edt 'tpu-edt-on)

(autoload 'tpu-edt-on "tpu-edt" "\
Turn on TPU/edt emulation." t nil)

;;;***

;;;### (autoloads nil "tpu-extras" "emulators/tpu-extras.el" (12376 19402))
;;; Generated autoloads from emulators/tpu-extras.el

;;;***

;;;### (autoloads nil "tpu-mapper" "emulators/tpu-mapper.el" (12376 19403))
;;; Generated autoloads from emulators/tpu-mapper.el

;;;***

;;;### (autoloads nil "ws-mode" "emulators/ws-mode.el" (12376 19402))
;;; Generated autoloads from emulators/ws-mode.el

;;;***

;;;### (autoloads nil "backtrace-logging" "energize/backtrace-logging.el" (12376 19404))
;;; Generated autoloads from energize/backtrace-logging.el

;;;***

;;;### (autoloads nil "energize-advice" "energize/energize-advice.el" (12376 19404))
;;; Generated autoloads from energize/energize-advice.el

;;;***

;;;### (autoloads nil "energize-annoprint" "energize/energize-annoprint.el" (12376 19409))
;;; Generated autoloads from energize/energize-annoprint.el

;;;***

;;;### (autoloads nil "energize-font-lock" "energize/energize-font-lock.el" (12376 19408))
;;; Generated autoloads from energize/energize-font-lock.el

;;;***

;;;### (autoloads nil "energize-font-size" "energize/energize-font-size.el" (12376 19408))
;;; Generated autoloads from energize/energize-font-size.el

;;;***

;;;### (autoloads nil "energize-init" "energize/energize-init.el" (12376 19405))
;;; Generated autoloads from energize/energize-init.el

;;;***

;;;### (autoloads nil "energize-load" "energize/energize-load.el" (12376 19405))
;;; Generated autoloads from energize/energize-load.el

;;;***

;;;### (autoloads nil "energize-menus" "energize/energize-menus.el" (12376 19407))
;;; Generated autoloads from energize/energize-menus.el

;;;***

;;;### (autoloads nil "energize-mode" "energize/energize-mode.el" (12720 56483))
;;; Generated autoloads from energize/energize-mode.el

;;;### (autoloads nil "energize-shell" "energize/energize-shell.el" (12110 33408))
;;; Generated autoloads from energize/energize-shell.el

;;;### (autoloads nil "energize-uimx" "energize/energize-uimx.el" (12376 19409))
;;; Generated autoloads from energize/energize-uimx.el

;;;***

;;;### (autoloads nil "energize-vi" "energize/energize-vi.el" (12376 19407))
;;; Generated autoloads from energize/energize-vi.el

;;;***

;;;### (autoloads nil "energize-visit-use" "energize/energize-visit-use.el" (12376 19407))
;;; Generated autoloads from energize/energize-visit-use.el

;;;***

;;;### (autoloads nil "energize-windows" "energize/energize-windows.el" (12376 19409))
;;; Generated autoloads from energize/energize-windows.el

;;;***

;;;### (autoloads nil "ostore" "energize/ostore.el" (12376 19408))
;;; Generated autoloads from energize/ostore.el

;;;***

;;;### (autoloads nil "write-file" "energize/write-file.el" (12376 19404))
;;; Generated autoloads from energize/write-file.el

;;;***

;;;### (autoloads nil "eos" "eos/eos.el" (12740 25241))
;;; Generated autoloads from eos/eos.el

;;;***

;;;### (autoloads nil "loaddefs-eos" "eos/loaddefs-eos.el" (12748 8006))
;;; Generated autoloads from eos/loaddefs-eos.el

;;;***

;;;### (autoloads nil "sun-eos-browser" "eos/sun-eos-browser.el" (12740 25240))
;;; Generated autoloads from eos/sun-eos-browser.el

;;;***

;;;### (autoloads nil "sun-eos-common" "eos/sun-eos-common.el" (12345 62348))
;;; Generated autoloads from eos/sun-eos-common.el

;;;### (autoloads nil "sun-eos-debugger-extra" "eos/sun-eos-debugger-extra.el" (12740 25236))
;;; Generated autoloads from eos/sun-eos-debugger-extra.el

;;;***

;;;### (autoloads nil "sun-eos-debugger" "eos/sun-eos-debugger.el" (12740 25235))
;;; Generated autoloads from eos/sun-eos-debugger.el

;;;***

;;;### (autoloads nil "sun-eos-editor" "eos/sun-eos-editor.el" (12740 25242))
;;; Generated autoloads from eos/sun-eos-editor.el

;;;***

;;;### (autoloads nil "sun-eos-init" "eos/sun-eos-init.el" (12740 25241))
;;; Generated autoloads from eos/sun-eos-init.el

;;;***

;;;### (autoloads nil "sun-eos-load" "eos/sun-eos-load.el" (12740 25239))
;;; Generated autoloads from eos/sun-eos-load.el

;;;***

;;;### (autoloads nil "sun-eos-menubar" "eos/sun-eos-menubar.el" (12740 25243))
;;; Generated autoloads from eos/sun-eos-menubar.el

;;;***

;;;### (autoloads nil "sun-eos-toolbar" "eos/sun-eos-toolbar.el" (12740 25234))
;;; Generated autoloads from eos/sun-eos-toolbar.el

;;;***

;;;### (autoloads nil "sun-eos" "eos/sun-eos.el" (12740 25239))
;;; Generated autoloads from eos/sun-eos.el

;;;### (autoloads nil "epoch" "epoch/epoch.el" (12376 19557))
;;; Generated autoloads from epoch/epoch.el

;;;### (autoloads (term make-term) "term" "eterm/term.el" (12254 10521))
;;; Generated autoloads from eterm/term.el

(autoload 'make-term "term" "\
Make a term process NAME in a buffer, running PROGRAM.
The name of the buffer is made by surrounding NAME with `*'s.
If there is already a running process in that buffer, it is not restarted.
Optional third arg STARTFILE is the name of a file to send the contents of to 
the process.  Any more args are arguments to PROGRAM." nil nil)

(autoload 'term "term" "\
Start a terminal-emulator in a new buffer." t nil)

;;;***

;;;### (autoloads (tperldb txdb tdbx tsdb tgdb) "tgud" "eterm/tgud.el" (12147 26618))
;;; Generated autoloads from eterm/tgud.el

(autoload 'tgdb "tgud" "\
Run gdb on program FILE in buffer *tgud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger." t nil)

(autoload 'tsdb "tgud" "\
Run sdb on program FILE in buffer *tgud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger." t nil)

(autoload 'tdbx "tgud" "\
Run dbx on program FILE in buffer *tgud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger." t nil)

(autoload 'txdb "tgud" "\
Run xdb on program FILE in buffer *tgud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger.

You can set the variable 'tgud-xdb-directories' to a list of program source
directories if your program contains sources from more than one directory." t nil)

(autoload 'tperldb "tgud" "\
Run perldb on program FILE in buffer *tgud-FILE*.
The directory containing FILE becomes the initial working directory
and source-file directory for your debugger." t nil)

;;;***

;;;### (autoloads (tshell) "tshell" "eterm/tshell.el" (12147 26618))
;;; Generated autoloads from eterm/tshell.el

(defvar tshell-prompt-pattern "^[^#$%>\n]*[#$%>] *" "\
Regexp to match prompts in the inferior shell.
Defaults to \"^[^#$%>\\n]*[#$%>] *\", which works pretty well.
This variable is used to initialise `term-prompt-regexp' in the 
shell buffer.

The pattern should probably not match more than one line.  If it does,
tshell-mode may become confused trying to distinguish prompt from input
on lines which don't start with a prompt.

This is a fine thing to set in your `.emacs' file.")

(autoload 'tshell "tshell" "\
Run an inferior shell, with I/O through buffer *shell*.
If buffer exists but shell process is not running, make new shell.
If buffer exists and shell process is running, just switch to buffer `*shell*'.
Program used comes from variable `explicit-shell-file-name',
 or (if that is nil) from the ESHELL environment variable,
 or else from SHELL if there is no ESHELL.
If a file `~/.emacs_SHELLNAME' exists, it is given as initial input
 (Note that this may lose due to a timing error if the shell
  discards input when it starts up.)
The buffer is put in Tshell mode, giving commands for sending input
and controlling the subjobs of the shell.  See `tshell-mode'.
See also the variable `tshell-prompt-pattern'.

The shell file name (sans directories) is used to make a symbol name
such as `explicit-csh-args'.  If that symbol is a variable,
its value is used as a list of arguments when invoking the shell.
Otherwise, one argument `-i' is passed to the shell.

\(Type \\[describe-mode] in the shell buffer for a list of commands.)" t nil)

;;;***

;;;### (autoloads nil "NeXTify" "games/NeXTify.el" (12376 19620))
;;; Generated autoloads from games/NeXTify.el

;;;### (autoloads (blackbox) "blackbox" "games/blackbox.el" (12376 19614))
;;; Generated autoloads from games/blackbox.el

(autoload 'blackbox "blackbox" "\
Play blackbox.  Optional prefix argument is the number of balls;
the default is 4.

What is blackbox?

Blackbox is a game of hide and seek played on an 8 by 8 grid (the
Blackbox).  Your opponent (Emacs, in this case) has hidden several
balls (usually 4) within this box.  By shooting rays into the box and
observing where they emerge it is possible to deduce the positions of
the hidden balls.  The fewer rays you use to find the balls, the lower
your score.

Overview of play:

\\<blackbox-mode-map>To play blackbox, type \\[blackbox].  An optional prefix argument
specifies the number of balls to be hidden in the box; the default is
four.

The cursor can be moved around the box with the standard cursor
movement keys.

To shoot a ray, move the cursor to the edge of the box and press SPC.
The result will be determined and the playfield updated.

You may place or remove balls in the box by moving the cursor into the
box and pressing \\[bb-romp].

When you think the configuration of balls you have placed is correct,
press \\[bb-done].  You will be informed whether you are correct or
not, and be given your score.  Your score is the number of letters and
numbers around the outside of the box plus five for each incorrectly
placed ball.  If you placed any balls incorrectly, they will be
indicated with `x', and their actual positions indicated with `o'.

Details:

There are three possible outcomes for each ray you send into the box:

	Detour: the ray is deflected and emerges somewhere other than
		where you sent it in.  On the playfield, detours are
		denoted by matching pairs of numbers -- one where the
		ray went in, and the other where it came out.

	Reflection: the ray is reflected and emerges in the same place
		it was sent in.  On the playfield, reflections are
		denoted by the letter `R'.

	Hit:	the ray strikes a ball directly and is absorbed.  It does
		not emerge from the box.  On the playfield, hits are
		denoted by the letter `H'.

The rules for how balls deflect rays are simple and are best shown by
example.

As a ray approaches a ball it is deflected ninety degrees.  Rays can
be deflected multiple times.  In the diagrams below, the dashes
represent empty box locations and the letter `O' represents a ball.
The entrance and exit points of each ray are marked with numbers as
described under \"Detour\" above.  Note that the entrance and exit
points are always interchangeable.  `*' denotes the path taken by the
ray.

Note carefully the relative positions of the ball and the ninety
degree deflection it causes.

    1                                            
  - * - - - - - -         - - - - - - - -         - - - - - - - -       
  - * - - - - - -         - - - - - - - -         - - - - - - - -       
1 * * - - - - - -         - - - - - - - -         - O - - - - O -       
  - - O - - - - -         - - O - - - - -         - - * * * * - -
  - - - - - - - -         - - - * * * * * 2     3 * * * - - * - -
  - - - - - - - -         - - - * - - - -         - - - O - * - -      
  - - - - - - - -         - - - * - - - -         - - - - * * - -       
  - - - - - - - -         - - - * - - - -         - - - - * - O -       
                                2                         3

As mentioned above, a reflection occurs when a ray emerges from the same point
it was sent in.  This can happen in several ways:

                                                                           
  - - - - - - - -         - - - - - - - -          - - - - - - - -
  - - - - O - - -         - - O - O - - -          - - - - - - - -
R * * * * - - - -         - - - * - - - -          O - - - - - - -
  - - - - O - - -         - - - * - - - -        R - - - - - - - -
  - - - - - - - -         - - - * - - - -          - - - - - - - -
  - - - - - - - -         - - - * - - - -          - - - - - - - -
  - - - - - - - -       R * * * * - - - -          - - - - - - - -
  - - - - - - - -         - - - - O - - -          - - - - - - - -

In the first example, the ray is deflected downwards by the upper
ball, then left by the lower ball, and finally retraces its path to
its point of origin.  The second example is similar.  The third
example is a bit anomalous but can be rationalized by realizing the
ray never gets a chance to get into the box.  Alternatively, the ray
can be thought of as being deflected downwards and immediately
emerging from the box.

A hit occurs when a ray runs straight into a ball:

  - - - - - - - -         - - - - - - - -          - - - - - - - -
  - - - - - - - -         - - - - - - - -          - - - - O - - -
  - - - - - - - -         - - - - O - - -        H * * * * - - - -
  - - - - - - - -       H * * * * O - - -          - - - * - - - -
  - - - - - - - -         - - - - O - - -          - - - O - - - -
H * * * O - - - -         - - - - - - - -          - - - - - - - -
  - - - - - - - -         - - - - - - - -          - - - - - - - -
  - - - - - - - -         - - - - - - - -          - - - - - - - -

Be sure to compare the second example of a hit with the first example of
a reflection." t nil)

;;;***

;;;### (autoloads (conx-load conx conx-region conx-buffer) "conx" "games/conx.el" (12376 19618))
;;; Generated autoloads from games/conx.el

(autoload 'conx-buffer "conx" "\
Absorb the text in the current buffer into the tree." t nil)

(autoload 'conx-region "conx" "\
Absorb the text in the current region into the tree." t nil)

(autoload 'conx "conx" "\
Generate some random sentences in the *conx* buffer." t nil)

(autoload 'conx-load "conx" "\
Load in a CONX database written by the \\[conx-save] command.
This clears the database currently in memory." t nil)

;;;***

;;;### (autoloads (shuffle-vector cookie-snarf cookie-insert cookie) "cookie1" "games/cookie1.el" (12376 19615))
;;; Generated autoloads from games/cookie1.el

(autoload 'cookie "cookie1" "\
Return a random phrase from PHRASE-FILE.  When the phrase file
is read in, display STARTMSG at beginning of load, ENDMSG at end." nil nil)

(autoload 'cookie-insert "cookie1" "\
Insert random phrases from PHRASE-FILE; COUNT of them.  When the phrase file
is read in, display STARTMSG at beginning of load, ENDMSG at end." nil nil)

(autoload 'cookie-snarf "cookie1" "\
Reads in the PHRASE-FILE, returns it as a vector of strings.
Emit STARTMSG and ENDMSG before and after.  Caches the result; second
and subsequent calls on the same file won't go to disk." nil nil)

(autoload 'shuffle-vector "cookie1" "\
Randomly permute the elements of VECTOR (all permutations equally likely)" nil nil)

;;;***

;;;### (autoloads (dissociated-press) "dissociate" "games/dissociate.el" (12376 19615))
;;; Generated autoloads from games/dissociate.el

(autoload 'dissociated-press "dissociate" "\
Dissociate the text of the current buffer.
Output goes in buffer named *Dissociation*,
which is redisplayed each time text is added to it.
Every so often the user must say whether to continue.
If ARG is positive, require ARG chars of continuity.
If ARG is negative, require -ARG words of continuity.
Default is 2." t nil)

;;;***

;;;### (autoloads (doctor) "doctor" "games/doctor.el" (12376 19616))
;;; Generated autoloads from games/doctor.el

(autoload 'doctor "doctor" "\
Switch to *doctor* buffer and start giving psychotherapy." t nil)

;;;***

;;;### (autoloads (dunnet) "dunnet" "games/dunnet.el" (12376 19621))
;;; Generated autoloads from games/dunnet.el

(autoload 'dunnet "dunnet" "\
Switch to *dungeon* buffer and start game." t nil)

;;;***

;;;### (autoloads (flame) "flame" "games/flame.el" (12376 19618))
;;; Generated autoloads from games/flame.el

(autoload 'flame "flame" "\
Generate ARG (default 1) sentences of half-crazed gibberish." t nil)

;;;***

;;;### (autoloads (gomoku) "gomoku" "games/gomoku.el" (12546 50547))
;;; Generated autoloads from games/gomoku.el

(autoload 'gomoku "gomoku" "\
Start a Gomoku game between you and Emacs.
If a game is in progress, this command allow you to resume it.
If optional arguments N and M are given, an N by M board is used.

You and Emacs play in turn by marking a free square. You mark it with X
and Emacs marks it with O. The winner is the first to get five contiguous
marks horizontally, vertically or in diagonal.
You play by moving the cursor over the square you choose and hitting
\\<gomoku-mode-map>\\[gomoku-human-plays].
Use \\[describe-mode] for more info." t nil)

;;;***

;;;### (autoloads (hanoi) "hanoi" "games/hanoi.el" (12546 50548))
;;; Generated autoloads from games/hanoi.el

(autoload 'hanoi "hanoi" "\
Towers of Hanoi diversion.  Argument is number of rings." t nil)

;;;***

;;;### (autoloads (life) "life" "games/life.el" (12743 11635))
;;; Generated autoloads from games/life.el

(autoload 'life "life" "\
Run Conway's Life simulation.
The starting pattern is randomly selected.  Prefix arg (optional first
arg non-nil from a program) is the number of seconds to sleep between
generations (this defaults to 1)." t nil)

;;;***

;;;### (autoloads (mpuz) "mpuz" "games/mpuz.el" (12546 50549))
;;; Generated autoloads from games/mpuz.el

(autoload 'mpuz "mpuz" "\
Multiplication puzzle with GNU Emacs." t nil)

;;;***

;;;### (autoloads (snarf-spooks spook) "spook" "games/spook.el" (12546 50549))
;;; Generated autoloads from games/spook.el

(autoload 'spook "spook" "\
Adds that special touch of class to your outgoing mail." t nil)

(autoload 'snarf-spooks "spook" "\
Return a vector containing the lines from `spook-phrases-file'." nil nil)

;;;***

;;;### (autoloads nil "studly" "games/studly.el" (12376 19619))
;;; Generated autoloads from games/studly.el

;;;### (autoloads (psychoanalyze-pinhead insert-zippyism yow) "yow" "games/yow.el" (12546 50550))
;;; Generated autoloads from games/yow.el

(autoload 'yow "yow" "\
Return or display a random Zippy quotation.  With prefix arg, insert it." t nil)

(autoload 'insert-zippyism "yow" "\
Prompt with completion for a known Zippy quotation, and insert it at point." t nil)

(autoload 'psychoanalyze-pinhead "yow" "\
Zippy goes to the analyst." t nil)

;;;***

;;;### (autoloads nil "custom" "gnus/custom.el" (12747 23775))
;;; Generated autoloads from gnus/custom.el

;;;### (autoloads nil "dgnushack" "gnus/dgnushack.el" (12740 29825))
;;; Generated autoloads from gnus/dgnushack.el

;;;### (autoloads (gnus-cache-generate-nov-databases gnus-cache-generate-active gnus-jog-cache) "gnus-cache" "gnus/gnus-cache.el" (12740 29819))
;;; Generated autoloads from gnus/gnus-cache.el

(autoload 'gnus-jog-cache "gnus-cache" "\
Go through all groups and put the articles into the cache." t nil)

(autoload 'gnus-cache-generate-active "gnus-cache" "\
Generate the cache active file." t nil)

(autoload 'gnus-cache-generate-nov-databases "gnus-cache" "\
Generate NOV files recursively starting in DIR." t nil)

;;;***

;;;### (autoloads nil "gnus-cite" "gnus/gnus-cite.el" (12747 7527))
;;; Generated autoloads from gnus/gnus-cite.el

;;;### (autoloads nil "gnus-cus" "gnus/gnus-cus.el" (12740 29826))
;;; Generated autoloads from gnus/gnus-cus.el

;;;### (autoloads nil "gnus-demon" "gnus/gnus-demon.el" (12740 29833))
;;; Generated autoloads from gnus/gnus-demon.el

;;;***

;;;### (autoloads nil "gnus-edit" "gnus/gnus-edit.el" (12740 29827))
;;; Generated autoloads from gnus/gnus-edit.el

;;;***

;;;### (autoloads nil "gnus-ems" "gnus/gnus-ems.el" (12743 11575))
;;; Generated autoloads from gnus/gnus-ems.el

;;;### (autoloads nil "gnus-gl" "gnus/gnus-gl.el" (12730 62156))
;;; Generated autoloads from gnus/gnus-gl.el

;;;### (autoloads nil "gnus-kill" "gnus/gnus-kill.el" (12727 30112))
;;; Generated autoloads from gnus/gnus-kill.el

;;;***

;;;### (autoloads nil "gnus-mh" "gnus/gnus-mh.el" (12740 29827))
;;; Generated autoloads from gnus/gnus-mh.el

;;;***

;;;### (autoloads nil "gnus-msg" "gnus/gnus-msg.el" (12747 7527))
;;; Generated autoloads from gnus/gnus-msg.el

;;;### (autoloads nil "gnus-nocem" "gnus/gnus-nocem.el" (12727 30244))
;;; Generated autoloads from gnus/gnus-nocem.el

;;;***

;;;### (autoloads nil "gnus-picon" "gnus/gnus-picon.el" (12747 7528))
;;; Generated autoloads from gnus/gnus-picon.el

;;;### (autoloads nil "gnus-salt" "gnus/gnus-salt.el" (12745 38025))
;;; Generated autoloads from gnus/gnus-salt.el

;;;### (autoloads (gnus-score-mode) "gnus-scomo" "gnus/gnus-scomo.el" (12740 29833))
;;; Generated autoloads from gnus/gnus-scomo.el

(autoload 'gnus-score-mode "gnus-scomo" "\
Mode for editing Gnus score files.
This mode is an extended emacs-lisp mode.

\\{gnus-score-mode-map}" t nil)

;;;***

;;;### (autoloads nil "gnus-score" "gnus/gnus-score.el" (12744 54649))
;;; Generated autoloads from gnus/gnus-score.el

;;;***

;;;### (autoloads nil "gnus-setup" "gnus/gnus-setup.el" (12727 30246))
;;; Generated autoloads from gnus/gnus-setup.el

;;;### (autoloads (gnus-batch-brew-soup) "gnus-soup" "gnus/gnus-soup.el" (12743 11596))
;;; Generated autoloads from gnus/gnus-soup.el

(autoload 'gnus-batch-brew-soup "gnus-soup" "\
Brew a SOUP packet from groups mention on the command line.
Will use the remaining command line arguments as regular expressions
for matching on group names.

For instance, if you want to brew on all the nnml groups, as well as
groups with \"emacs\" in the name, you could say something like:

$ emacs -batch -f gnus-batch-brew-soup ^nnml \".*emacs.*\"" t nil)

;;;***

;;;### (autoloads nil "gnus-srvr" "gnus/gnus-srvr.el" (12744 54653))
;;; Generated autoloads from gnus/gnus-srvr.el

;;;***

;;;### (autoloads nil "gnus-topic" "gnus/gnus-topic.el" (12727 30243))
;;; Generated autoloads from gnus/gnus-topic.el

;;;***

;;;### (autoloads nil "gnus-uu" "gnus/gnus-uu.el" (12743 11578))
;;; Generated autoloads from gnus/gnus-uu.el

;;;***

;;;### (autoloads nil "gnus-vis" "gnus/gnus-vis.el" (12747 7527))
;;; Generated autoloads from gnus/gnus-vis.el

;;;### (autoloads nil "gnus-vm" "gnus/gnus-vm.el" (12727 30121))
;;; Generated autoloads from gnus/gnus-vm.el

;;;***

;;;### (autoloads nil "gnus-xmas" "gnus/gnus-xmas.el" (12747 65521))
;;; Generated autoloads from gnus/gnus-xmas.el

;;;### (autoloads (gnus-batch-score gnus-fetch-group gnus gnus-other-frame gnus-slave gnus-no-server gnus-slave-no-server gnus-add-configuration gnus-update-format) "gnus" "gnus/gnus.el" (12748 38))
;;; Generated autoloads from gnus/gnus.el

(autoload 'gnus-update-format "gnus" "\
Update the format specification near point." t nil)

(autoload 'gnus-add-configuration "gnus" "\
Add the window configuration CONF to `gnus-buffer-configuration'." nil nil)

(autoload 'gnus-slave-no-server "gnus" "\
Read network news as a slave, without connecting to local server" t nil)

(autoload 'gnus-no-server "gnus" "\
Read network news.
If ARG is a positive number, Gnus will use that as the
startup level.	If ARG is nil, Gnus will be started at level 2.
If ARG is non-nil and not a positive number, Gnus will
prompt the user for the name of an NNTP server to use.
As opposed to `gnus', this command will not connect to the local server." t nil)

(autoload 'gnus-slave "gnus" "\
Read news as a slave." t nil)

(autoload 'gnus-other-frame "gnus" "\
Pop up a frame to read news." t nil)

(autoload 'gnus "gnus" "\
Read network news.
If ARG is non-nil and a positive number, Gnus will use that as the
startup level.	If ARG is non-nil and not a positive number, Gnus will
prompt the user for the name of an NNTP server to use." t nil)

(autoload 'gnus-fetch-group "gnus" "\
Start Gnus if necessary and enter GROUP.
Returns whether the fetching was successful or not." t nil)

(defalias 'gnus-batch-kill 'gnus-batch-score)

(autoload 'gnus-batch-score "gnus" "\
Run batched scoring.
Usage: emacs -batch -l gnus -f gnus-batch-score <newsgroups> ...
Newsgroups is a list of strings in Bnews format.  If you want to score
the comp hierarchy, you'd say \"comp.all\".  If you would not like to
score the alt hierarchy, you'd say \"!alt.all\"." t nil)

;;;***

;;;### (autoloads nil "mailheader" "gnus/mailheader.el" (12727 30250))
;;; Generated autoloads from gnus/mailheader.el

;;;### (autoloads (unbold-region bold-region message-news-other-frame message-news-other-window message-mail-other-frame message-mail-other-window message-bounce message-resend message-forward message-recover message-supersede message-cancel-news message-followup message-wide-reply message-reply message-news message-mail message-mode) "message" "gnus/message.el" (12747 7528))
;;; Generated autoloads from gnus/message.el

(defvar message-directory "~/Mail/" "\
*Directory from which all other mail file variables are derived.")

(defvar message-fcc-handler-function 'rmail-output "\
*A function called to save outgoing articles.
This function will be called with the name of the file to store the
article in. The default function is `rmail-output' which saves in Unix
mailbox format.")

(defvar message-courtesy-message "The following message is a courtesy copy of an article\nthat has been posted as well.\n\n" "\
*This is inserted at the start of a mailed copy of a posted message.
If this variable is nil, no such courtesy message will be added.")

(defvar message-ignored-bounced-headers "^\\(Received\\|Return-Path\\):" "\
*Regexp that matches headers to be removed in resent bounced mail.")

(defvar message-from-style 'default "\
*Specifies how \"From\" headers look.

If `nil', they contain just the return address like:
	king@grassland.com
If `parens', they look like:
	king@grassland.com (Elvis Parsley)
If `angles', they look like:
	Elvis Parsley <king@grassland.com>

Otherwise, most addresses look like `angles', but they look like
`parens' if `angles' would need quoting and `parens' would not.")

(defvar message-syntax-checks nil "\
Controls what syntax checks should not be performed on outgoing posts.
To disable checking of long signatures, for instance, add
 `(signature . disabled)' to this list.

Don't touch this variable unless you really know what you're doing.

Checks include subject-cmsg multiple-headers sendsys message-id from
long-lines control-chars size new-text redirected-followup signature
approved sender empty empty-headers message-id from subject.")

(defvar message-required-news-headers '(From Newsgroups Subject Date Message-ID (optional . Organization) Lines (optional . X-Newsreader)) "\
*Headers to be generated or prompted for when posting an article.
RFC977 and RFC1036 require From, Date, Newsgroups, Subject,
Message-ID.  Organization, Lines, In-Reply-To, Expires, and
X-Newsreader are optional.  If don't you want message to insert some
header, remove it from this list.")

(defvar message-required-mail-headers '(From Subject Date (optional . In-Reply-To) Message-ID Lines (optional . X-Mailer)) "\
*Headers to be generated or prompted for when mailing a message.
RFC822 required that From, Date, To, Subject and Message-ID be
included.  Organization, Lines and X-Mailer are optional.")

(defvar message-deletable-headers '(Message-ID Date) "\
*Headers to be deleted if they already exist and were generated by message previously.")

(defvar message-ignored-news-headers "^NNTP-Posting-Host:\\|^Xref:\\|^Bcc:\\|^Gcc:\\|^Fcc:" "\
*Regexp of headers to be removed unconditionally before posting.")

(defvar message-ignored-mail-headers "^Gcc:\\|^Fcc:" "\
*Regexp of headers to be removed unconditionally before mailing.")

(defvar message-ignored-supersedes-headers "^Path:\\|^Date\\|^NNTP-Posting-Host:\\|^Xref:\\|^Lines:\\|^Received:\\|^X-From-Line:\\|Return-Path:\\|^Supersedes:" "\
*Header lines matching this regexp will be deleted before posting.
It's best to delete old Path and Date headers before posting to avoid
any confusion.")

(defvar message-signature-separator "^-- *$" "\
Regexp matching the signature separator.")

(defvar message-interactive nil "\
Non-nil means when sending a message wait for and display errors.
nil means let mailer mail back a message to report errors.")

(defvar message-generate-new-buffers t "\
*Non-nil means that a new message buffer will be created whenever `mail-setup' is called.
If this is a function, call that function with three parameters:  The type,
the to address and the group name.  (Any of these may be nil.)  The function
should return the new buffer name.")

(defvar message-kill-buffer-on-exit nil "\
*Non-nil means that the message buffer will be killed after sending a message.")

(defvar message-user-organization-file "/usr/lib/news/organization" "\
*Local news organization file.")

(defvar message-autosave-directory (concat (file-name-as-directory message-directory) "drafts/") "\
*Directory where message autosaves buffers.
If nil, message won't autosave.")

(defvar message-signature-before-forwarded-message t "\
*If non-nil, put the signature before any included forwarded message.")

(defvar message-included-forward-headers "^From:\\|^Newsgroups:\\|^Subject:\\|^Date:\\|^Followup-To:\\|^Reply-To:\\|^Organization:\\|^Summary:\\|^Keywords:\\|^To:\\|^Cc:\\|^Posted-To:\\|^Mail-Copies-To:\\|^Apparently-To:\\|^Gnus-Warning:\\|^Resent-\\|^Message-ID:\\|^References:" "\
*Regexp matching headers to be included in forwarded messages.")

(defvar message-ignored-resent-headers "^Return-receipt" "\
*All headers that match this regexp will be deleted when resending a message.")

(defvar message-ignored-cited-headers "." "\
Delete these headers from the messages you yank.")

(defvar message-send-mail-function 'message-send-mail-with-sendmail "\
Function to call to send the current buffer as mail.
The headers should be delimited by a line whose contents match the
variable `mail-header-separator'.

Legal values include `message-send-mail-with-mh' and
`message-send-mail-with-sendmail', which is the default.")

(defvar message-send-news-function 'message-send-news "\
Function to call to send the current buffer as news.
The headers should be delimited by a line whose contents match the
variable `mail-header-separator'.")

(defvar message-reply-to-function nil "\
Function that should return a list of headers.
This function should pick out addresses from the To, Cc, and From headers
and respond with new To and Cc headers.")

(defvar message-wide-reply-to-function nil "\
Function that should return a list of headers.
This function should pick out addresses from the To, Cc, and From headers
and respond with new To and Cc headers.")

(defvar message-followup-to-function nil "\
Function that should return a list of headers.
This function should pick out addresses from the To, Cc, and From headers
and respond with new To and Cc headers.")

(defvar message-use-followup-to 'ask "\
*Specifies what to do with Followup-To header.
If nil, ignore the header. If it is t, use its value, but query before
using the \"poster\" value.  If it is the symbol `ask', query the user
whether to ignore the \"poster\" value.  If it is the symbol `use',
always use the value.")

(defvar message-post-method (cond ((and (boundp 'gnus-post-method) gnus-post-method) gnus-post-method) ((boundp 'gnus-select-method) gnus-select-method) (t '(nnspool ""))) "\
Method used to post news.")

(defvar message-generate-headers-first nil "\
*If non-nil, generate all possible headers before composing.")

(defvar message-citation-line-function 'message-insert-citation-line "\
*Function called to insert the \"Whomever writes:\" line.")

(defvar message-yank-prefix "> " "\
*Prefix inserted on the lines of yanked messages.
nil means use indentation.")

(defvar message-cite-function 'message-cite-original "\
*Function for citing an original message.")

(defvar message-indent-citation-function 'message-indent-citation "\
*Function for modifying a citation just inserted in the mail buffer.
This can also be a list of functions.  Each function can find the
citation between (point) and (mark t).  And each function should leave
point and mark around the citation text as modified.")

(defvar message-signature t "\
*String to be inserted at the end of the message buffer.
If t, the `message-signature-file' file will be inserted instead.
If a function, the result from the function will be used instead.
If a form, the result from the form will be used instead.")

(defvar message-signature-file "~/.signature" "\
*File containing the text inserted at end of message. buffer.")

(defvar message-default-headers nil "\
*A string containing header lines to be inserted in outgoing messages.
It is inserted before you edit the message, so you can edit or delete
these lines.")

(defvar message-default-mail-headers nil "\
*A string of header lines to be inserted in outgoing mails.")

(defvar message-default-news-headers nil "\
*A string of header lines to be inserted in outgoing news articles.")

(autoload 'message-mode "message" "\
Major mode for editing mail and news to be sent.
Like Text Mode but with these additional commands:
C-c C-s  message-send (send the message)    C-c C-c  message-send-and-exit
C-c C-f  move to a header field (and create it if there isn't):
	 C-c C-f C-t  move to To	C-c C-f C-s  move to Subject
	 C-c C-f C-c  move to Cc	C-c C-f C-b  move to Bcc
	 C-c C-f C-f  move to Fcc	C-c C-f C-r  move to Reply-To
	 C-c C-f C-u  move to Summary	C-c C-f C-n  move to Newsgroups
	 C-c C-f C-k  move to Keywords	C-c C-f C-d  move to Distribution
	 C-c C-f C-o  move to Followup-To
C-c C-t  message-insert-to (add a To header to a news followup)
C-c C-n  message-insert-newsgroups (add a Newsgroup header to a news reply)
C-c C-b  message-goto-body (move to beginning of message text).
C-c C-i  message-goto-signature (move to the beginning of the signature).
C-c C-w  message-insert-signature (insert `message-signature-file' file).
C-c C-y  message-yank-original (insert current message, if any).
C-c C-q  message-fill-yanked-message (fill what was yanked).
C-c C-r  message-ceasar-buffer-body (rot13 the message body)." t nil)

(autoload 'message-mail "message" "\
Start editing a mail message to be sent." t nil)

(autoload 'message-news "message" "\
Start editing a news article to be sent." t nil)

(autoload 'message-reply "message" "\
Start editing a reply to the article in the current buffer." t nil)

(autoload 'message-wide-reply "message" nil t nil)

(autoload 'message-followup "message" nil t nil)

(autoload 'message-cancel-news "message" "\
Cancel an article you posted." t nil)

(autoload 'message-supersede "message" "\
Start composing a message to supersede the current message.
This is done simply by taking the old article and adding a Supersedes
header line with the old Message-ID." t nil)

(autoload 'message-recover "message" "\
Reread contents of current buffer from its last auto-save file." t nil)

(autoload 'message-forward "message" "\
Forward the current message via mail.  
Optional NEWS will use news to forward instead of mail." t nil)

(autoload 'message-resend "message" "\
Resend the current article to ADDRESS." t nil)

(autoload 'message-bounce "message" "\
Re-mail the current message.
This only makes sense if the current message is a bounce message than
contains some mail you have written which has been bounced back to
you." t nil)

(autoload 'message-mail-other-window "message" "\
Like `message-mail' command, but display mail buffer in another window." t nil)

(autoload 'message-mail-other-frame "message" "\
Like `message-mail' command, but display mail buffer in another frame." t nil)

(autoload 'message-news-other-window "message" "\
Start editing a news article to be sent." t nil)

(autoload 'message-news-other-frame "message" "\
Start editing a news article to be sent." t nil)

(autoload 'bold-region "message" "\
Bold all nonblank characters in the region.
Works by overstriking characters.
Called from program, takes two arguments START and END
which specify the range to operate on." t nil)

(autoload 'unbold-region "message" "\
Remove all boldness (overstruck characters) in the region.
Called from program, takes two arguments START and END
which specify the range to operate on." t nil)

;;;***

;;;### (autoloads nil "messagexmas" "gnus/messagexmas.el" (12743 11600))
;;; Generated autoloads from gnus/messagexmas.el

;;;### (autoloads nil "nnbabyl" "gnus/nnbabyl.el" (12740 29830))
;;; Generated autoloads from gnus/nnbabyl.el

;;;***

;;;### (autoloads nil "nndb" "gnus/nndb.el" (12740 29835))
;;; Generated autoloads from gnus/nndb.el

;;;### (autoloads nil "nndir" "gnus/nndir.el" (12727 30126))
;;; Generated autoloads from gnus/nndir.el

;;;***

;;;### (autoloads nil "nndoc" "gnus/nndoc.el" (12727 30128))
;;; Generated autoloads from gnus/nndoc.el

;;;***

;;;### (autoloads nil "nndraft" "gnus/nndraft.el" (12727 30254))
;;; Generated autoloads from gnus/nndraft.el

;;;***

;;;### (autoloads nil "nneething" "gnus/nneething.el" (12727 30128))
;;; Generated autoloads from gnus/nneething.el

;;;### (autoloads (nnfolder-generate-active-file) "nnfolder" "gnus/nnfolder.el" (12740 29830))
;;; Generated autoloads from gnus/nnfolder.el

(autoload 'nnfolder-generate-active-file "nnfolder" "\
Look for mbox folders in the nnfolder directory and make them into groups." t nil)

;;;***

;;;### (autoloads nil "nnheader" "gnus/nnheader.el" (12747 7528))
;;; Generated autoloads from gnus/nnheader.el

;;;***

;;;### (autoloads nil "nnheaderems" "gnus/nnheaderems.el" (12740 29835))
;;; Generated autoloads from gnus/nnheaderems.el

;;;### (autoloads nil "nnheaderxm" "gnus/nnheaderxm.el" (12740 29473))
;;; Generated autoloads from gnus/nnheaderxm.el

;;;### (autoloads (nnkiboze-generate-groups) "nnkiboze" "gnus/nnkiboze.el" (12727 30131))
;;; Generated autoloads from gnus/nnkiboze.el

(autoload 'nnkiboze-generate-groups "nnkiboze" "\
Usage: emacs -batch -l nnkiboze -f nnkiboze-generate-groups
Finds out what articles are to be part of the nnkiboze groups." t nil)

;;;***

;;;### (autoloads nil "nnmail" "gnus/nnmail.el" (12743 11580))
;;; Generated autoloads from gnus/nnmail.el

;;;### (autoloads nil "nnmbox" "gnus/nnmbox.el" (12740 29831))
;;; Generated autoloads from gnus/nnmbox.el

;;;***

;;;### (autoloads nil "nnmh" "gnus/nnmh.el" (12745 38025))
;;; Generated autoloads from gnus/nnmh.el

;;;### (autoloads (nnml-generate-nov-databases) "nnml" "gnus/nnml.el" (12745 38025))
;;; Generated autoloads from gnus/nnml.el

(autoload 'nnml-generate-nov-databases "nnml" "\
Generate nov databases in all nnml directories." t nil)

;;;***

;;;### (autoloads nil "nnoo" "gnus/nnoo.el" (12727 30255))
;;; Generated autoloads from gnus/nnoo.el

;;;### (autoloads (nnsoup-revert-variables nnsoup-set-variables nnsoup-pack-replies) "nnsoup" "gnus/nnsoup.el" (12727 30256))
;;; Generated autoloads from gnus/nnsoup.el

(autoload 'nnsoup-pack-replies "nnsoup" "\
Make an outbound package of SOUP replies." t nil)

(autoload 'nnsoup-set-variables "nnsoup" "\
Use the SOUP methods for posting news and mailing mail." t nil)

(autoload 'nnsoup-revert-variables "nnsoup" "\
Revert posting and mailing methods to the standard Emacs methods." t nil)

;;;***

;;;### (autoloads nil "nnspool" "gnus/nnspool.el" (12743 11584))
;;; Generated autoloads from gnus/nnspool.el

;;;***

;;;### (autoloads nil "nntp" "gnus/nntp.el" (12744 54653))
;;; Generated autoloads from gnus/nntp.el

;;;### (autoloads nil "nnvirtual" "gnus/nnvirtual.el" (12740 29832))
;;; Generated autoloads from gnus/nnvirtual.el

;;;### (autoloads (gnus-smiley-display) "smiley" "gnus/smiley.el" (12743 11601))
;;; Generated autoloads from gnus/smiley.el

(autoload 'gnus-smiley-display "smiley" nil t nil)

;;;***

;;;### (autoloads nil "adapt" "hm--html-menus/adapt.el" (12351 16418))
;;; Generated autoloads from hm--html-menus/adapt.el

;;;### (autoloads nil "hm--date" "hm--html-menus/hm--date.el" (12376 19696))
;;; Generated autoloads from hm--html-menus/hm--date.el

;;;***

;;;### (autoloads nil "hm--html-configuration" "hm--html-menus/hm--html-configuration.el" (12583 60715))
;;; Generated autoloads from hm--html-menus/hm--html-configuration.el

;;;***

;;;### (autoloads nil "hm--html-keys" "hm--html-menus/hm--html-keys.el" (12583 60717))
;;; Generated autoloads from hm--html-menus/hm--html-keys.el

;;;***

;;;### (autoloads nil "hm--html-menu" "hm--html-menus/hm--html-menu.el" (12583 60713))
;;; Generated autoloads from hm--html-menus/hm--html-menu.el

;;;***

;;;### (autoloads nil "hm--html" "hm--html-menus/hm--html.el" (12639 8719))
;;; Generated autoloads from hm--html-menus/hm--html.el

;;;***

;;;### (autoloads nil "html-mode" "hm--html-menus/html-mode.el" (12639 8719))
;;; Generated autoloads from hm--html-menus/html-mode.el

;;;***

;;;### (autoloads (html-view-get-display html-view-goto-url html-view-view-buffer html-view-view-file html-view-start-mosaic) "html-view" "hm--html-menus/html-view.el" (12318 54261))
;;; Generated autoloads from hm--html-menus/html-view.el

(autoload 'html-view-start-mosaic "html-view" "\
Start Mosaic." t nil)

(autoload 'html-view-view-file "html-view" "\
View an html file with Mosaic." t nil)

(autoload 'html-view-view-buffer "html-view" "\
View html buffer with Mosaic.
If BUFFER-TO-VIEW is nil, then the current buffer is used." t nil)

(autoload 'html-view-goto-url "html-view" "\
Goto an URL in Mosaic." t nil)

(autoload 'html-view-get-display "html-view" "\
Get the display for Mosaic." t nil)

;;;***

;;;### (autoloads nil "tmpl-minor-mode" "hm--html-menus/tmpl-minor-mode.el" (12639 8720))
;;; Generated autoloads from hm--html-menus/tmpl-minor-mode.el

;;;### (autoloads nil "hact" "hyperbole/hact.el" (12376 19672))
;;; Generated autoloads from hyperbole/hact.el

;;;***

;;;### (autoloads nil "hactypes" "hyperbole/hactypes.el" (12558 60986))
;;; Generated autoloads from hyperbole/hactypes.el

;;;***

;;;### (autoloads nil "hargs" "hyperbole/hargs.el" (12384 54290))
;;; Generated autoloads from hyperbole/hargs.el

;;;### (autoloads nil "hbdata" "hyperbole/hbdata.el" (12376 19673))
;;; Generated autoloads from hyperbole/hbdata.el

;;;***

;;;### (autoloads nil "hbmap" "hyperbole/hbmap.el" (12558 60987))
;;; Generated autoloads from hyperbole/hbmap.el

;;;***

;;;### (autoloads nil "hbut" "hyperbole/hbut.el" (12558 60988))
;;; Generated autoloads from hyperbole/hbut.el

;;;***

;;;### (autoloads nil "hgnus" "hyperbole/hgnus.el" (12558 60988))
;;; Generated autoloads from hyperbole/hgnus.el

;;;### (autoloads nil "hhist" "hyperbole/hhist.el" (12376 19675))
;;; Generated autoloads from hyperbole/hhist.el

;;;***

;;;### (autoloads nil "hib-doc-id" "hyperbole/hib-doc-id.el" (12376 19675))
;;; Generated autoloads from hyperbole/hib-doc-id.el

;;;***

;;;### (autoloads nil "hib-kbd" "hyperbole/hib-kbd.el" (12558 60988))
;;; Generated autoloads from hyperbole/hib-kbd.el

;;;***

;;;### (autoloads nil "hibtypes" "hyperbole/hibtypes.el" (12558 60986))
;;; Generated autoloads from hyperbole/hibtypes.el

;;;***

;;;### (autoloads nil "hinit" "hyperbole/hinit.el" (12558 60989))
;;; Generated autoloads from hyperbole/hinit.el

;;;### (autoloads nil "hlvar" "hyperbole/hlvar.el" (12376 19677))
;;; Generated autoloads from hyperbole/hlvar.el

;;;### (autoloads (hmail:compose) "hmail" "hyperbole/hmail.el" (12558 60989))
;;; Generated autoloads from hyperbole/hmail.el

(autoload 'hmail:compose "hmail" "\
Compose mail with ADDRESS and evaluation of EXPR.
Optional SUBJECT and HELP message may also be given." t nil)

;;;***

;;;### (autoloads nil "hmh" "hyperbole/hmh.el" (12376 19677))
;;; Generated autoloads from hyperbole/hmh.el

;;;***

;;;### (autoloads nil "hmoccur" "hyperbole/hmoccur.el" (12376 19678))
;;; Generated autoloads from hyperbole/hmoccur.el

;;;### (autoloads (Info-handle-in-note smart-info-assist smart-info) "hmous-info" "hyperbole/hmous-info.el" (12558 60990))
;;; Generated autoloads from hyperbole/hmous-info.el

(autoload 'smart-info "hmous-info" "\
Walks through Info documentation networks using one key or mouse key.

If key is pressed within:
 (1) the first line of an Info Menu Entry or Cross Reference, the desired node
       is found;
 (2) the Up, Next, or Previous entries of a Node Header (first line),
       the desired node is found;
 (3) the File entry of a Node Header (first line),       
       the 'Top' node within that file is found;
 (4) at the end of the current node, the Next node is found (this will
       descend subtrees if the function 'Info-global-next' is bound);
 (5) anywhere else (e.g. at the end of a line), the current node entry is
       scrolled up one windowful.

Returns t if key is pressed within an Info Node Header, Cross Reference,
or a Menu; otherwise returns nil." t nil)

(autoload 'smart-info-assist "hmous-info" "\
Walks through Info documentation networks using one assist-key or mouse assist-key.

If assist-key is pressed within:
 (1) the first line of an Info Menu Entry or Cross Reference, the desired node
       is found;
 (2) the Up, Next, or Previous entries of a Node Header (first line),
       the last node in the history list is found;
 (3) the File entry of a Node Header (first line),       
       the 'DIR' root-level node is found;
 (4) at the end of the current node, the Previous node is found (this will
       return from subtrees if the function 'Info-global-prev is bound);
 (5) anywhere else (e.g. at the end of a line), the current node entry is
       scrolled down one windowful.

Returns t if assist-key is pressed within an Info Node Header, Cross Reference,
or a Menu; otherwise returns nil." t nil)

(autoload 'Info-handle-in-note "hmous-info" "\
Follows an Info cross-reference.
If point is within the first line of an Info note (cross-reference), follows
cross-reference and returns t; otherwise returns nil." nil nil)

;;;***

;;;### (autoloads nil "hmouse-drv" "hyperbole/hmouse-drv.el" (12558 60992))
;;; Generated autoloads from hyperbole/hmouse-drv.el

;;;***

;;;### (autoloads nil "hmouse-key" "hyperbole/hmouse-key.el" (12384 54293))
;;; Generated autoloads from hyperbole/hmouse-key.el

;;;### (autoloads nil "hmouse-mod" "hyperbole/hmouse-mod.el" (12376 19680))
;;; Generated autoloads from hyperbole/hmouse-mod.el

;;;***

;;;### (autoloads nil "hmouse-reg" "hyperbole/hmouse-reg.el" (12384 54294))
;;; Generated autoloads from hyperbole/hmouse-reg.el

;;;***

;;;### (autoloads nil "hmouse-sh" "hyperbole/hmouse-sh.el" (12384 54294))
;;; Generated autoloads from hyperbole/hmouse-sh.el

;;;***

;;;### (autoloads (smart-tags-file smart-tags-file-path smart-objc-oobr smart-objc smart-fortran-at-tag-p smart-c++-oobr smart-c++ smart-c-at-tag-p smart-asm-at-tag-p) "hmouse-tag" "hyperbole/hmouse-tag.el" (12384 54295))
;;; Generated autoloads from hyperbole/hmouse-tag.el

(autoload 'smart-asm-at-tag-p "hmouse-tag" "\
Return assembly tag name that point is within, else nil." nil nil)

(autoload 'smart-c-at-tag-p "hmouse-tag" "\
Return C tag name that point is within, else nil." nil nil)

(autoload 'smart-c++ "hmouse-tag" "\
Jumps to the definition of optional C++ IDENTIFIER or the one at point.
Optional second arg NEXT means jump to next matching C++ tag.

It assumes that its caller has already checked that the key was pressed in an
appropriate buffer and has moved the cursor to the selected buffer.

If:
 (1) on a '#include' statement, the include file is displayed;
     Look for include file in directory lists 'smart-c-cpp-include-dirs'
     and 'smart-c-include-dirs'.
 (2) on a C++ identifier, the identifier definition is displayed,
     assuming the identifier is found within an 'etags' generated tag file
     in the current directory or any of its ancestor directories.
 (3) if 'smart-c-use-lib-man' is non-nil, the C++ identifier is
     recognized as a library symbol, and a man page is found for the
     identifier, then the man page is displayed." t nil)

(autoload 'smart-c++-oobr "hmouse-tag" "\
Jumps to the definition of selected C++ construct via OO-Browser support.
Optional JUNK is ignored.  Does nothing if the OO-Browser is not available.

It assumes that its caller has already checked that the key was pressed in an
appropriate buffer and has moved the cursor to the selected buffer.

If key is pressed:
 (1) on a '#include' statement, the include file is displayed;
     Look for include file in directory lists 'smart-c-cpp-include-dirs'
     and 'smart-c-include-dirs'.
 (2) within a method declaration, its definition is displayed;
 (3) on a class name, the class definition is shown.

 (2) and (3) require that an OO-Browser Environment has been loaded with
     the {M-x br-env-load RTN} command." t nil)

(autoload 'smart-fortran-at-tag-p "hmouse-tag" "\
Return Fortran tag name that point is within, else nil." nil nil)

(autoload 'smart-objc "hmouse-tag" "\
Jumps to the definition of optional Objective-C IDENTIFIER or the one at point.
Optional second arg NEXT means jump to next matching Objective-C tag.

It assumes that its caller has already checked that the key was pressed in an
appropriate buffer and has moved the cursor to the selected buffer.

If:
 (1) on a '#include' statement, the include file is displayed;
     Look for include file in directory lists 'smart-c-cpp-include-dirs'
     and 'smart-c-include-dirs'.
 (2) on an Objective-C identifier, the identifier definition is displayed,
     assuming the identifier is found within an 'etags' generated tag file
     in the current directory or any of its ancestor directories.
 (3) if 'smart-c-use-lib-man' is non-nil, the Objective-C identifier is
     recognized as a library symbol, and a man page is found for the
     identifier, then the man page is displayed." t nil)

(autoload 'smart-objc-oobr "hmouse-tag" "\
Jumps to the definition of selected Objective-C construct via OO-Browser support.
Optional JUNK is ignored.  Does nothing if the OO-Browser is not available.

It assumes that its caller has already checked that the key was pressed in an
appropriate buffer and has moved the cursor to the selected buffer.

If key is pressed:
 (1) on a '#include' statement, the include file is displayed;
     Look for include file in directory lists 'smart-c-cpp-include-dirs'
     and 'smart-c-include-dirs'.
 (2) within a method declaration, its definition is displayed;
 (3) on a class name, the class definition is shown.

 (2) and (3) require that an OO-Browser Environment has been loaded with
     the {M-x br-env-load RTN} command." t nil)

(autoload 'smart-tags-file-path "hmouse-tag" "\
Expand relative FILE name by looking it up in the nearest tags file.
Return FILE unchanged if it exists relative to the current directory or
cannot be expanded via a tags file." nil nil)

(autoload 'smart-tags-file "hmouse-tag" "\
Return appropriate tags file name for CURR-FILENAME or 'tags-file-name'." nil nil)

;;;***

;;;### (autoloads nil "hpath" "hyperbole/hpath.el" (12558 60991))
;;; Generated autoloads from hyperbole/hpath.el

;;;### (autoloads nil "hrmail" "hyperbole/hrmail.el" (12376 19683))
;;; Generated autoloads from hyperbole/hrmail.el

;;;***

;;;### (autoloads nil "hsite-ex" "hyperbole/hsite-ex.el" (12558 60993))
;;; Generated autoloads from hyperbole/hsite-ex.el

;;;***

;;;### (autoloads nil "hsite" "hyperbole/hsite.el" (12558 61003))
;;; Generated autoloads from hyperbole/hsite.el

;;;### (autoloads nil "hsmail" "hyperbole/hsmail.el" (12376 19682))
;;; Generated autoloads from hyperbole/hsmail.el

;;;***

;;;### (autoloads nil "hsys-hbase" "hyperbole/hsys-hbase.el" (12558 60994))
;;; Generated autoloads from hyperbole/hsys-hbase.el

;;;***

;;;### (autoloads nil "hsys-w3" "hyperbole/hsys-w3.el" (12558 60995))
;;; Generated autoloads from hyperbole/hsys-w3.el

;;;***

;;;### (autoloads nil "hsys-wais" "hyperbole/hsys-wais.el" (12558 60995))
;;; Generated autoloads from hyperbole/hsys-wais.el

;;;### (autoloads nil "hsys-www" "hyperbole/hsys-www.el" (12376 19685))
;;; Generated autoloads from hyperbole/hsys-www.el

;;;***

;;;### (autoloads nil "htz" "hyperbole/htz.el" (12376 19685))
;;; Generated autoloads from hyperbole/htz.el

;;;***

;;;### (autoloads nil "hui-em19-b" "hyperbole/hui-em19-b.el" (12558 60996))
;;; Generated autoloads from hyperbole/hui-em19-b.el

;;;### (autoloads nil "hui-ep-but" "hyperbole/hui-ep-but.el" (12376 19685))
;;; Generated autoloads from hyperbole/hui-ep-but.el

;;;***

;;;### (autoloads nil "hui-epV4-b" "hyperbole/hui-epV4-b.el" (12376 19687))
;;; Generated autoloads from hyperbole/hui-epV4-b.el

;;;***

;;;### (autoloads nil "hui-menu" "hyperbole/hui-menu.el" (12558 61004))
;;; Generated autoloads from hyperbole/hui-menu.el

;;;***

;;;### (autoloads nil "hui-mini" "hyperbole/hui-mini.el" (12558 61074))
;;; Generated autoloads from hyperbole/hui-mini.el

;;;***

;;;### (autoloads nil "hui-mouse" "hyperbole/hui-mouse.el" (12558 60994))
;;; Generated autoloads from hyperbole/hui-mouse.el

;;;***

;;;### (autoloads nil "hui-window" "hyperbole/hui-window.el" (12558 60997))
;;; Generated autoloads from hyperbole/hui-window.el

;;;***

;;;### (autoloads nil "hui-xe-but" "hyperbole/hui-xe-but.el" (12558 60998))
;;; Generated autoloads from hyperbole/hui-xe-but.el

;;;### (autoloads nil "hui" "hyperbole/hui.el" (12376 19688))
;;; Generated autoloads from hyperbole/hui.el

;;;### (autoloads (var:append) "hvar" "hyperbole/hvar.el" (12376 19689))
;;; Generated autoloads from hyperbole/hvar.el

(autoload 'var:append "hvar" "\
Appends to value held by VAR-SYMBOL-NAME, LIST-TO-ADD.  Returns new value.
If VAR-SYMBOL-NAME is unbound, it is set to LIST-TO-ADD.
Often used to append to 'hook' variables." nil nil)

;;;***

;;;### (autoloads nil "hversion" "hyperbole/hversion.el" (12558 60998))
;;; Generated autoloads from hyperbole/hversion.el

;;;### (autoloads nil "hvm" "hyperbole/hvm.el" (12376 19689))
;;; Generated autoloads from hyperbole/hvm.el

;;;### (autoloads (hypb:configuration) "hypb" "hyperbole/hypb.el" (12558 61000))
;;; Generated autoloads from hyperbole/hypb.el

(autoload 'hypb:configuration "hypb" "\
Insert Emacs configuration information at the end of optional OUT-BUF or the current buffer." nil nil)

;;;***

;;;### (autoloads nil "hyperbole" "hyperbole/hyperbole.el" (12558 60999))
;;; Generated autoloads from hyperbole/hyperbole.el

(defvar action-key-url-function 'w3-fetch "\
Value is a function of one argument, a url, which displays the url referent.
Possible values are:
  w3-fetch - display using the W3 Emacs web browser;
  highlight-headers-follow-url-netscape - display in Netscape;
  highlight-headers-follow-url-mosaic - display in Mosaic.")

(defvar kimport:mode-alist '((t . kimport:text) (outline-mode . kimport:star-outline)) "\
Alist of (major-mode . importation-function) elements.
This determines the type of importation done on a file when `kimport:file' is
called if the major mode of the import file matches the car of an element in
this list.  If there is no match, then `kimport:suffix-alist' is checked.  If
that yields no match, the element in this list whose car is 't is used.  It
normally does an import of a koutline or text file.

Each importation-function must take two arguments, a buffer/file to import
and a buffer/file into which to insert the imported elements and a third
optional argument, CHILDREN-P, which when non-nil means insert imported cells
as the initial set of children of the current cell, if any.

   outline-mode  - imported as an Emacs outline whose entries begin with
                   asterisks; 
   .kot
   .kotl         - imported as a structured koutline

   all others    - imported as text.")

(defvar kimport:suffix-alist '(("\\.otl$" . kimport:star-outline) ("\\.aug$" . kimport:aug-post-outline)) "\
Alist of (buffer-name-suffix-regexp . importation-function) elements.
This determines the type of importation done on a file when `kimport:file' is
called.  Each importation-function must take two arguments, a buffer/file to
import and a buffer/file into which to insert the imported elements and a
third optional argument, CHILDREN-P, which when non-nil means insert imported
cells as the initial set of children of the current cell, if any.

   .otl  - imported as an Emacs outline whose entries begin with asterisks;
   .kot
   .kotl - imported as a structured koutline
   .aug  - imported as an Augment post-numbered outline.")

;;;***

;;;### (autoloads nil "set" "hyperbole/set.el" (12376 19690))
;;; Generated autoloads from hyperbole/set.el

;;;### (autoloads (wconfig-yank-pop wconfig-ring-save wconfig-delete-pop wconfig-restore-by-name wconfig-delete-by-name wconfig-add-by-name) "wconfig" "hyperbole/wconfig.el" (12376 19691))
;;; Generated autoloads from hyperbole/wconfig.el

(autoload 'wconfig-add-by-name "wconfig" "\
Saves the current window configuration under the string NAME.
When called interactively and a window configuration already exists under
NAME, confirms whether or not to replace it." t nil)

(autoload 'wconfig-delete-by-name "wconfig" "\
Deletes window configuration saved under NAME." t nil)

(autoload 'wconfig-restore-by-name "wconfig" "\
Restores window configuration saved under NAME." t nil)

(autoload 'wconfig-delete-pop "wconfig" "\
Replaces current window config with most recently saved config in ring.
Then deletes this new configuration from the ring." t nil)

(autoload 'wconfig-ring-save "wconfig" "\
Saves the current window configuration onto the save ring.
Use {\\[wconfig-yank-pop]} to restore it at a later time." t nil)

(autoload 'wconfig-yank-pop "wconfig" "\
Replaces current window config with prefix arg Nth prior one in save ring.
Interactively, default value of N = 1, meaning the last saved window
configuration is displayed.

The sequence of window configurations wraps around, so that after the oldest
one comes the newest one." t nil)

;;;***

;;;### (autoloads (rolo-logic) "wrolo-logic" "hyperbole/wrolo-logic.el" (12376 19691))
;;; Generated autoloads from hyperbole/wrolo-logic.el

(autoload 'rolo-logic "wrolo-logic" "\
Apply FUNC to all entries in optional IN-BUFS, display entries where FUNC is non-nil.
If IN-BUFS is nil, 'rolo-file-list' is used.  If optional COUNT-ONLY is
non-nil, don't display entries, return count of matching entries only.  If
optional INCLUDE-SUB-ENTRIES flag is non-nil, FUNC will be applied across all
sub-entries at once.  Default is to apply FUNC to each entry and sub-entry
separately.  Entries are displayed with all of their sub-entries unless
INCLUDE-SUB-ENTRIES is nil and optional NO-SUB-ENTRIES-OUT flag is non-nil.
FUNC should use the free variables 'start' and 'end' which contain the limits
of the region on which it should operate.  Returns number of applications of
FUNC that return non-nil." t nil)

;;;***

;;;### (autoloads nil "wrolo-menu" "hyperbole/wrolo-menu.el" (12558 61004))
;;; Generated autoloads from hyperbole/wrolo-menu.el

;;;***

;;;### (autoloads (rolo-yank rolo-sort rolo-kill rolo-grep rolo-fgrep rolo-edit rolo-display-matches rolo-add) "wrolo" "hyperbole/wrolo.el" (12558 61000))
;;; Generated autoloads from hyperbole/wrolo.el

(autoload 'rolo-add "wrolo" "\
Adds a new entry in personal rolodex for NAME.
Last name first is best, e.g. \"Smith, John\".
With prefix argument, prompts for optional FILE to add entry within.
NAME may be of the form: parent/child to insert child below a parent
entry which begins with the parent string." t nil)

(autoload 'rolo-display-matches "wrolo" "\
Display optional DISPLAY-BUF buffer of previously found rolodex matches.
If DISPLAY-BUF is nil, use the value in 'rolo-display-buffer'.
Second arg RETURN-TO-BUFFER is the buffer to leave point within after the display." t nil)

(autoload 'rolo-edit "wrolo" "\
Edits a rolodex entry given by optional NAME within 'rolo-file-list'.
With prefix argument, prompts for optional FILE to locate entry within.
With no NAME arg, simply displays FILE or first entry in 'rolo-file-list' in an
editable mode.  NAME may be of the form: parent/child to edit child below a
parent entry which begins with the parent string." t nil)

(autoload 'rolo-fgrep "wrolo" "\
Display rolodex entries matching STRING.
To a maximum of optional prefix arg MAX-MATCHES, in file(s) from optional
ROLO-FILE or rolo-file-list.  Default is to find all matching entries.  Each
entry is displayed with all of its sub-entries.  Optional COUNT-ONLY non-nil
means don't retrieve and don't display matching entries.  Optional NO-DISPLAY
non-nil means retrieve entries but don't display.

Nil value of MAX-MATCHES means find all matches, t value means find all matches
but omit file headers, negative values mean find up to the inverse of that
number of entries and omit file headers.

Returns number of entries matched.  See also documentation for the variable
rolo-file-list." t nil)

(autoload 'rolo-grep "wrolo" "\
Display rolodex entries matching REGEXP.
To a maximum of prefix arg MAX-MATCHES, in buffer(s) from optional ROLO-BUFS or
rolo-file-list.  Default is to find all matching entries.  Each entry is
displayed with all of its sub-entries.  Optional COUNT-ONLY non-nil means don't
retrieve and don't display matching entries.  Optional NO-DISPLAY non-nil
means retrieve entries but don't display.

Nil value of MAX-MATCHES means find all matches, t value means find all matches
but omit file headers, negative values mean find up to the inverse of that
number of entries and omit file headers.

Returns number of entries matched.  See also documentation for the variable
rolo-file-list." t nil)

(autoload 'rolo-kill "wrolo" "\
Kills a rolodex entry given by NAME within 'rolo-file-list'.
With prefix argument, prompts for optional FILE to locate entry within.
NAME may be of the form: parent/child to kill child below a parent entry
which begins with the parent string.
Returns t if entry is killed, nil otherwise." t nil)

(autoload 'rolo-sort "wrolo" "\
Sorts up to 14 levels of entries in ROLO-FILE (default is personal rolo).
Assumes entries are delimited by one or more '*'characters.
Returns list of number of groupings at each entry level." t nil)

(autoload 'rolo-yank "wrolo" "\
Inserts at point the first rolodex entry matching NAME.
With optional prefix arg, REGEXP-P, treats NAME as a regular expression instead
of a string." t nil)

;;;***

;;;### (autoloads nil "bridge" "ilisp/bridge.el" (12376 19421))
;;; Generated autoloads from ilisp/bridge.el

;;;***

;;;### (autoloads nil "comint-ipc" "ilisp/comint-ipc.el" (12376 19418))
;;; Generated autoloads from ilisp/comint-ipc.el

;;;***

;;;### (autoloads nil "comint-v18" "ilisp/comint-v18.el" (12376 19423))
;;; Generated autoloads from ilisp/comint-v18.el

;;;***

;;;### (autoloads nil "completer" "ilisp/completer.el" (12675 57068))
;;; Generated autoloads from ilisp/completer.el

;;;### (autoloads nil "ilcompat" "ilisp/ilcompat.el" (12376 19424))
;;; Generated autoloads from ilisp/ilcompat.el

;;;### (autoloads nil "ilfsf18" "ilisp/ilfsf18.el" (12376 19438))
;;; Generated autoloads from ilisp/ilfsf18.el

;;;***

;;;### (autoloads nil "ilfsf19" "ilisp/ilfsf19.el" (12376 19439))
;;; Generated autoloads from ilisp/ilfsf19.el

;;;***

;;;### (autoloads nil "ilisp-acl" "ilisp/ilisp-acl.el" (12376 19427))
;;; Generated autoloads from ilisp/ilisp-acl.el

;;;***

;;;### (autoloads nil "ilisp-aut" "ilisp/ilisp-aut.el" (12376 19428))
;;; Generated autoloads from ilisp/ilisp-aut.el

;;;***

;;;### (autoloads nil "ilisp-bat" "ilisp/ilisp-bat.el" (12376 19419))
;;; Generated autoloads from ilisp/ilisp-bat.el

;;;***

;;;### (autoloads nil "ilisp-bug" "ilisp/ilisp-bug.el" (12376 19425))
;;; Generated autoloads from ilisp/ilisp-bug.el

;;;***

;;;### (autoloads nil "ilisp-chs" "ilisp/ilisp-chs.el" (12376 19439))
;;; Generated autoloads from ilisp/ilisp-chs.el

;;;***

;;;### (autoloads nil "ilisp-cl" "ilisp/ilisp-cl.el" (12376 19427))
;;; Generated autoloads from ilisp/ilisp-cl.el

;;;***

;;;### (autoloads nil "ilisp-cmp" "ilisp/ilisp-cmp.el" (12376 19428))
;;; Generated autoloads from ilisp/ilisp-cmp.el

;;;***

;;;### (autoloads nil "ilisp-cmt" "ilisp/ilisp-cmt.el" (12376 19428))
;;; Generated autoloads from ilisp/ilisp-cmt.el

;;;***

;;;### (autoloads nil "ilisp-cmu" "ilisp/ilisp-cmu.el" (12376 19431))
;;; Generated autoloads from ilisp/ilisp-cmu.el

;;;***

;;;### (autoloads nil "ilisp-cpat" "ilisp/ilisp-cpat.el" (12376 19423))
;;; Generated autoloads from ilisp/ilisp-cpat.el

;;;***

;;;### (autoloads nil "ilisp-def" "ilisp/ilisp-def.el" (12376 19429))
;;; Generated autoloads from ilisp/ilisp-def.el

;;;***

;;;### (autoloads nil "ilisp-dia" "ilisp/ilisp-dia.el" (12376 19429))
;;; Generated autoloads from ilisp/ilisp-dia.el

;;;***

;;;### (autoloads nil "ilisp-doc" "ilisp/ilisp-doc.el" (12376 19429))
;;; Generated autoloads from ilisp/ilisp-doc.el

;;;***

;;;### (autoloads nil "ilisp-el" "ilisp/ilisp-el.el" (12376 19431))
;;; Generated autoloads from ilisp/ilisp-el.el

;;;***

;;;### (autoloads nil "ilisp-ext" "ilisp/ilisp-ext.el" (12376 19420))
;;; Generated autoloads from ilisp/ilisp-ext.el

;;;***

;;;### (autoloads nil "ilisp-hi" "ilisp/ilisp-hi.el" (12376 19431))
;;; Generated autoloads from ilisp/ilisp-hi.el

;;;***

;;;### (autoloads nil "ilisp-hlw" "ilisp/ilisp-hlw.el" (12376 19440))
;;; Generated autoloads from ilisp/ilisp-hlw.el

;;;***

;;;### (autoloads nil "ilisp-hnd" "ilisp/ilisp-hnd.el" (12376 19431))
;;; Generated autoloads from ilisp/ilisp-hnd.el

;;;***

;;;### (autoloads nil "ilisp-ind" "ilisp/ilisp-ind.el" (12376 19430))
;;; Generated autoloads from ilisp/ilisp-ind.el

;;;***

;;;### (autoloads nil "ilisp-inp" "ilisp/ilisp-inp.el" (12376 19431))
;;; Generated autoloads from ilisp/ilisp-inp.el

;;;***

;;;### (autoloads nil "ilisp-kcl" "ilisp/ilisp-kcl.el" (12376 19433))
;;; Generated autoloads from ilisp/ilisp-kcl.el

;;;***

;;;### (autoloads nil "ilisp-key" "ilisp/ilisp-key.el" (12376 19433))
;;; Generated autoloads from ilisp/ilisp-key.el

;;;***

;;;### (autoloads nil "ilisp-kil" "ilisp/ilisp-kil.el" (12376 19433))
;;; Generated autoloads from ilisp/ilisp-kil.el

;;;***

;;;### (autoloads nil "ilisp-low" "ilisp/ilisp-low.el" (12376 19433))
;;; Generated autoloads from ilisp/ilisp-low.el

;;;***

;;;### (autoloads nil "ilisp-luc" "ilisp/ilisp-luc.el" (12376 19433))
;;; Generated autoloads from ilisp/ilisp-luc.el

;;;***

;;;### (autoloads nil "ilisp-mak" "ilisp/ilisp-mak.el" (12376 19424))
;;; Generated autoloads from ilisp/ilisp-mak.el

;;;***

;;;### (autoloads nil "ilisp-menu" "ilisp/ilisp-menu.el" (12376 19419))
;;; Generated autoloads from ilisp/ilisp-menu.el

;;;***

;;;### (autoloads nil "ilisp-mnb" "ilisp/ilisp-mnb.el" (12376 19440))
;;; Generated autoloads from ilisp/ilisp-mnb.el

;;;***

;;;### (autoloads nil "ilisp-mod" "ilisp/ilisp-mod.el" (12558 60883))
;;; Generated autoloads from ilisp/ilisp-mod.el

;;;### (autoloads nil "ilisp-mov" "ilisp/ilisp-mov.el" (12376 19435))
;;; Generated autoloads from ilisp/ilisp-mov.el

;;;***

;;;### (autoloads nil "ilisp-out" "ilisp/ilisp-out.el" (12376 19435))
;;; Generated autoloads from ilisp/ilisp-out.el

;;;***

;;;### (autoloads nil "ilisp-prc" "ilisp/ilisp-prc.el" (12376 19435))
;;; Generated autoloads from ilisp/ilisp-prc.el

;;;***

;;;### (autoloads nil "ilisp-prn" "ilisp/ilisp-prn.el" (12376 19434))
;;; Generated autoloads from ilisp/ilisp-prn.el

;;;***

;;;### (autoloads nil "ilisp-rng" "ilisp/ilisp-rng.el" (12376 19436))
;;; Generated autoloads from ilisp/ilisp-rng.el

;;;***

;;;### (autoloads nil "ilisp-s2c" "ilisp/ilisp-s2c.el" (12376 19425))
;;; Generated autoloads from ilisp/ilisp-s2c.el

;;;***

;;;### (autoloads nil "ilisp-sch" "ilisp/ilisp-sch.el" (12376 19436))
;;; Generated autoloads from ilisp/ilisp-sch.el

;;;***

;;;### (autoloads nil "ilisp-snd" "ilisp/ilisp-snd.el" (12376 19437))
;;; Generated autoloads from ilisp/ilisp-snd.el

;;;***

;;;### (autoloads nil "ilisp-src" "ilisp/ilisp-src.el" (12376 19421))
;;; Generated autoloads from ilisp/ilisp-src.el

;;;***

;;;### (autoloads nil "ilisp-sym" "ilisp/ilisp-sym.el" (12376 19437))
;;; Generated autoloads from ilisp/ilisp-sym.el

;;;***

;;;### (autoloads nil "ilisp-utl" "ilisp/ilisp-utl.el" (12376 19437))
;;; Generated autoloads from ilisp/ilisp-utl.el

;;;***

;;;### (autoloads nil "ilisp-val" "ilisp/ilisp-val.el" (12376 19438))
;;; Generated autoloads from ilisp/ilisp-val.el

;;;***

;;;### (autoloads nil "ilisp-xfr" "ilisp/ilisp-xfr.el" (12376 19438))
;;; Generated autoloads from ilisp/ilisp-xfr.el

;;;***

;;;### (autoloads nil "ilisp" "ilisp/ilisp.el" (12376 19421))
;;; Generated autoloads from ilisp/ilisp.el

;;;***

;;;### (autoloads nil "illuc19" "ilisp/illuc19.el" (12376 19441))
;;; Generated autoloads from ilisp/illuc19.el

;;;***

;;;### (autoloads nil "ilxemacs" "ilisp/ilxemacs.el" (12657 40696))
;;; Generated autoloads from ilisp/ilxemacs.el

;;;***

;;;### (autoloads nil "iso-ascii" "iso/iso-ascii.el" (12282 38391))
;;; Generated autoloads from iso/iso-ascii.el

;;;### (autoloads nil "iso-cvt" "iso/iso-cvt.el" (12376 19651))
;;; Generated autoloads from iso/iso-cvt.el

;;;***

;;;### (autoloads nil "iso-insert" "iso/iso-insert.el" (12376 19650))
;;; Generated autoloads from iso/iso-insert.el

;;;***

;;;### (autoloads nil "iso-swed" "iso/iso-swed.el" (12376 19650))
;;; Generated autoloads from iso/iso-swed.el

;;;***

;;;### (autoloads nil "iso-syntax" "iso/iso-syntax.el" (12376 19652))
;;; Generated autoloads from iso/iso-syntax.el

;;;***

;;;### (autoloads nil "iso8859-1" "iso/iso8859-1.el" (12376 19650))
;;; Generated autoloads from iso/iso8859-1.el

;;;***

;;;### (autoloads nil "swedish" "iso/swedish.el" (12376 19651))
;;; Generated autoloads from iso/swedish.el

;;;***

;;;### (autoloads nil "site-start" "lisp/site-start.el" (12605 44813))
;;; Generated autoloads from lisp/site-start.el

;;;### (autoloads (mc-deactivate-passwd mc-install-write-mode mc-install-read-mode) "mailcrypt" "mailcrypt/mailcrypt.el" (12675 57176))
;;; Generated autoloads from mailcrypt/mailcrypt.el

(autoload 'mc-install-read-mode "mailcrypt" nil t nil)

(autoload 'mc-install-write-mode "mailcrypt" nil t nil)

(autoload 'mc-deactivate-passwd "mailcrypt" "\
*Deactivate the passphrase cache." t nil)

;;;***

;;;### (autoloads (mc-pgp-fetch-key mc-scheme-pgp) "mc-pgp" "mailcrypt/mc-pgp.el" (12639 8702))
;;; Generated autoloads from mailcrypt/mc-pgp.el

(autoload 'mc-scheme-pgp "mc-pgp" nil nil nil)

(autoload 'mc-pgp-fetch-key "mc-pgp" "\
Attempt to fetch a key for addition to PGP keyring.  Interactively,
prompt for string matching key to fetch.

Non-interactively, ID must be a pair.  The CAR must be a bare Email
address and the CDR a keyID (with \"0x\" prefix).  Either, but not
both, may be nil.

Return t if we think we were successful; nil otherwise.  Note that nil
is not necessarily an error, since we may have merely fired off an Email
request for the key." t nil)

;;;***

;;;### (autoloads (mc-remailer-insert-response-block mc-remailer-encrypt-for-chain mc-remailer-insert-pseudonym) "mc-remail" "mailcrypt/mc-remail.el" (12639 8701))
;;; Generated autoloads from mailcrypt/mc-remail.el

(autoload 'mc-remailer-insert-pseudonym "mc-remail" "\
Insert pseudonym as a From field in the hash-mark header.

See the documentation for the variable `mc-remailer-pseudonyms' for
more information." t nil)

(autoload 'mc-remailer-encrypt-for-chain "mc-remail" "\
Encrypt message for a remailer chain, prompting for chain to use.

With \\[universal-argument], pause before each encryption." t nil)

(autoload 'mc-remailer-insert-response-block "mc-remail" "\
Insert response block at point, prompting for chain to use.

With \\[universal-argument], enter a recursive edit of the innermost
layer of the block before encrypting it." t nil)

;;;***

;;;### (autoloads (mc-mh-snarf-keys mc-mh-verify-signature mc-mh-decrypt-message mc-gnus-decrypt-message mc-gnus-snarf-keys mc-gnus-verify-signature mc-vm-snarf-keys mc-vm-decrypt-message mc-vm-verify-signature mc-rmail-decrypt-message mc-rmail-verify-signature mc-rmail-summary-snarf-keys mc-rmail-summary-decrypt-message mc-rmail-summary-verify-signature mc-snarf-keys mc-snarf mc-insert-public-key mc-verify-signature mc-verify mc-sign-message mc-sign mc-decrypt-message mc-decrypt mc-encrypt-message mc-encrypt mc-cleanup-recipient-headers) "mc-toplev" "mailcrypt/mc-toplev.el" (12639 8701))
;;; Generated autoloads from mailcrypt/mc-toplev.el

(autoload 'mc-cleanup-recipient-headers "mc-toplev" nil nil nil)

(autoload 'mc-encrypt "mc-toplev" "\
*Encrypt the current buffer.

Exact behavior depends on current major mode.

With \\[universal-argument], prompt for User ID to sign as.

With \\[universal-argument] \\[universal-argument], prompt for encryption scheme to use." t nil)

(autoload 'mc-encrypt-message "mc-toplev" "\
*Encrypt a message for RECIPIENTS using the given encryption SCHEME.
RECIPIENTS is a comma separated string. If SCHEME is nil, use the value
of `mc-default-scheme'.  Returns t on success, nil otherwise." nil nil)

(autoload 'mc-decrypt "mc-toplev" "\
*Decrypt a message in the current buffer.

Exact behavior depends on current major mode." t nil)

(autoload 'mc-decrypt-message "mc-toplev" "\
Decrypt whatever message is in the current buffer.
Returns a pair (SUCCEEDED . VERIFIED) where SUCCEEDED is t if the encryption
succeeded and VERIFIED is t if it had a valid signature." nil nil)

(autoload 'mc-sign "mc-toplev" "\
*Sign a message in the current buffer.

Exact behavior depends on current major mode.

With one prefix arg, prompts for private key to use, with two prefix args,
also prompts for encryption scheme to use.  With negative prefix arg,
inhibits clearsigning (pgp)." t nil)

(autoload 'mc-sign-message "mc-toplev" "\
Clear sign the message." nil nil)

(autoload 'mc-verify "mc-toplev" "\
*Verify a message in the current buffer.

Exact behavior depends on current major mode." t nil)

(autoload 'mc-verify-signature "mc-toplev" "\
*Verify the signature of the signed message in the current buffer.
Show the result as a message in the minibuffer. Returns t if the signature
is verified." nil nil)

(autoload 'mc-insert-public-key "mc-toplev" "\
*Insert your public key at point.
With one prefix arg, prompts for user id to use. With two prefix
args, prompts for encryption scheme." t nil)

(autoload 'mc-snarf "mc-toplev" "\
*Add all public keys in the buffer to your keyring.

Exact behavior depends on current major mode." t nil)

(autoload 'mc-snarf-keys "mc-toplev" "\
*Add all public keys in the buffer to your keyring." t nil)

(autoload 'mc-rmail-summary-verify-signature "mc-toplev" "\
*Verify the signature in the current message." t nil)

(autoload 'mc-rmail-summary-decrypt-message "mc-toplev" "\
*Decrypt the contents of this message" t nil)

(autoload 'mc-rmail-summary-snarf-keys "mc-toplev" "\
*Adds keys from current message to public key ring" t nil)

(autoload 'mc-rmail-verify-signature "mc-toplev" "\
*Verify the signature in the current message." t nil)

(autoload 'mc-rmail-decrypt-message "mc-toplev" "\
*Decrypt the contents of this message" t nil)

(autoload 'mc-vm-verify-signature "mc-toplev" "\
*Verify the signature in the current VM message" t nil)

(autoload 'mc-vm-decrypt-message "mc-toplev" "\
*Decrypt the contents of the current VM message" t nil)

(autoload 'mc-vm-snarf-keys "mc-toplev" "\
*Snarf public key from the contents of the current VM message" t nil)

(autoload 'mc-gnus-verify-signature "mc-toplev" nil t nil)

(autoload 'mc-gnus-snarf-keys "mc-toplev" nil t nil)

(autoload 'mc-gnus-decrypt-message "mc-toplev" nil t nil)

(autoload 'mc-mh-decrypt-message "mc-toplev" "\
Decrypt the contents of the current MH message in the show buffer." t nil)

(autoload 'mc-mh-verify-signature "mc-toplev" "\
*Verify the signature in the current MH message." t nil)

(autoload 'mc-mh-snarf-keys "mc-toplev" nil t nil)

;;;***

;;;### (autoloads (mh-letter-mode mh-smail-other-window mh-smail-batch mh-smail) "mh-comp" "mh-e/mh-comp.el" (12657 40772))
;;; Generated autoloads from mh-e/mh-comp.el

(autoload 'mh-smail "mh-comp" "\
Compose and send mail with the MH mail system.
This function is an entry point to mh-e, the Emacs front end
to the MH mail system.

See documentation of `\\[mh-send]' for more details on composing mail." t nil)

(autoload 'mh-smail-batch "mh-comp" "\
Set up a mail composition draft with the MH mail system.
This function is an entry point to mh-e, the Emacs front end
to the MH mail system.  This function does not prompt the user
for any header fields, and thus is suitable for use by programs
that want to create a mail buffer.
Users should use `\\[mh-smail]' to compose mail." nil nil)

(autoload 'mh-smail-other-window "mh-comp" "\
Compose and send mail in other window with the MH mail system.
This function is an entry point to mh-e, the Emacs front end
to the MH mail system.

See documentation of `\\[mh-send]' for more details on composing mail." t nil)

(autoload 'mh-letter-mode "mh-comp" "\
Mode for composing letters in mh-e.\\<mh-letter-mode-map>
When you have finished composing, type \\[mh-send-letter] to send the message
using the MH mail handling system.
See the documentation for \\[mh-edit-mhn] for information on composing MIME
messages.

\\{mh-letter-mode-map}

Variables controlling this mode (defaults in parentheses):

 mh-delete-yanked-msg-window (nil)
    If non-nil, \\[mh-yank-cur-msg] will delete any windows displaying
    the yanked message.

 mh-yank-from-start-of-msg (t)
    If non-nil, \\[mh-yank-cur-msg] will include the entire message.
    If `body', just yank the body (no header).
    If nil, only the portion of the message following the point will be yanked.
    If there is a region, this variable is ignored.

 mh-ins-buf-prefix (\"> \")
    String to insert before each non-blank line of a message as it is
    inserted in a draft letter.

 mh-signature-file-name (\"~/.signature\")
    File to be inserted into message by \\[mh-insert-signature].

Upon invoking mh-letter-mode, text-mode-hook and mh-letter-mode-hook are
invoked with no args, if those values are non-nil." t nil)

;;;***

;;;### (autoloads (mh-version mh-rmail) "mh-e" "mh-e/mh-e.el" (12747 30157))
;;; Generated autoloads from mh-e/mh-e.el

(autoload 'mh-rmail "mh-e" "\
Inc(orporate) new mail with MH, or, with arg, scan an MH mail folder.
This function is an entry point to mh-e, the Emacs front end
to the MH mail system." t nil)

(autoload 'mh-version "mh-e" "\
Display version information about mh-e and the MH mail handling system." t nil)

;;;***

;;;### (autoloads nil "mh-funcs" "mh-e/mh-funcs.el" (12657 40774))
;;; Generated autoloads from mh-e/mh-funcs.el

;;;### (autoloads nil "mh-mime" "mh-e/mh-mime.el" (12657 40774))
;;; Generated autoloads from mh-e/mh-mime.el

(defvar mh-mime-content-types '(("text/plain") ("text/richtext") ("multipart/mixed") ("multipart/alternative") ("multipart/digest") ("multipart/parallel") ("message/rfc822") ("message/partial") ("message/external-body") ("application/octet-stream") ("application/postscript") ("image/jpeg") ("image/gif") ("audio/basic") ("video/mpeg")) "\
Legal MIME content types.  See documentation for \\[mh-edit-mhn].")

;;;***

;;;### (autoloads nil "mh-pick" "mh-e/mh-pick.el" (12657 40775))
;;; Generated autoloads from mh-e/mh-pick.el

;;;***

;;;### (autoloads nil "mh-seq" "mh-e/mh-seq.el" (12657 40776))
;;; Generated autoloads from mh-e/mh-seq.el

;;;### (autoloads nil "mh-utils" "mh-e/mh-utils.el" (12657 40777))
;;; Generated autoloads from mh-e/mh-utils.el

(put 'mh-progs 'risky-local-variable t)

(put 'mh-lib 'risky-local-variable t)

;;;***

;;;### (autoloads nil "abbrev" "modes/abbrev.el" (12639 8573))
;;; Generated autoloads from modes/abbrev.el

;;;### (autoloads (ada-make-filename-from-adaname ada-mode) "ada-mode" "modes/ada-mode.el" (12559 34908))
;;; Generated autoloads from modes/ada-mode.el

(autoload 'ada-mode "ada-mode" "\
Ada Mode is the major mode for editing Ada code.

Bindings are as follows: (Note: 'LFD' is control-j.)

 Indent line                                          '\\[ada-tab]'
 Indent line, insert newline and indent the new line. '\\[newline-and-indent]'

 Re-format the parameter-list point is in             '\\[ada-format-paramlist]'
 Indent all lines in region                           '\\[ada-indent-region]'
 Call external pretty printer program                 '\\[ada-call-pretty-printer]'

 Adjust case of identifiers and keywords in region    '\\[ada-adjust-case-region]'
 Adjust case of identifiers and keywords in buffer    '\\[ada-adjust-case-buffer]'

 Call EXTERNAL pretty printer (if you have one)       '\\[ada-call-pretty-printer]'

 Fill comment paragraph                               '\\[ada-fill-comment-paragraph]'
 Fill comment paragraph and justify each line         '\\[ada-fill-comment-paragraph-justify]'
 Fill comment paragraph, justify and append postfix   '\\[ada-fill-comment-paragraph-postfix]'

 Next func/proc/task '\\[ada-next-procedure]'    Previous func/proc/task '\\[ada-previous-procedure]'
 Next package        '\\[ada-next-package]'  Previous package        '\\[ada-previous-package]'

 Goto matching start of current 'end ...;'            '\\[ada-move-to-start]'
 Goto end of current block                            '\\[ada-move-to-end]'

Comments are handled using standard GNU Emacs conventions, including:
 Start a comment                                      '\\[indent-for-comment]'
 Comment region                                       '\\[comment-region]'
 Uncomment region                                     '\\[ada-uncomment-region]'
 Continue comment on next line                        '\\[indent-new-comment-line]'

If you use imenu.el:
 Display index-menu of functions & procedures         '\\[imenu]'

If you use find-file.el:
 Switch to other file (Body <-> Spec)                 '\\[ff-find-other-file]'
                                                   or '\\[ff-mouse-find-other-file]
 Switch to other file in other window                 '\\[ada-ff-other-window]'
                                                   or '\\[ff-mouse-find-other-file-other-window]
 If you use this function in a spec and no body is available, it gets created
 with body stubs.

If you use ada-xref.el:
 Goto declaration:          '\\[ada-point-and-xref]' on the identifier
                         or '\\[ada-goto-declaration]' with point on the identifier
 Complete identifier:       '\\[ada-complete-identifier]'
 Execute Gnatf:             '\\[ada-gnatf-current]'" t nil)

(autoload 'ada-make-filename-from-adaname "ada-mode" "\
determine the filename of a package/procedure from its own Ada name." t nil)

;;;***

;;;### (autoloads (archive-mode) "arc-mode" "modes/arc-mode.el" (12559 34909))
;;; Generated autoloads from modes/arc-mode.el

(autoload 'archive-mode "arc-mode" "\
Major mode for viewing an archive file in a dired-like way.
You can move around using the usual cursor motion commands.
Letters no longer insert themselves.
Type `e' to pull a file out of the archive and into its own buffer;
or click mouse-2 on the file's line in the archive mode buffer.

If you edit a sub-file of this archive (as with the `e' command) and
save it, the contents of that buffer will be saved back into the
archive.

\\{archive-mode-map}" nil nil)

;;;***

;;;### (autoloads (asm-mode) "asm-mode" "modes/asm-mode.el" (12558 60896))
;;; Generated autoloads from modes/asm-mode.el

(autoload 'asm-mode "asm-mode" "\
Major mode for editing typical assembler code.
Features a private abbrev table and the following bindings:

\\[asm-colon]	outdent a preceding label, tab to next tab stop.
\\[tab-to-tab-stop]	tab to next tab stop.
\\[asm-newline]	newline, then tab to next tab stop.
\\[asm-comment]	smart placement of assembler comments.

The character used for making comments is set by the variable
`asm-comment-char' (which defaults to `?;').

Alternatively, you may set this variable in `asm-mode-set-comment-hook',
which is called near the beginning of mode initialization.

Turning on Asm mode runs the hook `asm-mode-hook' at the end of initialization.

Special commands:\\{asm-mode-map}
" t nil)

;;;***

;;;### (autoloads nil "auto-show" "modes/auto-show.el" (12698 33539))
;;; Generated autoloads from modes/auto-show.el

;;;***

;;;### (autoloads (awk-mode) "awk-mode" "modes/awk-mode.el" (12559 34897))
;;; Generated autoloads from modes/awk-mode.el

(autoload 'awk-mode "awk-mode" "\
Major mode for editing AWK code.
This is much like C mode except for the syntax of comments.  It uses
the same keymap as C mode and has the same variables for customizing
indentation.  It has its own abbrev table and its own syntax table.

Turning on AWK mode calls the value of the variable `awk-mode-hook'
with no args, if that value is non-nil." t nil)

;;;***

;;;### (autoloads nil "bib-mode" "modes/bib-mode.el" (12559 34898))
;;; Generated autoloads from modes/bib-mode.el

;;;***

;;;### (autoloads (bibtex-mode) "bibtex" "modes/bibtex.el" (12147 25394))
;;; Generated autoloads from modes/bibtex.el

(autoload 'bibtex-mode "bibtex" "\
Major mode for editing bibtex files.

\\{bibtex-mode-map}

A command such as \\[bibtex-Book] will outline the fields for a BibTeX book entry.

The optional fields start with the string OPT, and thus ignored by BibTeX.
The OPT string may be removed from a field with \\[bibtex-remove-OPT].
\\[bibtex-kill-optional-field] kills the current optional field entirely.
\\[bibtex-remove-double-quotes] removes the double-quotes around the text of
the current field.  \\[bibtex-empty-field] replaces the text of the current
field with the default \"\".

The command \\[bibtex-clean-entry] cleans the current entry, i.e. (i) removes
double-quotes from entirely numerical fields, (ii) removes OPT from all
non-empty optional fields, (iii) removes all empty optional fields, and (iv)
checks that no non-optional fields are empty.

Use \\[bibtex-find-text] to position the dot at the end of the current field.
Use \\[bibtex-next-field] to move to end of the next field.

The following may be of interest as well:

  Functions:
    find-bibtex-duplicates
    find-bibtex-entry-location
    hide-bibtex-entry-bodies
    sort-bibtex-entries
    validate-bibtex-buffer

  Variables:
    bibtex-clean-entry-zap-empty-opts
    bibtex-entry-field-alist
    bibtex-include-OPTannote
    bibtex-include-OPTcrossref
    bibtex-include-OPTkey
    bibtex-maintain-sorted-entries
    bibtex-mode-user-optional-fields

Fields:
    address
           Publisher's address
    annote
           Long annotation used for annotated bibliographies (begins sentence)
    author
           Name(s) of author(s), in BibTeX name format
    booktitle
           Book title when the thing being referenced isn't the whole book.
           For book entries, the title field should be used instead.
    chapter
           Chapter number
    crossref
	   The database key of the entry being cross referenced.
    edition
           Edition of a book (e.g., \"second\")
    editor
           Name(s) of editor(s), in BibTeX name format.
           If there is also an author field, then the editor field should be
           for the book or collection that the work appears in
    howpublished
            How something strange has been published (begins sentence)
    institution
           Sponsoring institution
    journal
           Journal name (macros are provided for many)
    key
           Alphabetizing and labeling key (needed when no author or editor)
    month
           Month (macros are provided)
    note
           To help the reader find a reference (begins sentence)
    number
           Number of a journal or technical report
    organization
           Organization (sponsoring a conference)
    pages
           Page number or numbers (use `--' to separate a range)
    publisher
           Publisher name
    school
           School name (for theses)
    series
           The name of a series or set of books.
           An individual book will will also have it's own title
    title
           The title of the thing being referenced
    type
           Type of a technical report (e.g., \"Research Note\") to be used
           instead of the default \"Technical Report\"
    volume
           Volume of a journal or multivolume work
    year
           Year---should contain only numerals
---------------------------------------------------------
Entry to this mode calls the value of bibtex-mode-hook if that value is
non-nil." t nil)

;;;***

;;;### (autoloads nil "c-comment" "modes/c-comment.el" (12690 2171))
;;; Generated autoloads from modes/c-comment.el

;;;### (autoloads nil "c-fill" "modes/c-fill.el" (12559 34900))
;;; Generated autoloads from modes/c-fill.el

;;;***

;;;### (autoloads nil "c-style" "modes/c-style.el" (12559 34900))
;;; Generated autoloads from modes/c-style.el

;;;***

;;;### (autoloads nil "cc-compat" "modes/cc-compat.el" (12559 34905))
;;; Generated autoloads from modes/cc-compat.el

;;;***

;;;### (autoloads nil "cc-guess" "modes/cc-guess.el" (12559 34906))
;;; Generated autoloads from modes/cc-guess.el

;;;***

;;;### (autoloads nil "cc-lobotomy" "modes/cc-lobotomy.el" (12559 34904))
;;; Generated autoloads from modes/cc-lobotomy.el

;;;***

;;;### (autoloads nil "cc-mode" "modes/cc-mode.el" (12740 25201))
;;; Generated autoloads from modes/cc-mode.el

;;;### (autoloads (common-lisp-indent-function) "cl-indent" "modes/cl-indent.el" (12147 25404))
;;; Generated autoloads from modes/cl-indent.el

(autoload 'common-lisp-indent-function "cl-indent" nil nil nil)

;;;***

;;;### (autoloads (c-macro-expand) "cmacexp" "modes/cmacexp.el" (12546 50436))
;;; Generated autoloads from modes/cmacexp.el

(autoload 'c-macro-expand "cmacexp" "\
Expand C macros in the region, using the C preprocessor.
Normally display output in temp buffer, but
prefix arg means replace the region with it.

`c-macro-preprocessor' specifies the preprocessor to use.
Prompt for arguments to the preprocessor (e.g. `-DDEBUG -I ./include')
if the user option `c-macro-prompt-flag' is non-nil.

Noninteractive args are START, END, SUBST.
For use inside Lisp programs, see also `c-macro-expansion'." t nil)

;;;***

;;;### (autoloads nil "cperl-mode" "modes/cperl-mode.el" (12675 57223))
;;; Generated autoloads from modes/cperl-mode.el

;;;### (autoloads (enriched-decode enriched-encode enriched-mode) "enriched" "modes/enriched.el" (12648 18668))
;;; Generated autoloads from modes/enriched.el

(autoload 'enriched-mode "enriched" "\
Minor mode for editing text/enriched files.
These are files with embedded formatting information in the MIME standard
text/enriched format.
Turning the mode on runs `enriched-mode-hooks'.

More information about enriched-mode is available in the file 
etc/enriched.doc  in the Emacs distribution directory.

Commands:

\\<enriched-mode-map>\\{enriched-mode-map}" t nil)

(autoload 'enriched-encode "enriched" nil nil nil)

(autoload 'enriched-decode "enriched" nil nil nil)

;;;***

;;;### (autoloads (f90-mode) "f90" "modes/f90.el" (12675 57071))
;;; Generated autoloads from modes/f90.el

(autoload 'f90-mode "f90" "\
Major mode for editing Fortran 90 code in free format.

\\[f90-indent-new-line] corrects current indentation and creates new indented line.
\\[f90-indent-line] indents the current line correctly. 
\\[f90-indent-subprogram] indents the current subprogram. 

Type `? or `\\[help-command] to display a list of built-in abbrevs for F90 keywords.

Key definitions:
\\{f90-mode-map}

Variables controlling indentation style and extra features:

 f90-do-indent
    Extra indentation within do blocks.  (default 3)
 f90-if-indent
    Extra indentation within if/select case/where/forall blocks. (default 3)
 f90-type-indent
    Extra indentation within type/interface/block-data blocks.  (default 3)
 f90-program-indent
    Extra indentation within program/module/subroutine/function blocks.
      (default 2)
 f90-continuation-indent
    Extra indentation applied to continuation lines.  (default 5)
 f90-comment-region
    String inserted by \\[f90-comment-region] at start of each line in 
    region.  (default \"!!!$\")
 f90-indented-comment-re
    Regexp determining the type of comment to be intended like code.
    (default \"!\")
 f90-directive-comment-re
    Regexp of comment-like directive like \"!HPF\\\\$\", not to be indented.
    (default \"!hpf\\\\$\")
 f90-break-delimiters
    Regexp holding list of delimiters at which lines may be broken.
    (default \"[-+*/><=,% \\t]\")
 f90-break-before-delimiters
    Non-nil causes `f90-do-auto-fill' to break lines before delimiters.
    (default t)
 f90-beginning-ampersand 
    Automatic insertion of & at beginning of continuation lines. (default t)
 f90-smart-end 
    From an END statement, check and fill the end using matching block start.
    Allowed values are 'blink, 'no-blink, and nil, which determine
    whether to blink the matching beginning.) (default 'blink)
 f90-auto-keyword-case
    Automatic change of case of keywords. (default nil)
    The possibilities are 'downcase-word, 'upcase-word, 'capitalize-word.
 f90-leave-line-no
    Do not left-justify line numbers. (default nil)
 f90-startup-message
    Set to nil to inhibit message first time F90 mode is used. (default t)
 f90-keywords
    List of keywords used for highlighting/upcase-keywords etc.

Turning on F90 mode calls the value of the variable `f90-mode-hook'
with no args, if that value is non-nil." t nil)

;;;***

;;;### (autoloads (follow-delete-other-windows-and-split follow-mode turn-off-follow-mode turn-on-follow-mode) "follow" "modes/follow.el" (12720 56486))
;;; Generated autoloads from modes/follow.el

(add-minor-mode 'follow-mode nil 'follow-mode-map)

(autoload 'turn-on-follow-mode "follow" "\
Turn on Follow mode. Please see the function `follow-mode'." t nil)

(autoload 'turn-off-follow-mode "follow" "\
Turn off Follow mode. Please see the function `follow-mode'." t nil)

(autoload 'follow-mode "follow" "\
Minor mode which combines windows into one tall virtual window.

The feeling of a \"virtual window\" has been accomplished by the use
of two major techniques:

* The windows always displays adjacent sections of the buffer. 
  This means that whenever one window is moved, all the
  others will follow.  (Hence the name Follow Mode.)

* Should the point (cursor) end up outside a window, another 
  window displaying that point is selected, if possible.  This 
  makes it possible to walk between windows using normal cursor 
  movement commands.

Follow mode comes to its prime when used on a large screen and two
side-by-side window are used. The user can, with the help of Follow
mode, use two full-height windows as though they would have been
one. Imagine yourself editing a large function, or section of text,
and beeing able to use 144 lines instead of the normal 72... (your
mileage may vary).

To split one large window into two side-by-side windows, the commands
`\\[split-window-horizontally]' or `M-x follow-delete-other-windows-and-split' can be used.

Only windows displayed in the same frame follow each-other.

If the variable `follow-intercept-processes' is non-nil, Follow mode
will listen to the output of processes and redisplay accordingly.
\(This is the default.)

When Follow mode is switched on, the hook `follow-mode-hook'
is called.  When turned off, `follow-mode-off-hook' is called.

Keys specific to Follow mode:
\\{follow-mode-map}" t nil)

(autoload 'follow-delete-other-windows-and-split "follow" "\
Create two side by side windows and enter Follow Mode.

Execute this command to display as much as possible of the text
in the selected window.  All other windows, in the current 
frame, are deleted and the selected window is split in two
side-by-side windows. Follow Mode is activated, hence the 
two windows always will display two successive pages.
\(If one window is moved, the other one will follow.)

If ARG is positive, the leftmost window is selected.  If it negative,
the rightmost is selected.  If ARG is nil, the leftmost window is
selected if the original window is the first one in the frame.

To bind this command to a hotkey, place the following line
in your `~/.emacs' file, replacing [f7] by your favourite key:
    (global-set-key [f7] 'follow-delete-other-windows-and-split)" t nil)

;;;***

;;;### (autoloads nil "fortran-misc" "modes/fortran-misc.el" (12376 19458))
;;; Generated autoloads from modes/fortran-misc.el

;;;### (autoloads (fortran-mode) "fortran" "modes/fortran.el" (12558 60889))
;;; Generated autoloads from modes/fortran.el

(defvar fortran-tab-mode-default nil "\
*Default tabbing/carriage control style for empty files in Fortran mode.
A value of t specifies tab-digit style of continuation control.
A value of nil specifies that continuation lines are marked
with a character in column 6.")

(autoload 'fortran-mode "fortran" "\
Major mode for editing Fortran code.
\\[fortran-indent-line] indents the current Fortran line correctly. 
DO statements must not share a common CONTINUE.

Type ;? or ;\\[help-command] to display a list of built-in abbrevs for Fortran keywords.

Key definitions:
\\{fortran-mode-map}

Variables controlling indentation style and extra features:

 comment-start
    Normally nil in Fortran mode.  If you want to use comments
    starting with `!', set this to the string \"!\".
 fortran-do-indent
    Extra indentation within do blocks.  (default 3)
 fortran-if-indent
    Extra indentation within if blocks.  (default 3)
 fortran-structure-indent
    Extra indentation within structure, union, map and interface blocks.
    (default 3)
 fortran-continuation-indent
    Extra indentation applied to continuation statements.  (default 5)
 fortran-comment-line-extra-indent
    Amount of extra indentation for text within full-line comments. (default 0)
 fortran-comment-indent-style
    nil    means don't change indentation of text in full-line comments,
    fixed  means indent that text at `fortran-comment-line-extra-indent' beyond
           the value of `fortran-minimum-statement-indent-fixed' (for fixed
           format continuation style) or `fortran-minimum-statement-indent-tab'
           (for TAB format continuation style).
    relative  means indent at `fortran-comment-line-extra-indent' beyond the
 	      indentation for a line of code.
    (default 'fixed)
 fortran-comment-indent-char
    Single-character string to be inserted instead of space for
    full-line comment indentation.  (default \" \")
 fortran-minimum-statement-indent-fixed
    Minimum indentation for Fortran statements in fixed format mode. (def.6)
 fortran-minimum-statement-indent-tab
    Minimum indentation for Fortran statements in TAB format mode. (default 9)
 fortran-line-number-indent
    Maximum indentation for line numbers.  A line number will get
    less than this much indentation if necessary to avoid reaching
    column 5.  (default 1)
 fortran-check-all-num-for-matching-do
    Non-nil causes all numbered lines to be treated as possible \"continue\"
    statements.  (default nil)
 fortran-blink-matching-if 
    Non-nil causes \\[fortran-indent-line] on an ENDIF statement to blink on
    matching IF.  Also, from an ENDDO statement, blink on matching DO [WHILE]
    statement.  (default nil)
 fortran-continuation-string
    Single-character string to be inserted in column 5 of a continuation
    line.  (default \"$\")
 fortran-comment-region
    String inserted by \\[fortran-comment-region] at start of each line in 
    region.  (default \"c$$$\")
 fortran-electric-line-number
    Non-nil causes line number digits to be moved to the correct column 
    as typed.  (default t)
 fortran-break-before-delimiters
    Non-nil causes `fortran-fill' breaks lines before delimiters.
    (default t)
 fortran-startup-message
    Set to nil to inhibit message first time Fortran mode is used.

Turning on Fortran mode calls the value of the variable `fortran-mode-hook'
with no args, if that value is non-nil." t nil)

;;;***

;;;### (autoloads nil "hideif" "modes/hideif.el" (12639 8574))
;;; Generated autoloads from modes/hideif.el

(add-minor-mode 'hide-ifdef-mode " Ifdef")

;;;***

;;;### (autoloads (hs-minor-mode hs-hide-block hs-hide-all) "hideshow" "modes/hideshow.el" (12648 18570))
;;; Generated autoloads from modes/hideshow.el

(defvar hs-minor-mode nil "\
Non-nil if using hideshow mode as a minor mode of some other mode.
Use the command `hs-minor-mode' to toggle this variable.")

(autoload 'hs-hide-all "hideshow" "\
Hides all top-level blocks, displaying only first and last lines.
When done, point is repositioned at the beginning of the line, and
hs-hide-hooks is called.  See documentation for `run-hooks'." t nil)

(autoload 'hs-hide-block "hideshow" "\
Selects a block and hides it.  With prefix arg, reposition at end.
Block is defined as a sexp for lispish modes, mode-specific otherwise.
Comments are blocks, too.  Upon completion, point is at repositioned and
hs-hide-hooks is called.  See documentation for `run-hooks'." t nil)

(autoload 'hs-minor-mode "hideshow" "\
Toggle hideshow minor mode.
With ARG, turn hideshow minor mode on if ARG is positive, off otherwise.
When hideshow minor mode is on, the menu bar is augmented with hideshow
commands and the hideshow commands are enabled.  The variables

	selective-display
	selective-display-ellipses

are set to t.  Lastly, the hooks set in hs-minor-mode-hook are called.
See documentation for `run-hooks'.

Turning hideshow minor mode off reverts the menu bar and the
variables to default values and disables the hideshow commands." t nil)

(add-minor-mode 'hs-minor-mode " hs" 'hs-minor-mode-map)

;;;***

;;;### (autoloads (icon-mode) "icon" "modes/icon.el" (12559 34901))
;;; Generated autoloads from modes/icon.el

(autoload 'icon-mode "icon" "\
Major mode for editing Icon code.
Expression and list commands understand all Icon brackets.
Tab indents for Icon code.
Paragraphs are separated by blank lines only.
Delete converts tabs to spaces as it moves back.
\\{icon-mode-map}
Variables controlling indentation style:
 icon-tab-always-indent
    Non-nil means TAB in Icon mode should always reindent the current line,
    regardless of where in the line point is when the TAB command is used.
 icon-auto-newline
    Non-nil means automatically newline before and after braces
    inserted in Icon code.
 icon-indent-level
    Indentation of Icon statements within surrounding block.
    The surrounding block's indentation is the indentation
    of the line on which the open-brace appears.
 icon-continued-statement-offset
    Extra indentation given to a substatement, such as the
    then-clause of an if or body of a while.
 icon-continued-brace-offset
    Extra indentation given to a brace that starts a substatement.
    This is in addition to `icon-continued-statement-offset'.
 icon-brace-offset
    Extra indentation for line if it starts with an open brace.
 icon-brace-imaginary-offset
    An open brace following other text is treated as if it were
    this far to the right of the start of its line.

Turning on Icon mode calls the value of the variable `icon-mode-hook'
with no args, if that value is non-nil." t nil)

;;;***

;;;### (autoloads (ksh-mode) "ksh-mode" "modes/ksh-mode.el" (12714 11717))
;;; Generated autoloads from modes/ksh-mode.el

(autoload 'ksh-mode "ksh-mode" "\
ksh-mode 2.6 - Major mode for editing (Bourne, Korn or Bourne again)
shell scripts.
Special key bindings and commands:
\\{ksh-mode-map}
Variables controlling indentation style:
ksh-indent
    Indentation of ksh statements with respect to containing block.
    Default value is 2.
ksh-case-indent
    Additional indentation for statements under case items.
    Default value is nil which will align the statements one position 
    past the \")\" of the pattern.
ksh-case-item-offset
    Additional indentation for case items within a case statement.
    Default value is 2.
ksh-group-offset
    Additional indentation for keywords \"do\" and \"then\".
    Default value is -2.
ksh-brace-offset
    Additional indentation of \"{\" under functions or brace groupings.
    Default value is 0.
ksh-multiline-offset
   Additional indentation of line that is preceded of a line ending with a
   \\ to make it continue on next line.
ksh-tab-always-indent
    Controls the operation of the TAB key. If t (the default), always
    reindent the current line.  If nil, indent the current line only if
    point is at the left margin or in the line's indentation; otherwise
    insert a tab.
ksh-match-and-tell
    If non-nil echo in the minibuffer the matching compound command
    for the \"done\", \"}\", \"fi\", or \"esac\". Default value is t.

ksh-align-to-keyword
    Controls whether nested constructs align from the keyword or
    the current indentation. If non-nil, indentation will be relative to
    the column the keyword starts. If nil, indentation will be relative to
    the current indentation of the line the keyword is on.
    The default value is non-nil.

ksh-comment-regexp
  Regular expression used to recognize comments. Customize to support
  ksh-like languages. Default value is \"\\s *#\".

Style Guide.
 By setting
    (setq ksh-indent default-tab-width)
    (setq ksh-group-offset 0)

    The following style is obtained:

    if [ -z $foo ]
	    then
		    bar    # <-- ksh-group-offset is additive to ksh-indent
		    foo
    fi

 By setting
    (setq ksh-indent default-tab-width)
    (setq ksh-group-offset (- 0 ksh-indent))

    The following style is obtained:

    if [ -z $foo ]
    then
	    bar
	    foo
    fi

 By setting
    (setq ksh-case-item-offset 1)
    (setq ksh-case-indent nil)

    The following style is obtained:

    case x in *
     foo) bar           # <-- ksh-case-item-offset
          baz;;         # <-- ksh-case-indent aligns with \")\"
     foobar) foo
             bar;;
    esac

 By setting
    (setq ksh-case-item-offset 1)
    (setq ksh-case-indent 6)

    The following style is obtained:

    case x in *
     foo) bar           # <-- ksh-case-item-offset
           baz;;        # <-- ksh-case-indent
     foobar) foo
           bar;;
    esac
    

Installation:
  Put ksh-mode.el in some directory in your load-path.
  Put the following forms in your .emacs file.

 (setq auto-mode-alist
      (append auto-mode-alist
              (list
               '(\"\\\\.sh$\" . ksh-mode)
               '(\"\\\\.ksh$\" . ksh-mode)
               '(\"\\\\.bashrc\" . ksh-mode)
               '(\"\\\\..*profile\" . ksh-mode))))

 (setq ksh-mode-hook
      (function (lambda ()
         (font-lock-mode 1)             ;; font-lock the buffer
         (setq ksh-indent 8)
	 (setq ksh-group-offset -8))
	 (setq ksh-brace-offset -8)   
         (setq ksh-tab-always-indent t)
         (setq ksh-match-and-tell t)
         (setq ksh-align-to-keyword t)	;; Turn on keyword alignment
	 )))" t nil)

;;;***

;;;### (autoloads nil "lisp-mnt" "modes/lisp-mnt.el" (12546 50449))
;;; Generated autoloads from modes/lisp-mnt.el

;;;***

;;;### (autoloads nil "lisp-mode" "modes/lisp-mode.el" (12728 13792))
;;; Generated autoloads from modes/lisp-mode.el

;;;***

;;;### (autoloads nil "list-mode" "modes/list-mode.el" (12740 25196))
;;; Generated autoloads from modes/list-mode.el

;;;### (autoloads (makefile-mode) "make-mode" "modes/make-mode.el" (12558 60885))
;;; Generated autoloads from modes/make-mode.el

(autoload 'makefile-mode "make-mode" "\
Major mode for editing Makefiles.
This function ends by invoking the function(s) `makefile-mode-hook'.

\\{makefile-mode-map}

In the browser, use the following keys:

\\{makefile-browser-map}

Makefile mode can be configured by modifying the following variables:

makefile-browser-buffer-name:
    Name of the macro- and target browser buffer.

makefile-target-colon:
    The string that gets appended to all target names
    inserted by `makefile-insert-target'.
    \":\" or \"::\" are quite common values.

makefile-macro-assign:
   The string that gets appended to all macro names
   inserted by `makefile-insert-macro'.
   The normal value should be \" = \", since this is what
   standard make expects. However, newer makes such as dmake
   allow a larger variety of different macro assignments, so you
   might prefer to use \" += \" or \" := \" .

makefile-tab-after-target-colon:
   If you want a TAB (instead of a space) to be appended after the
   target colon, then set this to a non-nil value.

makefile-browser-leftmost-column:
   Number of blanks to the left of the browser selection mark.

makefile-browser-cursor-column:
   Column in which the cursor is positioned when it moves
   up or down in the browser.

makefile-browser-selected-mark:
   String used to mark selected entries in the browser.

makefile-browser-unselected-mark:
   String used to mark unselected entries in the browser.

makefile-browser-auto-advance-after-selection-p:
   If this variable is set to a non-nil value the cursor
   will automagically advance to the next line after an item
   has been selected in the browser.

makefile-pickup-everything-picks-up-filenames-p:
   If this variable is set to a non-nil value then
   `makefile-pickup-everything' also picks up filenames as targets
   (i.e. it calls `makefile-find-filenames-as-targets'), otherwise
   filenames are omitted.

makefile-cleanup-continuations-p:
   If this variable is set to a non-nil value then makefile-mode
   will assure that no line in the file ends with a backslash
   (the continuation character) followed by any whitespace.
   This is done by silently removing the trailing whitespace, leaving
   the backslash itself intact.
   IMPORTANT: Please note that enabling this option causes makefile-mode
   to MODIFY A FILE WITHOUT YOUR CONFIRMATION when \"it seems necessary\".

makefile-browser-hook:
   A function or list of functions to be called just before the
   browser is entered. This is executed in the makefile buffer.

makefile-special-targets-list:
   List of special targets. You will be offered to complete
   on one of those in the minibuffer whenever you enter a `.'.
   at the beginning of a line in Makefile mode." t nil)

;;;***

;;;### (autoloads nil "old-c++-mode" "modes/old-c++-mode.el" (12376 19469))
;;; Generated autoloads from modes/old-c++-mode.el

;;;***

;;;### (autoloads nil "old-c-mode" "modes/old-c-mode.el" (12546 50448))
;;; Generated autoloads from modes/old-c-mode.el

;;;### (autoloads (define-mail-alias build-mail-aliases mail-aliases-setup) "mail-abbrevs" "modes/mail-abbrevs.el" (12376 19450))
;;; Generated autoloads from modes/mail-abbrevs.el

(defvar mail-abbrev-mailrc-file nil "\
Name of file with mail aliases.   If nil, ~/.mailrc is used.")

(defvar mail-aliases nil "\
Word-abbrev table of mail address aliases.
If this is nil, it means the aliases have not yet been initialized and
should be read from the .mailrc file.  (This is distinct from there being
no aliases, which is represented by this being a table with no entries.)")

(autoload 'mail-aliases-setup "mail-abbrevs" nil nil nil)

(autoload 'build-mail-aliases "mail-abbrevs" "\
Read mail aliases from .mailrc and set mail-aliases." nil nil)

(autoload 'define-mail-alias "mail-abbrevs" "\
Define NAME as a mail-alias that translates to DEFINITION.
If DEFINITION contains multiple addresses, separate them with commas." t nil)

;;;***

;;;### (autoloads (modula-2-mode) "modula2" "modes/modula2.el" (12376 19453))
;;; Generated autoloads from modes/modula2.el

(autoload 'modula-2-mode "modula2" "\
This is a mode intended to support program development in Modula-2.
All control constructs of Modula-2 can be reached by typing
Control-C followed by the first character of the construct.
\\{m2-mode-map}
  Control-c b begin         Control-c c case
  Control-c d definition    Control-c e else
  Control-c f for           Control-c h header
  Control-c i if            Control-c m module
  Control-c l loop          Control-c o or
  Control-c p procedure     Control-c Control-w with
  Control-c r record        Control-c s stdio
  Control-c t type          Control-c u until
  Control-c v var           Control-c w while
  Control-c x export        Control-c y import
  Control-c { begin-comment Control-c } end-comment
  Control-c Control-z suspend-emacs     Control-c Control-t toggle
  Control-c Control-c compile           Control-x ` next-error
  Control-c Control-l link

   m2-indent controls the number of spaces for each indentation.
   m2-compile-command holds the command to compile a Modula-2 program.
   m2-link-command holds the command to link a Modula-2 program." t nil)

;;;***

;;;### (autoloads (electric-nroff-mode nroff-mode) "nroff-mode" "modes/nroff-mode.el" (12648 18567))
;;; Generated autoloads from modes/nroff-mode.el

(autoload 'nroff-mode "nroff-mode" "\
Major mode for editing text intended for nroff to format.
\\{nroff-mode-map}
Turning on Nroff mode runs `text-mode-hook', then `nroff-mode-hook'.
Also, try `nroff-electric-mode', for automatically inserting
closing requests for requests that are used in matched pairs." t nil)

(autoload 'electric-nroff-mode "nroff-mode" "\
Toggle `nroff-electric-newline' minor mode.
`nroff-electric-newline' forces Emacs to check for an nroff request at the
beginning of the line, and insert the matching closing request if necessary.
This command toggles that mode (off->on, on->off), with an argument,
turns it on iff arg is positive, otherwise off." t nil)

(defvar nroff-electric-mode nil "\
Non-nil if in electric-nroff minor mode.")

(add-minor-mode 'nroff-electric-mode " Electric" nil nil 'electric-nroff-mode)

;;;***

;;;### (autoloads (outl-mouse-minor-mode outl-mouse-mode) "outl-mouse" "modes/outl-mouse.el" (12376 19464))
;;; Generated autoloads from modes/outl-mouse.el

(autoload 'outl-mouse-mode "outl-mouse" "\
Calls outline-mode, with outl-mouse extensions" t nil)

(autoload 'outl-mouse-minor-mode "outl-mouse" "\
Toggles outline-minor-mode, with outl-mouse extensions" t nil)

;;;***

;;;### (autoloads (outline-minor-mode outline-mode) "outline" "modes/outline.el" (12648 18566))
;;; Generated autoloads from modes/outline.el

(defvar outline-minor-mode nil "\
Non-nil if using Outline mode as a minor mode of some other mode.")

(make-variable-buffer-local 'outline-minor-mode)

(put 'outline-minor-mode 'permanent-local t)

(add-minor-mode 'outline-minor-mode " Outl")

(autoload 'outline-mode "outline" "\
Set major mode for editing outlines with selective display.
Headings are lines which start with asterisks: one for major headings,
two for subheadings, etc.  Lines not starting with asterisks are body lines. 

Body text or subheadings under a heading can be made temporarily
invisible, or visible again.  Invisible lines are attached to the end 
of the heading, so they move with it, if the line is killed and yanked
back.  A heading with text hidden under it is marked with an ellipsis (...).

Commands:\\<outline-mode-map>
\\[outline-next-visible-heading]   outline-next-visible-heading      move by visible headings
\\[outline-previous-visible-heading]   outline-previous-visible-heading
\\[outline-forward-same-level]   outline-forward-same-level        similar but skip subheadings
\\[outline-backward-same-level]   outline-backward-same-level
\\[outline-up-heading]   outline-up-heading		    move from subheading to heading

\\[hide-body]	make all text invisible (not headings).
\\[show-all]	make everything in buffer visible.

The remaining commands are used when point is on a heading line.
They apply to some of the body or subheadings of that heading.
\\[hide-subtree]   hide-subtree	make body and subheadings invisible.
\\[show-subtree]   show-subtree	make body and subheadings visible.
\\[show-children]   show-children	make direct subheadings visible.
		 No effect on body, or subheadings 2 or more levels down.
		 With arg N, affects subheadings N levels down.
\\[hide-entry]	   make immediately following body invisible.
\\[show-entry]	   make it visible.
\\[hide-leaves]	   make body under heading and under its subheadings invisible.
		     The subheadings remain visible.
\\[show-branches]  make all subheadings at all levels visible.

The variable `outline-regexp' can be changed to control what is a heading.
A line is a heading if `outline-regexp' matches something at the
beginning of the line.  The longer the match, the deeper the level.

Turning on outline mode calls the value of `text-mode-hook' and then of
`outline-mode-hook', if they are non-nil." t nil)

(autoload 'outline-minor-mode "outline" "\
Toggle Outline minor mode.
With arg, turn Outline minor mode on if arg is positive, off otherwise.
See the command `outline-mode' for more information on this mode." t nil)

;;;***

;;;### (autoloads (pascal-mode) "pascal" "modes/pascal.el" (12698 33540))
;;; Generated autoloads from modes/pascal.el

(autoload 'pascal-mode "pascal" "\
Major mode for editing Pascal code. \\<pascal-mode-map>
TAB indents for Pascal code.  Delete converts tabs to spaces as it moves back.

\\[pascal-complete-word] completes the word around current point with respect to position in code
\\[pascal-show-completions] shows all possible completions at this point.

Other useful functions are:

\\[pascal-mark-defun]	- Mark function.
\\[pascal-insert-block]	- insert begin ... end;
\\[pascal-star-comment]	- insert (* ... *)
\\[pascal-comment-area]	- Put marked area in a comment, fixing nested comments.
\\[pascal-uncomment-area]	- Uncomment an area commented with \\[pascal-comment-area].
\\[pascal-beg-of-defun]	- Move to beginning of current function.
\\[pascal-end-of-defun]	- Move to end of current function.
\\[pascal-goto-defun]	- Goto function prompted for in the minibuffer.
\\[pascal-outline]	- Enter pascal-outline-mode (see also pascal-outline).

Variables controlling indentation/edit style:

 pascal-indent-level      (default 3)
    Indentation of Pascal statements with respect to containing block.
 pascal-case-indent       (default 2)
    Indentation for case statements.
 pascal-auto-newline      (default nil)
    Non-nil means automatically newline after simcolons and the punctation mark
    after an end.
 pascal-tab-always-indent (default t)
    Non-nil means TAB in Pascal mode should always reindent the current line,
    regardless of where in the line point is when the TAB command is used.
 pascal-auto-endcomments  (default t)
    Non-nil means a comment { ... } is set after the ends which ends cases and
    functions. The name of the function or case will be set between the braces.
 pascal-auto-lineup       (default t)
    List of contexts where auto lineup of :'s or ='s hould be done.

See also the user variables pascal-type-keywords, pascal-start-keywords and
pascal-separator-keywords.

Turning on Pascal mode calls the value of the variable pascal-mode-hook with
no args, if that value is non-nil." t nil)

;;;***

;;;### (autoloads (perl-mode) "perl-mode" "modes/perl-mode.el" (12573 5998))
;;; Generated autoloads from modes/perl-mode.el

(autoload 'perl-mode "perl-mode" "\
Major mode for editing Perl code.
Expression and list commands understand all Perl brackets.
Tab indents for Perl code.
Comments are delimited with # ... \\n.
Paragraphs are separated by blank lines only.
Delete converts tabs to spaces as it moves back.
\\{perl-mode-map}
Variables controlling indentation style:
 perl-tab-always-indent
    Non-nil means TAB in Perl mode should always indent the current line,
    regardless of where in the line point is when the TAB command is used.
 perl-tab-to-comment
    Non-nil means that for lines which don't need indenting, TAB will
    either delete an empty comment, indent an existing comment, move 
    to end-of-line, or if at end-of-line already, create a new comment.
 perl-nochange
    Lines starting with this regular expression are not auto-indented.
 perl-indent-level
    Indentation of Perl statements within surrounding block.
    The surrounding block's indentation is the indentation
    of the line on which the open-brace appears.
 perl-continued-statement-offset
    Extra indentation given to a substatement, such as the
    then-clause of an if or body of a while.
 perl-continued-brace-offset
    Extra indentation given to a brace that starts a substatement.
    This is in addition to `perl-continued-statement-offset'.
 perl-brace-offset
    Extra indentation for line if it starts with an open brace.
 perl-brace-imaginary-offset
    An open brace following other text is treated as if it were
    this far to the right of the start of its line.
 perl-label-offset
    Extra indentation for line that is a label.

Various indentation styles:       K&R  BSD  BLK  GNU  LW
  perl-indent-level                5    8    0    2    4
  perl-continued-statement-offset  5    8    4    2    4
  perl-continued-brace-offset      0    0    0    0   -4
  perl-brace-offset               -5   -8    0    0    0
  perl-brace-imaginary-offset      0    0    4    0    0
  perl-label-offset               -5   -8   -2   -2   -2

Turning on Perl mode runs the normal hook `perl-mode-hook'." t nil)

;;;***

;;;### (autoloads (picture-mode) "picture" "modes/picture.el" (12639 8578))
;;; Generated autoloads from modes/picture.el

(autoload 'picture-mode "picture" "\
Switch to Picture mode, in which a quarter-plane screen model is used.
Printing characters replace instead of inserting themselves with motion
afterwards settable by these commands:
  C-c <	  Move left after insertion.
  C-c >	  Move right after insertion.
  C-c ^	  Move up after insertion.
  C-c .	  Move down after insertion.
  C-c `	  Move northwest (nw) after insertion.
  C-c '	  Move northeast (ne) after insertion.
  C-c /	  Move southwest (sw) after insertion.
  C-c \\   Move southeast (se) after insertion.
The current direction is displayed in the modeline.  The initial
direction is right.  Whitespace is inserted and tabs are changed to
spaces when required by movement.  You can move around in the buffer
with these commands:
  \\[picture-move-down]	  Move vertically to SAME column in previous line.
  \\[picture-move-up]	  Move vertically to SAME column in next line.
  \\[picture-end-of-line]	  Move to column following last non-whitespace character.
  \\[picture-forward-column]	  Move right inserting spaces if required.
  \\[picture-backward-column]	  Move left changing tabs to spaces if required.
  C-c C-f Move in direction of current picture motion.
  C-c C-b Move in opposite direction of current picture motion.
  Return  Move to beginning of next line.
You can edit tabular text with these commands:
  M-Tab	  Move to column beneath (or at) next interesting character.
	    `Indents' relative to a previous line.
  Tab	  Move to next stop in tab stop list.
  C-c Tab Set tab stops according to context of this line.
	    With ARG resets tab stops to default (global) value.
	    See also documentation of variable	picture-tab-chars
	    which defines \"interesting character\".  You can manually
	    change the tab stop list with command \\[edit-tab-stops].
You can manipulate text with these commands:
  C-d	  Clear (replace) ARG columns after point without moving.
  C-c C-d Delete char at point - the command normally assigned to C-d.
  \\[picture-backward-clear-column]  Clear (replace) ARG columns before point, moving back over them.
  \\[picture-clear-line]	  Clear ARG lines, advancing over them.	 The cleared
	    text is saved in the kill ring.
  \\[picture-open-line]	  Open blank line(s) beneath current line.
You can manipulate rectangles with these commands:
  C-c C-k Clear (or kill) a rectangle and save it.
  C-c C-w Like C-c C-k except rectangle is saved in named register.
  C-c C-y Overlay (or insert) currently saved rectangle at point.
  C-c C-x Like C-c C-y except rectangle is taken from named register.
  \\[copy-rectangle-to-register]   Copies a rectangle to a register.
  \\[advertised-undo]   Can undo effects of rectangle overlay commands
	    commands if invoked soon enough.
You can return to the previous mode with:
  C-c C-c Which also strips trailing whitespace from every line.
	    Stripping is suppressed by supplying an argument.

Entry to this mode calls the value of  picture-mode-hook  if non-nil.

Note that Picture mode commands will work outside of Picture mode, but
they are not defaultly assigned to keys." t nil)

(defalias 'edit-picture 'picture-mode)

;;;***

;;;### (autoloads (postscript-mode) "postscript" "modes/postscript.el" (12558 60895))
;;; Generated autoloads from modes/postscript.el

(autoload 'postscript-mode "postscript" "\
Major mode for editing PostScript files.

\\[ps-execute-buffer] will send the contents of the buffer to the NeWS
server using psh(1).  \\[ps-execute-region] sends the current region.
\\[ps-shell] starts an interactive psh(1) window which will be used for
subsequent \\[ps-execute-buffer] or \\[ps-execute-region] commands.

In this mode, TAB and \\[indent-region] attempt to indent code
based on the position of {}, [], and begin/end pairs.  The variable
ps-indent-level controls the amount of indentation used inside
arrays and begin/end pairs.  

\\{ps-mode-map}

\\[postscript-mode] calls the value of the variable postscript-mode-hook 
with no args, if that value is non-nil." t nil)

;;;***

;;;### (autoloads (run-prolog inferior-prolog-mode prolog-mode) "prolog" "modes/prolog.el" (12558 60895))
;;; Generated autoloads from modes/prolog.el

(autoload 'prolog-mode "prolog" "\
Major mode for editing Prolog code for Prologs.
Blank lines and `%%...' separate paragraphs.  `%'s start comments.
Commands:
\\{prolog-mode-map}
Entry to this mode calls the value of `prolog-mode-hook'
if that value is non-nil." t nil)

(autoload 'inferior-prolog-mode "prolog" "\
Major mode for interacting with an inferior Prolog process.

The following commands are available:
\\{inferior-prolog-mode-map}

Entry to this mode calls the value of `prolog-mode-hook' with no arguments,
if that value is non-nil.  Likewise with the value of `comint-mode-hook'.
`prolog-mode-hook' is called after `comint-mode-hook'.

You can send text to the inferior Prolog from other buffers
using the commands `send-region', `send-string' and \\[prolog-consult-region].

Commands:
Tab indents for Prolog; with argument, shifts rest
 of expression rigidly with the current line.
Paragraphs are separated only by blank lines and '%%'.
'%'s start comments.

Return at end of buffer sends line as input.
Return not at end copies rest of line to end and sends it.
\\[comint-kill-input] and \\[backward-kill-word] are kill commands, imitating normal Unix input editing.
\\[comint-interrupt-subjob] interrupts the shell or its current subjob if any.
\\[comint-stop-subjob] stops. \\[comint-quit-subjob] sends quit signal." t nil)

(autoload 'run-prolog "prolog" "\
Run an inferior Prolog process, input and output via buffer *prolog*." t nil)

;;;***

;;;### (autoloads (rexx-mode) "rexx-mode" "modes/rexx-mode.el" (12558 60898))
;;; Generated autoloads from modes/rexx-mode.el

(autoload 'rexx-mode "rexx-mode" "\
Major mode for editing REXX code.
\\{rexx-mode-map}

Variables controlling indentation style:
 rexx-indent
	The basic indentation for do-blocks.
 rexx-end-indent
	The relative offset of the \"end\" statement. 0 places it in the
	same column as the statements of the block. Setting it to the same
	value as rexx-indent places the \"end\" under the do-line.
 rexx-cont-indent
	The indention for lines following \"then\", \"else\" and \",\"
	(continued) lines.
 rexx-tab-always-indent
	Non-nil means TAB in REXX mode should always reindent the current 
	line, regardless of where in the line the point is when the TAB
	command is used.

If you have set rexx-end-indent to a nonzero value, you probably want to
remap RETURN to rexx-indent-newline-indent. It makes sure that lines
indents correctly when you press RETURN.

An extensive abbrevation table consisting of all the keywords of REXX are
supplied. Expanded keywords are converted into upper case making it
easier to distinguish them. To use this feature the buffer must be in
abbrev-mode. (See example below.)

Turning on REXX mode calls the value of the variable rexx-mode-hook with
no args, if that value is non-nil.

For example:
\(setq rexx-mode-hook '(lambda ()
			(setq rexx-indent 4)
			(setq rexx-end-indent 4)
			(setq rexx-cont-indent 4)
			(local-set-key \"\\C-m\" 'rexx-indent-newline-indent)
			(abbrev-mode 1)
			))

will make the END aligned with the DO/SELECT. It will indent blocks and
IF-statenents four steps and make sure that the END jumps into the
correct position when RETURN is pressed. Finaly it will use the abbrev
table to convert all REXX keywords into upper case." t nil)

;;;***

;;;### (autoloads (resize-minibuffer-mode) "rsz-minibuf" "modes/rsz-minibuf.el" (12546 50449))
;;; Generated autoloads from modes/rsz-minibuf.el

(defvar resize-minibuffer-mode nil "\
*If non-`nil', resize the minibuffer so its entire contents are visible.")

(defvar resize-minibuffer-window-max-height nil "\
*Maximum size the minibuffer window is allowed to become.
If less than 1 or not a number, the limit is the height of the frame in
which the active minibuffer window resides.")

(defvar resize-minibuffer-window-exactly t "\
*If non-`nil', make minibuffer exactly the size needed to display all its contents.
Otherwise, the minibuffer window can temporarily increase in size but
never get smaller while it is active.")

(defvar resize-minibuffer-frame nil "\
*If non-`nil' and the active minibuffer is the sole window in its frame, allow changing the frame height.")

(defvar resize-minibuffer-frame-max-height nil "\
*Maximum size the minibuffer frame is allowed to become.
If less than 1 or not a number, there is no limit.")

(defvar resize-minibuffer-frame-exactly nil "\
*If non-`nil', make minibuffer frame exactly the size needed to display all its contents.
Otherwise, the minibuffer frame can temporarily increase in size but
never get smaller while it is active.")

(autoload 'resize-minibuffer-mode "rsz-minibuf" "\
Enable or disable resize-minibuffer mode.
A negative prefix argument disables this mode.  A positive argument or
argument of 0 enables it.

When this minor mode is enabled, the minibuffer is dynamically resized to
contain the entire region of text put in it as you type.

The variable `resize-minibuffer-mode' is set to t or nil depending on
whether this mode is active or not.

The maximum height to which the minibuffer can grow is controlled by the
variable `resize-minibuffer-window-max-height'.

The variable `resize-minibuffer-window-exactly' determines whether the
minibuffer window should ever be shrunk to make it no larger than needed to
display its contents.

When using a window system, it is possible for a minibuffer to tbe the sole
window in a frame.  Since that window is already its maximum size, the only
way to make more text visible at once is to increase the size of the frame.
The variable `resize-minibuffer-frame' controls whether this should be
done.  The variables `resize-minibuffer-frame-max-height' and
`resize-minibuffer-frame-exactly' are analogous to their window
counterparts." t nil)

;;;***

;;;### (autoloads (scheme-mode) "scheme" "modes/scheme.el" (12376 19455))
;;; Generated autoloads from modes/scheme.el

(autoload 'scheme-mode "scheme" "\
Major mode for editing Scheme code.
Editing commands are similar to those of lisp-mode.

In addition, if an inferior Scheme process is running, some additional
commands will be defined, for evaluating expressions and controlling
the interpreter, and the state of the process will be displayed in the
modeline of all Scheme buffers.  The names of commands that interact
with the Scheme process start with \"xscheme-\".  For more information
see the documentation for xscheme-interaction-mode.

Commands:
Delete converts tabs to spaces as it moves back.
Blank lines separate paragraphs.  Semicolons start comments.
\\{scheme-mode-map}
Entry to this mode calls the value of scheme-mode-hook
if that value is non-nil." t nil)

;;;***

;;;### (autoloads (scribe-mode) "scribe" "modes/scribe.el" (12376 19456))
;;; Generated autoloads from modes/scribe.el

(autoload 'scribe-mode "scribe" "\
Major mode for editing files of Scribe (a text formatter) source.
Scribe-mode is similar text-mode, with a few extra commands added.
\\{scribe-mode-map}

Interesting variables:

scribe-fancy-paragraphs
  Non-nil makes Scribe mode use a different style of paragraph separation.

scribe-electric-quote
  Non-nil makes insert of double quote use `` or '' depending on context.

scribe-electric-parenthesis
  Non-nil makes an open-parenthesis char (one of `([<{')
  automatically insert its close if typed after an @Command form." t nil)

;;;***

;;;### (autoloads (mail-other-frame mail-other-window mail mail-mode) "sendmail" "modes/sendmail.el" (12686 31716))
;;; Generated autoloads from modes/sendmail.el

(defvar mail-from-style 'angles "\
*Specifies how \"From:\" fields look.

If `nil', they contain just the return address like:
	king@grassland.com
If `parens', they look like:
	king@grassland.com (Elvis Parsley)
If `angles', they look like:
	Elvis Parsley <king@grassland.com>")

(defvar mail-self-blind nil "\
Non-nil means insert BCC to self in messages to be sent.
This is done when the message is initialized,
so you can remove or alter the BCC field to override the default.")

(defvar mail-interactive nil "\
Non-nil means when sending a message wait for and display errors.
nil means let mailer mail back a message to report errors.")

(defvar mail-dir nil "\
*Default directory for saving messages.")

(defvar rmail-ignored-headers (purecopy (concat "^\\(" (mapconcat 'identity '("Sender:" "References:" "Return-Path:" "Received:" "[^: 	\n]*Message-ID:" "Errors-To:" "Path:" "Expires:" "Xref:" "Lines:" "Approved:" "Distribution:" "Content-Length:" "Mime-Version:" "Content-Type:" "Content-Transfer-Encoding:" "X400-Received:" "X400-Originator:" "X400-Mts-Identifier:" "X400-Content-Type:" "Content-Identifier:" "Status:" "Summary-Line:" "X-Attribution:" "Via:" "Sent-Via:" "Mail-From:" "Origin:" "Comments:" "Originator:" "NF-ID:" "NF-From:" "Posting-Version:" "Posted:" "Posted-Date:" "Date-Received:" "Relay-Version:" "Article-I\\.D\\.:" "NNTP-Version:" "NNTP-Posting-Host:" "X-Mailer:" "X-Newsreader:" "News-Software:" "X-Received:" "X-References:" "X-Envelope-To:" "X-VMS-" "Remailed-" "X-Plantation:" "X-Windows:" "X-Pgp-") "\\|") "\\)")) "\
*Gubbish header fields one would rather not see.")

(defvar mail-yank-ignored-headers (purecopy (concat rmail-ignored-headers "\\|" "^\\(" (mapconcat 'identity '("Resent-To:" "Resent-By:" "Resent-CC:" "To:" "Subject:" "In-Reply-To:") "\\|") "\\)")) "\
Delete these headers from old message when it's inserted in a reply.")

(defvar send-mail-function 'sendmail-send-it "\
Function to call to send the current buffer as mail.
The headers should be delimited by a line whose contents
match the variable `mail-header-separator'.")

(defvar mail-header-separator (purecopy "--text follows this line--") "\
*Line used to separate headers from text in messages being composed.")

(defvar mail-archive-file-name nil "\
*Name of file to write all outgoing messages in, or nil for none.
This can be an inbox file or an Rmail file.")

(defvar mail-default-reply-to nil "\
*Address to insert as default Reply-to field of outgoing messages.
If nil, it will be initialized from the REPLYTO environment variable
when you first send mail.")

(defvar mail-alias-file nil "\
*If non-nil, the name of a file to use instead of `/usr/lib/aliases'.
This file defines aliases to be expanded by the mailer; this is a different
feature from that of defining aliases in `.mailrc' to be expanded in Emacs.
This variable has no effect unless your system uses sendmail as its mailer.")

(defvar mail-yank-prefix "> " "\
*Prefix insert on lines of yanked message being replied to.
nil means use indentation.")

(defvar mail-signature nil "\
*Text inserted at end of mail buffer when a message is initialized.
If t, it means to insert the contents of the file `mail-signature-file'.")

(autoload 'mail-mode "sendmail" "\
Major mode for editing mail to be sent.
Like Text Mode but with these additional commands:
C-c C-s  mail-send (send the message)    C-c C-c  mail-send-and-exit
C-c C-f  move to a header field (and create it if there isn't):
	 C-c C-f C-t  move to To:	C-c C-f C-s  move to Subj:
	 C-c C-f C-b  move to BCC:	C-c C-f C-c  move to CC:
	 C-c C-f C-f  move to FCC:	C-c C-f C-r  move to Reply-To:
C-c C-t  mail-text (move to beginning of message text).
C-c C-w  mail-signature (insert `mail-signature-file' file).
C-c C-y  mail-yank-original (insert current message, in Rmail).
C-c C-q  mail-fill-yanked-message (fill what was yanked).
C-c C-v  mail-sent-via (add a sent-via field for each To or CC)." t nil)

(autoload 'mail "sendmail" "\
Edit a message to be sent.  Prefix arg means resume editing (don't erase).
When this function returns, the buffer `*mail*' is selected.
The value is t if the message was newly initialized; otherwise, nil.

Optionally, the signature file `mail-signature-file' can be inserted at the
end; see the variable `mail-signature'.

\\<mail-mode-map>
While editing message, type \\[mail-send-and-exit] to send the message and exit.

Various special commands starting with C-c are available in sendmail mode
to move to message header fields:
\\{mail-mode-map}

The variable `mail-signature' controls whether the signature file
`mail-signature-file' is inserted immediately.

If `mail-signature' is nil, use \\[mail-signature] to insert the
signature in `mail-signature-file'.

If `mail-self-blind' is non-nil, a BCC to yourself is inserted
when the message is initialized.

If `mail-default-reply-to' is non-nil, it should be an address (a string);
a Reply-to: field with that address is inserted.

If `mail-archive-file-name' is non-nil, an FCC field with that file name
is inserted.

The normal hook `mail-setup-hook' is run after the message is
initialized.  It can add more default fields to the message.

When calling from a program, the first argument if non-nil says
not to erase the existing contents of the `*mail*' buffer.

The second through fifth arguments,
 TO, SUBJECT, IN-REPLY-TO and CC, specify if non-nil
 the initial contents of those header fields.
 These arguments should not have final newlines.
The sixth argument REPLYBUFFER is a buffer whose contents
 should be yanked if the user types C-c C-y.
The seventh argument ACTIONS is a list of actions to take
 if/when the message is sent.  Each action looks like (FUNCTION . ARGS);
 when the message is sent, we apply FUNCTION to ARGS.
 This is how Rmail arranges to mark messages `answered'." t nil)

(autoload 'mail-other-window "sendmail" "\
Like `mail' command, but display mail buffer in another window." t nil)

(autoload 'mail-other-frame "sendmail" "\
Like `mail' command, but display mail buffer in another frame." t nil)

(define-key ctl-x-map "m" 'mail)

(define-key ctl-x-4-map "m" 'mail-other-window)

(define-key ctl-x-5-map "m" 'mail-other-frame)

(add-hook 'same-window-buffer-names "*mail*")

;;;***

;;;### (autoloads nil "simula" "modes/simula.el" (12546 50444))
;;; Generated autoloads from modes/simula.el

;;;### (autoloads (tcl-help-on-word inferior-tcl tcl-mode) "tcl" "modes/tcl.el" (12639 8579))
;;; Generated autoloads from modes/tcl.el

(autoload 'tcl-mode "tcl" "\
Major mode for editing Tcl code.
Expression and list commands understand all Tcl brackets.
Tab indents for Tcl code.
Paragraphs are separated by blank lines only.
Delete converts tabs to spaces as it moves back.

Variables controlling indentation style:
  tcl-indent-level
    Indentation of Tcl statements within surrounding block.
  tcl-continued-indent-level
    Indentation of continuation line relative to first line of command.

Variables controlling user interaction with mode (see variable
documentation for details):
  tcl-tab-always-indent
    Controls action of TAB key.
  tcl-auto-newline
    Non-nil means automatically newline before and after braces, brackets,
    and semicolons inserted in Tcl code.
  tcl-electric-hash-style
    Controls action of `#' key.
  tcl-use-hairy-comment-detector
    If t, use more complicated, but slower, comment detector.
    This variable is only used in GNU Emacs 19.
  tcl-use-smart-word-finder
    If not nil, use a smarter, Tcl-specific way to find the current
    word when looking up help on a Tcl command.

Turning on Tcl mode calls the value of the variable `tcl-mode-hook'
with no args, if that value is non-nil.  Read the documentation for
`tcl-mode-hook' to see what kinds of interesting hook functions
already exist.

Commands:
\\{tcl-mode-map}" t nil)

(autoload 'inferior-tcl "tcl" "\
Run inferior Tcl process.
Prefix arg means enter program name interactively.
See documentation for function `inferior-tcl-mode' for more information." t nil)

(autoload 'tcl-help-on-word "tcl" "\
Get help on Tcl command.  Default is word at point.
Prefix argument means invert sense of `tcl-use-smart-word-finder'." t nil)

;;;***

;;;### (autoloads (latex-mode plain-tex-mode tex-mode) "tex-mode" "modes/tex-mode.el" (12376 19459))
;;; Generated autoloads from modes/tex-mode.el

(autoload 'tex-mode "tex-mode" "\
Major mode for editing files of input for TeX, LaTeX, or SliTeX.
Tries to determine (by looking at the beginning of the file) whether
this file is for plain TeX, LaTeX, or SliTeX and calls plain-tex-mode,
latex-mode, or slitex-mode, respectively.  If it cannot be determined,
such as if there are no commands in the file, the value of tex-default-mode
is used." t nil)

(fset 'TeX-mode 'tex-mode)

(fset 'LaTeX-mode 'latex-mode)

(autoload 'plain-tex-mode "tex-mode" "\
Major mode for editing files of input for plain TeX.
Makes $ and } display the characters they match.
Makes \" insert `` when it seems to be the beginning of a quotation,
and '' when it appears to be the end; it inserts \" only after a \\.

Use \\[tex-region] to run TeX on the current region, plus a \"header\"
copied from the top of the file (containing macro definitions, etc.),
running TeX under a special subshell.  \\[tex-buffer] does the whole buffer.
\\[tex-file] saves the buffer and then processes the file.
\\[tex-print] prints the .dvi file made by any of these.
\\[tex-view] previews the .dvi file made by any of these.
\\[tex-bibtex-file] runs bibtex on the file of the current buffer.

Use \\[validate-tex-buffer] to check buffer for paragraphs containing
mismatched $'s or braces.

Special commands:
\\{tex-mode-map}

Mode variables:
tex-run-command
	Command string used by \\[tex-region] or \\[tex-buffer].
tex-directory
	Directory in which to create temporary files for TeX jobs
	run by \\[tex-region] or \\[tex-buffer].
tex-dvi-print-command
	Command string used by \\[tex-print] to print a .dvi file.
tex-alt-dvi-print-command
	Alternative command string used by \\[tex-print] (when given a prefix
	argument) to print a .dvi file.
tex-dvi-view-command
	Command string used by \\[tex-view] to preview a .dvi file.
tex-show-queue-command
	Command string used by \\[tex-show-print-queue] to show the print
	queue that \\[tex-print] put your job on.

Entering Plain-tex mode calls the value of text-mode-hook, then the value of
tex-mode-hook, and then the value of plain-tex-mode-hook.  When the special
subshell is initiated, the value of tex-shell-hook is called." t nil)

(fset 'plain-TeX-mode 'plain-tex-mode)

(autoload 'latex-mode "tex-mode" "\
Major mode for editing files of input for LaTeX.
Makes $ and } display the characters they match.
Makes \" insert `` when it seems to be the beginning of a quotation,
and '' when it appears to be the end; it inserts \" only after a \\.

Use \\[tex-region] to run LaTeX on the current region, plus the preamble
copied from the top of the file (containing \\documentstyle, etc.),
running LaTeX under a special subshell.  \\[tex-buffer] does the whole buffer.
\\[tex-file] saves the buffer and then processes the file.
\\[tex-print] prints the .dvi file made by any of these.
\\[tex-view] previews the .dvi file made by any of these.
\\[tex-bibtex-file] runs bibtex on the file of the current buffer.

Use \\[validate-tex-buffer] to check buffer for paragraphs containing
mismatched $'s or braces.

Special commands:
\\{tex-mode-map}

Mode variables:
latex-run-command
	Command string used by \\[tex-region] or \\[tex-buffer].
tex-directory
	Directory in which to create temporary files for LaTeX jobs
	run by \\[tex-region] or \\[tex-buffer].
tex-dvi-print-command
	Command string used by \\[tex-print] to print a .dvi file.
tex-alt-dvi-print-command
	Alternative command string used by \\[tex-print] (when given a prefix
	argument) to print a .dvi file.
tex-dvi-view-command
	Command string used by \\[tex-view] to preview a .dvi file.
tex-show-queue-command
	Command string used by \\[tex-show-print-queue] to show the print
	queue that \\[tex-print] put your job on.

Entering Latex mode calls the value of text-mode-hook, then the value of
tex-mode-hook, and then the value of latex-mode-hook.  When the special
subshell is initiated, the value of tex-shell-hook is called." t nil)

;;;***

;;;### (autoloads (texinfo-mode) "texinfo" "modes/texinfo.el" (12558 60898))
;;; Generated autoloads from modes/texinfo.el

(autoload 'texinfo-mode "texinfo" "\
Major mode for editing Texinfo files.

  It has these extra commands:
\\{texinfo-mode-map}

  These are files that are used as input for TeX to make printed manuals
and also to be turned into Info files with \\[makeinfo-buffer] or
the `makeinfo' program.  These files must be written in a very restricted and
modified version of TeX input format.

  Editing commands are like text-mode except that the syntax table is
set up so expression commands skip Texinfo bracket groups.  To see
what the Info version of a region of the Texinfo file will look like,
use \\[makeinfo-region], which runs `makeinfo' on the current region.

  You can show the structure of a Texinfo file with \\[texinfo-show-structure].
This command shows the structure of a Texinfo file by listing the
lines with the @-sign commands for @chapter, @section, and the like.
These lines are displayed in another window called the *Occur* window.
In that window, you can position the cursor over one of the lines and
use \\[occur-mode-goto-occurrence], to jump to the corresponding spot
in the Texinfo file.

  In addition, Texinfo mode provides commands that insert various
frequently used @-sign commands into the buffer.  You can use these
commands to save keystrokes.  And you can insert balanced braces with
\\[texinfo-insert-braces] and later use the command \\[up-list] to
move forward past the closing brace.

Also, Texinfo mode provides functions for automatically creating or
updating menus and node pointers.  These functions

  * insert the `Next', `Previous' and `Up' pointers of a node,
  * insert or update the menu for a section, and
  * create a master menu for a Texinfo source file.

Here are the functions:

    texinfo-update-node                \\[texinfo-update-node]
    texinfo-every-node-update          \\[texinfo-every-node-update]
    texinfo-sequential-node-update 

    texinfo-make-menu                  \\[texinfo-make-menu]
    texinfo-all-menus-update           \\[texinfo-all-menus-update]
    texinfo-master-menu

    texinfo-indent-menu-description (column &optional region-p)

The `texinfo-column-for-description' variable specifies the column to
which menu descriptions are indented. 

Passed an argument (a prefix argument, if interactive), the
`texinfo-update-node' and `texinfo-make-menu' functions do their jobs
in the region.

To use the updating commands, you must structure your Texinfo file
hierarchically, such that each `@node' line, with the exception of the
Top node, is accompanied by some kind of section line, such as an
`@chapter' or `@section' line.

If the file has a `top' node, it must be called `top' or `Top' and
be the first node in the file.

Entering Texinfo mode calls the value of text-mode-hook, and then the
value of texinfo-mode-hook." t nil)

;;;***

;;;### (autoloads nil "text-mode" "modes/text-mode.el" (12546 50443))
;;; Generated autoloads from modes/text-mode.el

;;;### (autoloads (tc-recenter tc-scroll-down tc-scroll-up tc-scroll-line tc-associated-buffer tc-merge tc-dissociate tc-split tc-associate-buffer tc-two-columns) "two-column" "modes/two-column.el" (12648 18571))
;;; Generated autoloads from modes/two-column.el

(defvar tc-mode-map nil "\
Keymap for commands for two-column mode.")

(if tc-mode-map nil (setq tc-mode-map (make-sparse-keymap)) (define-key tc-mode-map "1" 'tc-merge) (define-key tc-mode-map "2" 'tc-two-columns) (define-key tc-mode-map "b" 'tc-associate-buffer) (define-key tc-mode-map "d" 'tc-dissociate) (define-key tc-mode-map "\^L" 'tc-recenter) (define-key tc-mode-map "o" 'tc-associated-buffer) (define-key tc-mode-map "s" 'tc-split) (define-key tc-mode-map "{" 'shrink-window-horizontally) (define-key tc-mode-map "}" 'enlarge-window-horizontally) (define-key tc-mode-map " " 'tc-scroll-up) (define-key tc-mode-map "" 'tc-scroll-down) (define-key tc-mode-map "
" 'tc-scroll-line))

(global-set-key "6" tc-mode-map)

(defvar tc-other nil "\
Marker to the associated buffer, if non-nil.")

(make-variable-buffer-local 'tc-other)

(put 'tc-other 'permanent-local t)

(autoload 'tc-two-columns "two-column" "\
Split current window vertically for two-column editing.

When called the first time, associates a buffer with the current
buffer.  Both buffers are put in two-column minor mode and
tc-mode-hook gets called on both.  These buffers remember
about one another, even when renamed.

When called again, restores the screen layout with the current buffer
first and the associated buffer to it's right.

If you include long lines, i.e which will span both columns (eg.
source code), they should be in what will be the first column, with
the associated buffer having empty lines next to them.

You have the following commands at your disposal:

\\[tc-two-columns]   Rearrange screen
\\[tc-associate-buffer]   Reassociate buffer after changing major mode
\\[tc-scroll-up]   Scroll both buffers up by a screenfull
\\[tc-scroll-down]   Scroll both buffers down by a screenful
\\[tc-scroll-line]   Scroll both buffers up by one or more lines
\\[tc-recenter]   Recenter and realign other buffer
\\[shrink-window-horizontally], \\[enlarge-window-horizontally]   Shrink, enlarge current column
\\[tc-associated-buffer]   Switch to associated buffer
\\[tc-merge]   Merge both buffers

These keybindings can be customized in your ~/.emacs by `tc-prefix'
and `tc-mode-map'.

The appearance of the screen can be customized by the variables
`tc-window-width', `tc-beyond-fill-column',
`tc-mode-line-format' and `truncate-partial-width-windows'." t nil)

(add-minor-mode 'tc-other " 2C" nil nil 'tc-two-columns)

(autoload 'tc-associate-buffer "two-column" "\
Associate another buffer with this one in two-column minor mode.
Can also be used to associate a just previously visited file, by
accepting the proposed default buffer.

See  \\[tc-two-columns]  and  `lisp/two-column.el'  for further details." t nil)

(autoload 'tc-split "two-column" "\
Unmerge a two-column text into two buffers in two-column minor mode.
The text is unmerged at the cursor's column which becomes the local
value of `tc-window-width'.  Only lines that have the ARG same
preceding characters at that column get split.  The ARG preceding
characters without any leading whitespace become the local value for
`tc-separator'.  This way lines that continue across both
columns remain untouched in the first buffer.

This function can be used with a prototype line, to set up things as
you like them.  You write the first line of each column with the
separator you like and then unmerge that line.  E.g.:

First column's text    sSs  Second columns text
		       \\___/\\
			/    \\
   5 character Separator      You type  M-5 \\[tc-split]  with the point here

See  \\[tc-two-columns]  and  `lisp/two-column.el'  for further details." t nil)

(autoload 'tc-dissociate "two-column" "\
Turn off two-column minor mode in current and associated buffer.
If the associated buffer is unmodified and empty, it is killed." t nil)

(autoload 'tc-merge "two-column" "\
Merges the associated buffer with the current buffer.
They get merged at the column, which is the value of
`tc-window-width', i.e. usually at the vertical window
separator.  This separator gets replaced with white space.  Beyond
that the value of gets inserted on merged lines.  The two columns are
thus pasted side by side, in a single text.  If the other buffer is
not displayed to the left of this one, then this one becomes the left
column.

If you want `tc-separator' on empty lines in the second column,
you should put just one space in them.  In the final result, you can strip
off trailing spaces with \\[beginning-of-buffer] \\[replace-regexp] [ SPC TAB ] + $ RET RET" t nil)

(autoload 'tc-associated-buffer "two-column" "\
Switch to associated buffer." t nil)

(autoload 'tc-scroll-line "two-column" "\
Scroll current window upward by ARG lines.
The associated window gets scrolled to the same line." t nil)

(autoload 'tc-scroll-up "two-column" "\
Scroll current window upward by ARG screens.
The associated window gets scrolled to the same line." t nil)

(autoload 'tc-scroll-down "two-column" "\
Scroll current window downward by ARG screens.
The associated window gets scrolled to the same line." t nil)

(autoload 'tc-recenter "two-column" "\
Center point in window.  With ARG, put point on line ARG.
This counts from bottom if ARG is negative.  The associated window
gets scrolled to the same line." t nil)

;;;***

;;;### (autoloads (auto-view-mode view-major-mode view-mode view-minor-mode view-buffer-other-window view-file-other-window view-buffer view-file) "view-less" "modes/view-less.el" (12743 11603))
;;; Generated autoloads from modes/view-less.el

(autoload 'view-file "view-less" "\
Find FILE, enter view mode.  With prefix arg OTHER-P, use other window." t nil)

(autoload 'view-buffer "view-less" "\
Switch to BUF, enter view mode.  With prefix arg use other window." t nil)

(autoload 'view-file-other-window "view-less" "\
Find FILE in other window, and enter view mode." t nil)

(autoload 'view-buffer-other-window "view-less" "\
Switch to BUFFER in another window, and enter view mode." t nil)

(autoload 'view-minor-mode "view-less" "\
Minor mode for viewing text, with bindings like `less'.
Commands are:
\\<view-minor-mode-map>
0..9	prefix args
-	prefix minus
\\[scroll-up]	page forward
\\[scroll-down]	page back
\\[view-scroll-lines-up]	scroll prefix-arg lines forward, default 1.
\\[view-scroll-lines-down]	scroll prefix-arg lines backward, default 1.
\\[view-scroll-some-lines-down]	scroll prefix-arg lines backward, default 10.
\\[view-scroll-some-lines-up]	scroll prefix-arg lines forward, default 10.
\\[what-line]	print line number
\\[view-mode-describe]	print this help message
\\[view-search-forward]	regexp search, uses previous string if you just hit RET
\\[view-search-backward]	as above but searches backward
\\[view-repeat-search]	repeat last search
\\[view-goto-line]	goto line prefix-arg, default 1
\\[view-last-windowful]	goto line prefix-arg, default last line
\\[view-goto-percent]	goto a position by percentage
\\[toggle-truncate-lines]	toggle truncate-lines
\\[view-file]	view another file
\\[view-buffer]	view another buffer
\\[view-cleanup-backspaces]	cleanup backspace constructions
\\[shell-command]	execute a shell command
\\[shell-command-on-region]	execute a shell command with the region as input
\\[view-quit]	exit view-mode, and bury the current buffer.

If invoked with the optional (prefix) arg non-nil, view-mode cleans up
backspace constructions.

More precisely:
\\{view-minor-mode-map}" t nil)

(autoload 'view-mode "view-less" "\
View the current buffer using view-minor-mode.  This exists to be 99.9%
compatible with the implementations of `view-mode' in view.el and older
versions of view-less.el." t nil)

(autoload 'view-major-mode "view-less" "\
View the current buffer using view-mode, as a major mode.
This function has a nonstandard name because `view-mode' is wrongly
named but is like this for compatibility reasons." t nil)

(autoload 'auto-view-mode "view-less" "\
If the file of the current buffer is not writable, call view-mode.
This is meant to be added to `find-file-hooks'." nil nil)

;;;***

;;;### (autoloads nil "view" "modes/view.el" (12546 50445))
;;; Generated autoloads from modes/view.el

;;;### (autoloads (vrml-mode) "vrml-mode" "modes/vrml-mode.el" (12714 11717))
;;; Generated autoloads from modes/vrml-mode.el

(autoload 'vrml-mode "vrml-mode" "\
Major mode for editing VRML code.
Expression and list commands understand all VRML brackets.
Tab indents for VRML code.
Paragraphs are separated by blank lines only.
Delete converts tabs to spaces as it moves back.

Variables controlling indentation style:
  vrml-indent-level
    Indentation of VRML statements within surrounding block.

Variables controlling user interaction with mode (see variable
documentation for details):
  vrml-tab-always-indent
    Controls action of TAB key.
  vrml-auto-newline
    Non-nil means automatically newline before and after braces
    inserted in VRML code.

Turning on VRML mode calls the value of the variable `vrml-mode-hook'
with no args, if that value is non-nil.  Read the documentation for
`vrml-mode-hook' to see what kinds of interesting hook functions
already exist.

Commands:
\\{vrml-mode-map}" t nil)

;;;***

;;;### (autoloads (xpm-mode) "xpm-mode" "modes/xpm-mode.el" (12559 34912))
;;; Generated autoloads from modes/xpm-mode.el

(autoload 'xpm-mode "xpm-mode" "\
Treat the current buffer as an xpm file and colorize it.

  Shift-button-1 lets you paint by dragging the mouse.  Shift-button-1 on a
color definition line will change the current painting color to that line's
value.

  Characters inserted from the keyboard will NOT be colored properly yet.
Use the mouse, or do xpm-init (\\[xpm-init]) after making changes.

\\[xpm-add-color] Add a new color, prompting for character and value
\\[xpm-show-image] show the current image at the top of the buffer
\\[xpm-parse-color] parse the current line's color definition and add
   it to the color table.  Provided as a means of changing colors.
XPM minor mode bindings:
\\{xpm-mode-map}" t nil)

;;;***

;;;### (autoloads nil "ns-init" "ns/ns-init.el" (12528 24026))
;;; Generated autoloads from ns/ns-init.el

;;;### (autoloads nil "br-c++-ft" "oobr/br-c++-ft.el" (12376 19719))
;;; Generated autoloads from oobr/br-c++-ft.el

;;;***

;;;### (autoloads nil "br-c++" "oobr/br-c++.el" (12558 61009))
;;; Generated autoloads from oobr/br-c++.el

;;;***

;;;### (autoloads nil "br-c-ft" "oobr/br-c-ft.el" (12558 61026))
;;; Generated autoloads from oobr/br-c-ft.el

;;;### (autoloads nil "br-clos-ft" "oobr/br-clos-ft.el" (12376 19720))
;;; Generated autoloads from oobr/br-clos-ft.el

;;;***

;;;### (autoloads nil "br-clos" "oobr/br-clos.el" (12558 61010))
;;; Generated autoloads from oobr/br-clos.el

;;;### (autoloads nil "br-compl" "oobr/br-compl.el" (12376 19721))
;;; Generated autoloads from oobr/br-compl.el

;;;***

;;;### (autoloads nil "br-eif-ft" "oobr/br-eif-ft.el" (12376 19721))
;;; Generated autoloads from oobr/br-eif-ft.el

;;;***

;;;### (autoloads nil "br-eif" "oobr/br-eif.el" (12558 61010))
;;; Generated autoloads from oobr/br-eif.el

;;;***

;;;### (autoloads (br-env-load br-env-browse) "br-env" "oobr/br-env.el" (12714 11761))
;;; Generated autoloads from oobr/br-env.el

(autoload 'br-env-browse "br-env" "\
Invoke the OO-Browser on an existing or to be created Environment ENV-FILE." t nil)

(autoload 'br-env-load "br-env" "\
Load browser Environment or spec from optional ENV-FILE or 'br-env-file'.
Non-nil PROMPT means prompt user before building tables.
Non-nil NO-BUILD means skip build of Environment entirely.
Return t if load is successful, else nil." t nil)

;;;***

;;;### (autoloads nil "br-ftr" "oobr/br-ftr.el" (12376 19723))
;;; Generated autoloads from oobr/br-ftr.el

;;;***

;;;### (autoloads nil "br-info" "oobr/br-info.el" (12558 61013))
;;; Generated autoloads from oobr/br-info.el

;;;***

;;;### (autoloads nil "br-init" "oobr/br-init.el" (12714 11763))
;;; Generated autoloads from oobr/br-init.el

;;;### (autoloads nil "br-java-ft" "oobr/br-java-ft.el" (12558 61028))
;;; Generated autoloads from oobr/br-java-ft.el

;;;***

;;;### (autoloads nil "br-java" "oobr/br-java.el" (12558 61025))
;;; Generated autoloads from oobr/br-java.el

;;;***

;;;### (autoloads nil "br-lib" "oobr/br-lib.el" (12558 61015))
;;; Generated autoloads from oobr/br-lib.el

;;;***

;;;### (autoloads nil "br-menu" "oobr/br-menu.el" (12558 61025))
;;; Generated autoloads from oobr/br-menu.el

;;;### (autoloads nil "br-objc-ft" "oobr/br-objc-ft.el" (12376 19725))
;;; Generated autoloads from oobr/br-objc-ft.el

;;;***

;;;### (autoloads nil "br-objc" "oobr/br-objc.el" (12558 61016))
;;; Generated autoloads from oobr/br-objc.el

;;;***

;;;### (autoloads nil "br-python-ft" "oobr/br-python-ft.el" (12714 11823))
;;; Generated autoloads from oobr/br-python-ft.el

;;;***

;;;### (autoloads nil "br-python" "oobr/br-python.el" (12714 11822))
;;; Generated autoloads from oobr/br-python.el

;;;### (autoloads nil "br-site" "oobr/br-site.el" (12376 19725))
;;; Generated autoloads from oobr/br-site.el

;;;***

;;;### (autoloads nil "br-smt" "oobr/br-smt.el" (12558 61017))
;;; Generated autoloads from oobr/br-smt.el

;;;***

;;;### (autoloads (oo-browser) "br-start" "oobr/br-start.el" (12558 61017))
;;; Generated autoloads from oobr/br-start.el

(fset 'oobr 'oo-browser)

(autoload 'oo-browser "br-start" "\
Prompt for an Environment and language over which to run the OO-Browser.
Optional prefix argument SAME-ENV-FLAG means browse the current Environment,
if any, without prompting." t nil)

;;;***

;;;### (autoloads nil "br-tree" "oobr/br-tree.el" (12376 19726))
;;; Generated autoloads from oobr/br-tree.el

;;;***

;;;### (autoloads nil "br-vers" "oobr/br-vers.el" (12714 11765))
;;; Generated autoloads from oobr/br-vers.el

;;;### (autoloads (br-to-from-viewer br-add-class-file) "br" "oobr/br.el" (12714 11764))
;;; Generated autoloads from oobr/br.el

(autoload 'br-add-class-file "br" "\
Add a file of classes to the current Environment.
Interactively or when optional CLASS-PATH is nil, CLASS-PATH defaults to the
current buffer file pathname.  If optional LIB-TABLE-P is non-nil, add to
Library Environment, otherwise add to System Environment.  If optional
SAVE-FILE is t, the Environment is then stored to the filename given by
'br-env-file'.  If SAVE-FILE is non-nil and not t, its string value is used
as the file to which to save the Environment." t nil)

(autoload 'br-to-from-viewer "br" "\
Move point to viewer window or back to last recorded listing window." t nil)

;;;***

;;;### (autoloads (c++-browse) "c++-browse" "oobr/c++-browse.el" (12558 61020))
;;; Generated autoloads from oobr/c++-browse.el

(autoload 'c++-browse "c++-browse" "\
Invoke the C++ OO-Browser.
This allows browsing through C++ library and system class hierarchies.  With
an optional non-nil prefix argument ENV-FILE, prompt for Environment file to
use.  Alternatively, a string value of ENV-FILE is used as the Environment
file name.  See also the file \"br-help\"." t nil)

;;;***

;;;### (autoloads (clos-browse) "clos-brows" "oobr/clos-brows.el" (12558 61020))
;;; Generated autoloads from oobr/clos-brows.el

(autoload 'clos-browse "clos-brows" "\
Invoke the CLOS OO-Browser.
This allows browsing through CLOS library and system class hierarchies.  With
an optional non-nil prefix argument ENV-FILE, prompt for Environment file
to use.  Alternatively, a string value of ENV-FILE is used as the
Environment file name.  See also the file \"br-help\"." t nil)

;;;***

;;;### (autoloads (eif-browse) "eif-browse" "oobr/eif-browse.el" (12558 61022))
;;; Generated autoloads from oobr/eif-browse.el

(autoload 'eif-browse "eif-browse" "\
Invoke the Eiffel OO-Browser.
This allows browsing through Eiffel library and system class hierarchies.
With an optional prefix arg ENV-FILE equal to t, prompt for Environment file
to use.  Alternatively, a string value of ENV-FILE is used as the Environment
file name.  See also the file \"br-help\"." t nil)

;;;***

;;;### (autoloads nil "eif-calls" "oobr/eif-calls.el" (12376 19728))
;;; Generated autoloads from oobr/eif-calls.el

;;;***

;;;### (autoloads nil "eif-ise-er" "oobr/eif-ise-er.el" (12376 19729))
;;; Generated autoloads from oobr/eif-ise-er.el

;;;***

;;;### (autoloads nil "hash-test" "oobr/hash-test.el" (12376 19729))
;;; Generated autoloads from oobr/hash-test.el

;;;***

;;;### (autoloads nil "hasht" "oobr/hasht.el" (12558 61022))
;;; Generated autoloads from oobr/hasht.el

;;;***

;;;### (autoloads nil "hmouse-br" "oobr/hmouse-br.el" (12558 61023))
;;; Generated autoloads from oobr/hmouse-br.el

;;;### (autoloads (info-browse) "info-brows" "oobr/info-brows.el" (12558 61021))
;;; Generated autoloads from oobr/info-brows.el

(autoload 'info-browse "info-brows" "\
Invoke the Info OO-Browser.
This allows browsing through Info library and system class hierarchies.  With
an optional non-nil prefix argument ENV-FILE, prompt for Environment file to
use.  Alternatively, a string value of ENV-FILE is used as the Environment
file name.  See also the file \"br-help\"." t nil)

;;;***

;;;### (autoloads (java-browse) "java-brows" "oobr/java-brows.el" (12558 61028))
;;; Generated autoloads from oobr/java-brows.el

(autoload 'java-browse "java-brows" "\
Invoke the Java OO-Browser.
This allows browsing through Java library and system class hierarchies.  With
an optional non-nil prefix argument ENV-FILE, prompt for Environment file to
use.  Alternatively, a string value of ENV-FILE is used as the Environment
file name.  See also the file \"br-help\"." t nil)

;;;***

;;;### (autoloads (objc-browse) "objc-brows" "oobr/objc-brows.el" (12558 61024))
;;; Generated autoloads from oobr/objc-brows.el

(autoload 'objc-browse "objc-brows" "\
Invoke the Objective-C OO-Browser.
This allows browsing through Objective-C library and system class
hierarchies.  With an optional non-nil prefix argument ENV-FILE, prompt for
Environment file to use.  Alternatively, a string value of ENV-FILE is used
as the Environment file name.  See also the file \"br-help\"." t nil)

;;;***

;;;### (autoloads (python-browse) "python-browse" "oobr/python-browse.el" (12714 11823))
;;; Generated autoloads from oobr/python-browse.el

(autoload 'python-browse "python-browse" "\
Invoke the Python OO-Browser.
This allows browsing through Python library and system class hierarchies.
With an optional non-nil prefix argument ENV-FILE, prompt for Environment
file to use.  Alternatively, a string value of ENV-FILE is used as the
Environment file name.  See also the file \"br-help\"." t nil)

;;;***

;;;### (autoloads (smt-browse) "smt-browse" "oobr/smt-browse.el" (12558 61024))
;;; Generated autoloads from oobr/smt-browse.el

(autoload 'smt-browse "smt-browse" "\
Invoke the Smalltalk OO-Browser.
This allows browsing through Smalltalk library and system class hierarchies.
With an optional non-nil prefix argument ENV-FILE, prompt for Environment
file to use.  Alternatively, a string value of ENV-FILE is used as the
Environment file name.  See also the file \"br-help\"." t nil)

;;;***

;;;### (autoloads (add-log-current-defun change-log-mode add-change-log-entry-other-window add-change-log-entry find-change-log prompt-for-change-log-name) "add-log" "packages/add-log.el" (12558 60906))
;;; Generated autoloads from packages/add-log.el

(defvar change-log-default-name nil "\
*Name of a change log file for \\[add-change-log-entry].")

(defvar add-log-current-defun-function nil "\
*If non-nil, function to guess name of current function from surrounding text.
\\[add-change-log-entry] calls this function (if nil, `add-log-current-defun'
instead) with no arguments.  It returns a string or nil if it cannot guess.")

(defvar add-log-full-name nil "\
*Full name of user, for inclusion in ChangeLog daily headers.
This defaults to the value returned by the `user-full-name' function.")

(defvar add-log-mailing-address nil "\
*Electronic mail address of user, for inclusion in ChangeLog daily headers.
This defaults to the value of `user-mail-address'.")

(autoload 'prompt-for-change-log-name "add-log" "\
Prompt for a change log name." nil nil)

(autoload 'find-change-log "add-log" "\
Find a change log file for \\[add-change-log-entry] and return the name.
Optional arg FILE-NAME specifies the file to use.
If FILE-NAME is nil, use the value of `change-log-default-name'.
If 'change-log-default-name' is nil, behave as though it were 'ChangeLog'
\(or whatever we use on this operating system).

If 'change-log-default-name' contains a leading directory component, then
simply find it in the current directory.  Otherwise, search in the current 
directory and its successive parents for a file so named.

Once a file is found, `change-log-default-name' is set locally in the
current buffer to the complete file name." nil nil)

(autoload 'add-change-log-entry "add-log" "\
Find change log file and add an entry for today.
Optional arg (interactive prefix) non-nil means prompt for user name and site.
Second arg is file name of change log.  If nil, uses `change-log-default-name'.
Third arg OTHER-WINDOW non-nil means visit in other window.
Fourth arg NEW-ENTRY non-nil means always create a new entry at the front;
never append to an existing entry." t nil)

(autoload 'add-change-log-entry-other-window "add-log" "\
Find change log file in other window and add an entry for today.
Optional arg (interactive prefix) non-nil means prompt for user name and site.
Second arg is file name of change log.  If nil, uses `change-log-default-name'." t nil)

(define-key ctl-x-4-map "a" 'add-change-log-entry-other-window)

(autoload 'change-log-mode "add-log" "\
Major mode for editing change logs; like Indented Text Mode.
Prevents numeric backups and sets `left-margin' to 8 and `fill-column' to 74.
New log entries are usually made with \\[add-change-log-entry] or \\[add-change-log-entry-other-window].
Each entry behaves as a paragraph, and the entries for one day as a page.
Runs `change-log-mode-hook'." t nil)

(autoload 'add-log-current-defun "add-log" "\
Return name of function definition point is in, or nil.

Understands C, Lisp, LaTeX (\"functions\" are chapters, sections, ...),
Texinfo (@node titles), Perl, and Fortran.

Other modes are handled by a heuristic that looks in the 10K before
point for uppercase headings starting in the first column or
identifiers followed by `:' or `=', see variable
`add-log-current-defun-header-regexp'.

Has a preference of looking backwards." nil nil)

;;;***

;;;### (autoloads (apropos-documentation apropos-value apropos apropos-command) "apropos" "packages/apropos.el" (12740 25203))
;;; Generated autoloads from packages/apropos.el

(fset 'command-apropos 'apropos-command)

(autoload 'apropos-command "apropos" "\
Shows commands (interactively callable functions) that match REGEXP.
With optional prefix ARG or if `apropos-do-all' is non-nil, also show
variables." t nil)

(autoload 'apropos "apropos" "\
Show all bound symbols whose names match REGEXP.
With optional prefix ARG or if `apropos-do-all' is non-nil, also show unbound
symbols and key bindings, which is a little more time-consuming.
Returns list of symbols and documentation found." t nil)

(autoload 'apropos-value "apropos" "\
Show all symbols whose value's printed image matches REGEXP.
With optional prefix ARG or if `apropos-do-all' is non-nil, also looks
at the function and at the names and values of properties.
Returns list of symbols and values found." t nil)

(autoload 'apropos-documentation "apropos" "\
Show symbols whose documentation contain matches for REGEXP.
With optional prefix ARG or if `apropos-do-all' is non-nil, also use
documentation that is not stored in the documentation file and show key
bindings.
Returns list of symbols and documentation found." t nil)

;;;***

;;;### (autoloads nil "array" "packages/array.el" (12547 2408))
;;; Generated autoloads from packages/array.el

;;;### (autoloads nil "auto-save" "packages/auto-save.el" (12376 19476))
;;; Generated autoloads from packages/auto-save.el

;;;### (autoloads (define-auto-insert auto-insert) "autoinsert" "packages/autoinsert.el" (12546 50457))
;;; Generated autoloads from packages/autoinsert.el

(autoload 'auto-insert "autoinsert" "\
Insert default contents into a new file if `auto-insert' is non-nil.
Matches the visited file name against the elements of `auto-insert-alist'." t nil)

(autoload 'define-auto-insert "autoinsert" "\
Associate CONDITION with (additional) ACTION in `auto-insert-alist'.
Optional AFTER means to insert action after all existing actions for CONDITION,
or if CONDITION had no actions, after all other CONDITIONs." nil nil)

;;;***

;;;### (autoloads nil "avoid" "packages/avoid.el" (12558 60916))
;;; Generated autoloads from packages/avoid.el

(add-minor-mode 'mouse-avoidance-mode " Avoid")

;;;***

;;;### (autoloads nil "balloon-help" "packages/balloon-help.el" (12547 2421))
;;; Generated autoloads from packages/balloon-help.el

;;;### (autoloads nil "big-menubar" "packages/big-menubar.el" (12376 19509))
;;; Generated autoloads from packages/big-menubar.el

;;;### (autoloads (blink-cursor-mode) "blink-cursor" "packages/blink-cursor.el" (12639 8850))
;;; Generated autoloads from packages/blink-cursor.el

(autoload 'blink-cursor-mode "blink-cursor" "\
Enable or disable a blinking cursor.
If TIMEOUT is nil, toggle on or off.
If TIMEOUT is t, enable with the previous timeout value.
If TIMEOUT is 0, disable.
If TIMEOUT is greater than 0, then the cursor will blink once
each TIMEOUT secs (can be a float)." t nil)

;;;***

;;;### (autoloads nil "blink-paren" "packages/blink-paren.el" (12376 19480))
;;; Generated autoloads from packages/blink-paren.el

;;;### (autoloads (bookmark-menu-delete bookmark-menu-rename bookmark-menu-locate bookmark-menu-jump bookmark-menu-insert bookmark-bmenu-list bookmark-load bookmark-save bookmark-write bookmark-delete bookmark-insert bookmark-rename bookmark-insert-location bookmark-relocate bookmark-jump bookmark-set) "bookmark" "packages/bookmark.el" (12675 57084))
;;; Generated autoloads from packages/bookmark.el

(if (symbolp (key-binding "r")) nil (progn (define-key ctl-x-map "rb" 'bookmark-jump) (define-key ctl-x-map "rm" 'bookmark-set) (define-key ctl-x-map "rl" 'bookmark-bmenu-list)))

(defvar bookmark-map nil "\
Keymap containing bindings to bookmark functions.
It is not bound to any key by default: to bind it
so that you have a bookmark prefix, just use `global-set-key' and bind a
key of your choice to `bookmark-map'.  All interactive bookmark
functions have a binding in this keymap.")

(define-prefix-command 'bookmark-map)

(define-key bookmark-map "x" 'bookmark-set)

(define-key bookmark-map "m" 'bookmark-set)

(define-key bookmark-map "j" 'bookmark-jump)

(define-key bookmark-map "g" 'bookmark-jump)

(define-key bookmark-map "i" 'bookmark-insert)

(define-key bookmark-map "e" 'edit-bookmarks)

(define-key bookmark-map "f" 'bookmark-insert-location)

(define-key bookmark-map "r" 'bookmark-rename)

(define-key bookmark-map "d" 'bookmark-delete)

(define-key bookmark-map "l" 'bookmark-load)

(define-key bookmark-map "w" 'bookmark-write)

(define-key bookmark-map "s" 'bookmark-save)

(autoload 'bookmark-set "bookmark" "\
Set a bookmark named NAME inside a file.
If name is nil, then the user will be prompted.
With prefix arg, will not overwrite a bookmark that has the same name
as NAME if such a bookmark already exists, but instead will \"push\"
the new bookmark onto the bookmark alist.  Thus the most recently set
bookmark with name NAME would be the one in effect at any given time,
but the others are still there, should you decide to delete the most
recent one.

To yank words from the text of the buffer and use them as part of the
bookmark name, type C-w while setting a bookmark.  Successive C-w's
yank successive words.

Typing C-u inserts the name of the last bookmark used in the buffer
\(as an aid in using a single bookmark name to track your progress
through a large file).  If no bookmark was used, then C-u inserts the
name of the file being visited.

Use \\[bookmark-delete] to remove bookmarks (you give it a name,
and it removes only the first instance of a bookmark with that name from
the list of bookmarks.)" t nil)

(autoload 'bookmark-jump "bookmark" "\
Jump to bookmark BOOKMARK (a point in some file).  
You may have a problem using this function if the value of variable
`bookmark-alist' is nil.  If that happens, you need to load in some
bookmarks.  See help on function `bookmark-load' for more about
this.

If the file pointed to by BOOKMARK no longer exists, you will be asked
if you wish to give the bookmark a new location, and bookmark-jump
will then jump to the new location, as well as recording it in place
of the old one in the permanent bookmark record." t nil)

(autoload 'bookmark-relocate "bookmark" "\
Relocate BOOKMARK -- prompts for a filename, and makes an already
existing bookmark point to that file, instead of the one it used to
point at.  Useful when a file has been renamed after a bookmark was
set in it." t nil)

(autoload 'bookmark-insert-location "bookmark" "\
Insert the name of the file associated with BOOKMARK.
Optional second arg NO-HISTORY means don't record this in the
minibuffer history list `bookmark-history'." t nil)

(autoload 'bookmark-rename "bookmark" "\
Change the name of OLD bookmark to NEW name.  If called from
keyboard, prompts for OLD and NEW.  If called from menubar, OLD is
selected from a menu, and prompts for NEW.

If called from Lisp, prompts for NEW if only OLD was passed as an
argument.  If called with two strings, then no prompting is done.  You
must pass at least OLD when calling from Lisp.

While you are entering the new name, consecutive C-w's insert
consectutive words from the text of the buffer into the new bookmark
name." t nil)

(autoload 'bookmark-insert "bookmark" "\
Insert the text of the file pointed to by bookmark BOOKMARK.  
You may have a problem using this function if the value of variable
`bookmark-alist' is nil.  If that happens, you need to load in some
bookmarks.  See help on function `bookmark-load' for more about
this." t nil)

(autoload 'bookmark-delete "bookmark" "\
Delete BOOKMARK from the bookmark list.  
Removes only the first instance of a bookmark with that name.  If
there are one or more other bookmarks with the same name, they will
not be deleted.  Defaults to the \"current\" bookmark (that is, the
one most recently used in this file, if any).
Optional second arg BATCH means don't update the bookmark list buffer,
probably because we were called from there." t nil)

(autoload 'bookmark-write "bookmark" "\
Write bookmarks to a file (for which the user will be prompted
interactively).  Don't use this in Lisp programs; use bookmark-save
instead." t nil)

(autoload 'bookmark-save "bookmark" "\
Save currently defined bookmarks.
Saves by default in the file defined by the variable
`bookmark-default-file'.  With a prefix arg, save it in file FILE
\(second argument).

If you are calling this from Lisp, the two arguments are PREFIX-ARG
and FILE, and if you just want it to write to the default file, then
pass no arguments.  Or pass in nil and FILE, and it will save in FILE
instead.  If you pass in one argument, and it is non-nil, then the
user will be interactively queried for a file to save in.

When you want to load in the bookmarks from a file, use
`bookmark-load', \\[bookmark-load].  That function will prompt you
for a file, defaulting to the file defined by variable
`bookmark-default-file'." t nil)

(autoload 'bookmark-load "bookmark" "\
Load bookmarks from FILE (which must be in bookmark format).
Appends loaded bookmarks to the front of the list of bookmarks.  If
optional second argument REVERT is non-nil, existing bookmarks are
destroyed.  Optional third arg NO-MSG means don't display any messages
while loading.

If you load a file that doesn't contain a proper bookmark alist, you
will corrupt Emacs's bookmark list.  Generally, you should only load
in files that were created with the bookmark functions in the first
place.  Your own personal bookmark file, `~/.emacs.bmk', is
maintained automatically by Emacs; you shouldn't need to load it
explicitly." t nil)

(autoload 'bookmark-bmenu-list "bookmark" "\
Display a list of existing bookmarks.
The list is displayed in a buffer named `*Bookmark List*'.
The leftmost column displays a D if the bookmark is flagged for
deletion, or > if it is flagged for displaying." t nil)

(defalias 'list-bookmarks 'bookmark-bmenu-list)

(defalias 'edit-bookmarks 'bookmark-bmenu-list)

(autoload 'bookmark-menu-insert "bookmark" "\
Insert the text of the file pointed to by bookmark BOOKMARK.  
You may have a problem using this function if the value of variable
`bookmark-alist' is nil.  If that happens, you need to load in some
bookmarks.  See help on function `bookmark-load' for more about
this.

Warning: this function only takes an EVENT as argument.  Use the
corresponding bookmark function from Lisp (the one without the
\"-menu-\" in its name)." t nil)

(autoload 'bookmark-menu-jump "bookmark" "\
Jump to bookmark BOOKMARK (a point in some file).  
You may have a problem using this function if the value of variable
`bookmark-alist' is nil.  If that happens, you need to load in some
bookmarks.  See help on function `bookmark-load' for more about
this.

Warning: this function only takes an EVENT as argument.  Use the
corresponding bookmark function from Lisp (the one without the
\"-menu-\" in its name)." t nil)

(autoload 'bookmark-menu-locate "bookmark" "\
Insert the name of the file associated with BOOKMARK. 
\(This is not the same as the contents of that file).

Warning: this function only takes an EVENT as argument.  Use the
corresponding bookmark function from Lisp (the one without the
\"-menu-\" in its name)." t nil)

(autoload 'bookmark-menu-rename "bookmark" "\
Change the name of OLD-BOOKMARK to NEWNAME.  
If called from keyboard, prompts for OLD-BOOKMARK and NEWNAME.
If called from menubar, OLD-BOOKMARK is selected from a menu, and
prompts for NEWNAME. 
If called from Lisp, prompts for NEWNAME if only OLD-BOOKMARK was
passed as an argument.  If called with two strings, then no prompting
is done.  You must pass at least OLD-BOOKMARK when calling from Lisp.

While you are entering the new name, consecutive C-w's insert
consectutive words from the text of the buffer into the new bookmark
name.

Warning: this function only takes an EVENT as argument.  Use the
corresponding bookmark function from Lisp (the one without the
\"-menu-\" in its name)." t nil)

(autoload 'bookmark-menu-delete "bookmark" "\
Delete the bookmark named NAME from the bookmark list.  
Removes only the first instance of a bookmark with that name.  If
there are one or more other bookmarks with the same name, they will
not be deleted.  Defaults to the \"current\" bookmark (that is, the
one most recently used in this file, if any).

Warning: this function only takes an EVENT as argument.  Use the
corresponding bookmark function from Lisp (the one without the
\"-menu-\" in its name)." t nil)

;;;***

;;;### (autoloads nil "buff-menu" "packages/buff-menu.el" (12731 19066))
;;; Generated autoloads from packages/buff-menu.el

;;;### (autoloads (command-history-mode list-command-history repeat-matching-complex-command) "chistory" "packages/chistory.el" (12546 50458))
;;; Generated autoloads from packages/chistory.el

(autoload 'repeat-matching-complex-command "chistory" "\
Edit and re-evaluate complex command with name matching PATTERN.
Matching occurrences are displayed, most recent first, until you
select a form for evaluation.  If PATTERN is empty (or nil), every form
in the command history is offered.  The form is placed in the minibuffer
for editing and the result is evaluated." t nil)

(autoload 'list-command-history "chistory" "\
List history of commands typed to minibuffer.
The number of commands listed is controlled by `list-command-history-max'.
Calls value of `list-command-history-filter' (if non-nil) on each history
element to judge if that element should be excluded from the list.

The buffer is left in Command History mode." t nil)

(autoload 'command-history-mode "chistory" "\
Major mode for examining commands from `command-history'.
The number of commands listed is controlled by `list-command-history-max'.
The command history is filtered by `list-command-history-filter' if non-nil.
Use \\<command-history-map>\\[command-history-repeat] to repeat the command on the current line.

Otherwise much like Emacs-Lisp Mode except that there is no self-insertion
and digits provide prefix arguments.  Tab does not indent.
\\{command-history-map}
Calls the value of `command-history-hook' if that is non-nil.
The Command History listing is recomputed each time this mode is invoked." t nil)

;;;***

;;;### (autoloads nil "cmuscheme" "packages/cmuscheme.el" (12546 63751))
;;; Generated autoloads from packages/cmuscheme.el

(add-hook 'same-window-buffer-names "*scheme*")

;;;***

;;;### (autoloads (display-column-mode) "column" "packages/column.el" (12657 40702))
;;; Generated autoloads from packages/column.el

(autoload 'display-column-mode "column" "\
Toggle display column mode.
With prefix arg, turn display column mode on iff arg is positive.

When display column mode is on, the current column and line number are
displayed in the mode line." t nil)

;;;***

;;;### (autoloads (compare-windows) "compare-w" "packages/compare-w.el" (12547 2409))
;;; Generated autoloads from packages/compare-w.el

(autoload 'compare-windows "compare-w" "\
Compare text in current window with text in next window.
Compares the text starting at point in each window,
moving over text in each one as far as they match.

This command pushes the mark in each window
at the prior location of point in that window.
If both windows display the same buffer,
the mark is pushed twice in that buffer:
first in the other window, then in the selected window.

A prefix arg means ignore changes in whitespace.
The variable `compare-windows-whitespace' controls how whitespace is skipped.
If `compare-ignore-case' is non-nil, changes in case are also ignored." t nil)

;;;***

;;;### (autoloads (first-error previous-error next-error compilation-minor-mode grep compile) "compile" "packages/compile.el" (12743 22077))
;;; Generated autoloads from packages/compile.el

(defvar compilation-mode-hook nil "\
*List of hook functions run by `compilation-mode' (see `run-hooks').")

(defvar compilation-window-height nil "\
*Number of lines in a compilation window.  If nil, use Emacs default.")

(defvar compilation-buffer-name-function nil "\
Function to compute the name of a compilation buffer.
The function receives one argument, the name of the major mode of the
compilation buffer.  It should return a string.
nil means compute the name with `(concat \"*\" (downcase major-mode) \"*\")'.")

(defvar compilation-finish-function nil "\
*Function to call when a compilation process finishes.
It is called with two arguments: the compilation buffer, and a string
describing how the process finished.")

(defvar compilation-search-path '(nil) "\
*List of directories to search for source files named in error messages.
Elements should be directory names, not file names of directories.
nil as an element means to try the default directory.")

(autoload 'compile "compile" "\
Compile the program including the current buffer.  Default: run `make'.
Runs COMMAND, a shell command, in a separate process asynchronously
with output going to the buffer `*compilation*'.

You can then use the command \\[next-error] to find the next error message
and move to the source code that caused it.

Interactively, prompts for the command if `compilation-read-command' is
non-nil; otherwise uses `compile-command'.  With prefix arg, always prompts.

To run more than one compilation at once, start one and rename the
`*compilation*' buffer to some other name with \\[rename-buffer].
Then start the next one.

The name used for the buffer is actually whatever is returned by
the function in `compilation-buffer-name-function', so you can set that
to a function that generates a unique name." t nil)

(autoload 'grep "compile" "\
Run grep, with user-specified args, and collect output in a buffer.
While grep runs asynchronously, you can use the \\[next-error] command
to find the text that grep hits refer to.

This command uses a special history list for its arguments, so you can
easily repeat a grep command." t nil)

(autoload 'compilation-minor-mode "compile" "\
Toggle compilation minor mode.
With arg, turn compilation mode on if and only if arg is positive.
See `compilation-mode'.
! \\{compilation-mode-map}" t nil)

(autoload 'next-error "compile" "\
Visit next compilation error message and corresponding source code.
This operates on the output from the \\[compile] command.
If all preparsed error messages have been processed,
the error message buffer is checked for new ones.

A prefix arg specifies how many error messages to move;
negative means move back to previous error messages.
Just C-u as a prefix means reparse the error message buffer
and start at the first error.

\\[next-error] normally applies to the most recent compilation started,
but as long as you are in the middle of parsing errors from one compilation
output buffer, you stay with that compilation output buffer.

Use \\[next-error] in a compilation output buffer to switch to
processing errors from that compilation.

See variables `compilation-parse-errors-function' and
`compilation-error-regexp-alist' for customization ideas." t nil)

(autoload 'previous-error "compile" "\
Visit previous compilation error message and corresponding source code.
This operates on the output from the \\[compile] command." t nil)

(autoload 'first-error "compile" "\
Reparse the error message buffer and start at the first error
Visit corresponding source code.
This operates on the output from the \\[compile] command." t nil)

(define-key ctl-x-map "`" 'next-error)

;;;***

;;;### (autoloads nil "completion" "packages/completion.el" (12547 2470))
;;; Generated autoloads from packages/completion.el

;;;***

;;;### (autoloads nil "crypt" "packages/crypt.el" (12743 22078))
;;; Generated autoloads from packages/crypt.el

;;;### (autoloads (dabbrev-expand dabbrev-completion) "dabbrev" "packages/dabbrev.el" (12547 2413))
;;; Generated autoloads from packages/dabbrev.el

(define-key global-map [(meta /)] 'dabbrev-expand)

(define-key global-map [(meta control /)] 'dabbrev-completion)

(autoload 'dabbrev-completion "dabbrev" "\
Completion on current word.
Like \\[dabbrev-expand] but finds all expansions in the current buffer
and presents suggestions for completion.

With a prefix argument, it searches all buffers accepted by the
function pointed out by `dabbrev-friend-buffer-function' to find the
completions.

If the prefix argument is 16 (which comes from C-u C-u),
then it searches *all* buffers.

With no prefix argument, it reuses an old completion list
if there is a suitable one already." t nil)

(autoload 'dabbrev-expand "dabbrev" "\
Expand previous word \"dynamically\".

Expands to the most recent, preceding word for which this is a prefix.
If no suitable preceding word is found, words following point are
considered.  If still no suitable word is found, then look in the
buffers accepted by the function pointed out by variable
`dabbrev-friend-buffer-function'.

A positive prefix argument, N, says to take the Nth backward *distinct*
possibility.  A negative argument says search forward.

If the cursor has not moved from the end of the previous expansion and
no argument is given, replace the previously-made expansion
with the next possible expansion not yet tried.

The variable `dabbrev-backward-only' may be used to limit the
direction of search to backward if set non-nil.

See also `dabbrev-abbrev-char-regexp' and \\[dabbrev-completion]." t nil)

;;;***

;;;### (autoloads nil "desktop" "packages/desktop.el" (12547 2459))
;;; Generated autoloads from packages/desktop.el

;;;### (autoloads nil "detexinfo" "packages/detexinfo.el" (12376 19504))
;;; Generated autoloads from packages/detexinfo.el

;;;### (autoloads (diff-backup diff) "diff" "packages/diff.el" (12547 2411))
;;; Generated autoloads from packages/diff.el

(defvar diff-switches (purecopy "-c") "\
*A string or list of strings specifying switches to be be passed to diff.")

(autoload 'diff "diff" "\
Find and display the differences between OLD and NEW files.
Interactively the current buffer's file name is the default for NEW
and a backup file for NEW is the default for OLD.
With prefix arg, prompt for diff switches." t nil)

(autoload 'diff-backup "diff" "\
Diff this file with its backup file or vice versa.
Uses the latest backup, if there are several numerical backups.
If this file is a backup, diff it with its original.
The backup file is the first file given to `diff'." t nil)

;;;***

;;;### (autoloads (ediff-documentation ediff-version ediff-revision ediff-patch-buffer ediff-patch-file run-ediff-from-cvs-buffer ediff-merge-revisions-with-ancestor ediff-merge-revisions ediff-merge-buffers-with-ancestor ediff-merge-buffers ediff-merge-files-with-ancestor ediff-merge-files ediff-regions-linewise ediff-regions-wordwise ediff-windows-linewise ediff-windows-wordwise ediff-merge-directory-revisions-with-ancestor ediff-merge-directory-revisions ediff-merge-directories-with-ancestor ediff-merge-directories ediff-directories3 ediff-directory-revisions ediff-directories ediff-buffers3 ediff-buffers ediff-files3 ediff-files) "ediff" "ediff/ediff.el" (12740 25250))
;;; Generated autoloads from ediff/ediff.el

(autoload 'ediff-files "ediff" "\
Run Ediff on a pair of files, FILE-A and FILE-B." t nil)

(autoload 'ediff-files3 "ediff" "\
Run Ediff on three files, FILE-A, FILE-B, and FILE-C." t nil)

(defalias 'ediff3 'ediff-files3)

(defalias 'ediff 'ediff-files)

(autoload 'ediff-buffers "ediff" "\
Run Ediff on a pair of buffers, BUFFER-A and BUFFER-B." t nil)

(defalias 'ebuffers 'ediff-buffers)

(autoload 'ediff-buffers3 "ediff" "\
Run Ediff on three buffers, BUFFER-A, BUFFER-B, and BUFFER-C." t nil)

(defalias 'ebuffers3 'ediff-buffers3)

(autoload 'ediff-directories "ediff" "\
Run Ediff on a pair of directories, DIR1 and DIR2, comparing files that have
the same name in both. The third argument, REGEXP, is a regular expression that
can be used to filter out certain file names." t nil)

(defalias 'edirs 'ediff-directories)

(autoload 'ediff-directory-revisions "ediff" "\
Run Ediff on a directory, DIR1, comparing its files with their revisions.
The second argument, REGEXP, is a regular expression that filters the file
names. Only the files that are under revision control are taken into account." t nil)

(defalias 'edir-revisions 'ediff-directory-revisions)

(autoload 'ediff-directories3 "ediff" "\
Run Ediff on three directories, DIR1, DIR2, and DIR3, comparing files that
have the same name in all three. The last argument, REGEXP, is a regular
expression that can be used to filter out certain file names." t nil)

(defalias 'edirs3 'ediff-directories3)

(autoload 'ediff-merge-directories "ediff" "\
Run Ediff on a pair of directories, DIR1 and DIR2, merging files that have
the same name in both. The third argument, REGEXP, is a regular expression that
can be used to filter out certain file names." t nil)

(defalias 'edirs-merge 'ediff-merge-directories)

(autoload 'ediff-merge-directories-with-ancestor "ediff" "\
Merge files in directories DIR1 and DIR2 using files in ANCESTOR-DIR as ancestors.
Ediff merges files that have identical names in DIR1, DIR2. If a pair of files
in DIR1 and DIR2 doesn't have an ancestor in ANCESTOR-DIR, Ediff will merge
without ancestor. The fourth argument, REGEXP, is a regular expression that
can be used to filter out certain file names." t nil)

(autoload 'ediff-merge-directory-revisions "ediff" "\
Run Ediff on a directory, DIR1, merging its files with their revisions.
The second argument, REGEXP, is a regular expression that filters the file
names. Only the files that are under revision control are taken into account." t nil)

(defalias 'edir-merge-revisions 'ediff-merge-directory-revisions)

(autoload 'ediff-merge-directory-revisions-with-ancestor "ediff" "\
Run Ediff on a directory, DIR1, merging its files with their revisions and ancestors.
The second argument, REGEXP, is a regular expression that filters the file
names. Only the files that are under revision control are taken into account." t nil)

(defalias 'edir-merge-revisions-with-ancestor 'ediff-merge-directory-revisions-with-ancestor)

(defalias 'edirs-merge-with-ancestor 'ediff-merge-directories-with-ancestor)

(autoload 'ediff-windows-wordwise "ediff" "\
Compare WIND-A and WIND-B, which are selected by clicking, wordwise.
With prefix argument, DUMB-MODE, or on a non-windowing display, works as
follows:
If WIND-A is nil, use selected window.
If WIND-B is nil, use window next to WIND-A." t nil)

(autoload 'ediff-windows-linewise "ediff" "\
Compare WIND-A and WIND-B, which are selected by clicking, linewise.
With prefix argument, DUMB-MODE, or on a non-windowing display, works as
follows:
If WIND-A is nil, use selected window.
If WIND-B is nil, use window next to WIND-A." t nil)

(autoload 'ediff-regions-wordwise "ediff" "\
Run Ediff on a pair of regions in two different buffers.
Regions (i.e., point and mark) are assumed to be set in advance.
This function is effective only for relatively small regions, up to 200
lines. For large regions, use `ediff-regions-linewise'." t nil)

(autoload 'ediff-regions-linewise "ediff" "\
Run Ediff on a pair of regions in two different buffers.
Regions (i.e., point and mark) are assumed to be set in advance.
Each region is enlarged to contain full lines.
This function is effective for large regions, over 100-200
lines. For small regions, use `ediff-regions-wordwise'." t nil)

(defalias 'ediff-merge 'ediff-merge-files)

(autoload 'ediff-merge-files "ediff" "\
Merge two files without ancestor." t nil)

(autoload 'ediff-merge-files-with-ancestor "ediff" "\
Merge two files with ancestor." t nil)

(defalias 'ediff-merge-with-ancestor 'ediff-merge-files-with-ancestor)

(autoload 'ediff-merge-buffers "ediff" "\
Merge buffers without ancestor." t nil)

(autoload 'ediff-merge-buffers-with-ancestor "ediff" "\
Merge buffers with ancestor." t nil)

(autoload 'ediff-merge-revisions "ediff" "\
Run Ediff by merging two revisions of a file.
The file is the optional FILE argument or the file visited by the current
buffer." t nil)

(autoload 'ediff-merge-revisions-with-ancestor "ediff" "\
Run Ediff by merging two revisions of a file with a common ancestor.
The file is the the optional FILE argument or the file visited by the current
buffer." t nil)

(autoload 'run-ediff-from-cvs-buffer "ediff" "\
Run Ediff-merge on appropriate revisions of the selected file.
First run after `M-x cvs-update'. Then place the cursor on a lide describing a
file and then run `run-ediff-from-cvs-buffer'." t nil)

(autoload 'ediff-patch-file "ediff" "\
Run Ediff by patching SOURCE-FILENAME." t nil)

(autoload 'ediff-patch-buffer "ediff" "\
Run Ediff by patching BUFFER-NAME." t nil)

(defalias 'epatch 'ediff-patch-file)

(defalias 'epatch-buffer 'ediff-patch-buffer)

(autoload 'ediff-revision "ediff" "\
Run Ediff by comparing versions of a file.
The file is an optional FILE argument or the file visited by the current
buffer. Use `vc.el' or `rcs.el' depending on `ediff-version-control-package'." t nil)

(autoload 'ediff-version "ediff" "\
Return string describing the version of Ediff.
When called interactively, displays the version." t nil)

(autoload 'ediff-documentation "ediff" "\
Display Ediff's manual." t nil)

;;;***

;;;### (autoloads nil "doctex" "packages/doctex.el" (12376 19483))
;;; Generated autoloads from packages/doctex.el

;;;### (autoloads (edit-faces) "edit-faces" "packages/edit-faces.el" (12743 11607))
;;; Generated autoloads from packages/edit-faces.el

(autoload 'edit-faces "edit-faces" "\
Alter face characteristics by editing a list of defined faces.
Pops up a buffer containing a list of defined faces.

Editing commands:

\\{edit-faces-mode-map}" t nil)

;;;***

;;;### (autoloads (report-emacs-bug) "emacsbug" "packages/emacsbug.el" (12547 2416))
;;; Generated autoloads from packages/emacsbug.el

(autoload 'report-emacs-bug "emacsbug" "\
Report a bug in GNU Emacs.
Prompts for bug subject.  Leaves you in a mail buffer." t nil)

;;;***

;;;### (autoloads (emerge-merge-directories emerge-revisions-with-ancestor emerge-revisions emerge-files-with-ancestor-remote emerge-files-remote emerge-files-with-ancestor-command emerge-files-command emerge-buffers-with-ancestor emerge-buffers emerge-files-with-ancestor emerge-files) "emerge" "packages/emerge.el" (12547 2420))
;;; Generated autoloads from packages/emerge.el

(autoload 'emerge-files "emerge" "\
Run Emerge on two files." t nil)

(fset 'emerge 'emerge-files)

(autoload 'emerge-files-with-ancestor "emerge" "\
Run Emerge on two files, giving another file as the ancestor." t nil)

(autoload 'emerge-buffers "emerge" "\
Run Emerge on two buffers." t nil)

(autoload 'emerge-buffers-with-ancestor "emerge" "\
Run Emerge on two buffers, giving another buffer as the ancestor." t nil)

(autoload 'emerge-files-command "emerge" nil nil nil)

(autoload 'emerge-files-with-ancestor-command "emerge" nil nil nil)

(autoload 'emerge-files-remote "emerge" nil nil nil)

(autoload 'emerge-files-with-ancestor-remote "emerge" nil nil nil)

(autoload 'emerge-revisions "emerge" "\
Emerge two RCS revisions of a file." t nil)

(autoload 'emerge-revisions-with-ancestor "emerge" "\
Emerge two RCS revisions of a file, with another revision as ancestor." t nil)

(autoload 'emerge-merge-directories "emerge" nil t nil)

;;;***

;;;### (autoloads (tags-apropos list-tags tags-query-replace tags-search tags-loop-continue next-file find-tag-other-window find-tag visit-tags-table) "etags" "packages/etags.el" (12743 22077))
;;; Generated autoloads from packages/etags.el

(defvar tags-build-completion-table 'ask "\
*If this variable is nil, then tags completion is disabled.
If this variable is t, then things which prompt for tags will do so with 
 completion across all known tags.
If this variable is the symbol `ask', then you will be asked whether each
 tags table should be added to the completion list as it is read in.
 (With the exception that for very small tags tables, you will not be asked,
 since they can be parsed quickly.)")

(defvar tags-always-exact nil "\
*If this variable is non-nil, then tags always looks for exact matches.")

(defvar tag-table-alist nil "\
*A list which determines which tags files should be active for a 
given buffer.  This is not really an association list, in that all 
elements are checked.  The CAR of each element of this list is a 
pattern against which the buffer's file name is compared; if it 
matches, then the CDR of the list should be the name of the tags
table to use.  If more than one element of this list matches the
buffer's file name, then all of the associated tags tables will be
used.  Earlier ones will be searched first.

If the CAR of elements of this list are strings, then they are treated
as regular-expressions against which the file is compared (like the
auto-mode-alist).  If they are not strings, then they are evaluated.
If they evaluate to non-nil, then the current buffer is considered to
match.

If the CDR of the elements of this list are strings, then they are
assumed to name a TAGS file.  If they name a directory, then the string
\"TAGS\" is appended to them to get the file name.  If they are not 
strings, then they are evaluated, and must return an appropriate string.

For example:
  (setq tag-table-alist
	'((\"/usr/src/public/perl/\" . \"/usr/src/public/perl/perl-3.0/\")
	 (\"\\\\.el$\" . \"/usr/local/emacs/src/\")
	 (\"/jbw/gnu/\" . \"/usr15/degree/stud/jbw/gnu/\")
	 (\"\" . \"/usr/local/emacs/src/\")
	 ))

This means that anything in the /usr/src/public/perl/ directory should use
the TAGS file /usr/src/public/perl/perl-3.0/TAGS; and file ending in .el should
use the TAGS file /usr/local/emacs/src/TAGS; and anything in or below the
directory /jbw/gnu/ should use the TAGS file /usr15/degree/stud/jbw/gnu/TAGS.
A file called something like \"/usr/jbw/foo.el\" would use both the TAGS files
/usr/local/emacs/src/TAGS and /usr15/degree/stud/jbw/gnu/TAGS (in that order)
because it matches both patterns.

If the buffer-local variable `buffer-tag-table' is set, then it names a tags
table that is searched before all others when find-tag is executed from this
buffer.

If there is a file called \"TAGS\" in the same directory as the file in 
question, then that tags file will always be used as well (after the
`buffer-tag-table' but before the tables specified by this list.)

If the variable tags-file-name is set, then the tags file it names will apply
to all buffers (for backwards compatibility.)  It is searched first.
")

(autoload 'visit-tags-table "etags" "\
Tell tags commands to use tags table file FILE first.
FILE should be the name of a file created with the `etags' program.
A directory name is ok too; it means file TAGS in that directory." t nil)

(autoload 'find-tag "etags" "\
*Find tag whose name contains TAGNAME.
 Selects the buffer that the tag is contained in
and puts point at its definition.
 If TAGNAME is a null string, the expression in the buffer
around or before point is used as the tag name.
 If called interactively with a numeric argument, searches for the next tag
in the tag table that matches the tagname used in the previous find-tag.
 If second arg OTHER-WINDOW is non-nil, uses another window to display
the tag.

This version of this function supports multiple active tags tables,
and completion.

Variables of note:

  tag-table-alist		controls which tables apply to which buffers
  tags-file-name		a default tags table
  tags-build-completion-table   controls completion behavior
  buffer-tag-table		another way of specifying a buffer-local table
  make-tags-files-invisible	whether tags tables should be very hidden
  tag-mark-stack-max		how many tags-based hops to remember" t nil)

(autoload 'find-tag-other-window "etags" "\
*Find tag whose name contains TAGNAME.
 Selects the buffer that the tag is contained in in another window
and puts point at its definition.
 If TAGNAME is a null string, the expression in the buffer
around or before point is used as the tag name.
 If second arg NEXT is non-nil (interactively, with prefix arg),
searches for the next tag in the tag table
that matches the tagname used in the previous find-tag.

This version of this function supports multiple active tags tables,
and completion.

Variables of note:

  tag-table-alist		controls which tables apply to which buffers
  tags-file-name		a default tags table
  tags-build-completion-table   controls completion behavior
  buffer-tag-table		another way of specifying a buffer-local table
  make-tags-files-invisible	whether tags tables should be very hidden
  tag-mark-stack-max		how many tags-based hops to remember" t nil)

(autoload 'next-file "etags" "\
Select next file among files in current tag table(s).

A first argument of t (prefix arg, if interactive) initializes to the
beginning of the list of files in the (first) tags table.  If the argument
is neither nil nor t, it is evalled to initialize the list of files.

Non-nil second argument NOVISIT means use a temporary buffer
to save time and avoid uninteresting warnings.

Value is nil if the file was already visited;
if the file was newly read in, the value is the filename." t nil)

(autoload 'tags-loop-continue "etags" "\
Continue last \\[tags-search] or \\[tags-query-replace] command.
Used noninteractively with non-nil argument to begin such a command (the
argument is passed to `next-file', which see).
Two variables control the processing we do on each file:
the value of `tags-loop-scan' is a form to be executed on each file
to see if it is interesting (it returns non-nil if so)
and `tags-loop-operate' is a form to execute to operate on an interesting file
If the latter returns non-nil, we exit; otherwise we scan the next file." t nil)

(autoload 'tags-search "etags" "\
Search through all files listed in tags table for match for REGEXP.
Stops when a match is found.
To continue searching for next match, use command \\[tags-loop-continue].

See documentation of variable `tag-table-alist'." t nil)

(autoload 'tags-query-replace "etags" "\
Query-replace-regexp FROM with TO through all files listed in tags table.
Third arg DELIMITED (prefix arg) means replace only word-delimited matches.
If you exit (\\[keyboard-quit] or ESC), you can resume the query-replace
with the command \\[tags-loop-continue].

See documentation of variable `tag-table-alist'." t nil)

(autoload 'list-tags "etags" "\
Display list of tags in file FILE.
FILE should not contain a directory spec
unless it has one in the tag table." t nil)

(autoload 'tags-apropos "etags" "\
Display list of all tags in tag table REGEXP matches." t nil)

;;;***

;;;### (autoloads nil "fa-extras" "packages/fa-extras.el" (12547 2465))
;;; Generated autoloads from packages/fa-extras.el

;;;### (autoloads (turn-on-fast-lock fast-lock-mode) "fast-lock" "packages/fast-lock.el" (12558 60915))
;;; Generated autoloads from packages/fast-lock.el

(defvar fast-lock-mode nil)

(autoload 'fast-lock-mode "fast-lock" "\
Toggle Fast Lock mode.
With arg, turn Fast Lock mode on if and only if arg is positive and the buffer
is associated with a file.  Enable it automatically in your `~/.emacs' by:

 (add-hook 'font-lock-mode-hook 'turn-on-fast-lock)

If Fast Lock mode is enabled, and the current buffer does not contain any text
properties, any associated Font Lock cache is used if its timestamp matches the
buffer's file, and its `font-lock-keywords' match those that you are using.

Font Lock caches may be saved:
 - When you save the file's buffer.
 - When you kill an unmodified file's buffer.
 - When you exit Emacs, for all unmodified or saved buffers.
Depending on the value of `fast-lock-save-events'.
See also the commands `fast-lock-read-cache' and `fast-lock-save-cache'.

Use \\[font-lock-fontify-buffer] to fontify the buffer if the cache is bad.

Various methods of control are provided for the Font Lock cache.  In general,
see variable `fast-lock-cache-directories' and function `fast-lock-cache-name'.
For saving, see variables `fast-lock-minimum-size', `fast-lock-save-events',
`fast-lock-save-others' and `fast-lock-save-faces'.

Use \\[fast-lock-submit-bug-report] to send bug reports or feedback." t nil)

(autoload 'turn-on-fast-lock "fast-lock" "\
Unconditionally turn on Fast Lock mode." nil nil)

(add-minor-mode 'fast-lock-mode " Fast")

;;;***

;;;### (autoloads (feedmail-send-it) "feedmail" "packages/feedmail.el" (12698 33544))
;;; Generated autoloads from packages/feedmail.el

(autoload 'feedmail-send-it "feedmail" nil nil nil)

;;;***

;;;### (autoloads (make-file-part) "file-part" "packages/file-part.el" (12639 8595))
;;; Generated autoloads from packages/file-part.el

(autoload 'make-file-part "file-part" "\
Make a file part on buffer BUFFER out of the region.  Call it NAME.
This command creates a new buffer containing the contents of the
region and marks the buffer as referring to the specified buffer,
called the `master buffer'.  When the file-part buffer is saved,
its changes are integrated back into the master buffer.  When the
master buffer is deleted, all file parts are deleted with it.

When called from a function, expects four arguments, START, END,
NAME, and BUFFER, all of which are optional and default to the
beginning of BUFFER, the end of BUFFER, a name generated from
BUFFER's name, and the current buffer, respectively." t nil)

;;;***

;;;### (autoloads nil "filladapt" "packages/filladapt.el" (12731 4993))
;;; Generated autoloads from packages/filladapt.el

;;;### (autoloads (font-lock-fontify-buffer turn-off-font-lock turn-on-font-lock font-lock-mode) "font-lock" "packages/font-lock.el" (12730 62099))
;;; Generated autoloads from packages/font-lock.el

(defvar font-lock-auto-fontify t "\
*Whether font-lock should automatically fontify files as they're loaded.
This will only happen if font-lock has fontifying keywords for the major
mode of the file.  You can get finer-grained control over auto-fontification
by using this variable in combination with `font-lock-mode-enable-list' or
`font-lock-mode-disable-list'.")

(defvar font-lock-mode-enable-list nil "\
*List of modes to auto-fontify, if `font-lock-auto-fontify' is nil.")

(defvar font-lock-mode-disable-list nil "\
*List of modes not to auto-fontify, if `font-lock-auto-fontify' is t.")

(defvar font-lock-use-colors '(color) "\
*Specification for when Font Lock will set up color defaults.
Normally this should be '(color), meaning that Font Lock will set up
color defaults that are only used on color displays.  Set this to nil
if you don't want Font Lock to set up color defaults at all.  This
should be one of

-- a list of valid tags, meaning that the color defaults will be used
   when all of the tags apply. (e.g. '(color x))
-- a list whose first element is 'or and whose remaining elements are
   lists of valid tags, meaning that the defaults will be used when
   any of the tag lists apply.
-- nil, meaning that the defaults should not be set up at all.

\(If you specify face values in your init file, they will override any
that Font Lock specifies, regardless of whether you specify the face
values before or after loading Font Lock.)

See also `font-lock-use-fonts'.  If you want more control over the faces
used for fontification, see the documentation of `font-lock-mode' for
how to do it.")

(defvar font-lock-use-fonts '(or (mono) (grayscale)) "\
*Specification for when Font Lock will set up non-color defaults.

Normally this should be '(or (mono) (grayscale)), meaning that Font
Lock will set up non-color defaults that are only used on either mono
or grayscale displays.  Set this to nil if you don't want Font Lock to
set up non-color defaults at all.  This should be one of

-- a list of valid tags, meaning that the non-color defaults will be used
   when all of the tags apply. (e.g. '(grayscale x))
-- a list whose first element is 'or and whose remaining elements are
   lists of valid tags, meaning that the defaults will be used when
   any of the tag lists apply.
-- nil, meaning that the defaults should not be set up at all.

\(If you specify face values in your init file, they will override any
that Font Lock specifies, regardless of whether you specify the face
values before or after loading Font Lock.)

See also `font-lock-use-colors'.  If you want more control over the faces
used for fontification, see the documentation of `font-lock-mode' for
how to do it.")

(defvar font-lock-maximum-decoration nil "\
*If non-nil, the maximum decoration level for fontifying.
If nil, use the minimum decoration (equivalent to level 0).
If t, use the maximum decoration available.
If a number, use that level of decoration (or if not available the maximum).
If a list, each element should be a cons pair of the form (MAJOR-MODE . LEVEL),
where MAJOR-MODE is a symbol or t (meaning the default).  For example:
 ((c++-mode . 2) (c-mode . t) (t . 1))
means use level 2 decoration for buffers in `c++-mode', the maximum decoration
available for buffers in `c-mode', and level 1 decoration otherwise.")

(define-obsolete-variable-alias 'font-lock-use-maximal-decoration 'font-lock-maximum-decoration)

(defvar font-lock-maximum-size (* 250 1024) "\
*If non-nil, the maximum size for buffers for fontifying.
Only buffers less than this can be fontified when Font Lock mode is turned on.
If nil, means size is irrelevant.
If a list, each element should be a cons pair of the form (MAJOR-MODE . SIZE),
where MAJOR-MODE is a symbol or t (meaning the default).  For example:
 ((c++-mode . 256000) (c-mode . 256000) (rmail-mode . 1048576))
means that the maximum size is 250K for buffers in `c++-mode' or `c-mode', one
megabyte for buffers in `rmail-mode', and size is irrelevant otherwise.")

(defvar font-lock-keywords nil "\
*A list of the keywords to highlight.
Each element should be of the form:

 MATCHER
 (MATCHER . MATCH)
 (MATCHER . FACENAME)
 (MATCHER . HIGHLIGHT)
 (MATCHER HIGHLIGHT ...)

where HIGHLIGHT should be either MATCH-HIGHLIGHT or MATCH-ANCHORED.

For highlighting single items, typically only MATCH-HIGHLIGHT is required.
However, if an item or (typically) items is to be hightlighted following the
instance of another item (the anchor) then MATCH-ANCHORED may be required.

MATCH-HIGHLIGHT should be of the form:

 (MATCH FACENAME OVERRIDE LAXMATCH)

Where MATCHER can be either the regexp to search for, or the function name to
call to make the search (called with one argument, the limit of the search).
MATCH is the subexpression of MATCHER to be highlighted.  FACENAME is either
a symbol naming a face, or an expression whose value is the face name to use.
If you want FACENAME to be a symbol that evaluates to a face, use a form
like \"(progn sym)\".

OVERRIDE and LAXMATCH are flags.  If OVERRIDE is t, existing fontification may
be overwritten.  If `keep', only parts not already fontified are highlighted.
If `prepend' or `append', existing fontification is merged with the new, in
which the new or existing fontification, respectively, takes precedence.
If LAXMATCH is non-nil, no error is signalled if there is no MATCH in MATCHER.

For example, an element of the form highlights (if not already highlighted):

 \"\\\\\\=<foo\\\\\\=>\"		Discrete occurrences of \"foo\" in the value of the
			variable `font-lock-keyword-face'.
 (\"fu\\\\(bar\\\\)\" . 1)	Substring \"bar\" within all occurrences of \"fubar\" in
			the value of `font-lock-keyword-face'.
 (\"fubar\" . fubar-face)	Occurrences of \"fubar\" in the value of `fubar-face'.
 (\"foo\\\\|bar\" 0 foo-bar-face t)
			Occurrences of either \"foo\" or \"bar\" in the value
			of `foo-bar-face', even if already highlighted.

MATCH-ANCHORED should be of the form:

 (MATCHER PRE-MATCH-FORM POST-MATCH-FORM MATCH-HIGHLIGHT ...)

Where MATCHER is as for MATCH-HIGHLIGHT with one exception.  The limit of the
search is currently guaranteed to be (no greater than) the end of the line.
PRE-MATCH-FORM and POST-MATCH-FORM are evaluated before the first, and after
the last, instance MATCH-ANCHORED's MATCHER is used.  Therefore they can be
used to initialise before, and cleanup after, MATCHER is used.  Typically,
PRE-MATCH-FORM is used to move to some position relative to the original
MATCHER, before starting with MATCH-ANCHORED's MATCHER.  POST-MATCH-FORM might
be used to move, before resuming with MATCH-ANCHORED's parent's MATCHER.

For example, an element of the form highlights (if not already highlighted):

 (\"\\\\\\=<anchor\\\\\\=>\" (0 anchor-face) (\"\\\\\\=<item\\\\\\=>\" nil nil (0 item-face)))

 Discrete occurrences of \"anchor\" in the value of `anchor-face', and subsequent
 discrete occurrences of \"item\" (on the same line) in the value of `item-face'.
 (Here PRE-MATCH-FORM and POST-MATCH-FORM are nil.  Therefore \"item\" is
 initially searched for starting from the end of the match of \"anchor\", and
 searching for subsequent instance of \"anchor\" resumes from where searching
 for \"item\" concluded.)

Note that the MATCH-ANCHORED feature is experimental; in the future, we may
replace it with other ways of providing this functionality.

These regular expressions should not match text which spans lines.  While
\\[font-lock-fontify-buffer] handles multi-line patterns correctly, updating
when you edit the buffer does not, since it considers text one line at a time.

Be very careful composing regexps for this list;
the wrong pattern can dramatically slow things down!")

(make-variable-buffer-local 'font-lock-keywords)

(defvar font-lock-mode nil)

(defvar font-lock-mode-hook nil "\
Function or functions to run on entry to font-lock-mode.")

(autoload 'font-lock-mode "font-lock" "\
Toggle Font Lock Mode.
With arg, turn font-lock mode on if and only if arg is positive.

When Font Lock mode is enabled, text is fontified as you type it:

 - Comments are displayed in `font-lock-comment-face';
 - Strings are displayed in `font-lock-string-face';
 - Documentation strings (in Lisp-like languages) are displayed in
   `font-lock-doc-string-face';
 - Language keywords (\"reserved words\") are displayed in
   `font-lock-keyword-face';
 - Function names in their defining form are displayed in
   `font-lock-function-name-face';
 - Variable names in their defining form are displayed in
   `font-lock-variable-name-face';
 - Type names are displayed in `font-lock-type-face';
 - References appearing in help files and the like are displayed
   in `font-lock-reference-face';
 - Preprocessor declarations are displayed in
  `font-lock-preprocessor-face';

   and

 - Certain other expressions are displayed in other faces according
   to the value of the variable `font-lock-keywords'.

Where modes support different levels of fontification, you can use the variable
`font-lock-maximum-decoration' to specify which level you generally prefer.
When you turn Font Lock mode on/off the buffer is fontified/defontified, though
fontification occurs only if the buffer is less than `font-lock-maximum-size'.
To fontify a buffer without turning on Font Lock mode, and regardless of buffer
size, you can use \\[font-lock-fontify-buffer].

See the variable `font-lock-keywords' for customization." t nil)

(autoload 'turn-on-font-lock "font-lock" "\
Unconditionally turn on Font Lock mode." nil nil)

(autoload 'turn-off-font-lock "font-lock" "\
Unconditionally turn off Font Lock mode." nil nil)

(autoload 'font-lock-fontify-buffer "font-lock" "\
Fontify the current buffer the way `font-lock-mode' would.
See `font-lock-mode' for details.

This can take a while for large buffers." t nil)

(add-minor-mode 'font-lock-mode " Font")

;;;***

;;;### (autoloads nil "fontl-hooks" "packages/fontl-hooks.el" (12558 61068))
;;; Generated autoloads from packages/fontl-hooks.el

;;;***

;;;### (autoloads nil "func-menu" "packages/func-menu.el" (12721 2479))
;;; Generated autoloads from packages/func-menu.el

;;;### (autoloads (sc-mode) "generic-sc" "packages/generic-sc.el" (12147 25472))
;;; Generated autoloads from packages/generic-sc.el

(autoload 'sc-mode "generic-sc" "\
Toggle sc-mode.
SYSTEM can be sccs, rcs or cvs.
Cvs requires the pcl-cvs package.

The following commands are available
\\[sc-next-operation]	perform next logical source control operation on current file
\\[sc-show-changes]	compare the version being edited with an older one
\\[sc-version-diff-file]	compare two older versions of a file
\\[sc-show-history]		display change history of current file
\\[sc-visit-previous-revision]	display an older revision of current file
\\[sc-revert-file]		revert buffer to last checked-in version
\\[sc-list-all-locked-files]		show all files locked in current directory
\\[sc-list-locked-files]		show all files locked by you in current directory
\\[sc-list-registered-files]		show all files under source control in current directory
\\[sc-update-directory]		get fresh copies of files checked-in by others in current directory
\\[sc-rename-file]		rename the current file and its source control file


While you are entering a change log message for a check in, sc-log-entry-mode
will be in effect.

Global user options:
    sc-diff-command	A list consisting of the command and flags
			to be used for generating context diffs.
    sc-mode-expert	suppresses some conformation prompts,
			notably for delta aborts and file saves.
    sc-max-log-size	specifies the maximum allowable size
			of a log message plus one.


When using SCCS you have additional commands and options

\\[sccs-insert-headers]		insert source control headers in current file

When you generate headers into a buffer using \\[sccs-insert-headers],
the value of sc-insert-headers-hook is called before insertion. If the
file is recognized a C or Lisp source, sc-insert-c-header-hook or
sc-insert-lisp-header-hook is called after insertion respectively.

    sccs-headers-wanted	which %-keywords to insert when adding
			headers with C-c h
    sccs-insert-static	if non-nil, keywords inserted in C files
			get stuffed in a static string area so that
			what(1) can see them in the compiled object code.

When using CVS you have additional commands

\\[sc-cvs-update-directory]	update the current directory using pcl-cvs
\\[sc-cvs-file-status]		show the CVS status of current file
" t nil)

;;;***

;;;### (autoloads (gnuserv-start) "gnuserv" "packages/gnuserv.el" (12740 25205))
;;; Generated autoloads from packages/gnuserv.el

(autoload 'gnuserv-start "gnuserv" "\
Allow this Emacs process to be a server for client processes.
This starts a server communications subprocess through which
client \"editors\" (gnuclient and gnudoit) can send editing commands to 
this Emacs job. See the gnuserv(1) manual page for more details.

Prefix arg means just kill any existing server communications subprocess." t nil)

;;;***

;;;### (autoloads (gopher-atpoint gopher) "gopher" "packages/gopher.el" (12376 19490))
;;; Generated autoloads from packages/gopher.el

(autoload 'gopher "gopher" "\
Start a gopher session.  With C-u, prompt for a gopher server." t nil)

(autoload 'gopher-atpoint "gopher" "\
Try to interpret the text around point as a gopher bookmark, and dispatch
to that object." t nil)

;;;***

;;;### (autoloads (hexl-find-file hexl-mode) "hexl" "packages/hexl.el" (12675 57080))
;;; Generated autoloads from packages/hexl.el

(autoload 'hexl-mode "hexl" "\
\\<hexl-mode-map>
A major mode for editing binary files in hex dump format.

This function automatically converts a buffer into the hexl format
using the function `hexlify-buffer'.

Each line in the buffer has an \"address\" (displayed in hexadecimal)
representing the offset into the file that the characters on this line
are at and 16 characters from the file (displayed as hexadecimal
values grouped every 16 bits) and as their ASCII values.

If any of the characters (displayed as ASCII characters) are
unprintable (control or meta characters) they will be replaced as
periods.

If `hexl-mode' is invoked with an argument the buffer is assumed to be
in hexl format.

A sample format:

  HEX ADDR: 0001 0203 0405 0607 0809 0a0b 0c0d 0e0f     ASCII-TEXT
  --------  ---- ---- ---- ---- ---- ---- ---- ----  ----------------
  00000000: 5468 6973 2069 7320 6865 786c 2d6d 6f64  This is hexl-mod
  00000010: 652e 2020 4561 6368 206c 696e 6520 7265  e.  Each line re
  00000020: 7072 6573 656e 7473 2031 3620 6279 7465  presents 16 byte
  00000030: 7320 6173 2068 6578 6164 6563 696d 616c  s as hexadecimal
  00000040: 2041 5343 4949 0a61 6e64 2070 7269 6e74   ASCII.and print
  00000050: 6162 6c65 2041 5343 4949 2063 6861 7261  able ASCII chara
  00000060: 6374 6572 732e 2020 416e 7920 636f 6e74  cters.  Any cont
  00000070: 726f 6c20 6f72 206e 6f6e 2d41 5343 4949  rol or non-ASCII
  00000080: 2063 6861 7261 6374 6572 730a 6172 6520   characters.are 
  00000090: 6469 7370 6c61 7965 6420 6173 2070 6572  displayed as per
  000000a0: 696f 6473 2069 6e20 7468 6520 7072 696e  iods in the prin
  000000b0: 7461 626c 6520 6368 6172 6163 7465 7220  table character 
  000000c0: 7265 6769 6f6e 2e0a                      region..

Movement is as simple as movement in a normal emacs text buffer.  Most
cursor movement bindings are the same (ie. Use \\[hexl-backward-char], \\[hexl-forward-char], \\[hexl-next-line], and \\[hexl-previous-line]
to move the cursor left, right, down, and up).

Advanced cursor movement commands (ala \\[hexl-beginning-of-line], \\[hexl-end-of-line], \\[hexl-beginning-of-buffer], and \\[hexl-end-of-buffer]) are
also supported.

There are several ways to change text in hexl mode:

ASCII characters (character between space (0x20) and tilde (0x7E)) are
bound to self-insert so you can simply type the character and it will
insert itself (actually overstrike) into the buffer.

\\[hexl-quoted-insert] followed by another keystroke allows you to insert the key even if
it isn't bound to self-insert.  An octal number can be supplied in place
of another key to insert the octal number's ASCII representation.

\\[hexl-insert-hex-char] will insert a given hexadecimal value (if it is between 0 and 0xFF)
into the buffer at the current point.

\\[hexl-insert-octal-char] will insert a given octal value (if it is between 0 and 0377)
into the buffer at the current point.

\\[hexl-insert-decimal-char] will insert a given decimal value (if it is between 0 and 255)
into the buffer at the current point.

\\[hexl-mode-exit] will exit hexl-mode.

Note: saving the file with any of the usual Emacs commands
will actually convert it back to binary format while saving.

You can use \\[hexl-find-file] to visit a file in hexl-mode.

\\[describe-bindings] for advanced commands." t nil)

(autoload 'hexl-find-file "hexl" "\
Edit file FILENAME in hexl-mode.
Switch to a buffer visiting file FILENAME, creating one in none exists." t nil)

;;;***

;;;### (autoloads (hypropos-popup-menu hypropos-set-variable hyper-describe-function hyper-describe-variable hyper-apropos) "hyper-apropos" "packages/hyper-apropos.el" (12607 29142))
;;; Generated autoloads from packages/hyper-apropos.el

(defvar hypropos-show-brief-docs t "\
*If non-nil, `hyper-apropos' will display some documentation in the
\"*Hyper Apropos*\" buffer.  Setting this to nil will speed up searches.")

(autoload 'hyper-apropos "hyper-apropos" "\
Display lists of functions and variables matching REGEXP
in buffer \"*Hyper Apropos*\".  If optional prefix arg is given, then the value
of `hypropos-programming-apropos' is toggled for this search.
See also `hyper-apropos-mode'." t nil)

(autoload 'hyper-describe-variable "hyper-apropos" "\
Hypertext drop-in replacement for `describe-variable'.
See also `hyper-apropos' and `hyper-describe-function'." t nil)

(autoload 'hyper-describe-function "hyper-apropos" "\
Hypertext replacement for `describe-function'.  Unlike `describe-function'
in that the symbol under the cursor is the default if it is a function.
See also `hyper-apropos' and `hyper-describe-variable'." t nil)

(autoload 'hypropos-set-variable "hyper-apropos" "\
Interactively set the variable on the current line." t nil)

(autoload 'hypropos-popup-menu "hyper-apropos" nil t nil)

;;;***

;;;### (autoloads (icomplete-minibuffer-setup icomplete-mode) "icomplete" "packages/icomplete.el" (12547 2477))
;;; Generated autoloads from packages/icomplete.el

(autoload 'icomplete-mode "icomplete" "\
Activate incremental minibuffer completion for this emacs session,
or deactivate with negative prefix arg." t nil)

(autoload 'icomplete-minibuffer-setup "icomplete" "\
Run in minibuffer on activation to establish incremental completion.

Usually run by inclusion in minibuffer-setup-hook." nil nil)

;;;***

;;;### (autoloads (Info-elisp-ref Info-emacs-key Info-goto-emacs-key-command-node Info-goto-emacs-command-node Info-emacs-command Info-search Info-visit-file Info-goto-node Info-query info) "info" "packages/info.el" (12740 25202))
;;; Generated autoloads from packages/info.el

(autoload 'info "info" "\
Enter Info, the documentation browser.
Optional argument FILE specifies the file to examine;
the default is the top-level directory of Info.

In interactive use, a prefix argument directs this command
to read a file name from the minibuffer." t nil)

(autoload 'Info-query "info" "\
Enter Info, the documentation browser.  Prompt for name of Info file." t nil)

(autoload 'Info-goto-node "info" "\
Go to info node named NAME.  Give just NODENAME or (FILENAME)NODENAME.
Actually, the following interpretations of NAME are tried in order:
    (FILENAME)NODENAME
    (FILENAME)     (using Top node)
    NODENAME       (in current file)
    TAGNAME        (see below)
    FILENAME       (using Top node)
where TAGNAME is a string that appears in quotes: \"TAGNAME\", in an
annotation for any node of any file.  (See `a' and `x' commands.)" t nil)

(autoload 'Info-visit-file "info" "\
Directly visit an info file." t nil)

(autoload 'Info-search "info" "\
Search for REGEXP, starting from point, and select node it's found in." t nil)

(autoload 'Info-emacs-command "info" "\
Look up an Emacs command in the Emacs manual in the Info system.
This command is designed to be used whether you are already in Info or not." t nil)

(autoload 'Info-goto-emacs-command-node "info" "\
Look up an Emacs command in the Emacs manual in the Info system.
This command is designed to be used whether you are already in Info or not." t nil)

(autoload 'Info-goto-emacs-key-command-node "info" "\
Look up an Emacs key sequence in the Emacs manual in the Info system.
This command is designed to be used whether you are already in Info or not." t nil)

(autoload 'Info-emacs-key "info" "\
Look up an Emacs key sequence in the Emacs manual in the Info system.
This command is designed to be used whether you are already in Info or not." t nil)

(autoload 'Info-elisp-ref "info" "\
Look up an Emacs Lisp function in the Elisp manual in the Info system.
This command is designed to be used whether you are already in Info or not." t nil)

;;;***

;;;### (autoloads (batch-info-validate Info-validate Info-split Info-tagify) "informat" "packages/informat.el" (12547 2425))
;;; Generated autoloads from packages/informat.el

(autoload 'Info-tagify "informat" "\
Create or update Info-file tag table in current buffer." t nil)

(autoload 'Info-split "informat" "\
Split an info file into an indirect file plus bounded-size subfiles.
Each subfile will be up to 50,000 characters plus one node.

To use this command, first visit a large Info file that has a tag
table.  The buffer is modified into a (small) indirect info file which
should be saved in place of the original visited file.

The subfiles are written in the same directory the original file is
in, with names generated by appending `-' and a number to the original
file name.  The indirect file still functions as an Info file, but it
contains just the tag table and a directory of subfiles." t nil)

(autoload 'Info-validate "informat" "\
Check current buffer for validity as an Info file.
Check that every node pointer points to an existing node." t nil)

(autoload 'batch-info-validate "informat" "\
Runs `Info-validate' on the files remaining on the command line.
Must be used only with -batch, and kills Emacs on completion.
Each file will be processed even if an error occurred previously.
For example, invoke \"emacs -batch -f batch-info-validate $info/ ~/*.info\"" nil nil)

;;;***

;;;### (autoloads (ispell-message ispell-minor-mode ispell-complete-word-interior-frag ispell-complete-word ispell-continue ispell-buffer ispell-region ispell-change-dictionary ispell-kill-ispell ispell-help ispell-word) "ispell" "packages/ispell.el" (12727 30153))
;;; Generated autoloads from packages/ispell.el

(defvar ispell-personal-dictionary nil "\
*File name of your personal spelling dictionary, or nil.
If nil, the default personal dictionary, \"~/.ispell_DICTNAME\" is used,
where DICTNAME is the name of your default dictionary.")

(defvar ispell-dictionary-alist-1 '((nil "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B") nil) ("english" "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B") nil) ("british" "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B" "-d" "british") nil) ("deutsch" "[a-zA-Z\"]" "[^a-zA-Z\"]" "[']" t ("-C") "~tex") ("deutsch8" "[a-zA-Z]" "[^a-zA-Z]" "[']" t ("-C" "-d" "deutsch") "~latin1") ("nederlands" "[A-Za-z--------]" "[^A-Za-z--------]" "[']" t ("-C") nil) ("nederlands8" "[A-Za-z--------]" "[^A-Za-z--------]" "[']" t ("-C") nil)))

(defvar ispell-dictionary-alist-2 '(("svenska" "[A-Za-z}{|\\133\\135\\\\]" "[^A-Za-z}{|\\133\\135\\\\]" "[']" nil ("-C") nil) ("svenska8" "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-C" "-d" "svenska") "~list") ("francais7" "[A-Za-z]" "[^A-Za-z]" "[`'^---]" t nil nil) ("francais" "[A-Za-z]" "[^A-Za-z]" "[---']" t nil "~list") ("francais-tex" "[A-Za-z\\]" "[^A-Za-z\\]" "[---'^`\"]" t nil "~tex") ("dansk" "[A-Za-z]" "[^A-Za-z]" "" nil ("-C") nil)))

(defvar ispell-dictionary-alist (append ispell-dictionary-alist-1 ispell-dictionary-alist-2) "\
An alist of dictionaries and their associated parameters.

Each element of this list is also a list:

\(DICTIONARY-NAME CASECHARS NOT-CASECHARS OTHERCHARS MANY-OTHERCHARS-P
        ISPELL-ARGS EXTENDED-CHARACTER-MODE)

DICTIONARY-NAME is a possible value of variable `ispell-dictionary', nil
means the default dictionary.

CASECHARS is a regular expression of valid characters that comprise a
word.

NOT-CASECHARS is the opposite regexp of CASECHARS.

OTHERCHARS is a regular expression of other characters that are valid
in word constructs.  Otherchars cannot be adjacent to each other in a
word, nor can they begin or end a word.  This implies we can't check
\"Stevens'\" as a correct possessive and other correct formations.

Hint: regexp syntax requires the hyphen to be declared first here.

MANY-OTHERCHARS-P is non-nil if many otherchars are to be allowed in a
word instead of only one.

ISPELL-ARGS is a list of additional arguments passed to the ispell
subprocess.

EXTENDED-CHARACTER-MODE should be used when dictionaries are used which
have been configured in an Ispell affix file.  (For example, umlauts
can be encoded as \\\"a, a\\\", \"a, ...)  Defaults are ~tex and ~nroff
in English.  This has the same effect as the command-line `-T' option.
The buffer Major Mode controls Ispell's parsing in tex or nroff mode,
but the dictionary can control the extended character mode.
Both defaults can be overruled in a buffer-local fashion. See
`ispell-parsing-keyword' for details on this.

Note that the CASECHARS and OTHERCHARS slots of the alist should
contain the same character set as casechars and otherchars in the
language.aff file (e.g., english.aff).")

(defvar ispell-menu-map nil "\
Key map for ispell menu")

(defvar ispell-menu-lucid nil "\
Spelling menu for Lucid Emacs.")

(defconst ispell-menu-map-needed (and (not ispell-menu-map) (string-lessp "19" emacs-version) (not (string-match "XEmacs" emacs-version))))

(if ispell-menu-map-needed (let ((dicts (reverse (cons (cons "default" nil) ispell-dictionary-alist))) name) (setq ispell-menu-map (make-sparse-keymap "Spell")) (while dicts (setq name (car (car dicts)) dicts (cdr dicts)) (if (stringp name) (define-key ispell-menu-map (vector (intern name)) (cons (concat "Select " (capitalize name)) (list 'lambda nil '(interactive) (list 'ispell-change-dictionary name))))))))

(if ispell-menu-map-needed (progn (define-key ispell-menu-map [ispell-change-dictionary] '("Change Dictionary" . ispell-change-dictionary)) (define-key ispell-menu-map [ispell-kill-ispell] '("Kill Process" . ispell-kill-ispell)) (define-key ispell-menu-map [ispell-pdict-save] '("Save Dictionary" lambda nil (interactive) (ispell-pdict-save t t))) (define-key ispell-menu-map [ispell-complete-word] '("Complete Word" . ispell-complete-word)) (define-key ispell-menu-map [ispell-complete-word-interior-frag] '("Complete Word Frag" . ispell-complete-word-interior-frag))))

(if ispell-menu-map-needed (progn (define-key ispell-menu-map [ispell-continue] '("Continue Check" . ispell-continue)) (define-key ispell-menu-map [ispell-word] '("Check Word" . ispell-word)) (define-key ispell-menu-map [ispell-region] '("Check Region" . ispell-region)) (define-key ispell-menu-map [ispell-buffer] '("Check Buffer" . ispell-buffer))))

(if ispell-menu-map-needed (progn (define-key ispell-menu-map [ispell-message] '("Check Message" . ispell-message)) (define-key ispell-menu-map [ispell-help] '("Help" lambda nil (interactive) (describe-function 'ispell-help))) (put 'ispell-region 'menu-enable 'mark-active) (fset 'ispell-menu-map (symbol-value 'ispell-menu-map))))

(defvar ispell-local-pdict ispell-personal-dictionary "\
A buffer local variable containing the current personal dictionary.
If non-nil, the value must be a string, which is a file name.

If you specify a personal dictionary for the current buffer which is
different from the current personal dictionary, the effect is similar
to calling \\[ispell-change-dictionary].  This variable is automatically
set when defined in the file with either `ispell-pdict-keyword' or the
local variable syntax.")

(define-key global-map "" 'ispell-word)

(autoload 'ispell-word "ispell" "\
Check spelling of word under or before the cursor.
If the word is not found in dictionary, display possible corrections
in a window allowing you to choose one.

With a prefix argument (or if CONTINUE is non-nil),
resume interrupted spell-checking of a buffer or region.

If optional argument FOLLOWING is non-nil or if `ispell-following-word'
is non-nil when called interactively, then the following word
\(rather than preceding) is checked when the cursor is not over a word.
When the optional argument QUIETLY is non-nil or `ispell-quietly' is non-nil
when called interactively, non-corrective messages are suppressed.

Word syntax described by `ispell-dictionary-alist' (which see).

This will check or reload the dictionary.  Use \\[ispell-change-dictionary]
or \\[ispell-region] to update the Ispell process." t nil)

(autoload 'ispell-help "ispell" "\
Display a list of the options available when a misspelling is encountered.

Selections are:

DIGIT: Replace the word with a digit offered in the *Choices* buffer.
SPC:   Accept word this time.
`i':   Accept word and insert into private dictionary.
`a':   Accept word for this session.
`A':   Accept word and place in `buffer-local dictionary'.
`r':   Replace word with typed-in value.  Rechecked.
`R':   Replace word with typed-in value. Query-replaced in buffer. Rechecked.
`?':   Show these commands.
`x':   Exit spelling buffer.  Move cursor to original point.
`X':   Exit spelling buffer.  Leaves cursor at the current point, and permits
        the aborted check to be completed later.
`q':   Quit spelling session (Kills ispell process).
`l':   Look up typed-in replacement in alternate dictionary.  Wildcards okay.
`u':   Like `i', but the word is lower-cased first.
`m':   Like `i', but allows one to include dictionary completion information.
`C-l':  redraws screen
`C-r':  recursive edit
`C-z':  suspend emacs or iconify frame" nil nil)

(autoload 'ispell-kill-ispell "ispell" "\
Kill current Ispell process (so that you may start a fresh one).
With NO-ERROR, just return non-nil if there was no Ispell running." t nil)

(autoload 'ispell-change-dictionary "ispell" "\
Change `ispell-dictionary' (q.v.) and kill old Ispell process.
A new one will be started as soon as necessary.

By just answering RET you can find out what the current dictionary is.

With prefix argument, set the default directory." t nil)

(autoload 'ispell-region "ispell" "\
Interactively check a region for spelling errors." t nil)

(autoload 'ispell-buffer "ispell" "\
Check the current buffer for spelling errors interactively." t nil)

(autoload 'ispell-continue "ispell" nil t nil)

(autoload 'ispell-complete-word "ispell" "\
Look up word before or under point in dictionary (see lookup-words command)
and try to complete it.  If optional INTERIOR-FRAG is non-nil then the word
may be a character sequence inside of a word.

Standard ispell choices are then available." t nil)

(autoload 'ispell-complete-word-interior-frag "ispell" "\
Completes word matching character sequence inside a word." t nil)

(autoload 'ispell-minor-mode "ispell" "\
Toggle Ispell minor mode.
With prefix arg, turn Ispell minor mode on iff arg is positive.
 
In Ispell minor mode, pressing SPC or RET
warns you if the previous word is incorrectly spelled." t nil)

(autoload 'ispell-message "ispell" "\
Check the spelling of a mail message or news post.
Don't check spelling of message headers except the Subject field.
Don't check included messages.

To abort spell checking of a message REGION and send the message anyway,
use the `x' or `q' command.  (Any subsequent regions will be checked.)
The `X' command aborts the message send so that you can edit the buffer.

To spell-check whenever a message is sent, include the appropriate lines
in your .emacs file:
   (add-hook 'news-inews-hook 'ispell-message)
   (add-hook 'mail-send-hook  'ispell-message)
   (add-hook 'mh-before-send-letter-hook 'ispell-message)

You can bind this to the key C-c i in GNUS or mail by adding to
`news-reply-mode-hook' or `mail-mode-hook' the following lambda expression:
   (function (lambda () (local-set-key \"\\C-ci\" 'ispell-message)))" t nil)

;;;***

;;;### (autoloads (jka-compr-install toggle-auto-compression jka-compr-load) "jka-compr" "packages/jka-compr.el" (12639 8598))
;;; Generated autoloads from packages/jka-compr.el

(autoload 'jka-compr-load "jka-compr" "\
Documented as original." nil nil)

(autoload 'toggle-auto-compression "jka-compr" "\
Toggle automatic file compression and decompression.
With prefix argument ARG, turn auto compression on if positive, else off.
Returns the new status of auto compression (non-nil means on)." t nil)

(autoload 'jka-compr-install "jka-compr" "\
Install jka-compr.
This adds entries to `file-name-handler-alist' and `auto-mode-alist'
and `inhibit-first-line-modes-suffixes'." nil nil)

;;;***

;;;### (autoloads (turn-on-lazy-lock lazy-lock-mode) "lazy-lock" "packages/lazy-lock.el" (12743 11606))
;;; Generated autoloads from packages/lazy-lock.el

(defvar lazy-lock-mode nil)

(autoload 'lazy-lock-mode "lazy-lock" "\
Toggle Lazy Lock mode.
With arg, turn Lazy Lock mode on if and only if arg is positive and the buffer
is at least `lazy-lock-minimum-size' characters long.

When Lazy Lock mode is enabled, fontification is demand-driven and stealthy:

 - Fontification occurs in visible parts of buffers when necessary.
   Occurs if there is no input after pausing for `lazy-lock-continuity-time'.

 - Fontification occurs in invisible parts when Emacs has been idle.
   Occurs if there is no input after pausing for `lazy-lock-stealth-time'.

If `lazy-lock-hide-invisible' is non-nil, text is not displayed until it is
fontified, otherwise it is displayed in `lazy-lock-invisible-foreground'.

See also variables `lazy-lock-walk-windows' and `lazy-lock-ignore-commands' for
window (scroll) fontification, and `lazy-lock-stealth-lines',
`lazy-lock-stealth-nice' and `lazy-lock-stealth-verbose' for stealth
fontification.

Use \\[lazy-lock-submit-bug-report] to send bug reports or feedback." t nil)

(autoload 'turn-on-lazy-lock "lazy-lock" "\
Unconditionally turn on Lazy Lock mode." nil nil)

(add-minor-mode 'lazy-lock-mode " Lazy")

;;;***

;;;### (autoloads (ledit-from-lisp-mode ledit-mode) "ledit" "packages/ledit.el" (12547 2428))
;;; Generated autoloads from packages/ledit.el

(defconst ledit-save-files t "\
*Non-nil means Ledit should save files before transferring to Lisp.")

(defconst ledit-go-to-lisp-string "%?lisp" "\
*Shell commands to execute to resume Lisp job.")

(defconst ledit-go-to-liszt-string "%?liszt" "\
*Shell commands to execute to resume Lisp compiler job.")

(autoload 'ledit-mode "ledit" "\
\\<ledit-mode-map>Major mode for editing text and stuffing it to a Lisp job.
Like Lisp mode, plus these special commands:
  \\[ledit-save-defun]	-- record defun at or after point
	   for later transmission to Lisp job.
  \\[ledit-save-region] -- record region for later transmission to Lisp job.
  \\[ledit-go-to-lisp] -- transfer to Lisp job and transmit saved text.
  \\[ledit-go-to-liszt] -- transfer to Liszt (Lisp compiler) job
	   and transmit saved text.
\\{ledit-mode-map}
To make Lisp mode automatically change to Ledit mode,
do (setq lisp-mode-hook 'ledit-from-lisp-mode)" t nil)

(autoload 'ledit-from-lisp-mode "ledit" nil nil nil)

;;;***

;;;### (autoloads nil "lispm-fonts" "packages/lispm-fonts.el" (12376 19482))
;;; Generated autoloads from packages/lispm-fonts.el

;;;### (autoloads (print-region lpr-region print-buffer lpr-buffer) "lpr" "packages/lpr.el" (12546 50464))
;;; Generated autoloads from packages/lpr.el

(defvar lpr-switches nil "\
*List of strings to pass as extra switch args to `lpr' when it is invoked.")

(defvar lpr-command (if (memq system-type '(usg-unix-v dgux hpux irix)) "lp" "lpr") "\
*Name of program for printing a file.")

(autoload 'lpr-buffer "lpr" "\
Print buffer contents as with Unix command `lpr'.
`lpr-switches' is a list of extra switches (strings) to pass to lpr." t nil)

(autoload 'print-buffer "lpr" "\
Print buffer contents as with Unix command `lpr -p'.
`lpr-switches' is a list of extra switches (strings) to pass to lpr." t nil)

(autoload 'lpr-region "lpr" "\
Print region contents as with Unix command `lpr'.
`lpr-switches' is a list of extra switches (strings) to pass to lpr." t nil)

(autoload 'print-region "lpr" "\
Print region contents as with Unix command `lpr -p'.
`lpr-switches' is a list of extra switches (strings) to pass to lpr." t nil)

;;;***

;;;### (autoloads nil "makeinfo" "packages/makeinfo.el" (12547 2422))
;;; Generated autoloads from packages/makeinfo.el

;;;### (autoloads (make-command-summary) "makesum" "packages/makesum.el" (12547 2408))
;;; Generated autoloads from packages/makesum.el

(autoload 'make-command-summary "makesum" "\
Make a summary of current key bindings in the buffer *Summary*.
Previous contents of that buffer are killed first." t nil)

;;;***

;;;### (autoloads (manual-entry) "man" "packages/man.el" (12727 30154))
;;; Generated autoloads from packages/man.el

(autoload 'manual-entry "man" "\
Display the Unix manual entry (or entries) for TOPIC.
If prefix arg is given, modify the search according to the value:
  2 = complement default exact matching of the TOPIC name;
      exact matching default is specified by `Manual-match-topic-exactly'
  3 = force a search of the unformatted man directories
  4 = both 2 and 3
The manual entries are searched according to the variable
Manual-directory-list, which should be a list of directories.  If
Manual-directory-list is nil, \\[Manual-directory-list-init] is
invoked to create this list from the MANPATH environment variable.
See the variable Manual-topic-buffer which controls how the buffer
is named.  See also the variables Manual-match-topic-exactly,
Manual-query-multiple-pages, and Manual-buffer-view-mode." t nil)

;;;***

;;;### (autoloads nil "metamail" "packages/metamail.el" (12547 2454))
;;; Generated autoloads from packages/metamail.el

;;;***

;;;### (autoloads nil "mime-compose" "packages/mime-compose.el" (12727 30155))
;;; Generated autoloads from packages/mime-compose.el

;;;### (autoloads nil "mode-motion+" "packages/mode-motion+.el" (12583 60690))
;;; Generated autoloads from packages/mode-motion+.el

;;;***

;;;### (autoloads nil "netunam" "packages/netunam.el" (12547 2429))
;;; Generated autoloads from packages/netunam.el

;;;***

;;;### (autoloads nil "page-ext" "packages/page-ext.el" (12547 2433))
;;; Generated autoloads from packages/page-ext.el

;;;### (autoloads (blink-paren paren-set-mode) "paren" "packages/paren.el" (12648 18575))
;;; Generated autoloads from packages/paren.el

(defvar paren-mode nil "\
*Sets the style of parenthesis highlighting.
Valid values are nil, `blink-paren', `paren', and `sexp'.
  nil		no parenthesis highlighting.
  blink-paren	causes the matching paren to blink.
  paren		causes the matching paren to be highlighted but not to blink.
  sexp		whole expression enclosed by the local paren at its mate.
  nested	(not yet implemented) use variable shading to see the
		nesting of an expression.  Also groks regular expressions
		and shell quoting.

This variable is global by default, but you can make it buffer-local and
highlight parentheses differrently in different major modes.")

(autoload 'paren-set-mode "paren" "\
Cycles through possible values for `paren-mode', force off with negative arg.
When called from lisp, a symbolic value for `paren-mode' can be pased directly.
See also `paren-mode' and `paren-highlight'." t nil)

(make-obsolete 'blink-paren 'paren-set-mode)

(autoload 'blink-paren "paren" "\
Obsolete.  Use `paren-set-mode' instead." t nil)

;;;***

;;;### (autoloads (pending-delete pending-delete-off pending-delete-on) "pending-del" "packages/pending-del.el" (12547 2431))
;;; Generated autoloads from packages/pending-del.el

(autoload 'pending-delete-on "pending-del" "\
Turn on pending delete.
When it is ON, typed text replaces the selection if the selection is active.
When it is OFF, typed text is just inserted at point." t nil)

(autoload 'pending-delete-off "pending-del" "\
Turn off pending delete.
When it is ON, typed text replaces the selection if the selection is active.
When it is OFF, typed text is just inserted at point." t nil)

(autoload 'pending-delete "pending-del" "\
Toggle automatic deletion of the selected region.
With a positive argument, turns it on.
With a non-positive argument, turns it off.
When active, typed text replaces the selection." t nil)

;;;***

;;;### (autoloads (ps-despool ps-spool-region-with-faces ps-spool-region ps-spool-buffer-with-faces ps-spool-buffer ps-print-region-with-faces ps-print-region ps-print-buffer-with-faces ps-print-buffer) "ps-print" "packages/ps-print.el" (12743 11610))
;;; Generated autoloads from packages/ps-print.el

(defvar ps-paper-type 'ps-letter "\
*Specifies the size of paper to format for.  Should be one of
`ps-letter', `ps-legal', or `ps-a4'.")

(defvar ps-print-color-p (and (or (fboundp 'x-color-values) (fboundp 'color-instance-rgb-components)) (fboundp 'float)) "\
*If non-nil, print the buffer's text in color.")

(autoload 'ps-print-buffer "ps-print" "\
Generate and print a PostScript image of the buffer.

When called with a numeric prefix argument (C-u), prompts the user for
the name of a file to save the PostScript image in, instead of sending
it to the printer.

More specifically, the FILENAME argument is treated as follows: if it
is nil, send the image to the printer.  If FILENAME is a string, save
the PostScript image in a file with that name.  If FILENAME is a
number, prompt the user for the name of the file to save in." t nil)

(autoload 'ps-print-buffer-with-faces "ps-print" "\
Generate and print a PostScript image of the buffer.

Like `ps-print-buffer', but includes font, color, and underline
information in the generated image." t nil)

(autoload 'ps-print-region "ps-print" "\
Generate and print a PostScript image of the region.

Like `ps-print-buffer', but prints just the current region." t nil)

(autoload 'ps-print-region-with-faces "ps-print" "\
Generate and print a PostScript image of the region.

Like `ps-print-region', but includes font, color, and underline
information in the generated image." t nil)

(autoload 'ps-spool-buffer "ps-print" "\
Generate and spool a PostScript image of the buffer.

Like `ps-print-buffer' except that the PostScript image is saved in a
local buffer to be sent to the printer later.

Use the command `ps-despool' to send the spooled images to the printer." t nil)

(autoload 'ps-spool-buffer-with-faces "ps-print" "\
Generate and spool a PostScript image of the buffer.

Like `ps-spool-buffer', but includes font, color, and underline
information in the generated image.

Use the command `ps-despool' to send the spooled images to the printer." t nil)

(autoload 'ps-spool-region "ps-print" "\
Generate a PostScript image of the region and spool locally.

Like `ps-spool-buffer', but spools just the current region.

Use the command `ps-despool' to send the spooled images to the printer." t nil)

(autoload 'ps-spool-region-with-faces "ps-print" "\
Generate a PostScript image of the region and spool locally.

Like `ps-spool-region', but includes font, color, and underline
information in the generated image.

Use the command `ps-despool' to send the spooled images to the printer." t nil)

(autoload 'ps-despool "ps-print" "\
Send the spooled PostScript to the printer.

When called with a numeric prefix argument (C-u), prompt the user for
the name of a file to save the spooled PostScript in, instead of sending
it to the printer.

More specifically, the FILENAME argument is treated as follows: if it
is nil, send the image to the printer.  If FILENAME is a string, save
the PostScript image in a file with that name.  If FILENAME is a
number, prompt the user for the name of the file to save in." t nil)

;;;***

;;;### (autoloads (remote-compile) "rcompile" "packages/rcompile.el" (12720 56487))
;;; Generated autoloads from packages/rcompile.el

(autoload 'remote-compile "rcompile" "\
Compile the the current buffer's directory on HOST.  Log in as USER.
See \\[compile]." t nil)

;;;***

;;;### (autoloads nil "recent-files" "packages/recent-files.el" (12558 60919))
;;; Generated autoloads from packages/recent-files.el

;;;***

;;;### (autoloads nil "refbib" "packages/refbib.el" (12547 2434))
;;; Generated autoloads from packages/refbib.el

;;;***

;;;### (autoloads nil "remote" "packages/remote.el" (12547 2435))
;;; Generated autoloads from packages/remote.el

;;;***

;;;### (autoloads nil "reportmail" "packages/reportmail.el" (12743 22077))
;;; Generated autoloads from packages/reportmail.el

;;;### (autoloads nil "resume" "packages/resume.el" (12547 2437))
;;; Generated autoloads from packages/resume.el

;;;***

;;;### (autoloads nil "rnews" "packages/rnews.el" (12547 2438))
;;; Generated autoloads from packages/rnews.el

;;;***

;;;### (autoloads nil "rnewspost" "packages/rnewspost.el" (12547 2440))
;;; Generated autoloads from packages/rnewspost.el

;;;***

;;;### (autoloads nil "saveconf" "packages/saveconf.el" (12547 2439))
;;; Generated autoloads from packages/saveconf.el

;;;***

;;;### (autoloads nil "saveplace" "packages/saveplace.el" (12547 2414))
;;; Generated autoloads from packages/saveplace.el

;;;***

;;;### (autoloads nil "sccs" "packages/sccs.el" (12547 2430))
;;; Generated autoloads from packages/sccs.el

;;;***

;;;### (autoloads nil "scroll-in-place" "packages/scroll-in-place.el" (12547 2457))
;;; Generated autoloads from packages/scroll-in-place.el

;;;***

;;;### (autoloads nil "server" "packages/server.el" (12547 2441))
;;; Generated autoloads from packages/server.el

(make-obsolete 'server-start 'gnuserv-start)

;;;***

;;;### (autoloads nil "session" "packages/session.el" (12547 2442))
;;; Generated autoloads from packages/session.el

;;;***

;;;### (autoloads (install-shell-fonts) "shell-font" "packages/shell-font.el" (12547 2438))
;;; Generated autoloads from packages/shell-font.el

(autoload 'install-shell-fonts "shell-font" "\
Decorate the current interaction buffer with fonts.
This uses the faces called `shell-prompt', `shell-input' and `shell-output';
you can alter the graphical attributes of those with the normal
face-manipulation functions." nil nil)

;;;***

;;;### (autoloads nil "spell" "packages/spell.el" (12547 2436))
;;; Generated autoloads from packages/spell.el

;;;***

;;;### (autoloads nil "supercite" "packages/supercite.el" (12547 2458))
;;; Generated autoloads from packages/supercite.el

;;;### (autoloads (tar-mode) "tar-mode" "packages/tar-mode.el" (12547 2442))
;;; Generated autoloads from packages/tar-mode.el

(autoload 'tar-mode "tar-mode" "\
Major mode for viewing a tar file as a dired-like listing of its contents.
You can move around using the usual cursor motion commands. 
Letters no longer insert themselves.
Type 'e' to pull a file out of the tar file and into its own buffer.
Type 'c' to copy an entry from the tar file into another file on disk.

If you edit a sub-file of this archive (as with the 'e' command) and 
save it with Control-X Control-S, the contents of that buffer will be 
saved back into the tar-file buffer; in this way you can edit a file 
inside of a tar archive without extracting it and re-archiving it.

See also: variables tar-update-datestamp and tar-anal-blocksize.
\\{tar-mode-map}" nil nil)

;;;***

;;;### (autoloads (terminal-emulator) "terminal" "packages/terminal.el" (12743 11605))
;;; Generated autoloads from packages/terminal.el

(autoload 'terminal-emulator "terminal" "\
Under a display-terminal emulator in BUFFER, run PROGRAM on arguments ARGS.
ARGS is a list of argument-strings.  Remaining arguments are WIDTH and HEIGHT.
BUFFER's contents are made an image of the display generated by that program,
and any input typed when BUFFER is the current Emacs buffer is sent to that
program an keyboard input.

Interactively, BUFFER defaults to \"*terminal*\" and PROGRAM and ARGS
are parsed from an input-string using your usual shell.
WIDTH and HEIGHT are determined from the size of the current window
-- WIDTH will be one less than the window's width, HEIGHT will be its height.

To switch buffers and leave the emulator, or to give commands
to the emulator itself (as opposed to the program running under it),
type Control-^.  The following character is an emulator command.
Type Control-^ twice to send it to the subprogram.
This escape character may be changed using the variable `terminal-escape-char'.

`Meta' characters may not currently be sent through the terminal emulator.

Here is a list of some of the variables which control the behaviour
of the emulator -- see their documentation for more information:
terminal-escape-char, terminal-scrolling, terminal-more-processing,
terminal-redisplay-interval.

This function calls the value of terminal-mode-hook if that exists
and is non-nil after the terminal buffer has been set up and the
subprocess started.

Presently with `termcap' only; if somebody sends us code to make this
work with `terminfo' we will try to use it." t nil)

;;;***

;;;### (autoloads nil "tex-latin1" "packages/tex-latin1.el" (12698 33545))
;;; Generated autoloads from packages/tex-latin1.el

;;;***

;;;### (autoloads (batch-texinfo-format texinfo-format-region texinfo-format-buffer) "texinfmt" "packages/texinfmt.el" (12547 2445))
;;; Generated autoloads from packages/texinfmt.el

(autoload 'texinfo-format-buffer "texinfmt" "\
Process the current buffer as texinfo code, into an Info file.
The Info file output is generated in a buffer visiting the Info file
names specified in the @setfilename command.

Non-nil argument (prefix, if interactive) means don't make tag table
and don't split the file if large.  You can use Info-tagify and
Info-split to do these manually." t nil)

(autoload 'texinfo-format-region "texinfmt" "\
Convert the current region of the Texinfo file to Info format.
This lets you see what that part of the file will look like in Info.
The command is bound to \\[texinfo-format-region].  The text that is
converted to Info is stored in a temporary buffer." t nil)

(autoload 'batch-texinfo-format "texinfmt" "\
Runs  texinfo-format-buffer  on the files remaining on the command line.
Must be used only with -batch, and kills emacs on completion.
Each file will be processed even if an error occurred previously.
For example, invoke
  \"emacs -batch -funcall batch-texinfo-format $docs/ ~/*.texinfo\"." nil nil)

;;;***

;;;### (autoloads nil "texnfo-tex" "packages/texnfo-tex.el" (12657 40704))
;;; Generated autoloads from packages/texnfo-tex.el

;;;### (autoloads (texinfo-sequential-node-update texinfo-every-node-update texinfo-update-node) "texnfo-upd" "packages/texnfo-upd.el" (12547 2451))
;;; Generated autoloads from packages/texnfo-upd.el

(autoload 'texinfo-update-node "texnfo-upd" "\
Without any prefix argument, update the node in which point is located.
Non-nil argument (prefix, if interactive) means update the nodes in the
marked region.

The functions for creating or updating nodes and menus, and their
keybindings, are:

    texinfo-update-node (&optional region-p)    \\[texinfo-update-node]
    texinfo-every-node-update ()                \\[texinfo-every-node-update]
    texinfo-sequential-node-update (&optional region-p)

    texinfo-make-menu (&optional region-p)      \\[texinfo-make-menu]
    texinfo-all-menus-update ()                 \\[texinfo-all-menus-update]
    texinfo-master-menu ()

    texinfo-indent-menu-description (column &optional region-p)

The `texinfo-column-for-description' variable specifies the column to
which menu descriptions are indented. Its default value is 32." t nil)

(autoload 'texinfo-every-node-update "texnfo-upd" "\
Update every node in a Texinfo file." t nil)

(autoload 'texinfo-sequential-node-update "texnfo-upd" "\
Update one node (or many) in a Texinfo file with sequential pointers.

This function causes the `Next' or `Previous' pointer to point to the
immediately preceding or following node, even if it is at a higher or
lower hierarchical level in the document.  Continually pressing `n' or
`p' takes you straight through the file.

Without any prefix argument, update the node in which point is located.
Non-nil argument (prefix, if interactive) means update the nodes in the
marked region.

This command makes it awkward to navigate among sections and
subsections; it should be used only for those documents that are meant
to be read like a novel rather than a reference, and for which the
Info `g*' command is inadequate." t nil)

;;;***

;;;### (autoloads (time-stamp) "time-stamp" "packages/time-stamp.el" (12547 2415))
;;; Generated autoloads from packages/time-stamp.el

(autoload 'time-stamp "time-stamp" "\
Update the time stamp string in the buffer.
Only does its thing if the variable  time-stamp-active  is non-nil.
Typically used on  write-file-hooks  for automatic time-stamping.
The format of the time stamp is determined by the variable
time-stamp-format.  The first  time-stamp-line-limit  lines of the
buffer (normally 8) are searched for the time stamp template,
and if it is found, a new time stamp is written into it." t nil)

;;;***

;;;### (autoloads (display-time) "time" "packages/time.el" (12546 50463))
;;; Generated autoloads from packages/time.el

(defvar display-time-day-and-date nil "\
*Non-nil means \\[display-time] should display day and date as well as time.")

(autoload 'display-time "time" "\
Display current time, load level, and mail flag in mode line of each buffer.
Updates automatically every minute.
If `display-time-day-and-date' is non-nil, the current day and date
are displayed as well.
After each update, `display-time-hook' is run with `run-hooks'.
If `display-time-echo-area' is non-nil, the time is displayed in the
echo area instead of in the mode-line." t nil)

;;;***

;;;### (autoloads nil "uncompress" "packages/uncompress.el" (12547 2442))
;;; Generated autoloads from packages/uncompress.el

;;;### (autoloads (ununderline-and-unoverstrike-region overstrike-region unoverstrike-region ununderline-region underline-region) "underline" "packages/underline.el" (12546 50465))
;;; Generated autoloads from packages/underline.el

(autoload 'underline-region "underline" "\
Underline all nonblank characters in the region.
Works by overstriking underscores.
Called from program, takes two arguments START and END
which specify the range to operate on." t nil)

(autoload 'ununderline-region "underline" "\
Remove all underlining (overstruck underscores) in the region.
Called from program, takes two arguments START and END
which specify the range to operate on." t nil)

(autoload 'unoverstrike-region "underline" "\
Remove all overstriking (character-backspace-character) in the region.
Called from program, takes two arguments START and END which specify the
range to operate on." t nil)

(autoload 'overstrike-region "underline" "\
Overstrike (character-backspace-character) all nonblank characters in
the region. Called from program, takes two arguments START and END which
specify the range to operate on." t nil)

(autoload 'ununderline-and-unoverstrike-region "underline" "\
Remove underlining and overstriking in the region.  Called from a program,
takes two arguments START and END which specify the range to operate on." t nil)

;;;***

;;;### (autoloads (ask-to-update-copyright update-copyright) "upd-copyr" "packages/upd-copyr.el" (12546 50466))
;;; Generated autoloads from packages/upd-copyr.el

(defvar copyright-do-not-disturb "Free Software Foundation, Inc." "\
*If non-nil, the existing copyright holder is checked against this regexp.
If it does not match, then a new copyright line is added with the copyright
holder set to the value of `copyright-whoami'.")

(defvar copyright-whoami nil "\
*A string containing the name of the owner of new copyright notices.")

(defvar copyright-notice-file nil "\
*If non-nil, replace copying notices with this file.")

(autoload 'update-copyright "upd-copyr" "\
Update the copyright notice at the beginning of the buffer
to indicate the current year.  If optional arg REPLACE is given
\(interactively, with prefix arg) replace the years in the notice
rather than adding the current year after them.
If `copyright-notice-file' is set, the copying permissions following the
copyright are replaced as well.

If optional third argument ASK is non-nil, the user is prompted for whether
or not to update the copyright.  If optional fourth argument ASK-YEAR is
non-nil, the user is prompted for whether or not to replace the year rather
than adding to it." t nil)

(autoload 'ask-to-update-copyright "upd-copyr" "\
If the current buffer contains a copyright notice that is out of date,
ask the user if it should be updated with `update-copyright' (which see).
Put this on write-file-hooks." nil nil)

;;;***

;;;### (autoloads nil "vc-hooks" "packages/vc-hooks.el" (12728 13779))
;;; Generated autoloads from packages/vc-hooks.el

;;;***

;;;### (autoloads (vc-update-change-log vc-rename-file vc-cancel-version vc-revert-buffer vc-print-log vc-retrieve-snapshot vc-create-snapshot vc-directory vc-insert-headers vc-version-other-window vc-version-diff vc-diff vc-register vc-next-action vc-file-status) "vc" "packages/vc.el" (12675 57076))
;;; Generated autoloads from packages/vc.el

(defvar vc-checkin-hook nil "\
*List of functions called after a checkin is done.  See `run-hooks'.")

(autoload 'vc-file-status "vc" "\
Display the current status of the file being visited.
Currently, this is only defined for CVS.  The information provided in the
modeline is generally sufficient for RCS and SCCS." t nil)

(autoload 'vc-next-action "vc" "\
Do the next logical checkin or checkout operation on the current file.

For RCS and SCCS files:
   If the file is not already registered, this registers it for version
control and then retrieves a writable, locked copy for editing.
   If the file is registered and not locked by anyone, this checks out
a writable and locked file ready for editing.
   If the file is checked out and locked by the calling user, this
first checks to see if the file has changed since checkout.  If not,
it performs a revert.
   If the file has been changed, this pops up a buffer for entry
of a log message; when the message has been entered, it checks in the
resulting changes along with the log message as change commentary.  If
the variable `vc-keep-workfiles' is non-nil (which is its default), a
read-only copy of the changed file is left in place afterwards.
   If the file is registered and locked by someone else, you are given
the option to steal the lock.

For CVS files:
   If the file is not already registered, this registers it for version
control.  This does a \"cvs add\", but no \"cvs commit\".
   If the file is added but not committed, it is committed.
   If the file has not been changed, neither in your working area or
in the repository, a message is printed and nothing is done.
   If your working file is changed, but the repository file is
unchanged, this pops up a buffer for entry of a log message; when the
message has been entered, it checks in the resulting changes along
with the logmessage as change commentary.  A writable file is retained.
   If the repository file is changed, you are asked if you want to
merge in the changes into your working copy.

The following is true regardless of which version control system you
are using:

   If you call this from within a VC dired buffer with no files marked,
it will operate on the file in the current line.
   If you call this from within a VC dired buffer, and one or more
files are marked, it will accept a log message and then operate on
each one.  The log message will be used as a comment for any register
or checkin operations, but ignored when doing checkouts.  Attempted
lock steals will raise an error.

   For checkin, a prefix argument lets you specify the version number to use." t nil)

(autoload 'vc-register "vc" "\
Register the current file into your version-control system." t nil)

(autoload 'vc-diff "vc" "\
Display diffs between file versions.
Normally this compares the current file and buffer with the most recent 
checked in version of that file.  This uses no arguments.
With a prefix argument, it reads the file name to use
and two version designators specifying which versions to compare." t nil)

(autoload 'vc-version-diff "vc" "\
For FILE, report diffs between two stored versions REL1 and REL2 of it.
If FILE is a directory, generate diffs between versions for all registered
files in or below it." t nil)

(autoload 'vc-version-other-window "vc" "\
Visit version REV of the current buffer in another window.
If the current buffer is named `F', the version is named `F.~REV~'.
If `F.~REV~' already exists, it is used instead of being re-created." t nil)

(autoload 'vc-insert-headers "vc" "\
Insert headers in a file for use with your version-control system.
Headers desired are inserted at the start of the buffer, and are pulled from
the variable `vc-header-alist'." t nil)

(autoload 'vc-directory "vc" "\
Show version-control status of all files in the directory DIR.
If the second argument VERBOSE is non-nil, show all files;
otherwise show only files that current locked in the version control system.
Interactively, supply a prefix arg to make VERBOSE non-nil.

If the optional third argument NESTED is non-nil,
scan the entire tree of subdirectories of the current directory." t nil)

(autoload 'vc-create-snapshot "vc" "\
Make a snapshot called NAME.
The snapshot is made from all registered files at or below the current
directory.  For each file, the version level of its latest
version becomes part of the named configuration." t nil)

(autoload 'vc-retrieve-snapshot "vc" "\
Retrieve the snapshot called NAME.
This function fails if any files are locked at or below the current directory
Otherwise, all registered files are checked out (unlocked) at their version
levels in the snapshot." t nil)

(autoload 'vc-print-log "vc" "\
List the change log of the current buffer in a window." t nil)

(autoload 'vc-revert-buffer "vc" "\
Revert the current buffer's file back to the latest checked-in version.
This asks for confirmation if the buffer contents are not identical
to that version.
If the back-end is CVS, this will give you the most recent revision of
the file on the branch you are editing." t nil)

(autoload 'vc-cancel-version "vc" "\
Get rid of most recently checked in version of this file.
A prefix argument means do not revert the buffer afterwards." t nil)

(autoload 'vc-rename-file "vc" "\
Rename file OLD to NEW, and rename its master file likewise." t nil)

(autoload 'vc-update-change-log "vc" "\
Find change log file and add entries from recent RCS logs.
The mark is left at the end of the text prepended to the change log.
With prefix arg of C-u, only find log entries for the current buffer's file.
With any numeric prefix arg, find log entries for all files currently visited.
Otherwise, find log entries for all registered files in the default directory.
From a program, any arguments are passed to the `rcs2log' script." t nil)

;;;***

;;;### (autoloads nil "webster-ucb" "packages/webster-ucb.el" (12547 2446))
;;; Generated autoloads from packages/webster-ucb.el

;;;### (autoloads (webster-spell webster-endings webster) "webster" "packages/webster.el" (12547 2449))
;;; Generated autoloads from packages/webster.el

(autoload 'webster "webster" "\
Look up a word in the Webster's dictionary.
Open a network login connection to a webster host if necessary.
Communication with host is recorded in a buffer *webster*." t nil)

(autoload 'webster-endings "webster" "\
Look up endings for a word in the Webster's dictionary.
Open a network login connection to a webster host if necessary.
Communication with host is recorded in a buffer *webster*." t nil)

(autoload 'webster-spell "webster" "\
Look spelling for a word in the Webster's dictionary.
Open a network login connection to a webster host if necessary.
Communication with host is recorded in a buffer *webster*." t nil)

;;;***

;;;### (autoloads (run-scheme) "xscheme" "packages/xscheme.el" (12547 2448))
;;; Generated autoloads from packages/xscheme.el

(defvar scheme-program-name "scheme" "\
*Program invoked by the `run-scheme' command.")

(defvar scheme-band-name nil "\
*Band loaded by the `run-scheme' command.")

(defvar scheme-program-arguments nil "\
*Arguments passed to the Scheme program by the `run-scheme' command.")

(autoload 'run-scheme "xscheme" "\
Run an inferior Scheme process.
Output goes to the buffer `*scheme*'.
With argument, asks for a command line." t nil)

;;;***

;;;### (autoloads nil "compile-all" "pcl-cvs/compile-all.el" (12376 19549))
;;; Generated autoloads from pcl-cvs/compile-all.el

;;;***

;;;### (autoloads nil "cookie" "pcl-cvs/cookie.el" (12675 57120))
;;; Generated autoloads from pcl-cvs/cookie.el

;;;***

;;;### (autoloads nil "dll-debug" "pcl-cvs/dll-debug.el" (12675 57121))
;;; Generated autoloads from pcl-cvs/dll-debug.el

;;;***

;;;### (autoloads nil "dll" "pcl-cvs/dll.el" (12675 57122))
;;; Generated autoloads from pcl-cvs/dll.el

;;;***

;;;### (autoloads nil "elib-node" "pcl-cvs/elib-node.el" (12675 57122))
;;; Generated autoloads from pcl-cvs/elib-node.el

;;;***

;;;### (autoloads nil "pcl-cvs-lucid" "pcl-cvs/pcl-cvs-lucid.el" (12675 57126))
;;; Generated autoloads from pcl-cvs/pcl-cvs-lucid.el

;;;***

;;;### (autoloads nil "pcl-cvs-startup" "pcl-cvs/pcl-cvs-startup.el" (12675 57121))
;;; Generated autoloads from pcl-cvs/pcl-cvs-startup.el

;;;***

;;;### (autoloads (pcl-cvs-fontify) "pcl-cvs-xemacs" "pcl-cvs/pcl-cvs-xemacs.el" (12686 31800))
;;; Generated autoloads from pcl-cvs/pcl-cvs-xemacs.el

(autoload 'pcl-cvs-fontify "pcl-cvs-xemacs" nil nil nil)

;;;***

;;;### (autoloads (cvs-update-other-window cvs-update) "pcl-cvs" "pcl-cvs/pcl-cvs.el" (12686 31728))
;;; Generated autoloads from pcl-cvs/pcl-cvs.el

(autoload 'cvs-update "pcl-cvs" "\
Run a 'cvs update' in the current working directory.  Feed the
output to a *cvs* buffer and run cvs-mode on it.
If optional prefix argument LOCAL is non-nil, 'cvs update -l' is run." t nil)

(autoload 'cvs-update-other-window "pcl-cvs" "\
Run a 'cvs update' in the current working directory.  Feed the
output to a *cvs* buffer, display it in the other window, and run
cvs-mode on it.

If optional prefix argument LOCAL is non-nil, 'cvs update -l' is run." t nil)

;;;***

;;;### (autoloads nil "string" "pcl-cvs/string.el" (12675 57127))
;;; Generated autoloads from pcl-cvs/string.el

;;;***

;;;### (autoloads (about-xemacs) "about" "prim/about.el" (12748 7624))
;;; Generated autoloads from prim/about.el

(autoload 'about-xemacs "about" nil t nil)

;;;***

;;;### (autoloads (all-hail-emacs all-hail-xemacs praise-be-unto-emacs praise-be-unto-xemacs) "advocacy" "prim/advocacy.el" (12740 25220))
;;; Generated autoloads from prim/advocacy.el

(defvar xemacs-praise-sound-file "sounds/im_so_happy.au" "\
The name of an audio file containing something to play
when praising XEmacs")

(defvar xemacs-praise-message "All Hail XEmacs!\n" "\
What to praise XEmacs with")

(autoload 'praise-be-unto-xemacs "advocacy" "\
All Hail XEmacs!" t nil)

(autoload 'praise-be-unto-emacs "advocacy" nil t nil)

(autoload 'all-hail-xemacs "advocacy" "\
All Hail XEmacs!" t nil)

(autoload 'all-hail-emacs "advocacy" nil t nil)

;;;***

;;;### (autoloads nil "backquote" "prim/backquote.el" (12546 50509))
;;; Generated autoloads from prim/backquote.el

;;;***

;;;### (autoloads nil "buffer" "prim/buffer.el" (12639 8641))
;;; Generated autoloads from prim/buffer.el

;;;***

;;;### (autoloads (describe-buffer-case-table) "case-table" "prim/case-table.el" (12648 18583))
;;; Generated autoloads from prim/case-table.el

(autoload 'describe-buffer-case-table "case-table" "\
Describe the case table of the current buffer." t nil)

;;;***

;;;### (autoloads nil "cmdloop" "prim/cmdloop.el" (12648 18587))
;;; Generated autoloads from prim/cmdloop.el

;;;***

;;;### (autoloads nil "cmdloop1" "prim/cmdloop1.el" (12546 50501))
;;; Generated autoloads from prim/cmdloop1.el

;;;***

;;;### (autoloads nil "console" "prim/console.el" (12546 50529))
;;; Generated autoloads from prim/console.el

;;;### (autoloads nil "curmudgeon" "prim/curmudgeon.el" (12648 454))
;;; Generated autoloads from prim/curmudgeon.el

;;;### (autoloads (cancel-debug-on-entry debug-on-entry debug) "debug" "prim/debug.el" (12639 8609))
;;; Generated autoloads from prim/debug.el

(autoload 'debug "debug" "\
Enter debugger.  To return, type \\<debugger-mode-map>`\\[debugger-continue]'.
Arguments are mainly for use when this is called
 from the internals of the evaluator.
You may call with no args, or you may
 pass nil as the first arg and any other args you like.
 In that case, the list of args after the first will 
 be printed into the backtrace buffer." t nil)

(autoload 'debug-on-entry "debug" "\
Request FUNCTION to invoke debugger each time it is called.
If you tell the debugger to continue, FUNCTION's execution proceeds.
This works by modifying the definition of FUNCTION,
which must be written in Lisp, not predefined.
Use \\[cancel-debug-on-entry] to cancel the effect of this command.
Redefining FUNCTION also cancels it." t nil)

(autoload 'cancel-debug-on-entry "debug" "\
Undo effect of \\[debug-on-entry] on FUNCTION.
If argument is nil or an empty string, cancel for all functions." t nil)

;;;***

;;;### (autoloads nil "device" "prim/device.el" (12573 6019))
;;; Generated autoloads from prim/device.el

;;;***

;;;### (autoloads nil "dialog" "prim/dialog.el" (12573 6368))
;;; Generated autoloads from prim/dialog.el

;;;### (autoloads (standard-display-european standard-display-underline standard-display-graphic standard-display-g1 standard-display-ascii standard-display-default standard-display-8bit make-display-table describe-current-display-table) "disp-table" "prim/disp-table.el" (12648 18591))
;;; Generated autoloads from prim/disp-table.el

(autoload 'describe-current-display-table "disp-table" "\
Describe the display table in use in the selected window and buffer." t nil)

(autoload 'make-display-table "disp-table" "\
Return a new, empty display table." nil nil)

(autoload 'standard-display-8bit "disp-table" "\
Display characters in the range L to H literally." nil nil)

(autoload 'standard-display-default "disp-table" "\
Display characters in the range L to H using the default notation." nil nil)

(autoload 'standard-display-ascii "disp-table" "\
Display character C using printable string S." nil nil)

(autoload 'standard-display-g1 "disp-table" "\
Display character C as character SC in the g1 character set.
This function assumes that your terminal uses the SO/SI characters;
it is meaningless for an X frame." nil nil)

(autoload 'standard-display-graphic "disp-table" "\
Display character C as character GC in graphics character set.
This function assumes VT100-compatible escapes; it is meaningless for an
X frame." nil nil)

(autoload 'standard-display-underline "disp-table" "\
Display character C as character UC plus underlining." nil nil)

(autoload 'standard-display-european "disp-table" "\
Toggle display of European characters encoded with ISO 8859.
When enabled, characters in the range of 160 to 255 display not
as octal escapes, but as accented characters.
With prefix argument, enable European character display iff arg is positive." t nil)

;;;***

;;;### (autoloads (setenv) "env" "prim/env.el" (12546 50503))
;;; Generated autoloads from prim/env.el

(autoload 'setenv "env" "\
Set the value of the environment variable named VARIABLE to VALUE.
VARIABLE should be a string.  VALUE is optional; if not provided or is
`nil', the environment variable VARIABLE will be removed.  

Interactively, a prefix argument means to unset the variable.
Interactively, the current value (if any) of the variable
appears at the front of the history list when you type in the new value.

This function works by modifying `process-environment'." t nil)

;;;***

;;;### (autoloads nil "events" "prim/events.el" (12727 30162))
;;; Generated autoloads from prim/events.el

;;;***

;;;### (autoloads nil "extents" "prim/extents.el" (12639 8620))
;;; Generated autoloads from prim/extents.el

;;;***

;;;### (autoloads nil "faces" "prim/faces.el" (12743 11619))
;;; Generated autoloads from prim/faces.el

;;;***

;;;### (autoloads nil "files" "prim/files.el" (12743 11613))
;;; Generated autoloads from prim/files.el

;;;***

;;;### (autoloads nil "fill" "prim/fill.el" (12528 23894))
;;; Generated autoloads from prim/fill.el

;;;### (autoloads nil "float-sup" "prim/float-sup.el" (12546 50479))
;;; Generated autoloads from prim/float-sup.el

;;;***

;;;### (autoloads nil "format" "prim/format.el" (12648 18601))
;;; Generated autoloads from prim/format.el

;;;***

;;;### (autoloads nil "frame" "prim/frame.el" (12741 25956))
;;; Generated autoloads from prim/frame.el

;;;***

;;;### (autoloads nil "glyphs" "prim/glyphs.el" (12740 25218))
;;; Generated autoloads from prim/glyphs.el

;;;### (autoloads nil "gui" "prim/gui.el" (12648 18603))
;;; Generated autoloads from prim/gui.el

;;;***

;;;### (autoloads nil "help" "prim/help.el" (12747 13261))
;;; Generated autoloads from prim/help.el

;;;### (autoloads nil "inc-vers" "prim/inc-vers.el" (12546 50487))
;;; Generated autoloads from prim/inc-vers.el

;;;### (autoloads nil "indent" "prim/indent.el" (12648 18582))
;;; Generated autoloads from prim/indent.el

;;;***

;;;### (autoloads nil "isearch-mode" "prim/isearch-mode.el" (12698 33552))
;;; Generated autoloads from prim/isearch-mode.el

;;;### (autoloads nil "itimer" "prim/itimer.el" (12546 50498))
;;; Generated autoloads from prim/itimer.el

;;;***

;;;### (autoloads nil "keydefs" "prim/keydefs.el" (12639 8614))
;;; Generated autoloads from prim/keydefs.el

;;;***

;;;### (autoloads nil "keymap" "prim/keymap.el" (12558 60933))
;;; Generated autoloads from prim/keymap.el

;;;***

;;;### (autoloads nil "lisp" "prim/lisp.el" (12546 50478))
;;; Generated autoloads from prim/lisp.el

;;;***

;;;### (autoloads nil "loaddefs" "prim/loaddefs.el" (12748 8002))
;;; Generated autoloads from prim/loaddefs.el

;;;***

;;;### (autoloads nil "loadup-el" "prim/loadup-el.el" (12639 8618))
;;; Generated autoloads from prim/loadup-el.el

;;;### (autoloads nil "loadup" "prim/loadup.el" (12728 8250))
;;; Generated autoloads from prim/loadup.el

;;;### (autoloads (apply-macro-to-region-lines kbd-macro-query insert-kbd-macro name-last-kbd-macro) "macros" "prim/macros.el" (12546 50477))
;;; Generated autoloads from prim/macros.el

(autoload 'name-last-kbd-macro "macros" "\
Assign a name to the last keyboard macro defined.
Argument SYMBOL is the name to define.
The symbol's function definition becomes the keyboard macro string.
Such a \"function\" cannot be called from Lisp, but it is a valid
editor command." t nil)

(autoload 'insert-kbd-macro "macros" "\
Insert in buffer the definition of kbd macro NAME, as Lisp code.
Optional second argument KEYS means also record the keys it is on
\(this is the prefix argument, when calling interactively).

This Lisp code will, when executed, define the kbd macro with the
same definition it has now.  If you say to record the keys,
the Lisp code will also rebind those keys to the macro.
Only global key bindings are recorded since executing this Lisp code
always makes global bindings.

To save a kbd macro, visit a file of Lisp code such as your `~/.emacs',
use this command, and then save the file." t nil)

(autoload 'kbd-macro-query "macros" "\
Query user during kbd macro execution.
With prefix argument, enters recursive edit,
 reading keyboard commands even within a kbd macro.
 You can give different commands each time the macro executes.
Without prefix argument, asks whether to continue running the macro.
Your options are: \\<query-replace-map>
\\[act]	Finish this iteration normally and continue with the next.
\\[skip]	Skip the rest of this iteration, and start the next.
\\[exit]	Stop the macro entirely right now.
\\[recenter]	Redisplay the frame, then ask again.
\\[edit]	Enter recursive edit; ask again when you exit from that." t nil)

(autoload 'apply-macro-to-region-lines "macros" "\
For each complete line between point and mark, move to the beginning
of the line, and run the last keyboard macro.

When called from lisp, this function takes two arguments TOP and
BOTTOM, describing the current region.  TOP must be before BOTTOM.
The optional third argument MACRO specifies a keyboard macro to
execute.

This is useful for quoting or unquoting included text, adding and
removing comments, or producing tables where the entries are regular.

For example, in Usenet articles, sections of text quoted from another
author are indented, or have each line start with `>'.  To quote a
section of text, define a keyboard macro which inserts `>', put point
and mark at opposite ends of the quoted section, and use
`\\[apply-macro-to-region-lines]' to mark the entire section.

Suppose you wanted to build a keyword table in C where each entry
looked like this:

    { \"foo\", foo_data, foo_function }, 
    { \"bar\", bar_data, bar_function },
    { \"baz\", baz_data, baz_function },

You could enter the names in this format:

    foo
    bar
    baz

and write a macro to massage a word into a table entry:

    \\C-x (
       \\M-d { \"\\C-y\", \\C-y_data, \\C-y_function },
    \\C-x )

and then select the region of un-tablified names and use
`\\[apply-macro-to-region-lines]' to build the table from the names.
" t nil)

;;;***

;;;### (autoloads nil "menubar" "prim/menubar.el" (12648 18588))
;;; Generated autoloads from prim/menubar.el

;;;***

;;;### (autoloads nil "minibuf" "prim/minibuf.el" (12740 25211))
;;; Generated autoloads from prim/minibuf.el

;;;***

;;;### (autoloads nil "misc" "prim/misc.el" (12558 60927))
;;; Generated autoloads from prim/misc.el

;;;### (autoloads nil "mode-motion" "prim/mode-motion.el" (12657 40715))
;;; Generated autoloads from prim/mode-motion.el

;;;***

;;;### (autoloads nil "modeline" "prim/modeline.el" (12730 62103))
;;; Generated autoloads from prim/modeline.el

;;;***

;;;### (autoloads nil "mouse" "prim/mouse.el" (12744 55115))
;;; Generated autoloads from prim/mouse.el

;;;### (autoloads (disable-command enable-command disabled-command-hook) "novice" "prim/novice.el" (12546 50489))
;;; Generated autoloads from prim/novice.el

(autoload 'disabled-command-hook "novice" nil nil nil)

(autoload 'enable-command "novice" "\
Allow COMMAND to be executed without special confirmation from now on.
The user's .emacs file is altered so that this will apply
to future sessions." t nil)

(autoload 'disable-command "novice" "\
Require special confirmation to execute COMMAND from now on.
The user's .emacs file is altered so that this will apply
to future sessions." t nil)

;;;***

;;;### (autoloads nil "objects" "prim/objects.el" (12573 6927))
;;; Generated autoloads from prim/objects.el

;;;***

;;;### (autoloads nil "obsolete" "prim/obsolete.el" (12698 33559))
;;; Generated autoloads from prim/obsolete.el

;;;***

;;;### (autoloads (edit-options list-options) "options" "prim/options.el" (12546 50481))
;;; Generated autoloads from prim/options.el

(autoload 'list-options "options" "\
Display a list of XEmacs user options, with values and documentation." t nil)

(autoload 'edit-options "options" "\
Edit a list of XEmacs user option values.
Selects a buffer containing such a list,
in which there are commands to set the option values.
Type \\[describe-mode] in that buffer for a list of commands." t nil)

;;;***

;;;### (autoloads nil "page" "prim/page.el" (12546 50475))
;;; Generated autoloads from prim/page.el

;;;***

;;;### (autoloads nil "paragraphs" "prim/paragraphs.el" (12558 60931))
;;; Generated autoloads from prim/paragraphs.el

;;;***

;;;### (autoloads nil "process" "prim/process.el" (12727 30162))
;;; Generated autoloads from prim/process.el

;;;### (autoloads nil "profile" "prim/profile.el" (12607 29210))
;;; Generated autoloads from prim/profile.el

;;;***

;;;### (autoloads (clear-rectangle string-rectangle open-rectangle insert-rectangle yank-rectangle kill-rectangle extract-rectangle delete-extract-rectangle delete-rectangle) "rect" "prim/rect.el" (12546 50488))
;;; Generated autoloads from prim/rect.el

(autoload 'delete-rectangle "rect" "\
Delete (don't save) text in rectangle with point and mark as corners.
The same range of columns is deleted in each line
starting with the line where the region begins
and ending with the line where the region ends." t nil)

(autoload 'delete-extract-rectangle "rect" "\
Delete contents of rectangle and return it as a list of strings.
Arguments START and END are the corners of the rectangle.
The value is list of strings, one for each line of the rectangle." nil nil)

(autoload 'extract-rectangle "rect" "\
Return contents of rectangle with corners at START and END.
Value is list of strings, one for each line of the rectangle." nil nil)

(defvar killed-rectangle nil "\
Rectangle for yank-rectangle to insert.")

(autoload 'kill-rectangle "rect" "\
Delete rectangle with corners at point and mark; save as last killed one.
Calling from program, supply two args START and END, buffer positions.
But in programs you might prefer to use `delete-extract-rectangle'." t nil)

(autoload 'yank-rectangle "rect" "\
Yank the last killed rectangle with upper left corner at point." t nil)

(autoload 'insert-rectangle "rect" "\
Insert text of RECTANGLE with upper left corner at point.
RECTANGLE's first line is inserted at point,
its second line is inserted at a point vertically under point, etc.
RECTANGLE should be a list of strings.
After this command, the mark is at the upper left corner
and point is at the lower right corner." nil nil)

(autoload 'open-rectangle "rect" "\
Blank out rectangle with corners at point and mark, shifting text right.
The text previously in the region is not overwritten by the blanks,
but instead winds up to the right of the rectangle." t nil)

(autoload 'string-rectangle "rect" "\
Insert STRING on each line of the region-rectangle, shifting text right.
The left edge of the rectangle specifies the column for insertion.
This command does not delete or overwrite any existing text.

Called from a program, takes three args; START, END and STRING." t nil)

(autoload 'clear-rectangle "rect" "\
Blank out rectangle with corners at point and mark.
The text previously in the region is overwritten by the blanks.
When called from a program, requires two args which specify the corners." t nil)

;;;***

;;;### (autoloads nil "register" "prim/register.el" (12540 24673))
;;; Generated autoloads from prim/register.el

;;;***

;;;### (autoloads nil "replace" "prim/replace.el" (12740 25219))
;;; Generated autoloads from prim/replace.el

;;;***

;;;### (autoloads (reposition-window) "reposition" "prim/reposition.el" (12546 50489))
;;; Generated autoloads from prim/reposition.el

(autoload 'reposition-window "reposition" "\
Make the current definition and/or comment visible.
Further invocations move it to the top of the window or toggle the
visibility of comments that precede it.
  Point is left unchanged unless prefix ARG is supplied.
  If the definition is fully onscreen, it is moved to the top of the
window.  If it is partly offscreen, the window is scrolled to get the
definition (or as much as will fit) onscreen, unless point is in a comment
which is also partly offscreen, in which case the scrolling attempts to get
as much of the comment onscreen as possible.
  Initially `reposition-window' attempts to make both the definition and
preceding comments visible.  Further invocations toggle the visibility of
the comment lines.
  If ARG is non-nil, point may move in order to make the whole defun
visible (if only part could otherwise be made so), to make the defun line
visible (if point is in code and it could not be made so, or if only
comments, including the first comment line, are visible), or to make the
first comment line visible (if point is in a comment)." t nil)

;;;***

;;;### (autoloads nil "scrollbar" "prim/scrollbar.el" (12546 50512))
;;; Generated autoloads from prim/scrollbar.el

;;;***

;;;### (autoloads nil "simple" "prim/simple.el" (12690 2179))
;;; Generated autoloads from prim/simple.el

;;;### (autoloads (reverse-region sort-columns sort-regexp-fields sort-fields sort-float-fields sort-numeric-fields sort-pages sort-paragraphs sort-lines sort-subr) "sort" "prim/sort.el" (12546 50495))
;;; Generated autoloads from prim/sort.el

(autoload 'sort-subr "sort" "\
General text sorting routine to divide buffer into records and sort them.
Arguments are REVERSE NEXTRECFUN ENDRECFUN &optional STARTKEYFUN ENDKEYFUN.

We divide the accessible portion of the buffer into disjoint pieces
called sort records.  A portion of each sort record (perhaps all of
it) is designated as the sort key.  The records are rearranged in the
buffer in order by their sort keys.  The records may or may not be
contiguous.

Usually the records are rearranged in order of ascending sort key.
If REVERSE is non-nil, they are rearranged in order of descending sort key.

The next four arguments are functions to be called to move point
across a sort record.  They will be called many times from within sort-subr.

NEXTRECFUN is called with point at the end of the previous record.
It moves point to the start of the next record.
It should move point to the end of the buffer if there are no more records.
The first record is assumed to start at the position of point when sort-subr
is called.

ENDRECFUN is called with point within the record.
It should move point to the end of the record.

STARTKEYFUN moves from the start of the record to the start of the key.
It may return either a non-nil value to be used as the key, or
else the key is the substring between the values of point after
STARTKEYFUN and ENDKEYFUN are called.  If STARTKEYFUN is nil, the key
starts at the beginning of the record.

ENDKEYFUN moves from the start of the sort key to the end of the sort key.
ENDKEYFUN may be nil if STARTKEYFUN returns a value or if it would be the
same as ENDRECFUN." nil nil)

(autoload 'sort-lines "sort" "\
Sort lines in region alphabetically; argument means descending order.
Called from a program, there are three arguments:
REVERSE (non-nil means reverse order), BEG and END (region to sort)." t nil)

(autoload 'sort-paragraphs "sort" "\
Sort paragraphs in region alphabetically; argument means descending order.
Called from a program, there are three arguments:
REVERSE (non-nil means reverse order), BEG and END (region to sort)." t nil)

(autoload 'sort-pages "sort" "\
Sort pages in region alphabetically; argument means descending order.
Called from a program, there are three arguments:
REVERSE (non-nil means reverse order), BEG and END (region to sort)." t nil)

(autoload 'sort-numeric-fields "sort" "\
Sort lines in region numerically by the ARGth field of each line.
Fields are separated by whitespace and numbered from 1 up.
Specified field must contain a number in each line of the region.
With a negative arg, sorts by the ARGth field counted from the right.
Called from a program, there are three arguments:
FIELD, BEG and END.  BEG and END specify region to sort.
If you want to sort floating-point numbers, try `sort-float-fields'." t nil)

(autoload 'sort-float-fields "sort" "\
Sort lines in region numerically by the ARGth field of each line.
Fields are separated by whitespace and numbered from 1 up.  Specified field
must contain a floating point number in each line of the region.  With a
negative arg, sorts by the ARGth field counted from the right.  Called from a
program, there are three arguments: FIELD, BEG and END.  BEG and END specify
region to sort." t nil)

(autoload 'sort-fields "sort" "\
Sort lines in region lexicographically by the ARGth field of each line.
Fields are separated by whitespace and numbered from 1 up.
With a negative arg, sorts by the ARGth field counted from the right.
Called from a program, there are three arguments:
FIELD, BEG and END.  BEG and END specify region to sort." t nil)

(autoload 'sort-regexp-fields "sort" "\
Sort the region lexicographically as specified by RECORD-REGEXP and KEY.
RECORD-REGEXP specifies the textual units which should be sorted.
  For example, to sort lines RECORD-REGEXP would be \"^.*$\"
KEY specifies the part of each record (ie each match for RECORD-REGEXP)
  is to be used for sorting.
  If it is \"\\\\digit\" then the digit'th \"\\\\(...\\\\)\" match field from
  RECORD-REGEXP is used.
  If it is \"\\\\&\" then the whole record is used.
  Otherwise, it is a regular-expression for which to search within the record.
If a match for KEY is not found within a record then that record is ignored.

With a negative prefix arg sorts in reverse order.

For example: to sort lines in the region by the first word on each line
 starting with the letter \"f\",
 RECORD-REGEXP would be \"^.*$\" and KEY would be \"\\\\=\\<f\\\\w*\\\\>\"" t nil)

(autoload 'sort-columns "sort" "\
Sort lines in region alphabetically by a certain range of columns.
For the purpose of this command, the region includes
the entire line that point is in and the entire line the mark is in.
The column positions of point and mark bound the range of columns to sort on.
A prefix argument means sort into reverse order.

Note that `sort-columns' rejects text that contains tabs,
because tabs could be split across the specified columns
and it doesn't know how to handle that.  Also, when possible,
it uses the `sort' utility program, which doesn't understand tabs.
Use \\[untabify] to convert tabs to spaces before sorting." t nil)

(autoload 'reverse-region "sort" "\
Reverse the order of lines in a region.
From a program takes two point or marker arguments, BEG and END." t nil)

;;;***

;;;### (autoloads (load-default-sounds load-sound-file) "sound" "prim/sound.el" (12546 50494))
;;; Generated autoloads from prim/sound.el

(or sound-alist (setq sound-alist '((ready nil) (warp nil))))

(autoload 'load-sound-file "sound" "\
Read in an audio-file and add it to the sound-alist.

You can only play sound files if you are running on display 0 of the console
of a Sun SparcStation, SGI machine, or HP9000s700, or running a NetAudio
server.  The sound file must be in the Sun/NeXT U-LAW format." t nil)

(autoload 'load-default-sounds "sound" "\
Load and install some sound files as beep-types.
This only works if you're on display 0 of a Sun SparcStation, SGI machine,
or HP9000s700, or running a NetAudio server." t nil)

;;;***

;;;### (autoloads nil "specifier" "prim/specifier.el" (12639 8622))
;;; Generated autoloads from prim/specifier.el

;;;***

;;;### (autoloads nil "startup" "prim/startup.el" (12740 25213))
;;; Generated autoloads from prim/startup.el

;;;***

;;;### (autoloads nil "subr" "prim/subr.el" (12639 8613))
;;; Generated autoloads from prim/subr.el

;;;***

;;;### (autoloads nil "symbols" "prim/symbols.el" (12546 50529))
;;; Generated autoloads from prim/symbols.el

;;;### (autoloads nil "syntax" "prim/syntax.el" (12546 50506))
;;; Generated autoloads from prim/syntax.el

;;;### (autoloads (tabify untabify) "tabify" "prim/tabify.el" (12639 8615))
;;; Generated autoloads from prim/tabify.el

(autoload 'untabify "tabify" "\
Convert all tabs in region to multiple spaces, preserving columns.
Called non-interactively, the region is specified by arguments
START and END, rather than by the position of point and mark.
The variable `tab-width' controls the spacing of tab stops." t nil)

(autoload 'tabify "tabify" "\
Convert multiple spaces in region to tabs when possible.
A group of spaces is partially replaced by tabs
when this can be done without changing the column they end at.
Called non-interactively, the region is specified by arguments
START and END, rather than by the position of point and mark.
The variable `tab-width' controls the spacing of tab stops." t nil)

;;;***

;;;### (autoloads nil "toolbar" "prim/toolbar.el" (12573 6011))
;;; Generated autoloads from prim/toolbar.el

;;;***

;;;### (autoloads nil "undo-stack" "prim/undo-stack.el" (12639 8852))
;;; Generated autoloads from prim/undo-stack.el

;;;***

;;;### (autoloads nil "update-elc" "prim/update-elc.el" (12546 50504))
;;; Generated autoloads from prim/update-elc.el

;;;***

;;;### (autoloads (ask-user-about-supersession-threat ask-user-about-lock) "userlock" "prim/userlock.el" (12639 8617))
;;; Generated autoloads from prim/userlock.el

(autoload 'ask-user-about-lock "userlock" "\
Ask user what to do when he wants to edit FILE but it is locked by USER.
This function has a choice of three things to do:
  do (signal 'file-locked (list FILE USER))
    to refrain from editing the file
  return t (grab the lock on the file)
  return nil (edit the file even though it is locked).
You can rewrite it to use any criterion you like to choose which one to do." nil nil)

(autoload 'ask-user-about-supersession-threat "userlock" "\
Ask a user who is about to modify an obsolete buffer what to do.
This function has two choices: it can return, in which case the modification
of the buffer will proceed, or it can (signal 'file-supersession (file)),
in which case the proposed buffer modification will not be made.

You can rewrite this to use any criterion you like to choose which one to do.
The buffer in question is current when this function is called." nil nil)

;;;***

;;;### (autoloads nil "window" "prim/window.el" (12657 40725))
;;; Generated autoloads from prim/window.el

;;;***

;;;### (autoloads nil "iso-sgml" "psgml/iso-sgml.el" (12639 8920))
;;; Generated autoloads from psgml/iso-sgml.el

;;;***

;;;### (autoloads nil "psgml-api" "psgml/psgml-api.el" (12639 8921))
;;; Generated autoloads from psgml/psgml-api.el

;;;***

;;;### (autoloads nil "psgml-charent" "psgml/psgml-charent.el" (12639 8921))
;;; Generated autoloads from psgml/psgml-charent.el

;;;***

;;;### (autoloads nil "psgml-debug" "psgml/psgml-debug.el" (12639 8922))
;;; Generated autoloads from psgml/psgml-debug.el

;;;***

;;;### (autoloads nil "psgml-dtd" "psgml/psgml-dtd.el" (12639 8923))
;;; Generated autoloads from psgml/psgml-dtd.el

;;;***

;;;### (autoloads nil "psgml-edit" "psgml/psgml-edit.el" (12648 18637))
;;; Generated autoloads from psgml/psgml-edit.el

;;;### (autoloads (style-format) "psgml-fs" "psgml/psgml-fs.el" (12648 18670))
;;; Generated autoloads from psgml/psgml-fs.el

(autoload 'style-format "psgml-fs" nil t nil)

;;;***

;;;### (autoloads nil "psgml-html" "psgml/psgml-html.el" (12657 40787))
;;; Generated autoloads from psgml/psgml-html.el

(autoload 'html-mode "psgml-html" "\
HTML mode." t)

(autoload 'html3-mode "psgml-html" "\
HTML3 mode." t)

;;;***

;;;### (autoloads nil "psgml-info" "psgml/psgml-info.el" (12639 8925))
;;; Generated autoloads from psgml/psgml-info.el

;;;***

;;;### (autoloads nil "psgml-other" "psgml/psgml-other.el" (12639 8926))
;;; Generated autoloads from psgml/psgml-other.el

;;;***

;;;### (autoloads nil "psgml-parse" "psgml/psgml-parse.el" (12639 8928))
;;; Generated autoloads from psgml/psgml-parse.el

;;;***

;;;### (autoloads nil "psgml-style" "psgml/psgml-style.el" (12648 18671))
;;; Generated autoloads from psgml/psgml-style.el

;;;### (autoloads (tmpl-expand-templates-in-buffer tmpl-expand-templates-in-region) "psgml-tmpl-mode" "psgml/psgml-tmpl-mode.el" (12638 54483))
;;; Generated autoloads from psgml/psgml-tmpl-mode.el

(autoload 'tmpl-expand-templates-in-region "psgml-tmpl-mode" "\
Expand the templates in the region from BEGIN to END.
If BEGIN and and are nil, then the current region is used." t nil)

(autoload 'tmpl-expand-templates-in-buffer "psgml-tmpl-mode" "\
Expand all templates in the current buffer." t nil)

;;;***

;;;### (autoloads nil "psgml-xemacs" "psgml/psgml-xemacs.el" (12648 18639))
;;; Generated autoloads from psgml/psgml-xemacs.el

;;;### (autoloads (sgml-mode) "psgml" "psgml/psgml.el" (12648 18638))
;;; Generated autoloads from psgml/psgml.el

(autoload 'sgml-mode "psgml" "\
Major mode for editing SGML.\\<sgml-mode-map>
Makes > display the matching <.  Makes / display matching /.
Use \\[sgml-validate] to validate your document with an SGML parser.

You can find information with:
\\[sgml-show-context]  Show the nesting of elements at cursor position.
\\[sgml-list-valid-tags]  Show the tags valid at cursor position.

Insert tags with completion of contextually valid tags with \\[sgml-insert-tag].
End the current element with \\[sgml-insert-end-tag].  Insert an element (i.e.
both start and end tag) with \\[sgml-insert-element].  Or tag a region with 
\\[sgml-tag-region]. 

To tag a region with the mouse, use transient mark mode or secondary selection.

Structure editing:
\\[sgml-backward-element]  Moves backwards over the previous element.
\\[sgml-forward-element]  Moves forward over the nex element.
\\[sgml-down-element]  Move forward and down one level in the element structure.
\\[sgml-backward-up-element]  Move backward out of this element level.
\\[sgml-beginning-of-element]  Move to after the start tag of the current element.
\\[sgml-end-of-element]  Move to before the end tag of the current element.
\\[sgml-kill-element]  Kill the element following the cursor.

Finding interesting positions
\\[sgml-next-data-field]  Move forward to next point where data is allowed.
\\[sgml-next-trouble-spot]  Move forward to next point where something is 
	amiss with the structure.

Folding and unfolding
\\[sgml-fold-element]  Fold the lines comprising the current element, leaving 
	the first line visible.
\\[sgml-fold-subelement]  Fold the elements in the content of the current element.
	Leaving the first line of every element visible.
\\[sgml-unfold-line]  Show hidden lines in current line.

User options:

sgml-omittag  Set this to reflect OMITTAG in the SGML declaration.
sgml-shortag  Set this to reflect SHORTTAG in the SGML declaration.
sgml-auto-insert-required-elements  If non-nil, automatically insert required 
	elements in the content of an inserted element.
sgml-balanced-tag-edit  If non-nil, always insert start-end tag pairs.
sgml-omittag-transparent  If non-nil, will show legal tags inside elements
	with omitable start tags and legal tags beyond omitable end tags.
sgml-leave-point-after-insert  If non-nil, the point will remain after 
	inserted tag(s).
sgml-warn-about-undefined-elements  If non-nil, print a warning when a tag 
	for a undefined element is found.
sgml-max-menu-size  Max number of entries in Tags and Entities menus before
 	they are split into several panes.
sgml-always-quote-attributes  If non-nil, quote all attribute values 
	inserted after finishing edit attributes.
sgml-minimize-attributes  Determines minimization of attributes inserted by 
	edit-attributes.
sgml-normalize-trims  If non-nil, sgml-normalize will trim off white space 
	from end of element when adding end tag.
sgml-indent-step  How much to increament indent for every element level.
sgml-indent-data  If non-nil, indent in data/mixed context also.
sgml-set-face     If non-nil, psgml will set the face of parsed markup.
sgml-markup-faces The faces used when the above variable is non-nil.
sgml-system-path  List of directorys used to look for system identifiers.
sgml-public-map  Mapping from public identifiers to file names.
sgml-offer-save  If non-nil, ask about saving modified buffers before
		\\[sgml-validate] is run.

All bindings:
\\{sgml-mode-map}
" t nil)

;;;***

;;;### (autoloads nil "tempo" "psgml/tempo.el" (12648 18672))
;;; Generated autoloads from psgml/tempo.el

;;;***

;;;### (autoloads nil "rmail-kill" "rmail/rmail-kill.el" (11903 15862))
;;; Generated autoloads from rmail/rmail-kill.el

;;;### (autoloads nil "rmail-lucid" "rmail/rmail-lucid.el" (12376 19553))
;;; Generated autoloads from rmail/rmail-lucid.el

;;;### (autoloads (rmail-input rmail-mode rmail) "rmail" "rmail/rmail.el" (12727 30166))
;;; Generated autoloads from rmail/rmail.el

(defvar rmail-dont-reply-to-names nil "\
*A regexp specifying names to prune of reply to messages.
A value of nil means exclude your own name only.")

(defvar rmail-default-dont-reply-to-names "info-" "\
A regular expression specifying part of the value of the default value of
the variable `rmail-dont-reply-to-names', for when the user does not set
`rmail-dont-reply-to-names' explicitly.  (The other part of the default
value is the user's name.)
It is useful to set this variable in the site customization file.")

(defvar rmail-delete-after-output nil "\
*Non-nil means automatically delete a message that is copied to a file.")

(defvar rmail-primary-inbox-list nil "\
*List of files which are inboxes for user's primary mail file `~/RMAIL'.
`nil' means the default, which is (\"/usr/spool/mail/$USER\")
\(the name varies depending on the operating system,
and the value of the environment variable MAIL overrides it).")

(defvar rmail-mail-new-frame nil "\
*Non-nil means Rmail makes a new frame for composing outgoing mail.")

(defvar rmail-retry-setup-hook nil "\
Hook that `rmail-retry-failure' uses in place of `mail-setup-hook'.")

(defvar rmail-last-file nil)

(autoload 'rmail "rmail" "\
Read and edit incoming mail.
Moves messages into file named by `rmail-file-name' (a babyl format file)
 and edits that file in RMAIL Mode.
Type \\[describe-mode] once editing that file, for a list of RMAIL commands.

May be called with filename as argument; then performs rmail editing on
that file, but does not copy any new mail into the file." t nil)

(autoload 'rmail-mode "rmail" "\
Rmail Mode is used by \\<rmail-mode-map>\\[rmail] for editing Rmail files.
All normal editing commands are turned off.
Instead, these commands are available:

\\[rmail-beginning-of-message]	Move point to front of this message (same as \\[beginning-of-buffer]).
\\[scroll-up]	Scroll to next screen of this message.
\\[scroll-down]	Scroll to previous screen of this message.
\\[rmail-next-undeleted-message]	Move to Next non-deleted message.
\\[rmail-previous-undeleted-message]	Move to Previous non-deleted message.
\\[rmail-next-message]	Move to Next message whether deleted or not.
\\[rmail-previous-message]	Move to Previous message whether deleted or not.
\\[rmail-first-message]	Move to the first message in Rmail file.
\\[rmail-last-message]	Move to the last message in Rmail file.
\\[rmail-show-message]	Jump to message specified by numeric position in file.
\\[rmail-search]	Search for string and show message it is found in.
\\[rmail-delete-forward]	Delete this message, move to next nondeleted.
\\[rmail-delete-backward]	Delete this message, move to previous nondeleted.
\\[rmail-undelete-previous-message]	Undelete message.  Tries current message, then earlier messages
	till a deleted message is found.
\\[rmail-edit-current-message]	Edit the current message.  \\[rmail-cease-edit] to return to Rmail.
\\[rmail-expunge]	Expunge deleted messages.
\\[rmail-expunge-and-save]	Expunge and save the file.
\\[rmail-quit]       Quit Rmail: expunge, save, then switch to another buffer.
\\[save-buffer] Save without expunging.
\\[rmail-get-new-mail]	Move new mail from system spool directory into this file.
\\[rmail-mail]	Mail a message (same as \\[mail-other-window]).
\\[rmail-continue]	Continue composing outgoing message started before.
\\[rmail-reply]	Reply to this message.  Like \\[rmail-mail] but initializes some fields.
\\[rmail-retry-failure]	Send this message again.  Used on a mailer failure message.
\\[rmail-forward]	Forward this message to another user.
\\[rmail-output-to-rmail-file]       Output this message to an Rmail file (append it).
\\[rmail-output]	Output this message to a Unix-format mail file (append it).
\\[rmail-input]	Input Rmail file.  Run Rmail on that file.
\\[rmail-add-label]	Add label to message.  It will be displayed in the mode line.
\\[rmail-kill-label]	Kill label.  Remove a label from current message.
\\[rmail-next-labeled-message]   Move to Next message with specified label
          (label defaults to last one specified).
          Standard labels: filed, unseen, answered, forwarded, deleted.
          Any other label is present only if you add it with \\[rmail-add-label].
\\[rmail-previous-labeled-message]   Move to Previous message with specified label
\\[rmail-summary]	Show headers buffer, with a one line summary of each message.
\\[rmail-summary-by-labels]	Summarize only messages with particular label(s).
\\[rmail-summary-by-recipients]   Summarize only messages with particular recipient(s).
\\[rmail-summary-by-regexp]   Summarize only messages with particular regexp(s).
\\[rmail-summary-by-topic]   Summarize only messages with subject line regexp(s).
\\[rmail-toggle-header]	Toggle display of complete header." t nil)

(autoload 'rmail-input "rmail" "\
Run Rmail on file FILENAME." t nil)

;;;***

;;;### (autoloads nil "rmailedit" "rmail/rmailedit.el" (12376 19554))
;;; Generated autoloads from rmail/rmailedit.el

;;;***

;;;### (autoloads nil "rmailkwd" "rmail/rmailkwd.el" (12376 19555))
;;; Generated autoloads from rmail/rmailkwd.el

;;;***

;;;### (autoloads nil "rmailmsc" "rmail/rmailmsc.el" (12376 19555))
;;; Generated autoloads from rmail/rmailmsc.el

;;;### (autoloads (rmail-file-p) "rmailout" "rmail/rmailout.el" (12376 19556))
;;; Generated autoloads from rmail/rmailout.el

(autoload 'rmail-file-p "rmailout" nil nil nil)

;;;***

;;;### (autoloads nil "rmailsort" "rmail/rmailsort.el" (12376 19557))
;;; Generated autoloads from rmail/rmailsort.el

;;;***

;;;### (autoloads nil "rmailsum" "rmail/rmailsum.el" (12546 50532))
;;; Generated autoloads from rmail/rmailsum.el

;;;***

;;;### (autoloads nil "undigest" "rmail/undigest.el" (12546 50532))
;;; Generated autoloads from rmail/undigest.el

;;;***

;;;### (autoloads (unrmail batch-unrmail) "unrmail" "rmail/unrmail.el" (12147 25753))
;;; Generated autoloads from rmail/unrmail.el

(autoload 'batch-unrmail "unrmail" "\
Convert Rmail files to mailbox files.
Specify the input Rmail file names as command line arguments.
For each Rmail file, the corresponding output file name
is made by adding `.mail' at the end.
For example, invoke `emacs -batch -f batch-unrmail RMAIL'." nil nil)

(autoload 'unrmail "unrmail" "\
Convert Rmail file FILE to mailbox-format file TO-FILE." t nil)

;;;***

;;;### (autoloads nil "eserve" "sunpro/eserve.el" (12690 2205))
;;; Generated autoloads from sunpro/eserve.el

;;;***

;;;### (autoloads nil "sunpro-init" "sunpro/sunpro-init.el" (12740 25230))
;;; Generated autoloads from sunpro/sunpro-init.el

;;;### (autoloads nil "sunpro-keys" "sunpro/sunpro-keys.el" (11903 15927))
;;; Generated autoloads from sunpro/sunpro-keys.el

;;;***

;;;### (autoloads nil "sunpro-load" "sunpro/sunpro-load.el" (12256 51714))
;;; Generated autoloads from sunpro/sunpro-load.el

;;;***

;;;### (autoloads nil "sunpro-menubar" "sunpro/sunpro-menubar.el" (12740 25229))
;;; Generated autoloads from sunpro/sunpro-menubar.el

;;;***

;;;### (autoloads nil "sunpro-sparcworks" "sunpro/sunpro-sparcworks.el" (12740 25231))
;;; Generated autoloads from sunpro/sunpro-sparcworks.el

;;;***

;;;### (autoloads nil "workshop" "sunpro/workshop.el" (12690 2204))
;;; Generated autoloads from sunpro/workshop.el

;;;### (autoloads nil "AT386" "term/AT386.el" (12639 8852))
;;; Generated autoloads from term/AT386.el

;;;***

;;;### (autoloads nil "apollo" "term/apollo.el" (12639 8854))
;;; Generated autoloads from term/apollo.el

;;;***

;;;### (autoloads nil "bg-mouse" "term/bg-mouse.el" (12639 8854))
;;; Generated autoloads from term/bg-mouse.el

;;;***

;;;### (autoloads nil "bobcat" "term/bobcat.el" (12639 8855))
;;; Generated autoloads from term/bobcat.el

;;;***

;;;### (autoloads nil "internal" "term/internal.el" (12639 8855))
;;; Generated autoloads from term/internal.el

;;;***

;;;### (autoloads nil "keyswap" "term/keyswap.el" (12639 8855))
;;; Generated autoloads from term/keyswap.el

;;;***

;;;### (autoloads nil "linux" "term/linux.el" (12639 8856))
;;; Generated autoloads from term/linux.el

;;;***

;;;### (autoloads nil "lk201" "term/lk201.el" (12639 8857))
;;; Generated autoloads from term/lk201.el

;;;***

;;;### (autoloads nil "news" "term/news.el" (12639 8642))
;;; Generated autoloads from term/news.el

;;;***

;;;### (autoloads nil "pc-win" "term/pc-win.el" (12639 8858))
;;; Generated autoloads from term/pc-win.el

;;;***

;;;### (autoloads nil "scoansi" "term/scoansi.el" (12740 25282))
;;; Generated autoloads from term/scoansi.el

;;;### (autoloads nil "sun-mouse" "term/sun-mouse.el" (12639 8860))
;;; Generated autoloads from term/sun-mouse.el

;;;***

;;;### (autoloads nil "sun" "term/sun.el" (12639 8861))
;;; Generated autoloads from term/sun.el

;;;***

;;;### (autoloads nil "sup-mouse" "term/sup-mouse.el" (12639 8862))
;;; Generated autoloads from term/sup-mouse.el

;;;***

;;;### (autoloads nil "tty-init" "term/tty-init.el" (12698 33568))
;;; Generated autoloads from term/tty-init.el

;;;### (autoloads nil "tvi970" "term/tvi970.el" (12639 8863))
;;; Generated autoloads from term/tvi970.el

;;;### (autoloads nil "vt-control" "term/vt-control.el" (12376 19561))
;;; Generated autoloads from term/vt-control.el

;;;***

;;;### (autoloads nil "vt100-led" "term/vt100-led.el" (12376 19560))
;;; Generated autoloads from term/vt100-led.el

;;;***

;;;### (autoloads nil "vt100" "term/vt100.el" (12639 8643))
;;; Generated autoloads from term/vt100.el

;;;***

;;;### (autoloads nil "vt102" "term/vt102.el" (12639 8864))
;;; Generated autoloads from term/vt102.el

;;;***

;;;### (autoloads nil "vt125" "term/vt125.el" (12639 8864))
;;; Generated autoloads from term/vt125.el

;;;***

;;;### (autoloads nil "vt200" "term/vt200.el" (12639 8865))
;;; Generated autoloads from term/vt200.el

;;;***

;;;### (autoloads nil "vt201" "term/vt201.el" (12639 8866))
;;; Generated autoloads from term/vt201.el

;;;***

;;;### (autoloads nil "vt220" "term/vt220.el" (12639 8867))
;;; Generated autoloads from term/vt220.el

;;;***

;;;### (autoloads nil "vt240" "term/vt240.el" (12639 8867))
;;; Generated autoloads from term/vt240.el

;;;***

;;;### (autoloads nil "vt300" "term/vt300.el" (12639 8867))
;;; Generated autoloads from term/vt300.el

;;;***

;;;### (autoloads nil "vt320" "term/vt320.el" (12639 8868))
;;; Generated autoloads from term/vt320.el

;;;***

;;;### (autoloads nil "vt400" "term/vt400.el" (12639 8868))
;;; Generated autoloads from term/vt400.el

;;;***

;;;### (autoloads nil "vt420" "term/vt420.el" (12639 8869))
;;; Generated autoloads from term/vt420.el

;;;***

;;;### (autoloads nil "win32-win" "term/win32-win.el" (12639 8869))
;;; Generated autoloads from term/win32-win.el

;;;***

;;;### (autoloads nil "wyse50" "term/wyse50.el" (12639 8870))
;;; Generated autoloads from term/wyse50.el

;;;***

;;;### (autoloads nil "xterm" "term/xterm.el" (12639 8870))
;;; Generated autoloads from term/xterm.el

;;;***

;;;### (autoloads nil "tooltalk-init" "tooltalk/tooltalk-init.el" (12714 11728))
;;; Generated autoloads from tooltalk/tooltalk-init.el

;;;***

;;;### (autoloads nil "tooltalk-load" "tooltalk/tooltalk-load.el" (12142 8533))
;;; Generated autoloads from tooltalk/tooltalk-load.el

;;;### (autoloads nil "tooltalk-macros" "tooltalk/tooltalk-macros.el" (12376 19613))
;;; Generated autoloads from tooltalk/tooltalk-macros.el

;;;***

;;;### (autoloads nil "tooltalk-util" "tooltalk/tooltalk-util.el" (12376 19611))
;;; Generated autoloads from tooltalk/tooltalk-util.el

;;;***

;;;### (autoloads nil "base64" "url/base64.el" (12675 57192))
;;; Generated autoloads from url/base64.el

;;;***

;;;### (autoloads nil "md5" "url/md5.el" (12740 27391))
;;; Generated autoloads from url/md5.el

;;;***

;;;### (autoloads nil "mm" "url/mm.el" (12715 10594))
;;; Generated autoloads from url/mm.el

;;;***

;;;### (autoloads nil "ssl" "url/ssl.el" (12715 10594))
;;; Generated autoloads from url/ssl.el

;;;***

;;;### (autoloads nil "url-cookie" "url/url-cookie.el" (12727 30991))
;;; Generated autoloads from url/url-cookie.el

;;;***

;;;### (autoloads nil "url-file" "url/url-file.el" (12740 27393))
;;; Generated autoloads from url/url-file.el

;;;***

;;;### (autoloads nil "url-gopher" "url/url-gopher.el" (12639 8908))
;;; Generated autoloads from url/url-gopher.el

;;;### (autoloads nil "url-hash" "url/url-hash.el" (12648 18631))
;;; Generated autoloads from url/url-hash.el

;;;***

;;;### (autoloads nil "url-http" "url/url-http.el" (12740 27393))
;;; Generated autoloads from url/url-http.el

;;;***

;;;### (autoloads nil "url-irc" "url/url-irc.el" (12740 27199))
;;; Generated autoloads from url/url-irc.el

;;;### (autoloads nil "url-mail" "url/url-mail.el" (12727 30990))
;;; Generated autoloads from url/url-mail.el

;;;***

;;;### (autoloads nil "url-misc" "url/url-misc.el" (12740 27393))
;;; Generated autoloads from url/url-misc.el

;;;***

;;;### (autoloads nil "url-news" "url/url-news.el" (12720 56537))
;;; Generated autoloads from url/url-news.el

;;;***

;;;### (autoloads nil "url-nfs" "url/url-nfs.el" (12740 27199))
;;; Generated autoloads from url/url-nfs.el

;;;***

;;;### (autoloads nil "url-parse" "url/url-parse.el" (12639 8912))
;;; Generated autoloads from url/url-parse.el

;;;### (autoloads nil "url-pgp" "url/url-pgp.el" (12714 11780))
;;; Generated autoloads from url/url-pgp.el

;;;***

;;;### (autoloads nil "url-sysdp" "url/url-sysdp.el" (12727 30990))
;;; Generated autoloads from url/url-sysdp.el

;;;***

;;;### (autoloads nil "url-vars" "url/url-vars.el" (12740 27394))
;;; Generated autoloads from url/url-vars.el

;;;***

;;;### (autoloads nil "url-wais" "url/url-wais.el" (12714 11783))
;;; Generated autoloads from url/url-wais.el

;;;### (autoloads (url-retrieve url-cache-expired url-popup-info url-get-url-at-point url-buffer-visiting url-normalize-url url-file-attributes) "url" "url/url.el" (12740 27391))
;;; Generated autoloads from url/url.el

(autoload 'url-file-attributes "url" "\
Return a list of attributes of URL.
Value is nil if specified file cannot be opened.
Otherwise, list elements are:
 0. t for directory, string (name linked to) for symbolic link, or nil.
 1. Number of links to file.
 2. File uid.
 3. File gid.
 4. Last access time, as a list of two integers.
  First integer has high-order 16 bits of time, second has low 16 bits.
 5. Last modification time, likewise.
 6. Last status change time, likewise.
 7. Size in bytes. (-1, if number is out of range).
 8. File modes, as a string of ten letters or dashes as in ls -l.
    If URL is on an http server, this will return the content-type if possible.
 9. t iff file's gid would change if file were deleted and recreated.
10. inode number.
11. Device number.

If file does not exist, returns nil." nil nil)

(autoload 'url-normalize-url "url" "\
Return a 'normalized' version of URL.  This strips out default port
numbers, etc." nil nil)

(autoload 'url-buffer-visiting "url" "\
Return the name of a buffer (if any) that is visiting URL." nil nil)

(autoload 'url-get-url-at-point "url" "\
Get the URL closest to point, but don't change your
position. Has a preference for looking backward when not
directly on a symbol." nil nil)

(autoload 'url-popup-info "url" "\
Retrieve the HTTP/1.0 headers and display them in a temp buffer." nil nil)

(autoload 'url-cache-expired "url" "\
Return t iff a cached file has expired." nil nil)

(autoload 'url-retrieve "url" "\
Retrieve a document over the World Wide Web.
The document should be specified by its fully specified
Uniform Resource Locator.  No parsing is done, just return the
document as the server sent it.  The document is left in the
buffer specified by url-working-buffer.  url-working-buffer is killed
immediately before starting the transfer, so that no buffer-local
variables interfere with the retrieval.  HTTP/1.0 redirection will
be honored before this function exits." nil nil)

;;;***

;;;### (autoloads nil "urlauth" "url/urlauth.el" (12639 8898))
;;; Generated autoloads from url/urlauth.el

;;;***

;;;### (autoloads nil "abbrevlist" "utils/abbrevlist.el" (12559 34915))
;;; Generated autoloads from utils/abbrevlist.el

;;;### (autoloads (defadvice ad-add-advice) "advice" "utils/advice.el" (12559 34922))
;;; Generated autoloads from utils/advice.el

(defvar ad-redefinition-action 'warn "\
*Defines what to do with redefinitions during Advice de/activation.
Redefinition occurs if a previously activated function that already has an
original definition associated with it gets redefined and then de/activated.
In such a case we can either accept the current definition as the new
original definition, discard the current definition and replace it with the
old original, or keep it and raise an error.  The values `accept', `discard',
`error' or `warn' govern what will be done.  `warn' is just like `accept' but
it additionally prints a warning message.  All other values will be
interpreted as `error'.")

(defvar ad-default-compilation-action 'maybe "\
*Defines whether to compile advised definitions during activation.
A value of `always' will result in unconditional compilation, `never' will
always avoid compilation, `maybe' will compile if the byte-compiler is already
loaded, and `like-original' will compile if the original definition of the
advised function is compiled or a built-in function. Every other value will
be interpreted as `maybe'. This variable will only be considered if the 
COMPILE argument of `ad-activate' was supplied as nil.")

(autoload 'ad-add-advice "advice" "\
Adds a piece of ADVICE to FUNCTION's list of advices in CLASS.
If FUNCTION already has one or more pieces of advice of the specified
CLASS then POSITION determines where the new piece will go.  The value
of POSITION can either be `first', `last' or a number where 0 corresponds
to `first'.  Numbers outside the range will be mapped to the closest
extreme position.  If there was already a piece of ADVICE with the same
name, then the position argument will be ignored and the old advice
will be overwritten with the new one.
    If the FUNCTION was not advised already, then its advice info will be 
initialized.  Redefining a piece of advice whose name is part of the cache-id
will clear the cache." nil nil)

(autoload 'defadvice "advice" "\
Defines a piece of advice for FUNCTION (a symbol).
The syntax of `defadvice' is as follows:

  (defadvice FUNCTION (CLASS NAME [POSITION] [ARGLIST] FLAG...)
    [DOCSTRING] [INTERACTIVE-FORM]
    BODY... )

FUNCTION ::= Name of the function to be advised.
CLASS ::= `before' | `around' | `after' | `activation' | `deactivation'.
NAME ::= Non-nil symbol that names this piece of advice.
POSITION ::= `first' | `last' | NUMBER. Optional, defaults to `first',
    see also `ad-add-advice'.
ARGLIST ::= An optional argument list to be used for the advised function
    instead of the argument list of the original.  The first one found in
    before/around/after-advices will be used.
FLAG ::= `protect'|`disable'|`activate'|`compile'|`preactivate'|`freeze'.
    All flags can be specified with unambiguous initial substrings.
DOCSTRING ::= Optional documentation for this piece of advice.
INTERACTIVE-FORM ::= Optional interactive form to be used for the advised
    function.  The first one found in before/around/after-advices will be used.
BODY ::= Any s-expression.

Semantics of the various flags:
`protect': The piece of advice will be protected against non-local exits in
any code that precedes it.  If any around-advice of a function is protected
then automatically all around-advices will be protected (the complete onion).

`activate': All advice of FUNCTION will be activated immediately if
FUNCTION has been properly defined prior to this application of `defadvice'.

`compile': In conjunction with `activate' specifies that the resulting
advised function should be compiled.

`disable': The defined advice will be disabled, hence, it will not be used 
during activation until somebody enables it.

`preactivate': Preactivates the advised FUNCTION at macro-expansion/compile
time.  This generates a compiled advised definition according to the current
advice state that will be used during activation if appropriate.  Only use
this if the `defadvice' gets actually compiled.

`freeze': Expands the `defadvice' into a redefining `defun/defmacro' according
to this particular single advice.  No other advice information will be saved.
Frozen advices cannot be undone, they behave like a hard redefinition of
the advised function.  `freeze' implies `activate' and `preactivate'.  The
documentation of the advised function can be dumped onto the `DOC' file
during preloading.

Look at the file `advice.el' for comprehensive documentation." nil 'macro)

;;;***

;;;### (autoloads nil "assoc" "utils/assoc.el" (12559 34934))
;;; Generated autoloads from utils/assoc.el

;;;***

;;;### (autoloads nil "atomic-extents" "utils/atomic-extents.el" (12639 8649))
;;; Generated autoloads from utils/atomic-extents.el

;;;***

;;;### (autoloads (batch-update-autoloads update-directory-autoloads update-autoloads-here update-file-autoloads generate-file-autoloads) "autoload" "utils/autoload.el" (12639 8647))
;;; Generated autoloads from utils/autoload.el

(autoload 'generate-file-autoloads "autoload" "\
Insert at point a loaddefs autoload section for FILE.
autoloads are generated for defuns and defmacros in FILE
marked by `generate-autoload-cookie' (which see).
If FILE is being visited in a buffer, the contents of the buffer
are used." t nil)

(autoload 'update-file-autoloads "autoload" "\
Update the autoloads for FILE in `generated-autoload-file'
\(which FILE might bind in its local variables)." t nil)

(autoload 'update-autoloads-here "autoload" "\
Update sections of the current buffer generated by \\[update-file-autoloads]." t nil)

(autoload 'update-directory-autoloads "autoload" "\
Run \\[update-file-autoloads] on each .el file in DIR." t nil)

(autoload 'batch-update-autoloads "autoload" "\
Update the autoloads for the files or directories on the command line.
Runs \\[update-file-autoloads] on files and \\[update-directory-autoloads]
on directories.  Must be used only with -batch, and kills Emacs on completion.
Each file will be processed even if an error occurred previously.
For example, invoke `emacs -batch -f batch-update-autoloads *.el'." nil nil)

;;;***

;;;### (autoloads nil "blessmail" "utils/blessmail.el" (12559 34936))
;;; Generated autoloads from utils/blessmail.el

;;;***

;;;### (autoloads (browse-url-lynx-emacs browse-url-lynx-xterm browse-url-w3 browse-url-iximosaic browse-url-grail browse-url-mosaic browse-url-netscape) "browse-url" "utils/browse-url.el" (12745 43114))
;;; Generated autoloads from utils/browse-url.el

(defvar browse-url-browser-function 'browse-url-w3 "\
*Function to display the current buffer in a WWW browser.
Used by the `browse-url-at-point', `browse-url-at-mouse', and
`browse-url-of-file' commands.")

(autoload 'browse-url-netscape "browse-url" "\
Ask the Netscape WWW browser to load URL.

Default to the URL around or before point.  The strings in variable
`browse-url-netscape-arguments' are also passed to Netscape.

When called interactively, if variable `browse-url-new-window-p' is
non-nil, load the document in a new Netscape window, otherwise use a
random existing one.  A non-nil interactive prefix argument reverses
the effect of browse-url-new-window-p.

When called non-interactively, optional second argument NEW-WINDOW is
used instead of browse-url-new-window-p." t nil)

(autoload 'browse-url-mosaic "browse-url" "\
Ask the XMosaic WWW browser to load URL.
Default to the URL around or before point." t nil)

(autoload 'browse-url-grail "browse-url" "\
Ask the Grail WWW browser to load URL.
Default to the URL around or before point.  Runs the program in the
variable `browse-url-grail'." t nil)

(autoload 'browse-url-iximosaic "browse-url" "\
Ask the IXIMosaic WWW browser to load URL.
Default to the URL around or before point." t nil)

(autoload 'browse-url-w3 "browse-url" "\
Ask the w3 WWW browser to load URL.
Default to the URL around or before point." t nil)

(autoload 'browse-url-lynx-xterm "browse-url" "\
Ask the Lynx WWW browser to load URL.
Default to the URL around or before point.  A new Lynx process is run
in an Xterm window." t nil)

(autoload 'browse-url-lynx-emacs "browse-url" "\
Ask the Lynx WWW browser to load URL.
Default to the URL around or before point.  Run a new Lynx process in
an Emacs buffer." t nil)

;;;***

;;;### (autoloads nil "crontab" "utils/crontab.el" (12559 34936))
;;; Generated autoloads from utils/crontab.el

;;;***

;;;### (autoloads nil "delbackspace" "utils/delbackspace.el" (12740 50723))
;;; Generated autoloads from utils/delbackspace.el

;;;### (autoloads nil "derived" "utils/derived.el" (12639 8645))
;;; Generated autoloads from utils/derived.el

;;;### (autoloads (docref-setup) "docref" "utils/docref.el" (12559 34937))
;;; Generated autoloads from utils/docref.el

(autoload 'docref-setup "docref" "\
Process docref cross-references in the current buffer.
See also \\(f@docref-subst)." t nil)

;;;***

;;;### (autoloads (easy-menu-define) "easymenu" "utils/easymenu.el" (12686 31729))
;;; Generated autoloads from utils/easymenu.el

(autoload 'easy-menu-define "easymenu" "\
Define a menu bar submenu in maps MAPS, according to MENU.
The arguments SYMBOL and DOC are ignored; they are present for
compatibility only.  SYMBOL is not evaluated.  In other Emacs versions
these arguments may be used as a variable to hold the menu data, and a
doc string for that variable.

The first element of MENU must be a string.  It is the menu bar item name.
The rest of the elements are menu items.

A menu item is usually a vector of three elements:  [NAME CALLBACK ENABLE]

NAME is a string--the menu item name.

CALLBACK is a command to run when the item is chosen,
or a list to evaluate when the item is chosen.

ENABLE is an expression; the item is enabled for selection
whenever this expression's value is non-nil.

Alternatively, a menu item may have the form: 

   [ NAME CALLBACK [ KEYWORD ARG ] ... ]

Where KEYWORD is one of the symbol defined below.

   :keys KEYS

KEYS is a string; a complex keyboard equivalent to this menu item.

   :active ENABLE

ENABLE is an expression; the item is enabled for selection
whenever this expression's value is non-nil.

   :suffix NAME

NAME is a string; the name of an argument to CALLBACK.

   :style STYLE
   
STYLE is a symbol describing the type of menu item.  The following are
defined:  

toggle: A checkbox.  
        Currently just prepend the name with the string \"Toggle \".
radio: A radio button. 
nil: An ordinary menu item.

   :selected SELECTED

SELECTED is an expression; the checkbox or radio button is selected
whenever this expression's value is non-nil.
Currently just disable radio buttons, no effect on checkboxes.

A menu item can be a string.  Then that string appears in the menu as
unselectable text.  A string consisting solely of hyphens is displayed
as a solid horizontal line.

A menu item can be a list.  It is treated as a submenu.
The first element should be the submenu name.  That's used as the
menu item in the top-level menu.  The cdr of the submenu list
is a list of menu items, as above." nil 'macro)

;;;***

;;;### (autoloads (elp-submit-bug-report elp-results elp-instrument-package elp-instrument-list elp-restore-function elp-instrument-function) "elp" "utils/elp.el" (12559 34934))
;;; Generated autoloads from utils/elp.el

(autoload 'elp-instrument-function "elp" "\
Instrument FUNSYM for profiling.
FUNSYM must be a symbol of a defined function." t nil)

(autoload 'elp-restore-function "elp" "\
Restore an instrumented function to its original definition.
Argument FUNSYM is the symbol of a defined function." t nil)

(autoload 'elp-instrument-list "elp" "\
Instrument for profiling, all functions in `elp-function-list'.
Use optional LIST if provided instead." t nil)

(autoload 'elp-instrument-package "elp" "\
Instrument for profiling, all functions which start with PREFIX.
For example, to instrument all ELP functions, do the following:

    \\[elp-instrument-package] RET elp- RET" t nil)

(autoload 'elp-results "elp" "\
Display current profiling results.
If `elp-reset-after-results' is non-nil, then current profiling
information for all instrumented functions are reset after results are
displayed." t nil)

(autoload 'elp-submit-bug-report "elp" "\
Submit via mail, a bug report on elp." t nil)

;;;***

;;;### (autoloads (list-colors-display facemenu-read-color list-text-properties-at facemenu-remove-special facemenu-remove-props facemenu-set-read-only facemenu-set-intangible facemenu-set-invisible facemenu-make-much-smaller facemenu-make-much-larger facemenu-make-smaller facemenu-make-larger facemenu-set-size-default facemenu-set-face-from-menu facemenu-set-background facemenu-set-foreground facemenu-set-face) "facemenu" "utils/facemenu.el" (12657 40747))
;;; Generated autoloads from utils/facemenu.el

(defvar facemenu-menu nil "\
Facemenu top-level menu keymap.")

(defvar facemenu-keymap (let ((map (make-sparse-keymap "Set face"))) (define-key map 111 'facemenu-set-face) map) "\
Keymap for face-changing commands.
`Facemenu-update' fills in the keymap according to the bindings
requested in `facemenu-keybindings'.")

(autoload 'facemenu-set-face "facemenu" "\
Add FACE to the region or next character typed.
It will be added to the top of the face list; any faces lower on the list that
will not show through at all will be removed.

Interactively, the face to be used is read with the minibuffer.

If the region is active and there is no prefix argument,
this command sets the region to the requested face.

Otherwise, this command specifies the face for the next character
inserted.  Moving point or switching buffers before
typing a character to insert cancels the specification." t nil)

(autoload 'facemenu-set-foreground "facemenu" "\
Set the foreground color of the region or next character typed.
The color is prompted for.  A face named `fg:color' is used (or created).
If the region is active, it will be set to the requested face.  If
it is inactive (even if mark-even-if-inactive is set) the next
character that is typed (via `self-insert-command') will be set to
the selected face.  Moving point or switching buffers before
typing a character cancels the request." t nil)

(autoload 'facemenu-set-background "facemenu" "\
Set the background color of the region or next character typed.
The color is prompted for.  A face named `bg:color' is used (or created).
If the region is active, it will be set to the requested face.  If
it is inactive (even if mark-even-if-inactive is set) the next
character that is typed (via `self-insert-command') will be set to
the selected face.  Moving point or switching buffers before
typing a character cancels the request." t nil)

(autoload 'facemenu-set-face-from-menu "facemenu" "\
Set the face of the region or next character typed.
This function is designed to be called from a menu; the face to use
is the menu item's name.

If the region is active and there is no prefix argument,
this command sets the region to the requested face.

Otherwise, this command specifies the face for the next character
inserted.  Moving point or switching buffers before
typing a character to insert cancels the specification." nil nil)

(autoload 'facemenu-set-size-default "facemenu" nil t nil)

(autoload 'facemenu-make-larger "facemenu" nil t nil)

(autoload 'facemenu-make-smaller "facemenu" nil t nil)

(autoload 'facemenu-make-much-larger "facemenu" nil t nil)

(autoload 'facemenu-make-much-smaller "facemenu" nil t nil)

(autoload 'facemenu-set-invisible "facemenu" "\
Make the region invisible.
This sets the `invisible' text property; it can be undone with
`facemenu-remove-special'." t nil)

(autoload 'facemenu-set-intangible "facemenu" "\
Make the region intangible: disallow moving into it.
This sets the `intangible' text property; it can be undone with
`facemenu-remove-special'." t nil)

(autoload 'facemenu-set-read-only "facemenu" "\
Make the region unmodifiable.
This sets the `read-only' text property; it can be undone with
`facemenu-remove-special'." t nil)

(autoload 'facemenu-remove-props "facemenu" "\
Remove all text properties that facemenu added to region." t nil)

(autoload 'facemenu-remove-special "facemenu" "\
Remove all the \"special\" text properties from the region.
These special properties include `invisible', `intangible' and `read-only'." t nil)

(autoload 'list-text-properties-at "facemenu" "\
Pop up a buffer listing text-properties at LOCATION." t nil)

(autoload 'facemenu-read-color "facemenu" "\
Read a color using the minibuffer." nil nil)

(autoload 'list-colors-display "facemenu" "\
Display names of defined colors, and show what they look like.
If the optional argument LIST is non-nil, it should be a list of
colors to display.  Otherwise, this command computes a list
of colors that the current display can handle." t nil)

;;;***

;;;### (autoloads nil "find-gc" "utils/find-gc.el" (12546 50541))
;;; Generated autoloads from utils/find-gc.el

;;;### (autoloads nil "finder-inf" "utils/finder-inf.el" (12376 19577))
;;; Generated autoloads from utils/finder-inf.el

;;;***

;;;### (autoloads nil "finder" "utils/finder.el" (12546 50542))
;;; Generated autoloads from utils/finder.el

;;;***

;;;### (autoloads (enable-flow-control-on enable-flow-control) "flow-ctrl" "utils/flow-ctrl.el" (12639 8648))
;;; Generated autoloads from utils/flow-ctrl.el

(autoload 'enable-flow-control "flow-ctrl" "\
Toggle flow control handling.
When handling is enabled, user can type C-s as C-\\, and C-q as C-^.
With arg, enable flow control mode if arg is positive, otherwise disable." t nil)

(autoload 'enable-flow-control-on "flow-ctrl" "\
Enable flow control if using one of a specified set of terminal types.
Use `(enable-flow-control-on \"vt100\" \"h19\")' to enable flow control
on VT-100 and H19 terminals.  When flow control is enabled,
you must type C-\\ to get the effect of a C-s, and type C-^
to get the effect of a C-q." nil nil)

;;;***

;;;### (autoloads nil "foldout" "utils/foldout.el" (12558 60956))
;;; Generated autoloads from utils/foldout.el

;;;***

;;;### (autoloads nil "forms-d2" "utils/forms-d2.el" (12559 34931))
;;; Generated autoloads from utils/forms-d2.el

;;;***

;;;### (autoloads nil "forms-pass" "utils/forms-pass.el" (12559 34932))
;;; Generated autoloads from utils/forms-pass.el

;;;### (autoloads (forms-find-file-other-window forms-find-file forms-mode) "forms" "utils/forms.el" (12559 34930))
;;; Generated autoloads from utils/forms.el

(autoload 'forms-mode "forms" "\
Major mode to visit files in a field-structured manner using a form.

Commands:                        Equivalent keys in read-only mode:

 TAB          forms-next-field           TAB
 C-c TAB      forms-next-field
 C-c <        forms-first-record         <
 C-c >        forms-last-record          >
 C-c ?        describe-mode              ?
 C-c C-k      forms-delete-record
 C-c C-q      forms-toggle-read-only     q
 C-c C-o      forms-insert-record
 C-c C-l      forms-jump-record          l
 C-c C-n      forms-next-record          n
 C-c C-p      forms-prev-record          p
 C-c C-s      forms-search               s
 C-c C-x      forms-exit                 x" t nil)

(autoload 'forms-find-file "forms" "\
Visit a file in Forms mode." t nil)

(autoload 'forms-find-file-other-window "forms" "\
Visit a file in Forms mode in other window." t nil)

;;;***

;;;### (autoloads nil "hide-copyleft" "utils/hide-copyleft.el" (12743 20032))
;;; Generated autoloads from utils/hide-copyleft.el

;;;### (autoloads (highlight-headers-follow-url highlight-headers-follow-url-mosaic highlight-headers-follow-url-netscape highlight-headers) "highlight-headers" "utils/highlight-headers.el" (12743 22083))
;;; Generated autoloads from utils/highlight-headers.el

(autoload 'highlight-headers "highlight-headers" "\
Highlight message headers between start and end.
Faces used:
  message-headers			the part before the colon
  message-header-contents		the part after the colon
  message-highlighted-header-contents	contents of \"special\" headers
  message-cited-text			quoted text from other messages

Variables used:

  highlight-headers-regexp			what makes a \"special\" header
  highlight-headers-citation-regexp		matches lines of quoted text
  highlight-headers-citation-header-regexp	matches headers for quoted text

If HACK-SIG is true,then we search backward from END for something that
looks like the beginning of a signature block, and don't consider that a
part of the message (this is because signatures are often incorrectly
interpreted as cited text.)" nil nil)

(autoload 'highlight-headers-follow-url-netscape "highlight-headers" nil nil nil)

(autoload 'highlight-headers-follow-url-mosaic "highlight-headers" nil nil nil)

(autoload 'highlight-headers-follow-url "highlight-headers" nil t nil)

;;;***

;;;### (autoloads nil "lib-complete" "utils/lib-complete.el" (12675 57130))
;;; Generated autoloads from utils/lib-complete.el

;;;***

;;;### (autoloads nil "live-icon" "utils/live-icon.el" (12657 40747))
;;; Generated autoloads from utils/live-icon.el

;;;### (autoloads (unload-feature) "loadhist" "utils/loadhist.el" (12558 60957))
;;; Generated autoloads from utils/loadhist.el

(autoload 'unload-feature "loadhist" "\
Unload the library that provided FEATURE, restoring all its autoloads.
If the feature is required by any other loaded code, and optional FORCE
is nil, raise an error." t nil)

;;;***

;;;### (autoloads (what-domain mail-extract-address-components) "mail-extr" "utils/mail-extr.el" (12743 22084))
;;; Generated autoloads from utils/mail-extr.el

(autoload 'mail-extract-address-components "mail-extr" "\
Given an RFC-822 ADDRESS, extract full name and canonical address.
Returns a list of the form (FULL-NAME CANONICAL-ADDRESS).
If no name can be extracted, FULL-NAME will be nil.
ADDRESS may be a string or a buffer.  If it is a buffer, the visible 
 (narrowed) portion of the buffer will be interpreted as the address.
 (This feature exists so that the clever caller might be able to avoid
 consing a string.)
If ADDRESS contains more than one RFC-822 address, only the first is
 returned.  Some day this function may be extended to extract multiple
 addresses, or perhaps return the position at which parsing stopped." nil nil)

(autoload 'what-domain "mail-extr" "\
Prompts for a mail domain, and prints the country it corresponds to
in the minibuffer." t nil)

;;;***

;;;### (autoloads (mail-fetch-field mail-file-babyl-p) "mail-utils" "utils/mail-utils.el" (12743 11630))
;;; Generated autoloads from utils/mail-utils.el

(defvar mail-use-rfc822 nil "\
*If non-nil, use a full, hairy RFC822 parser on mail addresses.
Otherwise, (the default) use a smaller, somewhat faster, and
often correct parser.")

(autoload 'mail-file-babyl-p "mail-utils" nil nil nil)

(autoload 'mail-fetch-field "mail-utils" "\
Return the value of the header field FIELD-NAME.
The buffer is expected to be narrowed to just the headers of the message.
If second arg LAST is non-nil, use the last such field if there are several.
If third arg ALL is non-nil, concatenate all such fields with commas between." nil nil)

;;;***

;;;### (autoloads nil "mailpost" "utils/mailpost.el" (12559 46922))
;;; Generated autoloads from utils/mailpost.el

;;;***

;;;### (autoloads nil "map-ynp" "utils/map-ynp.el" (12740 25222))
;;; Generated autoloads from utils/map-ynp.el

;;;### (autoloads nil "meese" "utils/meese.el" (12559 34917))
;;; Generated autoloads from utils/meese.el

;;;### (autoloads (read-passwd) "passwd" "utils/passwd.el" (12559 34928))
;;; Generated autoloads from utils/passwd.el

(autoload 'read-passwd "passwd" "\
Prompts for a password in the minibuffer, and returns it as a string.
If PROMPT may be a prompt string or an alist of elements 
'(prompt . default).
If optional arg CONFIRM is true, then ask the user to type the password
again to confirm that they typed it correctly.
If optional arg DEFAULT is provided, then it is a string to insert as
the default choice (it is not, of course, displayed.)

If running under X, the keyboard will be grabbed (with XGrabKeyboard())
to reduce the possibility that evesdropping is occuring.

When reading a password, all keys self-insert, except for:
\\<read-passwd-map>
	\\[read-passwd-erase-line]	Erase the entire line.
	\\[quoted-insert]	Insert the next character literally.
	\\[delete-backward-char]	Delete the previous character.
	\\[exit-minibuffer]	Accept what you have typed.
	\\[keyboard-quit]	Abort the command.

The returned value is always a newly-created string.  No additional copies
of the password remain after this function has returned.

NOTE: unless great care is taken, the typed password will exist in plaintext
form in the running image for an arbitrarily long time.  Priveleged users may
be able to extract it from memory.  If emacs crashes, it may appear in the
resultant core file.

Some steps you can take to prevent the password from being copied around:

 - as soon as you are done with the returned string, destroy it with
   (fillarray string 0).  The same goes for any default passwords
   or password histories.

 - do not copy the string, as with concat or substring - if you do, be
   sure to keep track of and destroy all copies.

 - do not insert the password into a buffer - if you do, be sure to 
   overwrite the buffer text before killing it, as with the functions 
   `passwd-erase-buffer' or `passwd-kill-buffer'.  Note that deleting
   the text from the buffer does NOT necessarily remove the text from
   memory.

 - be careful of the undo history - if you insert the password into a 
   buffer which has undo recording turned on, the password will be 
   copied onto the undo list, and thus recoverable.

 - do not pass it as an argument to a shell command - anyone will be
   able to see it if they run `ps' at the right time.

Note that the password will be temporarily recoverable with the `view-lossage'
command.  This data will not be overwritten until another hundred or so 
characters are typed.  There's not currently a way around this." nil nil)

;;;***

;;;### (autoloads (pp) "pp" "utils/pp.el" (12559 34933))
;;; Generated autoloads from utils/pp.el

(autoload 'pp "pp" "\
Output the pretty-printed representation of OBJECT, any Lisp object.
Quoting characters are printed when needed to make output that `read'
can handle, whenever this is possible.
Output stream is STREAM, or value of `standard-output' (which see)." nil nil)

(defalias 'pprint 'pp)

;;;***

;;;### (autoloads nil "regi" "utils/regi.el" (12558 60951))
;;; Generated autoloads from utils/regi.el

;;;***

;;;### (autoloads (reporter-submit-bug-report) "reporter" "utils/reporter.el" (12559 34929))
;;; Generated autoloads from utils/reporter.el

(autoload 'reporter-submit-bug-report "reporter" nil nil nil)

;;;***

;;;### (autoloads nil "rfc822" "utils/rfc822.el" (12559 34920))
;;; Generated autoloads from utils/rfc822.el

;;;***

;;;### (autoloads nil "shadowfile" "utils/shadowfile.el" (12558 60958))
;;; Generated autoloads from utils/shadowfile.el

;;;### (autoloads (skeleton-pair-insert-maybe skeleton-insert skeleton-proxy define-skeleton) "skeleton" "utils/skeleton.el" (12675 57223))
;;; Generated autoloads from utils/skeleton.el

(defvar skeleton-filter 'identity "\
Function for transforming a skeleton-proxy's aliases' variable value.")

(autoload 'define-skeleton "skeleton" "\
Define a user-configurable COMMAND that enters a statement skeleton.
DOCUMENTATION is that of the command, while the variable of the same name,
which contains the skeleton, has a documentation to that effect.
INTERACTOR and ELEMENT ... are as defined under `skeleton-insert'." nil 'macro)

(autoload 'skeleton-proxy "skeleton" "\
Insert skeleton defined by variable of same name (see `skeleton-insert').
Prefix ARG allows wrapping around words or regions (see `skeleton-insert').
This command can also be an abbrev expansion (3rd and 4th columns in
\\[edit-abbrevs]  buffer: \"\"  command-name).

When called as a function, optional first argument STR may also be a string
which will be the value of `str' whereas the skeleton's interactor is then
ignored." t nil)

(autoload 'skeleton-insert "skeleton" "\
Insert the complex statement skeleton SKELETON describes very concisely.

With optional third REGIONS wrap first interesting point (`_') in skeleton
around next REGIONS words, if REGIONS is positive.  If REGIONS is negative,
wrap REGIONS preceding interregions into first REGIONS interesting positions
\(successive `_'s) in skeleton.  An interregion is the stretch of text between
two contiguous marked points.  If you marked A B C [] (where [] is the cursor)
in alphabetical order, the 3 interregions are simply the last 3 regions.  But
if you marked B A [] C, the interregions are B-A, A-[], []-C.

Optional fourth STR is the value for the variable `str' within the skeleton.
When this is non-`nil' the interactor gets ignored, and this should be a valid
skeleton element.

SKELETON is made up as (INTERACTOR ELEMENT ...).  INTERACTOR may be nil if
not needed, a prompt-string or an expression for complex read functions.

If ELEMENT is a string or a character it gets inserted (see also
`skeleton-transformation').  Other possibilities are:

	\\n	go to next line and indent according to mode
	_	interesting point, interregion here, point after termination
	>	indent line (or interregion if > _) according to major mode
	&	do next ELEMENT if previous moved point
	|	do next ELEMENT if previous didn't move point
	-num	delete num preceding characters (see `skeleton-untabify')
	resume:	skipped, continue here if quit is signaled
	nil	skipped

Further elements can be defined via `skeleton-further-elements'.  ELEMENT may
itself be a SKELETON with an INTERACTOR.  The user is prompted repeatedly for
different inputs.  The SKELETON is processed as often as the user enters a
non-empty string.  \\[keyboard-quit] terminates skeleton insertion, but
continues after `resume:' and positions at `_' if any.  If INTERACTOR in such
a subskeleton is a prompt-string which contains a \".. %s ..\" it is
formatted with `skeleton-subprompt'.  Such an INTERACTOR may also a list of
strings with the subskeleton being repeated once for each string.

Quoted lisp-expressions are evaluated evaluated for their side-effect.
Other lisp-expressions are evaluated and the value treated as above.
Note that expressions may not return `t' since this impplies an
endless loop.  Modes can define other symbols by locally setting them
to any valid skeleton element.  The following local variables are
available:

	str	first time: read a string according to INTERACTOR
		then: insert previously read string once more
	help	help-form during interaction with the user or `nil'
	input	initial input (string or cons with index) while reading str
	v1, v2	local variables for memorising anything you want

When done with skeleton, but before going back to `_'-point call
`skeleton-end-hook' if that is non-`nil'." nil nil)

(autoload 'skeleton-pair-insert-maybe "skeleton" "\
Insert the character you type ARG times.

With no ARG, if `skeleton-pair' is non-nil, and if
`skeleton-pair-on-word' is non-nil or we are not before or inside a
word, and if `skeleton-pair-filter' returns nil, pairing is performed.

If a match is found in `skeleton-pair-alist', that is inserted, else
the defaults are used.  These are (), [], {}, <> and `' for the
symmetrical ones, and the same character twice for the others." t nil)

;;;***

;;;### (autoloads nil "soundex" "utils/soundex.el" (12558 60959))
;;; Generated autoloads from utils/soundex.el

;;;***

;;;### (autoloads nil "symbol-syntax" "utils/symbol-syntax.el" (12573 6370))
;;; Generated autoloads from utils/symbol-syntax.el

;;;***

;;;### (autoloads nil "sysdep" "utils/sysdep.el" (12714 11722))
;;; Generated autoloads from utils/sysdep.el

;;;### (autoloads nil "text-props" "utils/text-props.el" (12639 8651))
;;; Generated autoloads from utils/text-props.el

;;;***

;;;### (autoloads nil "thing" "utils/thing.el" (12558 60956))
;;; Generated autoloads from utils/thing.el

;;;***

;;;### (autoloads nil "timezone" "utils/timezone.el" (12546 50543))
;;; Generated autoloads from utils/timezone.el

;;;### (autoloads (tq-create) "tq" "utils/tq.el" (12558 60960))
;;; Generated autoloads from utils/tq.el

(autoload 'tq-create "tq" "\
Create and return a transaction queue communicating with PROCESS.
PROCESS should be a subprocess capable of sending and receiving
streams of bytes.  It may be a local process, or it may be connected
to a tcp server on another machine." nil nil)

;;;***

;;;### (autoloads (trace-function-background trace-function) "trace" "utils/trace.el" (12546 50540))
;;; Generated autoloads from utils/trace.el

(defvar trace-buffer "*trace-output*" "\
*Trace output will by default go to that buffer.")

(autoload 'trace-function "trace" "\
Traces FUNCTION with trace output going to BUFFER.
For every call of FUNCTION Lisp-style trace messages that display argument
and return values will be inserted into BUFFER. This function generates the
trace advice for FUNCTION and activates it together with any other advice
there might be!! The trace BUFFER will popup whenever FUNCTION is called.
Do not use this to trace functions that switch buffers or do any other
display oriented stuff, use `trace-function-background' instead." t nil)

(autoload 'trace-function-background "trace" "\
Traces FUNCTION with trace output going quietly to BUFFER.
For every call of FUNCTION Lisp-style trace messages that display argument
and return values will be inserted into BUFFER. This function generates the
trace advice for FUNCTION and activates it together with any other advice
there might be!! Trace output will quietly go to BUFFER without changing
the window or buffer configuration at all." t nil)

;;;***

;;;### (autoloads nil "tree-menu" "utils/tree-menu.el" (12558 60954))
;;; Generated autoloads from utils/tree-menu.el

;;;### (autoloads (y-or-n-p-with-timeout yes-or-no-p-with-timeout with-timeout with-timeout-internal) "with-timeout" "utils/with-timeout.el" (12558 60948))
;;; Generated autoloads from utils/with-timeout.el

(autoload 'with-timeout-internal "with-timeout" nil nil nil)

(autoload 'with-timeout "with-timeout" "\
Usage: (with-timeout (seconds &rest timeout-forms) &rest body)
This is just like progn, but if the given number of seconds expires before
the body returns, then timeout-forms are evaluated and returned instead.
The body won't be interrupted in the middle of a computation: the check for 
the timer expiration only occurs when body does a redisplay, or prompts the
user for input, or calls accept-process-output." nil 'macro)

(autoload 'yes-or-no-p-with-timeout "with-timeout" "\
Just like yes-or-no-p, but will time out after TIMEOUT seconds
if the user has not yes answered, returning DEFAULT-VALUE." nil nil)

(autoload 'y-or-n-p-with-timeout "with-timeout" "\
Just like y-or-n-p, but will time out after TIMEOUT seconds
if the user has not yes answered, returning DEFAULT-VALUE." nil nil)

;;;***

;;;### (autoloads (xbm-button-create) "xbm-button" "utils/xbm-button.el" (12639 8650))
;;; Generated autoloads from utils/xbm-button.el

(autoload 'xbm-button-create "xbm-button" "\
Returns a list of XBM image instantiators for a button displaying TEXT.
The list is of the form
   (UP DOWN DISABLED)
where UP, DOWN, and DISABLED are the up, down and disabled image
instantiators for the button.

BORDER-THICKNESS specifies how many pixels should be used for the
borders on the edges of the buttons.  It should be a positive integer,
or 0 to mean no border." nil nil)

;;;***

;;;### (autoloads (xpm-button-create) "xpm-button" "utils/xpm-button.el" (12648 18605))
;;; Generated autoloads from utils/xpm-button.el

(autoload 'xpm-button-create "xpm-button" "\
Returns a list of XPM image instantiators for a button displaying TEXT.
The list is of the form
   (UP DOWN DISABLED)
where UP, DOWN, and DISABLED are the up, down and disabled image
instantiators for the button.

SHADOW-THICKNESS specifies how many pixels should be used for the
shadows on the edges of the buttons.  It should be a positive integer,
or 0 to mean no shadows on the edges.
FG-COLOR is the color used to display the text.  It should be a string.
BG-COLOR is the background color the text will be displayed upon.
It should be a string." nil nil)

;;;***

;;;### (autoloads nil "viper-ex" "viper/viper-ex.el" (12745 61913))
;;; Generated autoloads from viper/viper-ex.el

;;;***

;;;### (autoloads nil "viper-keym" "viper/viper-keym.el" (12639 8723))
;;; Generated autoloads from viper/viper-keym.el

;;;### (autoloads nil "viper-macs" "viper/viper-macs.el" (12728 13099))
;;; Generated autoloads from viper/viper-macs.el

;;;***

;;;### (autoloads nil "viper-mous" "viper/viper-mous.el" (12728 13099))
;;; Generated autoloads from viper/viper-mous.el

;;;***

;;;### (autoloads nil "viper-util" "viper/viper-util.el" (12745 61913))
;;; Generated autoloads from viper/viper-util.el

;;;### (autoloads (viper-mode) "viper" "viper/viper.el" (12745 61914))
;;; Generated autoloads from viper/viper.el

(autoload 'viper-mode "viper" "\
Turn on Viper emulation of Vi." t nil)

(defalias 'vip-mode 'viper-mode)

;;;***

;;;### (autoloads nil "tapestry" "vm/tapestry.el" (12376 19602))
;;; Generated autoloads from vm/tapestry.el

;;;***

;;;### (autoloads nil "vm-autoload" "vm/vm-autoload.el" (12744 48175))
;;; Generated autoloads from vm/vm-autoload.el

;;;***

;;;### (autoloads nil "vm-byteopts" "vm/vm-byteopts.el" (12244 51680))
;;; Generated autoloads from vm/vm-byteopts.el

;;;### (autoloads nil "vm-delete" "vm/vm-delete.el" (12740 28348))
;;; Generated autoloads from vm/vm-delete.el

;;;### (autoloads nil "vm-digest" "vm/vm-digest.el" (12376 19586))
;;; Generated autoloads from vm/vm-digest.el

;;;### (autoloads (vm-easy-menu-create-keymaps vm-easy-menu-define) "vm-easymenu" "vm/vm-easymenu.el" (12376 19602))
;;; Generated autoloads from vm/vm-easymenu.el

(autoload 'vm-easy-menu-define "vm-easymenu" "\
Define a menu bar submenu in maps MAPS, according to MENU.
The menu keymap is stored in symbol SYMBOL, both as its value
and as its function definition.   DOC is used as the doc string for SYMBOL.

The first element of MENU must be a string.  It is the menu bar item name.
The rest of the elements are menu items.

A menu item is usually a vector of three elements:  [NAME CALLBACK ENABLE]

NAME is a string--the menu item name.

CALLBACK is a command to run when the item is chosen,
or a list to evaluate when the item is chosen.

ENABLE is an expression; the item is enabled for selection
whenever this expression's value is non-nil.

Alternatively, a menu item may have the form: 

   [ NAME CALLBACK [ KEYWORD ARG ] ... ]

Where KEYWORD is one of the symbol defined below.

   :keys KEYS

KEYS is a string; a complex keyboard equivalent to this menu item.
This is normally not needed because keyboard equivalents are usually
computed automatically.

   :active ENABLE

ENABLE is an expression; the item is enabled for selection
whenever this expression's value is non-nil.

   :suffix NAME

NAME is a string; the name of an argument to CALLBACK.

   :style 
   
STYLE is a symbol describing the type of menu item.  The following are
defined:  

toggle: A checkbox.  
        Currently just prepend the name with the string \"Toggle \".
radio: A radio button. 
nil: An ordinary menu item.

   :selected SELECTED

SELECTED is an expression; the checkbox or radio button is selected
whenever this expression's value is non-nil.
Currently just disable radio buttons, no effect on checkboxes.

A menu item can be a string.  Then that string appears in the menu as
unselectable text.  A string consisting solely of hyphens is displayed
as a solid horizontal line.

A menu item can be a list.  It is treated as a submenu.
The first element should be the submenu name.  That's used as the
menu item in the top-level menu.  The cdr of the submenu list
is a list of menu items, as above." nil 'macro)

(autoload 'vm-easy-menu-create-keymaps "vm-easymenu" nil nil nil)

;;;***

;;;### (autoloads nil "vm-edit" "vm/vm-edit.el" (12376 19587))
;;; Generated autoloads from vm/vm-edit.el

;;;***

;;;### (autoloads nil "vm-folder" "vm/vm-folder.el" (12740 28349))
;;; Generated autoloads from vm/vm-folder.el

;;;### (autoloads nil "vm-license" "vm/vm-license.el" (12376 19590))
;;; Generated autoloads from vm/vm-license.el

;;;***

;;;### (autoloads nil "vm-mark" "vm/vm-mark.el" (12376 19590))
;;; Generated autoloads from vm/vm-mark.el

;;;***

;;;### (autoloads nil "vm-menu" "vm/vm-menu.el" (12376 19592))
;;; Generated autoloads from vm/vm-menu.el

;;;***

;;;### (autoloads nil "vm-message" "vm/vm-message.el" (12376 19589))
;;; Generated autoloads from vm/vm-message.el

;;;***

;;;### (autoloads nil "vm-minibuf" "vm/vm-minibuf.el" (12376 19600))
;;; Generated autoloads from vm/vm-minibuf.el

;;;***

;;;### (autoloads nil "vm-misc" "vm/vm-misc.el" (12740 28349))
;;; Generated autoloads from vm/vm-misc.el

;;;***

;;;### (autoloads nil "vm-motion" "vm/vm-motion.el" (12740 28349))
;;; Generated autoloads from vm/vm-motion.el

;;;### (autoloads nil "vm-mouse" "vm/vm-mouse.el" (12376 19602))
;;; Generated autoloads from vm/vm-mouse.el

;;;***

;;;### (autoloads nil "vm-page" "vm/vm-page.el" (12376 19594))
;;; Generated autoloads from vm/vm-page.el

;;;***

;;;### (autoloads nil "vm-pop" "vm/vm-pop.el" (12740 28349))
;;; Generated autoloads from vm/vm-pop.el

;;;### (autoloads nil "vm-reply" "vm/vm-reply.el" (12376 19587))
;;; Generated autoloads from vm/vm-reply.el

;;;***

;;;### (autoloads nil "vm-save" "vm/vm-save.el" (12376 19592))
;;; Generated autoloads from vm/vm-save.el

;;;***

;;;### (autoloads nil "vm-search" "vm/vm-search.el" (12740 30223))
;;; Generated autoloads from vm/vm-search.el

;;;### (autoloads nil "vm-search18" "vm/vm-search18.el" (12244 51678))
;;; Generated autoloads from vm/vm-search18.el

;;;### (autoloads nil "vm-search19" "vm/vm-search19.el" (12376 19598))
;;; Generated autoloads from vm/vm-search19.el

;;;***

;;;### (autoloads nil "vm-sort" "vm/vm-sort.el" (12376 19598))
;;; Generated autoloads from vm/vm-sort.el

;;;***

;;;### (autoloads nil "vm-startup" "vm/vm-startup.el" (12740 35805))
;;; Generated autoloads from vm/vm-startup.el

;;;***

;;;### (autoloads nil "vm-summary" "vm/vm-summary.el" (12740 28349))
;;; Generated autoloads from vm/vm-summary.el

;;;### (autoloads nil "vm-thread" "vm/vm-thread.el" (12376 19602))
;;; Generated autoloads from vm/vm-thread.el

;;;***

;;;### (autoloads nil "vm-toolbar" "vm/vm-toolbar.el" (12714 11723))
;;; Generated autoloads from vm/vm-toolbar.el

;;;***

;;;### (autoloads nil "vm-undo" "vm/vm-undo.el" (12244 51671))
;;; Generated autoloads from vm/vm-undo.el

;;;***

;;;### (autoloads nil "vm-vars" "vm/vm-vars.el" (12740 28350))
;;; Generated autoloads from vm/vm-vars.el

;;;***

;;;### (autoloads nil "vm-version" "vm/vm-version.el" (12740 28351))
;;; Generated autoloads from vm/vm-version.el

;;;***

;;;### (autoloads nil "vm-virtual" "vm/vm-virtual.el" (12312 61353))
;;; Generated autoloads from vm/vm-virtual.el

;;;***

;;;### (autoloads nil "vm-window" "vm/vm-window.el" (12740 28350))
;;; Generated autoloads from vm/vm-window.el

;;;***

;;;### (autoloads nil "docomp" "w3/docomp.el" (12727 30855))
;;; Generated autoloads from w3/docomp.el

;;;***

;;;### (autoloads nil "font" "w3/font.el" (12720 56497))
;;; Generated autoloads from w3/font.el

;;;***

;;;### (autoloads nil "images" "w3/images.el" (12714 11732))
;;; Generated autoloads from w3/images.el

;;;***

;;;### (autoloads nil "w3-about" "w3/w3-about.el" (12745 49143))
;;; Generated autoloads from w3/w3-about.el

;;;### (autoloads nil "w3-annotat" "w3/w3-annotat.el" (12727 30858))
;;; Generated autoloads from w3/w3-annotat.el

;;;***

;;;### (autoloads nil "w3-auto" "w3/w3-auto.el" (12727 30865))
;;; Generated autoloads from w3/w3-auto.el

;;;### (autoloads nil "w3-beta" "w3/w3-beta.el" (12686 31748))
;;; Generated autoloads from w3/w3-beta.el

;;;***

;;;### (autoloads nil "w3-draw" "w3/w3-draw.el" (12727 30859))
;;; Generated autoloads from w3/w3-draw.el

;;;***

;;;### (autoloads nil "w3-e19" "w3/w3-e19.el" (12727 30856))
;;; Generated autoloads from w3/w3-e19.el

;;;***

;;;### (autoloads nil "w3-emulate" "w3/w3-emulate.el" (12714 11733))
;;; Generated autoloads from w3/w3-emulate.el

;;;***

;;;### (autoloads nil "w3-forms" "w3/w3-forms.el" (12740 27003))
;;; Generated autoloads from w3/w3-forms.el

;;;***

;;;### (autoloads (w3-use-hotlist) "w3-hot" "w3/w3-hot.el" (12727 30861))
;;; Generated autoloads from w3/w3-hot.el

(autoload 'w3-use-hotlist "w3-hot" "\
Possibly go to a link in your W3/Mosaic hotlist.
This is part of the emacs World Wide Web browser.  It will prompt for
one of the items in your 'hotlist'.  A hotlist is a list of often
visited or interesting items you have found on the World Wide Web." t nil)

;;;***

;;;### (autoloads nil "w3-imap" "w3/w3-imap.el" (12715 10457))
;;; Generated autoloads from w3/w3-imap.el

;;;***

;;;### (autoloads nil "w3-keyword" "w3/w3-keyword.el" (12727 30860))
;;; Generated autoloads from w3/w3-keyword.el

;;;***

;;;### (autoloads nil "w3-latex" "w3/w3-latex.el" (12727 30593))
;;; Generated autoloads from w3/w3-latex.el

;;;***

;;;### (autoloads nil "w3-menu" "w3/w3-menu.el" (12727 30864))
;;; Generated autoloads from w3/w3-menu.el

;;;### (autoloads nil "w3-mouse" "w3/w3-mouse.el" (12720 56526))
;;; Generated autoloads from w3/w3-mouse.el

;;;***

;;;### (autoloads nil "w3-mule" "w3/w3-mule.el" (12686 31739))
;;; Generated autoloads from w3/w3-mule.el

;;;***

;;;### (autoloads nil "w3-parse" "w3/w3-parse.el" (12727 30863))
;;; Generated autoloads from w3/w3-parse.el

;;;***

;;;### (autoloads nil "w3-prefs" "w3/w3-prefs.el" (12740 27017))
;;; Generated autoloads from w3/w3-prefs.el

;;;### (autoloads nil "w3-print" "w3/w3-print.el" (12727 30863))
;;; Generated autoloads from w3/w3-print.el

;;;***

;;;### (autoloads nil "w3-speak" "w3/w3-speak.el" (12740 27018))
;;; Generated autoloads from w3/w3-speak.el

;;;### (autoloads nil "w3-style" "w3/w3-style.el" (12727 30864))
;;; Generated autoloads from w3/w3-style.el

;;;***

;;;### (autoloads nil "w3-sysdp" "w3/w3-sysdp.el" (12727 30857))
;;; Generated autoloads from w3/w3-sysdp.el

;;;### (autoloads nil "w3-toolbar" "w3/w3-toolbar.el" (12686 31734))
;;; Generated autoloads from w3/w3-toolbar.el

;;;***

;;;### (autoloads nil "w3-vars" "w3/w3-vars.el" (12745 49201))
;;; Generated autoloads from w3/w3-vars.el

;;;***

;;;### (autoloads nil "w3-widget" "w3/w3-widget.el" (12720 56504))
;;; Generated autoloads from w3/w3-widget.el

;;;### (autoloads nil "w3-xemac" "w3/w3-xemac.el" (12727 30860))
;;; Generated autoloads from w3/w3-xemac.el

;;;### (autoloads (w3-follow-link w3-follow-link-other-frame w3-do-setup w3 w3-preview-this-buffer w3-batch-fetch w3-follow-url-at-point w3-follow-url-at-point-other-frame w3-maybe-follow-link w3-maybe-follow-link-mouse w3-fetch w3-fetch-other-frame w3-find-file w3-open-local) "w3" "w3/w3.el" (12740 27016))
;;; Generated autoloads from w3/w3.el

(autoload 'w3-open-local "w3" "\
Find a local file, and interpret it as a hypertext document.
It will prompt for an existing file or directory, and retrieve it as a
hypertext document.  If it is a directory, and url-use-hypertext-dired
is non-nil, then an HTML directory listing is created on the fly.
Otherwise, dired-mode is used to visit the buffer." t nil)

(autoload 'w3-find-file "w3" "\
Find a local file, and interpret it as a hypertext document.
It will prompt for an existing file or directory, and retrieve it as a
hypertext document.  If it is a directory, and url-use-hypertext-dired
is non-nil, then an HTML directory listing is created on the fly.
Otherwise, dired-mode is used to visit the buffer." t nil)

(autoload 'w3-fetch-other-frame "w3" "\
Attempt to follow the hypertext reference under point in a new frame.
With prefix-arg P, ignore viewers and dump the link straight
to disk." t nil)

(autoload 'w3-fetch "w3" "\
Retrieve a document over the World Wide Web.
The World Wide Web is a global hypertext system started by CERN in
Switzerland in 1991.

The document should be specified by its fully specified
Uniform Resource Locator.  The document will be parsed, printed, or
passed to an external viewer as appropriate.  Variable
`mm-mime-info' specifies viewers for particular file types." t nil)

(autoload 'w3-maybe-follow-link-mouse "w3" "\
Maybe follow a hypertext link under point.
If there is no link under point, this will try using
url-get-url-at-point" t nil)

(autoload 'w3-maybe-follow-link "w3" "\
Maybe follow a hypertext link under point.
If there is no link under point, this will try using
url-get-url-at-point" t nil)

(autoload 'w3-follow-url-at-point-other-frame "w3" "\
Follow the URL under PT, defaults to link under (point)" t nil)

(autoload 'w3-follow-url-at-point "w3" "\
Follow the URL under PT, defaults to link under (point)" t nil)

(autoload 'w3-batch-fetch "w3" "\
Fetch all the URLs on the command line and save them to files in
the current directory.  The first argument after the -f w3-batch-fetch
on the command line should be a string specifying how to save the
information retrieved.  If it is \"html\", then the page will be
unformatted when it is written to disk.  If it is \"text\", then the
page will be formatted before it is written to disk.  If it is
\"binary\" it will not mess with the file extensions, and just save
the data in raw binary format.  If none of those, the default is
\"text\", and the first argument is treated as a normal URL." nil nil)

(autoload 'w3-preview-this-buffer "w3" "\
See what this buffer will look like when its formatted as HTML.
HTML is the HyperText Markup Language used by the World Wide Web to
specify formatting for text.  More information on HTML can be found at
ftp.w3.org:/pub/www/doc." t nil)

(autoload 'w3 "w3" "\
Retrieve the default World Wide Web home page.
The World Wide Web is a global hypertext system started by CERN in
Switzerland in 1991.

The home page is specified by the variable w3-default-homepage.  The
document should be specified by its fully specified Uniform Resource
Locator.  The document will be parsed as HTML (if appropriate) and
displayed in a new buffer." t nil)

(autoload 'w3-do-setup "w3" "\
Do setup - this is to avoid conflict with user settings when W3 is
dumped with emacs." nil nil)

(autoload 'w3-follow-link-other-frame "w3" "\
Attempt to follow the hypertext reference under point in a new frame.
With prefix-arg P, ignore viewers and dump the link straight
to disk." nil nil)

(autoload 'w3-follow-link "w3" "\
Attempt to follow the hypertext reference under point.
With prefix-arg P, ignore viewers and dump the link straight
to disk." t nil)

;;;***

;;;### (autoloads nil "widget-edit" "w3/widget-edit.el" (12740 27017))
;;; Generated autoloads from w3/widget-edit.el

;;;***

;;;### (autoloads nil "widget" "w3/widget.el" (12720 56523))
;;; Generated autoloads from w3/widget.el

;;;### (autoloads nil "x-compose" "x11/x-compose.el" (12202 12100))
;;; Generated autoloads from x11/x-compose.el

;;;### (autoloads nil "x-faces" "x11/x-faces.el" (12698 33571))
;;; Generated autoloads from x11/x-faces.el

;;;***

;;;### (autoloads (font-menu-weight-constructor font-menu-size-constructor font-menu-family-constructor reset-device-font-menus) "x-font-menu" "x11/x-font-menu.el" (12675 57137))
;;; Generated autoloads from x11/x-font-menu.el

(defvar font-menu-ignore-scaled-fonts t "\
*If non-nil, then the font menu will try to show only bitmap fonts.")

(defvar font-menu-this-frame-only-p t "\
*If non-nil, then changing the default font from the font menu will only
affect one frame instead of all frames.")

(fset 'install-font-menus 'reset-device-font-menus)

(autoload 'reset-device-font-menus "x-font-menu" "\
Generates the `Font', `Size', and `Weight' submenus for the Options menu.
This is run the first time that a font-menu is needed for each device.
If you don't like the lazy invocation of this function, you can add it to
`create-device-hook' and that will make the font menus respond more quickly
when they are selected for the first time.  If you add fonts to your system, 
or if you change your font path, you can call this to re-initialize the menus." nil nil)

(autoload 'font-menu-family-constructor "x-font-menu" nil nil nil)

(autoload 'font-menu-size-constructor "x-font-menu" nil nil nil)

(autoload 'font-menu-weight-constructor "x-font-menu" nil nil nil)

;;;***

;;;### (autoloads nil "x-init" "x11/x-init.el" (12698 33574))
;;; Generated autoloads from x11/x-init.el

;;;### (autoloads nil "x-iso8859-1" "x11/x-iso8859-1.el" (12639 8655))
;;; Generated autoloads from x11/x-iso8859-1.el

;;;***

;;;### (autoloads nil "x-menubar" "x11/x-menubar.el" (12743 24793))
;;; Generated autoloads from x11/x-menubar.el

;;;### (autoloads nil "x-misc" "x11/x-misc.el" (12639 8659))
;;; Generated autoloads from x11/x-misc.el

;;;### (autoloads nil "x-mouse" "x11/x-mouse.el" (12698 33572))
;;; Generated autoloads from x11/x-mouse.el

;;;### (autoloads nil "x-scrollbar" "x11/x-scrollbar.el" (12639 8659))
;;; Generated autoloads from x11/x-scrollbar.el

;;;***

;;;### (autoloads nil "x-select" "x11/x-select.el" (12546 50544))
;;; Generated autoloads from x11/x-select.el

;;;***

;;;### (autoloads nil "x-toolbar" "x11/x-toolbar.el" (12714 11727))
;;; Generated autoloads from x11/x-toolbar.el

;;;### (autoloads nil "x-win-sun" "x11/x-win-sun.el" (12690 2180))
;;; Generated autoloads from x11/x-win-sun.el

;;;### (autoloads nil "x-win-xfree86" "x11/x-win-xfree86.el" (12639 8662))
;;; Generated autoloads from x11/x-win-xfree86.el

;;; Generated autoloads from x11/x-win-sun.el

;;; Don't make backup versions of this file - most of it is generated
;;; automatically by autoload.el, and what isn't changes rarely.
;;; Local Variables:
;;; version-control: never
;;; no-byte-compile: t
;;; no-update-autoloads: t
;;; End:
;;; loaddefs.el ends here
